<?php
/**
 * Created by PhpStorm.
 * User: leonardocarvalho
 * Date: 29/02/2016
 * Time: 17:19
 */

namespace HCM\BTL\Services;

use Doctrine\ORM\EntityManager;
use Geral\Util\DateTime;
use HCM\Util\DateUtil;
use Teknisa\Libs\Util\Environment;
use HCM\Util\HCM;
use HCM\Util\Util;
use HCM\Util\HCMParameters;

class ProcessosService {

    const SET = 'set';
    const TIPOPARCEIRO_CANDIDATO = 'CANDIDATO';

    protected $entityManager;
    /** @var Environment */
    protected $environment;
    protected $hcm;
    protected $hcmParameters;

    public function __construct(EntityManager $entityManager, Environment $environment, HCM $hcm, HCMParameters $hcmParameters) {
        $this->entityManager = $entityManager;
        $this->environment = $environment;
        $this->hcm = $hcm;
        $this->hcmParameters = $hcmParameters;
    }

    public function getProcessos($parameters, $filters) {
        if(array_key_exists(0,$filters))
            if($filters[0]['columnName'] == "id")
                $filters = array();
        
        $filterPlaceHolder = Util::createQueryFiltered($filters);
        
        $nrorg = $this->hcmParameters->getNrorgTrabalho();
        $nrorgpadrao       = $this->hcmParameters->getNrorgPadrao();
        $dtiniciomes       = $this->hcmParameters->getPrimeiroDiaDoMes();
        $dtfimmes          = $this->hcmParameters->getUltimoDiaMes();
        $filterDashboard = "";
        $idStatusCancelado = 7;
        $idStatusSuspenso = 9;

        if(isset($parameters['FILTRO'])){
          if($parameters['FILTRO'] == 'SUSPENSO'){
            $filterDashboard = "(IDSTATUS = '$idStatusSuspenso' OR IDSTATUS IS NULL) AND DTSUSPENSAO BETWEEN '$dtiniciomes' AND '$dtfimmes'";
          } else if($parameters['FILTRO'] == 'CANCELADO'){
            $filterDashboard = "(IDSTATUS = '$idStatusCancelado' OR IDSTATUS IS NULL) AND DTCANCELAMENTO BETWEEN '$dtiniciomes' AND '$dtfimmes'";
          } else if($parameters['FILTRO'] == 'MES'){
            $filterDashboard = "DTSOLICITA BETWEEN '$dtiniciomes' AND '$dtfimmes'";
          } else if($parameters['FILTRO'] == 'TODOS'){
            $filterDashboard = " '$dtiniciomes' BETWEEN TRUNC(DTSOLICITA, 'MONTH') AND DTLIMSOLICITA
                                 AND (DTCONCLUSAO > LAST_DAY(TO_DATE('$dtiniciomes')) OR DTCONCLUSAO IS NULL) ";
          }else {
            $filterDashboard = '1 = 1';  
          }
        } else {
          $filterDashboard = '1 = 1';
        }

        if(array_key_exists('SALARIOMINIMO', $parameters)){
          $salarioMinimo = str_replace('%', '', $parameters['SALARIOMINIMO']);
        }
        else {
          $salarioMinimo = '';
        }

        if(array_key_exists('SALARIOMAXIMO', $parameters)){
          $salarioMaximo = str_replace('%', '', $parameters['SALARIOMAXIMO']);
        }
        else {
          $salarioMaximo = '';
        }

        if(array_key_exists('DTSOLICITA', $parameters)){
          $dtinicio = $parameters['DTSOLICITA'];
        }
        else {
          $dtinicio = '';
        }

        if(array_key_exists('DTLIMSOLICITA', $parameters)){
          $dtfim = $parameters['DTLIMSOLICITA'];
        }
        else {
          $dtfim = '';
        }
        $filterPeriodoPadrao = "";

        if (strlen($parameters['FILTRO']) == 0 && !isset($parameters['DTSOLICITA']) && !isset($parameters['DTLIMSOLICITA']) && !isset($parameters['DSSTATUS'])) {
            $filterPeriodoPadrao = "AND ((DTCONCLUSAO >= TRUNC(SYSDATE) OR DTCONCLUSAO IS NULL) AND (DTCANCELAMENTO >= TRUNC(SYSDATE) OR DTCANCELAMENTO IS NULL) AND (DTSUSPENSAO >= TRUNC(SYSDATE) OR DTSUSPENSAO IS NULL) AND (TRUNC(SYSDATE) BETWEEN DTSOLICITA AND DTLIMSOLICITA))";
            foreach($filters as $filtro ){
              if($filtro['columnName'] == "DTCONCLUSAO" || $filtro['columnName'] == "DTCANCELAMENTO" || $filtro['columnName'] == "DTSUSPENSAO" ){
                $filterPeriodoPadrao = "AND (TRUNC(SYSDATE) BETWEEN DTSOLICITA AND DTLIMSOLICITA)";
                break;
              }
            }
        }

        if (isset($parameters['DTSOLICITA']) || isset($parameters['DTLIMSOLICITA'])) {
              $filterPeriodoPadrao = "AND (DTSOLICITA >= TO_DATE('$dtinicio','DD/MM/YYYY') OR DTLIMSOLICITA <= TO_DATE('$dtfim','DD/MM/YYYY'))";
        }

        if (isset($parameters['DTSOLICITA']) && isset($parameters['DTLIMSOLICITA'])) {
              $filterPeriodoPadrao = "AND (DTSOLICITA >= TO_DATE('$dtinicio','DD/MM/YYYY') AND DTLIMSOLICITA <= TO_DATE('$dtfim','DD/MM/YYYY'))";
        }
        
        $query = "
            SELECT * FROM(
              SELECT
                PRO.NRPROCESSO,
                STATUS.CDSTATUS,
                STATUS.DSSTATUS,
                PRO.NMPROCESSO,
                PRO.DSDESCRICAO,
                PRO.DSMOTIVO,
                PRO.VRSALARIO,
                PRO.DTINCLUSAO,
                PRO.CDOPERINCLUSAO,
                PRO.NRORGINCLUSAO,
                PRO.DTULTATU,
                PRO.CDOPERULTATU,
                PRO.NRORGULTATU,
                TO_CHAR(PRO.DTCONCLUSAO, 'DD/MM/YYYY') AS DTCONCLUSAO,
                TO_CHAR(PRO.DTSUSPENSAO, 'DD/MM/YYYY') AS DTSUSPENSAO,
                TO_CHAR(PRO.DTCANCELAMENTO, 'DD/MM/YYYY') AS DTCANCELAMENTO,
                TO_CHAR(PRO.DTRETORNO, 'DD/MM/YYYY') AS DTRETORNO,
                PRO.NRVAGAS,
                PRO.NRTEMPLATE,
                PRO.NRPRIORIDADE,
                DES_PRIORIDADE.NMPRIORIDADE,
                DES_TEMPLATE.NMTEMPLATE,
                PRO.NRTIPOPROCESSO,
                PRO.NRPARCNEGOCIOSOLICITA,
                PARCSOLICITA.NMPRINCIPALPARC AS NMPARCNEGOCIOSOLICITA,
                PRO.NRPARCNEGOCIORESP,
                PARCRESPONSAVEL.NMPRINCIPALPARC AS NMPARCNEGOCIORESP,
                PRO.IDPUBLICAVAGA,
                PRO.NRORG,
                TPPRO.NMTIPOPROCESSO,
                TO_CHAR(PRO.DTSOLICITA, 'DD/MM/YYYY') DTSOLICITA,
                TO_CHAR(PRO.DTLIMSOLICITA, 'DD/MM/YYYY') DTLIMSOLICITA,
                ESC.NRESCALATRABM,
                ESC.NMESCALATRABH,
                OH.NROCUPACAOH,
                OH.NROCUPACAOM,
                INITCAP(OH.NMOCUPACAOH) NMOCUPACAOH,
                EM.NRESTRUTURAM AS NRESTRUTURAM,
                EH.NMESTRUTURAH AS NMESTRUTURAH,
                PRO.DSLOCALIDADE AS LOCALIDADE,
                MUN.CDMUNICIPIO,
                NVL((SELECT NMFASE
                  FROM DES_PROCESSO PR
                  JOIN DES_FASETEMPLATE FASE
                    ON PR.NRTEMPLATE = FASE.NRTEMPLATE
                   AND PR.NRORG      = FASE.NRORG
                  JOIN DES_FASE F
                    ON FASE.NRFASE = F.NRFASE
                   AND FASE.NRORG  = F.NRORG  
                 WHERE PR.NRORG      = :P_NRORG
                   AND PR.NRPROCESSO = PRO.NRPROCESSO
                   AND NRORDEM  = (SELECT MAX(NRORDEM)
                                     FROM DES_FASETEMPLATE FT
                                     JOIN DES_PARTICIPAFASE PF
                                       ON PF.NRPROCESSO = PR.NRPROCESSO
                                      AND PF.NRFASE     = FT.NRFASE
                                      AND PF.NRORG      = FT.NRORG
                                    WHERE FT.NRTEMPLATE   = FASE.NRTEMPLATE
                                      AND FT.NRORG        = FASE.NRORG)
                  ), 'Não iniciado') AS NMFASE,
                  CASE 
                    WHEN DTCONCLUSAO IS NULL
                    THEN (SELECT FLOOR(TO_NUMBER(SYSDATE - DTSOLICITA)) AS TEMPO FROM DES_PROCESSO P 
                             WHERE P.NRPROCESSO = PRO.NRPROCESSO  
                              AND P.NRORG      = PRO.NRORG)
                    ELSE (SELECT FLOOR(TO_NUMBER(DTCONCLUSAO - DTSOLICITA)) AS TEMPO FROM DES_PROCESSO P 
                            WHERE P.NRPROCESSO = PRO.NRPROCESSO
                              AND P.NRORG      = PRO.NRORG)
                    END  || ' dias' TEMPO,
                  NVL( (SELECT DECODE( SIGN((FLOOR(TO_NUMBER(SYSDATE - DTSOLICITA)) - QTDDIASLIMITE )), '-1', '0 dia', (FLOOR(TO_NUMBER(SYSDATE - DTSOLICITA)) - QTDDIASLIMITE ) || ' dias') AS FORAMETA FROM DES_PROCESSO P 
                            JOIN DES_PRIORIDADE DPR
                              ON P.NRPRIORIDADE = DPR.NRPRIORIDADE
                             AND P.NRORG = DPR.NRORG
                            WHERE P.NRPROCESSO = PRO.NRPROCESSO
                              AND P.NRORG      = PRO.NRORG
                              AND DTCONCLUSAO IS NULL
                              ), '0 dia') AS FORAMETA
               FROM DES_PROCESSO PRO
               LEFT JOIN DES_TIPOPROCESSO TPPRO
                 ON TPPRO.NRTIPOPROCESSO = PRO.NRTIPOPROCESSO
                AND TPPRO.NRORG          = PRO.NRORG
               JOIN DES_STATUS STATUS
                 ON STATUS.CDSTATUS = PRO.IDSTATUS
               LEFT JOIN DES_TEMPLATE
                 ON DES_TEMPLATE.NRTEMPLATE = PRO.NRTEMPLATE
                AND DES_TEMPLATE.NRORG      = PRO.NRORG
               LEFT JOIN GPE_OCUPACAOH OH
                 ON OH.NROCUPACAOM    = PRO.NROCUPACAODESEJADA
                AND (OH.NRORG = :P_NRORG)
                AND OH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC)
                                            FROM GPE_OCUPACAOH
                                           WHERE (NRORG     = :P_NRORG)
                                             AND NROCUPACAOM = OH.NROCUPACAOM)
               LEFT JOIN ESTRUTURAM EM
                 ON EM.NRESTRUTURAM = PRO.NRESTRUTGEREN
                AND EM.NRORG        = PRO.NRORG
               LEFT JOIN ESTRUTURAH EH
                 ON EH.NRESTRUTURAM   = EM.NRESTRUTURAM
                AND EH.NRORG          = EM.NRORG
                AND EH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC)
                                            FROM ESTRUTURAH
                                           WHERE NRORG        = EH.NRORG
                                             AND NRESTRUTURAM = EH.NRESTRUTURAM)
               LEFT JOIN ENDERECOPARC ENDE
                 ON ENDE.NRPARCNEGOCIO = EM.NRPARCNEGOCIO
                AND ENDE.NRORG         = EM.NRORG
               LEFT JOIN MUNICIPIO MUN
                 ON MUN.CDMUNICIPIO = ENDE.CDMUNICIPIO
                  AND MUN.CDPAIS = ENDE.CDPAIS
                  AND MUN.SGESTADO = ENDE.SGESTADO
               LEFT JOIN GPE_ESCALATRABH ESC
                 ON ESC.NRESCALATRABM  = PRO.NRESCALATRABM
                AND ESC.NRORG          = PRO.NRORG
                AND ESC.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC)
                                            FROM GPE_ESCALATRABH
                                           WHERE NRORG         = ESC.NRORG
                                             AND NRESCALATRABM = ESC.NRESCALATRABM)
               LEFT JOIN PARCNEGOCIO PARCSOLICITA
                 ON PARCSOLICITA.NRPARCNEGOCIO = PRO.NRPARCNEGOCIOSOLICITA
                AND PARCSOLICITA.NRORG         = PRO.NRORG
               LEFT JOIN PARCNEGOCIO PARCRESPONSAVEL
                 ON PARCRESPONSAVEL.NRPARCNEGOCIO = PRO.NRPARCNEGOCIORESP
                AND PARCRESPONSAVEL.NRORG         = PRO.NRORG
               LEFT JOIN DES_PRIORIDADE 
                 ON PRO.NRPRIORIDADE = DES_PRIORIDADE.NRPRIORIDADE
                AND PRO.NRORG        = DES_PRIORIDADE.NRORG
              WHERE $filterDashboard)
            WHERE NRORG = :P_NRORG
            $filterPeriodoPadrao
            AND (VRSALARIO BETWEEN NVL(:P_SALARIOMINIMO, 0) AND NVL(:P_SALARIOMAXIMO, 999999) OR (:P_SALARIOMINIMO IS NULL AND :P_SALARIOMAXIMO IS NULL) OR (:P_SALARIOMINIMO IS NULL AND VRSALARIO IS NULL))
            {$filterPlaceHolder}
            ORDER BY NMPROCESSO, DTLIMSOLICITA
        ";
        
        $result = $this->entityManager->getConnection()->prepare($query);  
        $result->execute(
            array(
                "P_NRORG" => $nrorg,
                "P_SALARIOMINIMO" => $salarioMinimo,
                "P_SALARIOMAXIMO" => $salarioMaximo
            )
        );

        $processos = $result->fetchAll();
        return $processos;
    }

    public function getProcessosAtivos($parameters, $conditions) {

        $nrorg = $this->hcmParameters->getNrorgTrabalho();
        $query = $this->entityManager->getConnection()->prepare("
          SELECT * FROM ( SELECT
                DES_PROCESSO.NRORG,
                NRPROCESSO,
                NMPROCESSO,
                TO_CHAR(DTSOLICITA, 'DD/MM/YYYY') DTSOLICITA,
                TO_CHAR(DTLIMSOLICITA, 'DD/MM/YYYY') DTLIMSOLICITA,
                STATUS.DSSTATUS,
                VRSALARIO,
                NRVAGAS,
                NRTEMPLATE
              FROM DES_PROCESSO
              JOIN DES_STATUS STATUS
                ON STATUS.CDSTATUS = DES_PROCESSO.IDSTATUS
             WHERE DES_PROCESSO.NRORG = :P_NRORG
               AND IDPUBLICAVAGA      = :P_IDPUBLICAVAGA )
          WHERE 1 = 1     
          $conditions     
        ");

        $query->execute(
            array(
                "P_NRORG"         => $nrorg,
                "P_IDPUBLICAVAGA" => 'S'
            )
        );

        $status = $query->fetchAll();
        return $status;

    }

    public function getStatus($parameters) {

        $nrorgpadrao = $this->hcmParameters->getNrorgPadrao();

        $query = $this->entityManager->getConnection()->prepare("
            SELECT
              CDSTATUS,
              DSSTATUS
              FROM DES_STATUS
             WHERE NRORG = :P_NRORGPADRAO
             ORDER BY DSSTATUS
        ");

        $query->execute(
            array(
                "P_NRORGPADRAO" => $nrorgpadrao
            )
        );

        $status = $query->fetchAll();
        return $status;
    }

    public function getTemplates($parameters) {

        $nrorg = $this->hcmParameters->getNrorgTrabalho();
        
        $query = $this->entityManager->getConnection()->prepare("
            SELECT
              *
              FROM DES_TEMPLATE
             WHERE NRORG = :P_NRORG
        ");

        $query->execute(
            array(
                "P_NRORG" => $nrorg
            )
        );

        $templates = $query->fetchAll();
        return $templates;

    }

    public function getTipoprocessos($parameters) {

        $nrorg = $this->hcmParameters->getNrorgTrabalho();

        $query = $this->entityManager->getConnection()->prepare("
            SELECT
              *
              FROM DES_TIPOPROCESSO
             WHERE NRORG = :P_NRORG
        ");

        $query->execute(
            array(
                "P_NRORG" => $nrorg
            )
        );

        $tipoprocesso = $query->fetchAll();
        return $tipoprocesso;

    }

    public function adicionarUpdateProcesso($dataset) {
        $nrorg = $this->hcmParameters->getNrorgTrabalho();
        $cdoperador = $this->environment->getCdOperador();
        $update = filter_var($dataset["update"], FILTER_VALIDATE_BOOLEAN);

        $dataset["IDPUBLICAVAGA"] = ($dataset["IDPUBLICAVAGA"] == 'Y' || $dataset["IDPUBLICAVAGA"] == 'S') ? 'S' : 'N';

        $idpublicavaga         = (isset($dataset["IDPUBLICAVAGA"]))         ? $dataset["IDPUBLICAVAGA"]         : null;
        $nrocupacaodesejada    = (isset($dataset["NROCUPACAOM"]))           ? $dataset["NROCUPACAOM"]           : null;
        $nrescalatrabm         = (isset($dataset["NRESCALATRABM"]))         ? $dataset["NRESCALATRABM"]         : null;
        $nrestrutgeren         = (isset($dataset["NRESTRUTURAM"]))          ? $dataset["NRESTRUTURAM"]          : null;
        $nrvagas               = (isset($dataset["NRVAGAS"]))               ? $dataset["NRVAGAS"]               : null;
        $idstatus              = (isset($dataset["CDSTATUS"]))              ? $dataset["CDSTATUS"]              : null;
        $dtlimsolicita         = (isset($dataset["DTLIMSOLICITA"]))         ? $dataset["DTLIMSOLICITA"]         : null;
        $dtsolicita            = (isset($dataset["DTSOLICITA"]))            ? $dataset["DTSOLICITA"]            : null;
        $nmprocesso            = (isset($dataset["NMPROCESSO"]))            ? $dataset["NMPROCESSO"]            : null;
        $vrsalario             = (isset($dataset["VRSALARIO"]))             ? $dataset["VRSALARIO"]             : null;
        $nrtemplate            = (isset($dataset["NRTEMPLATE"]))            ? $dataset["NRTEMPLATE"]            : null;
        $nrtipoprocesso        = (isset($dataset["NRTIPOPROCESSO"]))        ? $dataset["NRTIPOPROCESSO"]        : null;
        $nrparcnegociosolicita = (isset($dataset["NRPARCNEGOCIOSOLICITA"])) ? $dataset["NRPARCNEGOCIOSOLICITA"] : null;
        $nrparcnegocioresp     = (isset($dataset["NRPARCNEGOCIORESP"]))     ? $dataset["NRPARCNEGOCIORESP"]     : null;
        $dsmotivo              = (isset($dataset["DSMOTIVO"]))              ? $dataset["DSMOTIVO"]              : null;
        $dsdescricao           = (isset($dataset["DSDESCRICAO"]))           ? $dataset["DSDESCRICAO"]           : null;
        $dslocalidade          = (isset($dataset["LOCALIDADE"]))            ? $dataset["LOCALIDADE"]            : null;
        $dtsuspensao           = (isset($dataset["DTSUSPENSAO"]))           ? $dataset["DTSUSPENSAO"]           : null;
        $dtcancelamento        = (isset($dataset["DTCANCELAMENTO"]))        ? $dataset["DTCANCELAMENTO"]        : null;
        $dtretorno             = (isset($dataset["DTRETORNO"]))             ? $dataset["DTRETORNO"]             : null;
        $nrprioridade          = (isset($dataset["NRPRIORIDADE"]))          ? $dataset["NRPRIORIDADE"]          : null;
        $dtconclusao           = (isset($dataset["DTCONCLUSAO"]))           ? $dataset["DTCONCLUSAO"]           : null;


        $this->entityManager->beginTransaction();


        $dtlimsolicita = DateUtil::getDataDeString($dtlimsolicita, DateUtil::FORMATO_BRASILEIRO, true);
        $dtsolicita = DateUtil::getDataDeString($dtsolicita, DateUtil::FORMATO_BRASILEIRO, true);
        if (!empty($dtsuspensao)){
          $dtsuspensao = DateUtil::getDataDeString($dtsuspensao, DateUtil::FORMATO_BRASILEIRO, true);
        } else {
          $dtsuspensao = null;
        }
        if (!empty($dtcancelamento)){
          $dtcancelamento = DateUtil::getDataDeString($dtcancelamento, DateUtil::FORMATO_BRASILEIRO, true);
        } else {
          $dtcancelamento = null;
        }
        if (!empty($dtretorno)){
          $dtretorno = DateUtil::getDataDeString($dtretorno, DateUtil::FORMATO_BRASILEIRO, true);
        } else {
          $dtretorno = null;
        }
        if(!empty($dtconclusao)){
          $dtconclusao = DateUtil::getDataDeString($dtconclusao, DateUtil::FORMATO_BRASILEIRO, true);
        } else {
          $dtconclusao = null;
        }
          
        if ($update) {
            $nrprocesso = $dataset["NRPROCESSO"];

            /** @var $processo \Folha\Entities\DesProcesso */
            $processo = $this->entityManager->getRepository('\Folha\Entities\DesProcesso')
                ->findOneBy(array('nrprocesso' => $nrprocesso,
                                       'nrorg' => $nrorg));


        } else {
            $nrprocesso = \HCM\Util\NovoCodigo::geraCodigo($nrorg, 'DES_PROCESSO', 1, 22, true);

            /** @var $processo \Folha\Entities\DesProcesso */
            $nrorginclusao = (isset($nrorg)) ? $nrorg : null;
            $dtinclusao = (isset($nrorg)) ? $nrorg : null;
            $nrorginclusao = (isset($nrorg)) ? $nrorg : null;
            $processo = new \Folha\Entities\DesProcesso();
            $processo->setNrprocesso($nrprocesso);
            $processo->setNrorg($nrorg);
            $processo->setDtinclusao(DateUtil::getDataAtual());
            $processo->setCdoperinclusao($cdoperador);
            $processo->setNrorginclusao($nrorg);
        }
        $processo->setIdpublicavaga($idpublicavaga);
        $processo->setNrocupacaodesejada($nrocupacaodesejada);
        $processo->setNrescalatrabm($nrescalatrabm);
        $processo->setNrestrutgeren($nrestrutgeren);
        $processo->setNrvagas($nrvagas);
        $processo->setidstatus($idstatus);
        $processo->setDtlimsolicita($dtlimsolicita);
        $processo->setDtsolicita($dtsolicita);
        $processo->setNmprocesso($nmprocesso);
        $processo->setVrsalario($vrsalario);
        $processo->setNrtemplate($nrtemplate);
        $processo->setNrtipoprocesso($nrtipoprocesso);
        $processo->setNrparcnegociosolicita($nrparcnegociosolicita);
        $processo->setNrparcnegocioresp($nrparcnegocioresp);
        $processo->setDsmotivo($dsmotivo);
        $processo->setDsdescricao($dsdescricao);
        $processo->setDslocalidade($dslocalidade);        
        $processo->setDtsuspensao($dtsuspensao);
        $processo->setDtcancelamento($dtcancelamento);
        $processo->setDtretorno($dtretorno);
        $processo->setNrprioridade($nrprioridade);
        $processo->setDtconclusao($dtconclusao);
        $processo->setDtultatu(DateUtil::getDataAtual());
        $processo->setCdoperultatu($cdoperador);
        $processo->setNrorgultatu($nrorg);

        $this->entityManager->persist($processo);
        $this->entityManager->flush();
        $this->entityManager->commit();

        if ($update)
            return 'update';
        else
            return 'save';

    }

    public function deleteProcesso($dataset) {

        $nrorg = $this->hcmParameters->getNrorgTrabalho();
        $nrprocesso = $dataset["NRPROCESSO"];

        /** @var $participaProcesso \Folha\Entities\DesParticipaprocesso */
        $participaProcesso = $this->entityManager->getRepository('\Folha\Entities\DesParticipaprocesso')
            ->findOneBy(array('nrprocesso' => $nrprocesso,
                              'nrorg' => $nrorg));

        if (is_object($participaProcesso)) {
            throw new \Exception("Existem candidatos relacionados ao processo.");
        } else {
            $this->entityManager->beginTransaction();

            /** @var $processo \Folha\Entities\DesProcesso */
            $processo = $this->entityManager->getRepository('\Folha\Entities\DesProcesso')
                ->findOneBy(array('nrprocesso' => $nrprocesso,
                                  'nrorg' => $nrorg));

            $this->entityManager->remove($processo);
            $this->entityManager->flush();
            $this->entityManager->commit();
        }

    }

    public function getCandidatosParticipantes($parameters, $filters) {
        
        $nrorg = $this->hcmParameters->getNrorgTrabalho();
        $nrorgpadrao = $this->hcmParameters->getNrorgPadrao();
        $filterPlaceHolder = $this->hcm->getFilterPlaceHolder($filters);
        $selectFiltroCandidatos  = '';
        $queryFiltroCandidatos   = '';
        $whereFiltroSalarioIdade = '';
        $queryFiltroFormacao = '';

        if(isset($parameters['FILTRO'])){
          $selectFiltroCandidatos = "
              GPE_PESSOA.NRORG,
              OPERADOR.DSEMAILOPER,
              TO_CHAR(DES_CURRICULOGERAL.DTULTATUALIZACAO, 'DD/MM/YYYY') || ' - ' || TO_CHAR(TRUNC(SYSDATE) - TRUNC(DES_CURRICULOGERAL.DTULTATUALIZACAO)) || ' dias' AS DTULTATUALIZACAO,
              GPE_PESSOA.NRPESSOA,
              GPE_PESSOA.NRPARCNEGOCIO,
              GPE_PESSOAH.NMPESSOA,
              GPE_PESSOAH.NRCPFPESSOA,
              TO_CHAR(GPE_PESSOAH.DTNASCPESSOA, 'DD/MM/YYYY') AS DTNASCPESSOA,
              FLOOR(TO_NUMBER(SYSDATE - GPE_PESSOAH.DTNASCPESSOA) / 365.25) AS IDADE,
              GPE_PESSOAH.IDSEXOPESSOA,
              GPE_PESSOAH.CDESTACIVIL,
              GPE_NACIONALIDADE.NRNACIONALIDADE,
              GPE_PESSOAH.NRGRAUINSTR,
              GPE_PESSOAH.IDPOSSUIFILHO,
              GPE_PESSOAH.NRIDADEFILHO,
              COMUNICAPARC.CDFORMACOMU,
              COMUNICAPARC.CDCOMUNICAPARC,
              COMUNICAPARC.NRCOMUNICAPARC,
              LOGRADOURO.DSLOGRADOURO || ' ' || INITCAP(ENDERECOPARC.DSENDERECO) AS DSENDERECO,
              ENDERECOPARC.NRIMOVELENDERECO,
              ENDERECOPARC.DSCOMPLEENDERECO,
              ENDERECOPARC.NRCEPENDERECO,
              ENDERECOPARC.NMBAIRROENDERECO,
              ENDERECOPARC.CDMUNICIPIO,
              ENDERECOPARC.SGESTADO,
              ENDERECOPARC.CDPAIS,
              PAIS.NMPAIS,
              INITCAP(MUNICIPIO.NMMUNICIPIO) || ' - ' || ENDERECOPARC.SGESTADO AS LOCALIDADE,
              MUNICIPIO.NMMUNICIPIO,
              GPE_GRAUINSTR.NMGRAUINSTR,
              GPE_NACIONALIDADE.DSNACIONALIDADE,
              (SELECT COUNT(NRPROCPARTICIPA)
                 FROM DES_PARTICIPAPROCESSO
                WHERE NRORG             = :P_NRORG
                  AND NRPARCNEGOCIOCAND = GPE_PESSOA.NRPARCNEGOCIO) AS QTD_PROCESSOS,
          ";

          $queryFiltroCandidatos = "
                LEFT JOIN GPE_PESSOA
                  ON GPE_PESSOA.NRPARCNEGOCIO = PARC.NRPARCNEGOCIO
                 AND GPE_PESSOA.NRORG = PARC.NRORG
                LEFT JOIN GPE_PESSOAH
                  ON GPE_PESSOAH.NRPESSOA = GPE_PESSOA.NRPESSOA
                 AND GPE_PESSOAH.NRORG   = GPE_PESSOA.NRORG
                 AND GPE_PESSOAH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC)
                                                    FROM GPE_PESSOAH H
                                                   WHERE NRORG    = GPE_PESSOA.NRORG
                                                     AND NRPESSOA = GPE_PESSOA.NRPESSOA)
               INNER JOIN DES_CURRICULOGERAL
                  ON DES_CURRICULOGERAL.NRPARCNEGOCIO = GPE_PESSOA.NRPARCNEGOCIO
                 AND DES_CURRICULOGERAL.NRORG         = GPE_PESSOA.NRORG                                                       
                LEFT JOIN COMUNICAPARC
                  ON COMUNICAPARC.NRPARCNEGOCIO = GPE_PESSOA.NRPARCNEGOCIO
                 AND COMUNICAPARC.NRORG         = GPE_PESSOA.NRORG
                 AND COMUNICAPARC.CDFORMACOMU   = '05'
                 AND COMUNICAPARC.NRCOMUNICAPARC = (
                        SELECT MAX(COM.NRCOMUNICAPARC) FROM COMUNICAPARC COM
                        WHERE COM.NRPARCNEGOCIO = GPE_PESSOA.NRPARCNEGOCIO                  
                        AND COM.NRORG         = GPE_PESSOA.NRORG                  
                        AND COM.CDFORMACOMU   = '05')
                LEFT JOIN ENDERECOPARC
                  ON ENDERECOPARC.NRPARCNEGOCIO = GPE_PESSOA.NRPARCNEGOCIO
                 AND ENDERECOPARC.NRORG         = GPE_PESSOA.NRORG
                LEFT JOIN MUNICIPIO 
                  ON MUNICIPIO.CDMUNICIPIO = ENDERECOPARC.CDMUNICIPIO
                LEFT JOIN LOGRADOURO
                  ON LOGRADOURO.CDLOGRADOURO = ENDERECOPARC.CDLOGRADOURO
                LEFT JOIN PAIS
                  ON PAIS.CDPAIS = ENDERECOPARC.CDPAIS
                LEFT JOIN OPERADOR
                  ON OPERADOR.NRPARCNEGOCIO = GPE_PESSOA.NRPARCNEGOCIO
                 AND OPERADOR.NRORG         = GPE_PESSOA.NRORG               
                LEFT JOIN GPE_GRAUINSTR
                  ON GPE_PESSOAH.NRGRAUINSTR = GPE_GRAUINSTR.NRGRAUINSTR
                LEFT JOIN GPE_NACIONALIDADE
                  ON GPE_PESSOAH.NRNACIONALID = GPE_NACIONALIDADE.NRNACIONALIDADE
          ";

          if(isset($parameters['DSINSTITUICAO']) || isset($parameters['CDIDIOM'])){
            $dsinstituicao = '';
            $query_dsinstituicao = '';
            if(isset($parameters['DSINSTITUICAO'])){
              $dsinstituicao = $parameters['DSINSTITUICAO'];
              $query_dsinstituicao = "AND (NVL(DSINSTITUICAO, '0') LIKE '$dsinstituicao')";
            }

            $cdidioma = '';
            $query_cdidioma = '';
            if(isset($parameters['CDIDIOM'])){
              $cdidioma = $parameters['CDIDIOM'];
              $query_cdidioma = "AND (NVL(CDIDIOMA, 0) = '$cdidioma')";
            }
            
            $queryFiltroFormacao = "
                AND EXISTS (SELECT 1
                                FROM DES_FORMACAO
                               WHERE NRORG          = GPE_PESSOA.NRORG
                                 AND NRPARCNEGOCIO  = GPE_PESSOA.NRPARCNEGOCIO
                                  $query_dsinstituicao 
                                  $query_cdidioma 
                                  )
            ";
          }

          if(array_key_exists('SALARIOMAXIMO', $parameters)){
            $salarioMaximo = $parameters["SALARIOMAXIMO"];
            $salarioMaximo = trim ($salarioMaximo, "%");
          } else {
            $salarioMaximo  = "";
          }

          if(array_key_exists('SALARIOMINIMO', $parameters)) {
            $salarioMinimo = $parameters["SALARIOMINIMO"];
            $salarioMinimo = trim ($salarioMinimo, "%");
          } else {
            $salarioMinimo = "";
          }

          if(array_key_exists('IDADEMINIMA', $parameters)) {
            $idadeMinima = $parameters["IDADEMINIMA"];
            $idadeMinima = trim ($idadeMinima, "%");
          } else {
            $idadeMinima = "";
          }

          if(array_key_exists('IDADEMAXIMA', $parameters)){
            $idadeMaxima = $parameters["IDADEMAXIMA"];
            $idadeMaxima = trim ($idadeMaxima, "%");
          } else {
            $idadeMaxima = "";
          }

          $whereFiltroSalarioIdade = "
            AND ((FLOOR(TO_NUMBER(SYSDATE - GPE_PESSOAH.DTNASCPESSOA) / 365.25) >= '$idadeMinima' OR '$idadeMinima' IS NULL )
                   AND (FLOOR(TO_NUMBER(SYSDATE - GPE_PESSOAH.DTNASCPESSOA) / 365.25) <= '$idadeMaxima' OR '$idadeMaxima' IS NULL))
            AND (PART.VRPRETENSAOSALARIO BETWEEN NVL('$salarioMinimo', 0) AND NVL('$salarioMaximo', 999999))
          ";
        }

        $query = $this->entityManager->getConnection()->prepare("
            SELECT
              $selectFiltroCandidatos
              PART.NRPROCPARTICIPA,
              PART.NRPROCESSO,
              PARC.NRPARCNEGOCIO,
              PARC.NMPRINCIPALPARC,
              PART.VRPRETENSAOSALARIO,
              STATUS.CDSTATUS,
              STATUS.DSSTATUS
              FROM DES_PARTICIPAPROCESSO PART
              JOIN PARCNEGOCIO PARC
                ON PARC.NRPARCNEGOCIO = PART.NRPARCNEGOCIOCAND
               AND PARC.NRORG         = :P_NRORGPADRAO
              JOIN PARCERIA
                ON PARCERIA.NRPARCNEGOCIO  = PARC.NRPARCNEGOCIO
               AND PARCERIA.CDTIPOPARCEIRO = :P_CDTIPOPARCEIRO
              JOIN DES_STATUS STATUS
                ON STATUS.CDSTATUS = PART.CDSTATUS
               $queryFiltroCandidatos
             WHERE PART.NRORG      = :P_NRORG
               AND PART.NRPROCESSO = :P_NRPROCESSO
               AND $filterPlaceHolder
               $whereFiltroSalarioIdade
               $queryFiltroFormacao
             ORDER BY PARC.NMPRINCIPALPARC
        ");
        $query->execute(
            array(
                "P_NRORGPADRAO"    => $nrorgpadrao,
                "P_CDTIPOPARCEIRO" => self::TIPOPARCEIRO_CANDIDATO,
                "P_NRORG"          => $nrorg,
                "P_NRPROCESSO"     => $parameters["P_NRPROCESSO"]
            )
        );



        $participantes = $query->fetchAll();
        return $participantes;
    }

    public function getProcessosCandidato($parameters) {
        $nrorg = $this->hcmParameters->getNrorgTrabalho();
        $nrorgpadrao = $this->hcmParameters->getNrorgPadrao();
        $query = $this->entityManager->getConnection()->prepare("
            SELECT
              PRO.NRPROCESSO,
              INITCAP(PRO.NMPROCESSO) NMPROCESSO,
              INITCAP(OH.NMOCUPACAOH) NMOCUPACAOH,
              PART.CDSTATUS,
              DES_STATUS.DSSTATUS,
              PART.VRPRETENSAOSALARIO,
              PRO.NRTEMPLATE
              FROM DES_PROCESSO PRO
              JOIN DES_PARTICIPAPROCESSO PART
                ON PART.NRPROCESSO = PRO.NRPROCESSO
               AND PART.NRORG      = PRO.NRORG
              JOIN DES_STATUS
                ON DES_STATUS.CDSTATUS = PART.CDSTATUS
              LEFT JOIN GPE_OCUPACAOH OH
                ON OH.NROCUPACAOM = PRO.NROCUPACAODESEJADA
               AND OH.NRORG       = :P_NRORGPADRAO
               AND OH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC)
                                          FROM GPE_OCUPACAOH
                                         WHERE NRORG       = OH.NRORG
                                           AND NROCUPACAOM = OH.NROCUPACAOM)
             WHERE PART.NRPARCNEGOCIOCAND = :P_NRPARCNEGOCIO
             AND PRO.NRORG = :P_NRORG
        ");
        $query->execute(
            array(
                "P_NRORG"   => $nrorg,
                "P_NRORGPADRAO"   => $nrorgpadrao,
                "P_NRPARCNEGOCIO" => $parameters["P_NRPARCNEGOCIOCAND"]
            )
        );

        $processos = $query->fetchAll();
        return $processos;
    }

    public function alterarIdPublicaVaga($parameters) {
        $nrorg = $this->hcmParameters->getNrorgTrabalho();
        $query = $this->entityManager->getConnection()->prepare("
            UPDATE DES_PROCESSO
               SET IDPUBLICAVAGA = 'N'
             WHERE NRORG    = :P_NRORG
               AND NRPROCESSO = :P_NRPROCESSO
        ");

        $processos = $query->execute(
            array(
                "P_NRORG"   => $nrorg,
                "P_NRPROCESSO" => $parameters["P_NRPROCESSO"]
            )
        );
        return $processos;
    }

}