<?php

namespace HCM\FPA\Calculos\Apolo;

use Doctrine\ORM\EntityManager;
use HCM\Util\HCMParameters;
use HCM\Util\Messages;
use HCM\Util\Repositories;
use HCM\Util\DateUtil;
use HCM\Geral\Entity\TipohierarqorgEntity;
use HCM\Geral\Entity\HierarquiaorgEntity;
use HCM\Geral\Entity\GpevinculomEntity;
use HCM\Geral\Entity\GpealtesalarioEntity;
use HCM\FPA\Services\CalculoFolhaService;
use HCM\FPA\Entity\FparesumoctblEntity;
use HCM\FPA\Entity\FpaProcessoEntity;
use HCM\FPA\Entity\FpaTpvincprocessoEntity;
use HCM\FPA\Entity\FpaDireitotpdemisEntity;
use HCM\FPA\Entity\FpatipocalculoEntity;
use HCM\FPA\Entity\FpatpcalcmovEntity;
use HCM\FPA\Entity\FpatipomovimentEntity;
use HCM\FPA\Entity\FpaitecalcfolhaEntity;
use HCM\FPA\Entity\FpatipoincideEntity;
use HCM\FPA\Entity\FpaincideventoEntity;
use HCM\FPA\Entity\FpatipoassoceveEntity;
use HCM\FPA\Calculos\Geral\Inicializa;
use HCM\FPA\Calculos\Adiantamento\CalculoAdiantamento;
use HCM\FPA\Calculos\DecimoTerceiro\CalculoDecimoTerceiro;
use HCM\FPA\Calculos\Fechamento\CalculoFechamento;
use HCM\FPA\Calculos\Folha\CalculoFolha;
use HCM\FPA\Calculos\PLR\CalculoPLR;
use HCM\FPA\Calculos\Provisao\CalculoProvisao;
use HCM\FPA\Calculos\Rescisao\CalculoRescisao;
use HCM\FPA\Calculos\Geral\Dependencia;
use HCM\FPA\Calculos\Geral\MemoriaCalculo;
use HCM\FPA\Calculos\Geral\Preparacao;
use HCM\FPA\Calculos\Contabilizacao\Contabilizacao;
use HCM\Util\FPA;
use HCM\Util\TkDI;
use HCM\Util\MsgCalculo;
 
class PercorreCalculoConsumer{

    private $_parametros;
    private $_nrOrgPadrao;
    private $_nrOrgTrabalho;
    private $_nrEstruturaTrab;
    private $_idDebug;
    private $_ambienteSalarioOperador;
    private $_nrTipoModalidadeCalculo;
    private $_nrOcorrenciaCalculo;
    private $_mesCompetencia;
    private $_nrTipoCalculo;
    private $_simulacao;
    private $_retrocesso;
    private $_parcial;
    private $_dtPagamento;
    private $_ultPagamento;
    private $_dt_inicio;
    private $_dt_fim;
    private $_dsInicio;
    private $_dsFim;
    private $_id_retrocesso;
    private $_id_fazDifComp;
    private $_dsOrdem;
    private $_tipo_calculo;
    private $_calculo_folha;
    private $_tp_calc_mov;
    private $_vinculom;
    private $_tpVinculoProcesso;
    private $_tpRescisaoDireito;
    private $_pessoa;
    private $_dependencia;
    private $_formulario;
    private $_msg_idstatus;
    private $_msg_idabort;
    private $_msg_mensagens;
    private $_cadGozoFerias;
    private $nrGozoFerias;

    const TP_TRANSF_LEGAL = '1';
    const TP_TRANSF_GEREN = '2';
    
    public function __construct($entityManager, $hcmParameters, $dataSet){
        
        $this->entityManager = $entityManager;
        $this->hcmParameters = $hcmParameters;
        $this->dataSet = $dataSet;
        $this->_nrTipoCalculo = $dataSet['TP_CALCULO'];
        $this->_parametros = $hcmParameters;
        $this->_nrOrgTrabalho = $this->hcmParameters->getNrorgTrabalho();
        $this->_nrOrgPadrao = $this->hcmParameters->getNrorgPadrao();
        $this->_nrEstruturaTrab = FPA::getEstruturaTrabalho();
        $this->_mesCompetencia = DateUtil::getDataDeString($this->hcmParameters->getDtCompetencia());
        $this->_nrTipoModalidadeCalculo = $this->hcmParameters->getTipoModalidadeCalculo();
        $this->_nrOcorrenciaCalculo = $this->hcmParameters->getNrOcorrenciaCalculo();
        $this->_buscaDadosFormulario($dataSet);
        $this->_retrocesso = false;
        $this->_parcial = false;
        $this->_cadGozoFerias = false;
        $tp_processamento = strtoupper($dataSet['TP_PROCESSAMENTO']);
        switch ($tp_processamento){
            case FPA::PROC_SIMULACAO :
                $this->_simulacao = true; break;
            case FPA::PROC_CALCULO :
                $this->_simulacao = false; break;
            case FPA::PROC_APOLO_GRAVAR :
                $this->_simulacao = false; break;
            case FPA::PROC_APOLO_GRAVAR_PARCIAL :
                $this->_simulacao = false;
                $this->_parcial = true; break;
            case FPA::PROC_RETROCESSO :
                $this->_retrocesso = true; break;
            case FPA::PROC_CALCULO_PARCIAL :
                $this->_simulacao = false;
                $this->_parcial = true; break;
            case FPA::PROC_CADGOZOFERIAS :
                $this->_cadGozoFerias = true; break; 
            default:
                throw new \Exception('Tipo de processo '.$tp_processamento.' invalido');
        }

        Inicializa::Container($entityManager, $hcmParameters, $this->_dtPagamento);
        Inicializa::carregaVariaveis();
        
        $this->_vinculom = TkDI::getContainer()->vinculom;
        $this->_tipo_calculo = TkDI::getContainer()->tipo_calculo;
        $this->_calculo_folha = TkDI::getContainer()->calculo_folha;
        $this->_pessoa = TkDI::getContainer()->pessoa;
        $this->_tp_calc_mov = TkDI::getContainer()->tp_calc_mov;
        $this->_dependencia = new Dependencia($entityManager, $hcmParameters);
        $this->_tpVinculoProcesso = new FpaTpvincprocessoEntity($entityManager, $hcmParameters);
        $this->_tpRescisaoDireito = new FpaDireitotpdemisEntity($entityManager, $hcmParameters);
        $this->objTipoCalculo = new FpatipocalculoEntity($entityManager, $hcmParameters);
        $this->_tipo_hierarquia = new TipohierarqorgEntity($entityManager, $hcmParameters);
        $this->_resumoContabil = new FparesumoctblEntity($entityManager, $hcmParameters);
        $this->verificaTipoCalculo($this->_nrTipoCalculo);
        $this->_eventom = TkDI::getContainer()->eventom;
        $this->_assoc_eventos = TkDI::getContainer()->assoc_eventos;
        $this->_alteracaoSalario = new GpealtesalarioEntity($entityManager, $hcmParameters);
    }
    
    public function verificaTipoCalculo($nrTipoCalculo){
        
        switch ($nrTipoCalculo){
            case FpatipocalculoEntity::CALC_FOLHA:
                $this->tipoCalculo = 'HCM\FPA\Calculos\Folha\CalculoFolha';
                break;
            case FpatipocalculoEntity::CALC_FERIAS:
                $this->tipoCalculo = 'HCM\FPA\Calculos\Ferias\CalculoFerias';
                break;
            case FpatipocalculoEntity::CALC_13SAL:
                $this->tipoCalculo = 'HCM\FPA\Calculos\DecimoTerceiro\CalculoDecimoTerceiro';
                break;
            case FpatipocalculoEntity::CALC_ADIANT_13SAL:
                $this->tipoCalculo = 'HCM\FPA\Calculos\DecimoTerceiro\CalculoDecimoTerceiro';
                break;
            case FpatipocalculoEntity::CALC_ADIANTAMENTO:
                $this->tipoCalculo = 'HCM\FPA\Calculos\Adiantamento\CalculoAdiantamento';
                break;
            case FpatipocalculoEntity::CALC_RESCISAO:
                $this->tipoCalculo = 'HCM\FPA\Calculos\Rescisao\CalculoRescisao';
                break;
            case FpatipocalculoEntity::CALC_RESCISAO_COMPLEMENTAR:
                $this->tipoCalculo = 'HCM\FPA\Calculos\Rescisao\CalculoRescisao';
                break;
            case FpatipocalculoEntity::CALC_PROVISAO_CONTABIL:
                $this->tipoCalculo = 'HCM\FPA\Calculos\Provisao\CalculoProvisao';
                break;
            case FpatipocalculoEntity::CALC_PROVISAO_OTIMIZADA:
                $this->tipoCalculo = 'HCM\FPA\Calculos\Provisao\CalculoProvisaoOtimizada';
                break;
            case FpatipocalculoEntity::CALC_FECHAMENTO:
                $this->tipoCalculo = 'HCM\FPA\Calculos\Fechamento\CalculoFechamento';
                break;
            case FpatipocalculoEntity::CALC_PLR:
                $this->tipoCalculo = 'HCM\FPA\Calculos\PLR\CalculoPLR';
                break;
            case FpatipocalculoEntity::CALC_FOLHA_COMPLEMENTAR:
                $this->tipoCalculo = 'HCM\FPA\Calculos\FolhaComplementar\CalculoFolhaComplementar';
                break;
            case FpatipocalculoEntity::CALC_CONTABILIZACAO:
            break;
            default:
                throw new \Exception('Tipo de calculo '.$nrTipoCalculo.' invalido');
        }
    }
    
    public function percorreCalculoConsumerFolhaComplementar($vinculos, $fpaCalculoConsumer) {
        $resultado = array();
        $contador = 0;
        $qtparcela = 0;
        $idstatus = MsgCalculo::ID_PENDENTE;
        $idabort = false;
        $objTpCalc = $this->objTipoCalculo->retornaTipoCalculo($this->_nrTipoCalculo);
        $mesInicio = $this->_dtInicio->format('m');
        $anoInicio = $this->_dtInicio->format('Y');
        $mesFim = $this->_dtFim->format('m');
        $anoFim = $this->_dtFim->format('Y');
        $ocorrencia = 1;
        $this->_nrTipoModalidadeCalculo = 13; //Cenario III
        $tp_calc_movs = [];
        
        if($this->dataSet['QTOCORPROXPERI']){
            $qtparcela = $this->dataSet['QTOCORPROXPERI'];
        }
        
        for ($dataAtual = clone $this->_dtInicio; $dataAtual < $this->_dtFim; $dataAtual->modify('+1 month')) {
            $mesAtual = $dataAtual->format('m');
            $anoAtual = $dataAtual->format('Y');
            
            if($mesAtual >= $mesFim && $anoAtual == $anoFim){
                continue;
            }
        
            $tp_calc_movs[$mesAtual] = new FpatpcalcmovEntity($this->entityManager, $this->hcmParameters);
            $tp_calc_movs[$mesAtual]->retornaTipoCalculoMovimento($this->_nrTipoCalculo); 
                    
            $competencia = DateUtil::getDataDeString("01/".$mesAtual."/".$dataAtual->format("Y"));
                    
            $tp_calc_movs[$mesAtual]->setCompetencia($competencia);
                    
            $tp_calc_movs[$mesAtual]->atribuiNrCalculoFolha(
                $this->_nrOrgTrabalho, $this->_calculo_folha, $competencia, $this->_nrTipoModalidadeCalculo,
                $ocorrencia
            );
        }
        
        /*for($i = $mesInicio; $i < $mesFim; $i++){
            $tp_calc_movs[$i] = new FpatpcalcmovEntity($this->entityManager, $this->hcmParameters);
            $tp_calc_movs[$i]->retornaTipoCalculoMovimento($this->_nrTipoCalculo); 
            
            $competencia = DateUtil::getDataDeString("01/".$i."/".$this->_dtInicio->format("Y"));
            
            $tp_calc_movs[$i]->setCompetencia($competencia);
            
            $tp_calc_movs[$i]->atribuiNrCalculoFolha(
                $this->_nrOrgTrabalho, $this->_calculo_folha, $competencia, $this->_nrTipoModalidadeCalculo,
                $ocorrencia
            );
        }*/
        
        $this->_tpVinculoProcesso->carregaTipoVinculoPorProcesso(
            $this->_nrOrgTrabalho, $this->_nrOrgPadrao, $this->_mesCompetencia
        );
        $vinculosContemplados = [];
        
        $resultadoCalcular = null;
        foreach($tp_calc_movs as $tp_calc_mov){
            $dtCompetencia = $tp_calc_mov->getCompetencia();
            $this->hcmParameters->set("dtmescompetenc", substr($dtCompetencia, 0, 10));
            $this->_eventom->setMesCompetencia(substr($dtCompetencia, 0, 10));
            
            $calculo = new $this->tipoCalculo($this->entityManager, $this->hcmParameters, $this->dataSet, 1, $this->_simulacao, $this->_parcial, $dtCompetencia);
            
            $obj_tp_calc_mov = current($tp_calc_mov->retornaPosicaoTpMov($tp_calc_mov->retornaTpMovPrincipal()));
            $nrcalcprincipal = $obj_tp_calc_mov->getNrCalcFolha();
            $_mesCompetenciaAtual = DateUtil::getDataDeString($this->_mesCompetencia, DateUtil::FORMATO_BRASILEIRO, true); 
        
            foreach ($vinculos as $nrvinculom => $nmvinculom) {
                echo PHP_EOL."Iniciando calculo do vinculo $nrvinculom - $nmvinculom".PHP_EOL;
                
                $vinculo = $this->_vinculom->retornaVincMestreHistorico($this->_nrOrgTrabalho, $nrvinculom, $this->_mesCompetencia);
                
                $carregou = $this->_alteracaoSalario->carregaAlteracaoSalario($_mesCompetenciaAtual,  $vinculo->getNrvinculom(), 1);
                $dtAlteracao = $this->_alteracaoSalario->getDtalterasalario();
                $dataBase = $this->_alteracaoSalario->getDataBaseDissidio();
                
                if(!$dataBase){
                    continue;
                }
                
                if($dtAlteracao->format('m/Y') != $_mesCompetenciaAtual->format('m/Y')
                or $dtCompetencia < $dataBase){
                    continue;
                }
                
                //Verifica liquido
                $calc_folhaAtual = $this->entityManager->getRepository(Repositories::FPA_CALCULOFOLHA)->retornaCalculoFolha(
                    $this->_nrOrgTrabalho, FpatipomovimentEntity::TP_MOV_MENSAL, 1, 1, $_mesCompetenciaAtual, $_mesCompetenciaAtual
                )[0];
                $itemCalcCompetenciaAtual = new FpaitecalcfolhaEntity($this->entityManager, $this->hcmParameters, $calc_folhaAtual->getNrcalculofolha());
                $itemCalcCompetenciaAtual->retornaItensPorVinculo($vinculo->getNrorg(), $vinculo->getNrvinculom(), $calc_folhaAtual->getNrcalculofolha());
                $liquido = $itemCalcCompetenciaAtual->verificaLiquido(
                    $this->_assoc_eventos, $this->_eventom, $this->_nrorg
                );
                
                MsgCalculo::getInstance()->setDados($nrcalcprincipal, $vinculo->getNrvinculom(), $this->_nrOrgTrabalho, 'ERROSCALCULO');
                
                if ($liquido == true) {
                    MsgCalculo::getInstance()->adicionaMensagem('Vinculo possui liquido.');
                    $this->finalizaMensagens();
                    continue;
                }
                
                if($contador >= 10 && $this->_simulacao==true){
                    MsgCalculo::getInstance()->adicionaMensagem("Simulado e demonstrado apenas os 10 primeiros vínculos da lista. Para calcular mais vínculos será necessário gravar.");
                    break;
                }
                
                $listaProcessos = $this->_tpVinculoProcesso->getItem($vinculo->getNrtipovinculom());
            
                
                if (is_null($listaProcessos)) {
                    MsgCalculo::getInstance()->adicionaMensagem('Tipo de vínculo não possui processos de cálculo');
                    if (!$this->finalizaMensagens()) {
                        continue;
                    }
                    continue;
                }
                
                if ($this->_retrocesso === false) {
                    //ITENS SALARIO ANTIGO
                    $itensCalculoOrig = new FpaitecalcfolhaEntity($this->entityManager, $this->hcmParameters);
                    $calc_folha = $this->entityManager->getRepository(Repositories::FPA_CALCULOFOLHA)->retornaCalculoFolha(
                        $this->_nrOrgTrabalho, FpatipomovimentEntity::TP_MOV_MENSAL, 1, 1, $dtCompetencia, $dtCompetencia
                    )[0];
                    
                    $itensCalculoOrig->retornaItensPorVinculo($vinculo->getNrorg(), $vinculo->getNrvinculom(), $calc_folha->getNrcalculofolha());
                    $itemsOrig = $itensCalculoOrig->getItems();
                    
                    if(count($itemsOrig) <= 0){
                        MsgCalculo::getInstance()->adicionaMensagem('Vinculo não possui liquido na competencia '.$dtCompetencia);
                        $this->finalizaMensagens();
                        continue;
                    }
                    
                    //ITENS SALARIO NOVO
                    $tp_calc_mov->retornaItensCalculo($vinculo);
                    $obj_tp_calc_mov = current($tp_calc_mov->retornaPosicaoTpMov($tp_calc_mov->retornaTpMovPrincipal()));
                    $_itens_calculo_principal = $obj_tp_calc_mov->getInstancia();
                    $_itens_calculo_principal->setNrTipoCalculo(1);
                    $itemsPrincipal = $_itens_calculo_principal->getItems();
                    //LIMPA SEMPRE ITENS NOVOS
                    if(count($itemsPrincipal) > 0){
                        $_itens_calculo_principal->removeAllItens();
                    }
                    
                    $incideventoEntity = new FpaincideventoEntity($this->entityManager, $this->hcmParameters);
                    $incideEventosNaoRecalcula = $incideventoEntity->getEventosIncDissidio(
                        $dtCompetencia, FpatipoincideEntity::NÃO_RECALCULA_DISSIDIO
                    );
                    
                    $incideEventosRecalculaPerc = $incideventoEntity->getEventosIncDissidio(
                        $dtCompetencia, FpatipoincideEntity::RECALCULA_POR_DISSIDIO
                    );
                    
                    $eventosInseridos = [];
                    //Replica a folha com salario antigo para o novo calculo.
                    foreach($itemsOrig as $itemOrig){
                        
                        $item = clone $itemOrig;
                        $evento = $this->entityManager->getRepository(Repositories::FPA_EVENTOM)->retornaEvento($item->getNreventom(), $this->_nrOrgTrabalho, 0, $this->_mesCompetencia);
                        if(in_array($evento->getNreventom(), $incideEventosNaoRecalcula)){
                            continue;   
                        }
                        
                        if(array_key_exists($evento->getNreventom(), $eventosInseridos)){
                            $evento = $this->_eventom->retornaEvento($evento->getNreventom());
                            $indice = $_itens_calculo_principal->procuraEvento($evento);
                            $itemOld = $_itens_calculo_principal->getItem($indice);
                            $ref = $itemOld->getVrreffolha() + $item->getVrreffolha();
                            $val = $itemOld->getVreventofolha() + $item->getVreventofolha();
                            $itemOld->setVrreffolha($ref);
                            $itemOld->setVreventofolha($val);
                        }else{
                            $item->setNritemcalcfolha(\HCM\Util\NovoCodigo::geraCodigo($this->_nrOrgTrabalho, 'FPA_ITECALCFOLHA', 1, 12, true));
                            $item->setIdrefevalorcalc('NORMAL');
                            $item->setNrcalculofolha($_itens_calculo_principal->getNrcalculofolha());
                            $item->setDtocorrencia(DateUtil::getDataDeString($dtCompetencia));
                            $_itens_calculo_principal->addItem($item);
                        }    
                            
                        $eventosInseridos[$evento->getNreventom()] = true;       
                        
                    }
                    
                    //$_itens_calculo_principal->persistList();
                    $this->_dependencia->analisaDependencia($vinculo);
                    
                    try {
                        $calculo->setListaProcessos($listaProcessos);
                        $result = $calculo->calculaFolhaComplementar($vinculo, $tp_calc_mov, $qtparcela, $itemsOrig, $_itens_calculo_principal, $this->_mesCompetencia);
                        $resultadoCalcular[$vinculo->getNrvinculom()][] = $result;
                    } catch (\Exception $e) {
                        MsgCalculo::getInstance()->adicionaMensagem($e->getMessage());
                        if (!$this->finalizaMensagens()) {
                            continue;
                        }
                        continue;
                    }
                } else {
                    try {
                        
                        //ITENS SALARIO NOVO
                        $tp_calc_mov->retornaItensCalculo($vinculo);
                        $obj_tp_calc_mov = current($tp_calc_mov->retornaPosicaoTpMov($tp_calc_mov->retornaTpMovPrincipal()));
                        $_itens_calculo_principal = $obj_tp_calc_mov->getInstancia();
                        $_itens_calculo_principal->setNrTipoCalculo(1);
                        $itemsPrincipal = $_itens_calculo_principal->getItems();
                        //LIMPA ITENS CENARIO
                        if(count($itemsPrincipal) > 0){
                            $_itens_calculo_principal->removeAllItens();
                        }
                        
                        //Busca folha atual para deletar o evento de diferenca
                        $calc_folhaAtual = $this->entityManager->getRepository(Repositories::FPA_CALCULOFOLHA)->retornaCalculoFolha(
                            $this->_nrOrgTrabalho, FpatipomovimentEntity::TP_MOV_MENSAL, 1, 1, $_mesCompetenciaAtual, $_mesCompetenciaAtual
                        )[0];
                        $itemCalcCompetenciaAtual = new FpaitecalcfolhaEntity($this->entityManager, $this->hcmParameters, $calc_folhaAtual->getNrcalculofolha());
                        $itemCalcCompetenciaAtual->retornaItensPorVinculo($vinculo->getNrorg(), $vinculo->getNrvinculom(), $calc_folhaAtual->getNrcalculofolha());
                        
                        $evento_difSalario = $this->_eventom->retornaEvento(FpatipoassoceveEntity::ASSOC_DIF_SALARIO_DISSIDIO);
                        $indice = $itemCalcCompetenciaAtual->procuraEvento($evento_difSalario);
                        $eveDifSalario = $itemCalcCompetenciaAtual->getItem($indice);
                        if($eveDifSalario && $eveDifSalario->getIdrefevalorcalc() == 'VALOR'){
                            $itemCalcCompetenciaAtual->excluiItem($eveDifSalario, $indice);
                        }
                        
                        $evento_difSalarioMaternidade = $this->_eventom->retornaEvento(334);
                        $indice = $itemCalcCompetenciaAtual->procuraEvento($evento_difSalarioMaternidade);
                        $eveDifSalario = $itemCalcCompetenciaAtual->getItem($indice);
                        if($eveDifSalario && $eveDifSalario->getIdrefevalorcalc() == 'VALOR'){
                            $itemCalcCompetenciaAtual->excluiItem($eveDifSalario, $indice);
                        }
                        
                        $evento_difProvNaoIncide = $this->_eventom->retornaEvento(205510);
                        $indice = $itemCalcCompetenciaAtual->procuraEvento($evento_difProvNaoIncide);
                        $eveDifSalario = $itemCalcCompetenciaAtual->getItem($indice);
                        if($eveDifSalario && $eveDifSalario->getIdrefevalorcalc() == 'VALOR'){
                            $itemCalcCompetenciaAtual->excluiItem($eveDifSalario, $indice);
                        }
                        
                        $evento_difDesconto = $this->_eventom->retornaEvento(205580);
                        $indice = $itemCalcCompetenciaAtual->procuraEvento($evento_difDesconto);
                        $eveDifSalario = $itemCalcCompetenciaAtual->getItem($indice);
                        if($eveDifSalario && $eveDifSalario->getIdrefevalorcalc() == 'VALOR'){
                            $itemCalcCompetenciaAtual->excluiItem($eveDifSalario, $indice);
                        }
                        
                        $evento_difDescontoNaoIncide = $this->_eventom->retornaEvento(205559);
                        $indice = $itemCalcCompetenciaAtual->procuraEvento($evento_difDescontoNaoIncide);
                        $eveDifSalario = $itemCalcCompetenciaAtual->getItem($indice);
                        if($eveDifSalario && $eveDifSalario->getIdrefevalorcalc() == 'VALOR'){
                            $itemCalcCompetenciaAtual->excluiItem($eveDifSalario, $indice);
                        }
                    } catch (\Exception $e){
                        throw new \Exception($e->getMessage());
                    }
                }
                if (!$this->finalizaMensagens()) {
                    continue;
                }
                
                $contador++;
                
                if(!in_array($nrvinculom, $vinculosContemplados)){
                    $this->entityManager->getRepository(Repositories::FPA_CALCULOCONSUMER)->addNrtotprocessvinc($this->_nrOrgTrabalho, $fpaCalculoConsumer->getNrcalculoconsumer());
                    $vinculosContemplados[$nrvinculom] = $nrvinculom;
                    echo PHP_EOL."$nrvinculom - $nmvinculom - Contemplado".PHP_EOL;
                }
                
                echo "Finalizando calculo do vinculo $nrvinculom - $nmvinculom".PHP_EOL;
            }
        }
        if ($this->_msg_idabort === false) {
            try {
                $fpacalculofolha = $this->entityManager->getRepository(Repositories::FPA_CALCULOFOLHA)->findOneBy(array('nrcalculofolha' => $nrcalcprincipal, 'nrorg' => $this->_nrOrgTrabalho));
                if (($this->_msg_idstatus === MsgCalculo::ID_COM_ERRO) or ($this->_msg_idstatus === MsgCalculo::ID_COM_ALERTA)) {
                    $fpacalculofolha->setIdstatusproc('E');
                } else {
                    $fpacalculofolha->setIdstatusproc('C');
                }
                $fpacalculofolha->persist();
                \Setforms2\Shared\Registry::getEntityManager()->flush();
            } catch (\Exception $e) {
                \Setforms2\Shared\Registry::getEntityManager()->rollback();
                throw $e;
            }
        }
        
        \Setforms2\Shared\Registry::getEntityManager()->beginTransaction();

        $this->entityManager->getRepository(Repositories::FPA_CALCULOCONSUMER)->concluirCalculoConsumer($this->_nrOrgTrabalho, $fpaCalculoConsumer->getNrcalculoconsumer());
        $fpaCalculoConsumer->setNrcalculofolha($nrcalcprincipal);
        //$fpaCalculoConsumer->setIderro(($this->_msg_idstatus === MsgCalculo::ID_COM_ERRO) or ($this->_msg_idstatus === MsgCalculo::ID_COM_ALERTA));
        
        \Setforms2\Shared\Registry::getEntityManager()->persist($fpaCalculoConsumer);
        \Setforms2\Shared\Registry::getEntityManager()->flush();
        \Setforms2\Shared\Registry::getEntityManager()->commit();

        echo PHP_EOL."---------------".PHP_EOL;
        echo PHP_EOL.'Numero de Calculo: '.$nrcalcprincipal.PHP_EOL;
        echo ">>>>>>>>>> Processo Finalizado <<<<<<<<<<".PHP_EOL;

        return array(
            'CALCULO' => $nrcalcprincipal,
            'MENSAGENS' => ($this->_msg_idabort === true ? $this->_msg_mensagens : null),
            'ERRO' => (($this->_msg_idstatus === MsgCalculo::ID_COM_ERRO) or ($this->_msg_idstatus === MsgCalculo::ID_COM_ALERTA)),
            'ABORT' => $this->_msg_idabort
        );
    }
    
    public function percorreCalculoConsumer($vinculos, $fpaCalculoConsumer) {
        $resultado = array();
        $contador = 0;
        $idstatus = MsgCalculo::ID_PENDENTE;
        $idabort = false;
        
        $objTpCalc = $this->objTipoCalculo->retornaTipoCalculo($this->_nrTipoCalculo);
        
        try {
            $this->_tp_calc_mov->retornaTipoCalculoMovimento($this->_nrTipoCalculo);
        } catch (Exception $e){
            throw $e;
        }
        
        $this->_tp_calc_mov->atribuiNrCalculoFolha(
            $this->_nrOrgTrabalho, $this->_calculo_folha, $this->_mesCompetencia, $this->_nrTipoModalidadeCalculo,
            $this->_nrOcorrenciaCalculo
        );
        
        if ($this->_nrTipoCalculo === FpatipocalculoEntity::CALC_RESCISAO || $this->_nrTipoCalculo === FpatipocalculoEntity::CALC_RESCISAO_COMPLEMENTAR)  {
            $this->_tpRescisaoDireito->carregaTipoDireitoPorTipoDemissao(
                $this->_nrOrgTrabalho, $this->_nrOrgPadrao, $this->_mesCompetencia
            );
        }
        
        $this->_tpVinculoProcesso->carregaTipoVinculoPorProcesso(
            $this->_nrOrgTrabalho, $this->_nrOrgPadrao, $this->_mesCompetencia
        );
        
        $calculo = new $this->tipoCalculo($this->entityManager, $this->hcmParameters, $this->dataSet, $this->_nrTipoCalculo, $this->_simulacao, $this->_parcial);
        
        $preparacao = new Preparacao($this->entityManager, $this->hcmParameters, $this->dataSet, $this->_nrTipoCalculo, false, false);
        
        $obj_tp_calc_mov = current($this->_tp_calc_mov->retornaPosicaoTpMov($this->_tp_calc_mov->retornaTpMovPrincipal()));
        $nrcalcprincipal = $obj_tp_calc_mov->getNrCalcFolha();
        
        foreach ($vinculos as $nrvinculom => $nmvinculom) {
            echo PHP_EOL."Iniciando calculo do vinculo $nrvinculom - $nmvinculom".PHP_EOL;
            
            $vinculo = $this->_vinculom->retornaVincMestreHistorico($this->_nrOrgTrabalho, $nrvinculom, $this->_mesCompetencia);
        
            $processa_msg = MsgCalculo::getInstance();
            MsgCalculo::getInstance()->setDados($nrcalcprincipal, $vinculo->getNrvinculom(), $this->_nrOrgTrabalho, 'ERROSCALCULO');
        
            if ($this->_retrocesso === false) {
                try {
                    $this->_executaPreparacao($vinculo, $preparacao, $contador);
                } catch (\Exception $e) {
                    MsgCalculo::getInstance()->adicionaMensagem($e->getMessage());
                    if (!$this->finalizaMensagens()) {
                        continue;
                    }
                    continue;
                }
                
                if(!empty($this->nrGozoFerias)){
                    $existeGozoCalculado = $this->entityManager->getRepository(Repositories::FPA_ITECALCFOLHA)->retornaItemGozoFeriasCalculado($this->_nrOrgTrabalho, $nrcalcprincipal, $vinculo->getNrvinculom(), 3030, $this->nrGozoFerias);
                
                    if(!empty($existeGozoCalculado)){
                        echo PHP_EOL."Cálculo de Férias já realizado para o vínculo, gozo de férias: ".$existeGozoCalculado[0]['NRGOZOFERIAS']." calculado na competência: ".$existeGozoCalculado[0]['DTOCORRENCIA']."".PHP_EOL;
                        continue;
                    }
                }

                $this->_dependencia->analisaDependencia($vinculo);
        
                $listaProcessos = $this->_tpVinculoProcesso->getItem($vinculo->getNrtipovinculom());
                if (is_null($listaProcessos)) {
                    MsgCalculo::getInstance()->adicionaMensagem('Tipo de vínculo não possui processos de cálculo');
                    if (!$this->finalizaMensagens()) {
                        continue;
                    }
                    continue;
                }
                
                switch ($this->_nrTipoCalculo){
                    case FpatipocalculoEntity::CALC_FERIAS:
                        if (in_array(FpaProcessoEntity::FERIAS, $listaProcessos) === false){
                            continue;
                        }
                        break;
                    case FpatipocalculoEntity::CALC_13SAL:
                    case FpatipocalculoEntity::CALC_ADIANT_13SAL:
                        if (in_array(FpaProcessoEntity::DECIMO_TERCEIRO_SALARIO, $listaProcessos) === false){
                            continue;
                        }
                        break;
                    case FpatipocalculoEntity::CALC_ADIANTAMENTO:
                        if (in_array(FpaProcessoEntity::ADIANTAMENTO_SALARIO, $listaProcessos) === false){
                            continue;
                        }
                        break;
                    case FpatipocalculoEntity::CALC_RESCISAO:
                    case FpatipocalculoEntity::CALC_RESCISAO_COMPLEMENTAR:
                        if (in_array(FpaProcessoEntity::RESCISAO, $listaProcessos) === false){
                            continue;
                        } else {
                            $listaDireitos = $this->_tpRescisaoDireito->getItem($vinculo->getNrtpdemissao());
                            $calculo->setListaDireitos($listaDireitos);
                            if ($vinculo->getNrtpdemissao() == null){
                                MsgCalculo::getInstance()->adicionaMensagem('Vínculo sem tipo de demissão.');
                            }
                        }
                    break;
                    case FpatipocalculoEntity::CALC_PROVISAO_CONTABIL:
                        if (in_array(FpaProcessoEntity::PROVISAO, $listaProcessos) === false){
                            continue;
                        }
                        break;
                    case FpatipocalculoEntity::CALC_ADIANT_PLR:
                    case FpatipocalculoEntity::CALC_PLR:
                        if (in_array(FpaProcessoEntity::PLR, $listaProcessos) === false){
                            continue;
                        }
                        break;
                }

                try {
                    echo "Processando calculo do vinculo".PHP_EOL;
                    $calculo->setListaProcessos($listaProcessos);
                    
                    switch ($this->_nrTipoCalculo){
                        case FpatipocalculoEntity::CALC_FOLHA:
                            $calculo->calculaFolha($vinculo, $this->_tp_calc_mov, $contador);
                            break;
                        case FpatipocalculoEntity::CALC_FERIAS:
                            $calculo->calculaFerias($vinculo, $this->_tp_calc_mov, $contador, $this->nrGozoFerias, 'N');
                            break;
                        case FpatipocalculoEntity::CALC_13SAL:
                            $calculo->calculaDecimoTerceiro($vinculo, $this->_tp_calc_mov, $contador);
                            break;
                        case FpatipocalculoEntity::CALC_ADIANT_13SAL:
                            $calculo->calculaDecimoTerceiro($vinculo, $this->_tp_calc_mov, $contador);
                            break;
                        case FpatipocalculoEntity::CALC_ADIANTAMENTO:
                            $calculo->calculaAdiantamento($vinculo, $this->_tp_calc_mov, $contador);
                            break;
                        case FpatipocalculoEntity::CALC_RESCISAO:
                            $calculo->calculaRescisao($vinculo, $this->_tp_calc_mov, $contador);
                            break;
                        case FpatipocalculoEntity::CALC_RESCISAO_COMPLEMENTAR:
                            $calculo->calculaRescisao($vinculo, $this->_tp_calc_mov, $contador);
                            break;
                        case FpatipocalculoEntity::CALC_PROVISAO_CONTABIL:
                            $calculo->calculaProvisao($vinculo, $this->_tp_calc_mov, $contador);
                            break;
                        case FpatipocalculoEntity::CALC_PROVISAO_OTIMIZADA:
                            $calculo->calculaProvisaoOtimizada($vinculo, $this->_tp_calc_mov, $contador);
                            break;
                        case FpatipocalculoEntity::CALC_FECHAMENTO:
                            $calculo->calculaFechamento($vinculo, $this->_tp_calc_mov, $contador);
                            break;
                        case FpatipocalculoEntity::CALC_ADIANT_PLR:
                            $calculo->calculaPLR($vinculo, $this->_tp_calc_mov, $contador);
                            return;
                        case FpatipocalculoEntity::CALC_PLR:
                            $calculo->calculaPLR($vinculo, $this->_tp_calc_mov, $contador);
                            break;
                    }
                    
                } catch (Exception $e) {
                    MsgCalculo::getInstance()->adicionaMensagem($e->getMessage());

                    if (!$this->finalizaMensagens()) {
                        continue;
                    }
                    continue;
                }
            } else {
                try {
                    $calculo->retroceder($vinculo, $this->_tp_calc_mov);
                } catch (Exception $e){
                    throw new \Exception($e->getMessage());
                }
            }
            if (!$this->finalizaMensagens()) {
                continue;
            }
            
            $contador++;
            $this->entityManager->getRepository(Repositories::FPA_CALCULOCONSUMER)->addNrtotprocessvinc($this->_nrOrgTrabalho, $fpaCalculoConsumer->getNrcalculoconsumer());

            echo "Finalizando calculo do vinculo $nrvinculom - $nmvinculom".PHP_EOL;
        }   

        if ($this->_msg_idabort === false) {
            try {
                $fpacalculofolha = $this->entityManager->getRepository(Repositories::FPA_CALCULOFOLHA)->findOneBy(array('nrcalculofolha' => $nrcalcprincipal, 'nrorg' => $this->_nrOrgTrabalho));
                if (($this->_msg_idstatus === MsgCalculo::ID_COM_ERRO) or ($this->_msg_idstatus === MsgCalculo::ID_COM_ALERTA)) {
                    $fpacalculofolha->setIdstatusproc('E');
                } else {
                    $fpacalculofolha->setIdstatusproc('C');
                }
                $fpacalculofolha->persist();
                \Setforms2\Shared\Registry::getEntityManager()->flush();
            } catch (Exception $e) {
                \Setforms2\Shared\Registry::getEntityManager()->rollback();
                throw $e;
            }
        }

        \Setforms2\Shared\Registry::getEntityManager()->beginTransaction();

        $this->entityManager->getRepository(Repositories::FPA_CALCULOCONSUMER)->concluirCalculoConsumer($this->_nrOrgTrabalho, $fpaCalculoConsumer->getNrcalculoconsumer());
        $fpaCalculoConsumer->setNrcalculofolha($nrcalcprincipal);
        //$fpaCalculoConsumer->setIderro(($this->_msg_idstatus === MsgCalculo::ID_COM_ERRO) or ($this->_msg_idstatus === MsgCalculo::ID_COM_ALERTA));
        
        \Setforms2\Shared\Registry::getEntityManager()->persist($fpaCalculoConsumer);
        \Setforms2\Shared\Registry::getEntityManager()->flush();
        \Setforms2\Shared\Registry::getEntityManager()->commit();

        echo PHP_EOL."---------------".PHP_EOL;
        echo PHP_EOL.'Numero de Calculo: '.$nrcalcprincipal.PHP_EOL;
        echo ">>>>>>>>>> Processo Finalizado <<<<<<<<<<".PHP_EOL;

        return array(
            'CALCULO' => $nrcalcprincipal,
            'MENSAGENS' => ($this->_msg_idabort === true ? $this->_msg_mensagens : null),
            'ERRO' => (($this->_msg_idstatus === MsgCalculo::ID_COM_ERRO) or ($this->_msg_idstatus === MsgCalculo::ID_COM_ALERTA)),
            'ABORT' => $this->_msg_idabort
        );
    }
    
    public function percorreContabilizacaoConsumer($vinculos, $fpaCalculoConsumer){

        $demonstrativo = null;
        
        if ($this->_retrocesso === false) {
            if (!$this->_nrOcorrenciaCalc) {
                $listaNrCalculo = $this->_calculo_folha->carregaListaNrCalculoFolhaTodasOcorrencias(
                    $this->_nrOrgTrabalho, $this->_tipo_movimento, $this->_mesCompetencia, $this->_nrTipoModalidadeCalculo
                );

                foreach ($listaNrCalculo as $nrCalculo) {
                    $lista_calculo_folha[] = $nrCalculo;
                }
            } else {
                $arrayOcorrencias = explode(',',$this->_nrOcorrenciaCalc);
                foreach ($arrayOcorrencias as $nrocorrencia) {
                    $calculo_folha = $this->_calculo_folha->retornaCalculoFolha(
                        $this->_nrOrgTrabalho, $this->_tipo_movimento, $this->_mesCompetencia, $this->_nrTipoModalidadeCalculo, $nrocorrencia
                    );

                    foreach ($calculo_folha as $calc_folha) {
                        $lista_calculo_folha[] = $calc_folha->getNrcalculofolha();
                    }
                }
            }
        }
        
        $contabiliza = new Contabilizacao($this->entityManager, $this->hcmParameters, $this->_simulacao, $this->dataSet);
        
        $estruturasVinc = $this->_verificaEstruturasVinculos();
        
        $Associacao = $this->_verificaAssociacaoContabil($lista_calculo_folha);
        
        if($Associacao && $estruturasVinc){
            foreach ($vinculos  as $nrvinculom => $nmvinculom) {
                echo PHP_EOL."Iniciando contabilização do vinculo $nrvinculom - $nmvinculom".PHP_EOL;
                
                $vinculo = $this->_vinculom->retornaVincMestreHistorico($this->_nrOrgTrabalho, $nrvinculom, $this->_mesCompetencia);
                
                if ($this->_retrocesso === false) {
                    try {
                        echo "Processando contabilização do vinculo".PHP_EOL;
                        $result_parc = $contabiliza->contabilizar(
                            $vinculo, $lista_calculo_folha, 
                            $demonstrativo, $this->_nr_tipo_hierarquia
                        );
    
                    } catch(Exception $e) {
                        throw $e;
                    }
                } else {
                    $contabiliza->retrocedeResumoEventos($vinculo);
                }
    
                // adiciona + 1 ao contador de vinculos processados no registro do calculoconsumer
                $this->entityManager->getRepository(Repositories::FPA_CALCULOCONSUMER)->addNrtotprocessvinc($this->_nrOrgTrabalho, $fpaCalculoConsumer->getNrcalculoconsumer());
    
                echo "Finalizando contabilização do vinculo $nrvinculom - $nmvinculom".PHP_EOL;
         
                if (($this->_retrocesso === false) && ($this->_simulacao === false)) {
                    if ($this->_idFormaContab == 'C' || $this->_idFormaContab == 'T') {
                        $contabiliza->persistResumoContabil();
                    }
                    if ($this->_idFormaContab == 'D' || $this->_idFormaContab == 'T') {
                        $contabiliza->persistResumoDro();
                    }
                }
                
            }
    
            
        }else{
            $fpaCalculoConsumer->setNrtotprocessvinc($fpaCalculoConsumer->getNrtotvinculos());
        }  
        
        \Setforms2\Shared\Registry::getEntityManager()->beginTransaction();

        $this->entityManager->getRepository(Repositories::FPA_CALCULOCONSUMER)->concluirCalculoConsumer($this->_nrOrgTrabalho, $fpaCalculoConsumer->getNrcalculoconsumer());
        $fpaCalculoConsumer->setNrcalculofolha(null);
        //$fpaCalculoConsumer->setIderro(($this->_msg_idstatus === MsgCalculo::ID_COM_ERRO) or ($this->_msg_idstatus === MsgCalculo::ID_COM_ALERTA));
        
        \Setforms2\Shared\Registry::getEntityManager()->persist($fpaCalculoConsumer);
        \Setforms2\Shared\Registry::getEntityManager()->flush();
        \Setforms2\Shared\Registry::getEntityManager()->commit();

        echo PHP_EOL."---------------".PHP_EOL;
        echo ">>>>>>>>>> Processo Finalizado <<<<<<<<<<".PHP_EOL;

        $demonstrativo = $contabiliza->preparaEmail();
        array_unshift($demonstrativo, $this->msg);
        
        return $demonstrativo;
    }
    
    private function _buscaDadosFormulario($dataSet){
        
        switch ($this->_nrTipoCalculo) {
            case FpatipocalculoEntity::CALC_FERIAS :
                $this->_buscaDadosFormularioFerias($dataSet);
                break;

            case FpatipocalculoEntity::CALC_RESCISAO :
                $this->_buscaDadosFormularioRescisao($dataSet);
                break;

            case FpatipocalculoEntity::CALC_RESCISAO_COMPLEMENTAR :
                $this->_buscaDadosFormularioRescocomplementar($dataSet);
                break;
            case FpatipocalculoEntity::CALC_CONTABILIZACAO :
                $this->_buscaDadosFormularioContabilizacao($dataSet);
                break;
            case FpatipocalculoEntity::CALC_FOLHA_COMPLEMENTAR :
                $this->_buscaDadosFormularioCalculoComplementar($dataSet);
                break;
            default:
                $this->_buscaDadosFormularioCalculo($dataSet);
                break;
        }
    }
    
    private function _buscaDadosFormularioContabilizacao($dataSet){
        $this->_formulario = $dataSet;
        $this->_dsOrdem = $dataSet['DSORDEM'];
        $this->_dsInicio = $dataSet['DSINICIO'];
        $this->_dsFim = $dataSet['DSFIM'];
        $this->_nr_tipo_hierarquia = $dataSet['NRHIERARQORG'];
        $this->_tipo_movimento = $dataSet['NRTIPOMOVIMENT'];
        $this->_nrOcorrenciaCalc = $dataSet['NROCORRECAL'];
        $this->_idFormaContab = $dataSet['IDFORMACONTAB'];
    }
    
    private function _buscaDadosFormularioCalculoComplementar($dataSet){
        $this->_dsOrdem = trim($dataSet['DSORDEM']);
        $this->_dsInicio = is_string($dataSet['DSINICIO']) ? trim($dataSet['DSINICIO']) : $dataSet['DSINICIO'];
        $this->_dsFim = $dataSet['DSFIM'];
        $this->_dtPagamento = $dataSet['DTPAGAMENTO'];
        $this->_dtInicio = DateUtil::getDataDeString(substr($dataSet['DTINI'], 0, 10), DateUtil::FORMATO_BRASILEIRO, true);
        $this->_dtFim = DateUtil::getDataDeString(substr($dataSet['DTFIM'], 0, 10), DateUtil::FORMATO_BRASILEIRO, true);
        $dataSet['IDDIFFERIAS'] = 'S';
    }

    private function _buscaDadosFormularioCalculo($dataSet){
        $this->_dsOrdem = trim($dataSet['DSORDEM']);
        $this->_dsInicio = is_string($dataSet['DSINICIO']) ? trim($dataSet['DSINICIO']) : $dataSet['DSINICIO'];
        $this->_dsFim = $dataSet['DSFIM'];
        $this->_dtPagamento = $dataSet['DTPAGAMENTO'];
    }

    private function _buscaDadosFormularioFerias($dataSet){
        $this->_dsOrdem = trim($dataSet['DSORDEM']);
        $this->_dsInicio = is_string($dataSet['DSINICIO']) ? trim($dataSet['DSINICIO']) : $dataSet['DSINICIO'];
        $this->_dsFim = $dataSet['DSFIM'];
        $this->_dtPagamento = $dataSet['DTPAGAMENTO'];
        $this->_dt_inicio = DateUtil::getDataDeString(substr($dataSet['DTINIGOZOINI'],0, 10));
        $this->_dt_fim = DateUtil::getDataDeString(substr($dataSet['DTINIGOZOFIM'], 0, 10));
        $this->_dias_abono = DateUtil::getDataDeString($dataSet['QTDIASABONOFERI']);
        $this->nrFerias = $dataSet['NRFERIAS'];
    }

    private function _buscaDadosFormularioRescisao($dataSet){
        $this->_dsOrdem = trim($dataSet['DSORDEM']);
        $this->_dsInicio = is_string($dataSet['DSINICIO']) ? trim($dataSet['DSINICIO']) : $dataSet['DSINICIO'];
        $this->_dsFim = $dataSet['DSFIM'];
        $this->_dtPagamento = $dataSet['DTPAGAMENTO'];
        $this->_dt_inicio = DateUtil::getDataDeString(substr($dataSet['DTRESCISAOINI'],0, 10));
        $this->_dt_fim = DateUtil::getDataDeString(substr($dataSet['DTRESCISAOFIM'], 0, 10));
    }

    private function _buscaDadosFormularioRescocomplementar($dataSet){
        $this->_dsOrdem = trim($dataSet['DSORDEM']);
        $this->_dsInicio = is_string($dataSet['DSINICIO']) ? trim($dataSet['DSINICIO']) : $dataSet['DSINICIO'];
        $this->_dsFim = $dataSet['DSFIM'];
        $this->_dtPagamento = $dataSet['DTPAGAMENTO'];
        $this->_dt_inicio = DateUtil::getDataDeString(substr($dataSet['DTRESCISAOINI'],0, 10));
        $this->_dt_fim = DateUtil::getDataDeString(substr($dataSet['DTRESCISAOFIM'], 0, 10));
        $this->_id_fazDifComp = $dataSet['IDFAZDIFCOMP'];
    }

    private function _executaPreparacao(
         $vinculo, $preparacao, $contador
    ){
        if ($this->_nrTipoCalculo !== FpatipocalculoEntity::CALC_FOLHA &&
            $this->_nrTipoCalculo !== FpatipocalculoEntity::CALC_ADIANTAMENTO &&
            $this->_nrTipoCalculo !== FpatipocalculoEntity::CALC_FECHAMENTO
        ) {
            $this->nrGozoFerias = $preparacao->preparar($vinculo, $this->_tp_calc_mov, $contador);
        }
    }

    private function finalizaMensagens() {
        $msgcalculo = MsgCalculo::getInstance()->getDados();
        if ($msgcalculo['IDSTATUS'] === MsgCalculo::ID_COM_ERRO) {
            $this->_msg_idstatus = MsgCalculo::ID_COM_ERRO;
        } elseif ($msgcalculo['IDSTATUS'] === MsgCalculo::ID_COM_ALERTA and $this->_msg_idstatus != MsgCalculo::ID_COM_ERRO) {
            $this->_msg_idstatus = MsgCalculo::ID_COM_ALERTA;
        }
        if ($msgcalculo['IDABORT'] === true) {
            $this->_msg_idabort = true;
            $this->_msg_mensagens = $msgcalculo['MENSAGENS'];
            return false;
        } else {
            MsgCalculo::getInstance()->salvaMensagens();
        }
        MsgCalculo::getInstance()->clear();
        return true;
    }
    
    private function _verificaEstruturasVinculos(){

        $nrtpestrutlegal = FPA::getTipoEstruturaLegalOrg();
        $nrtpestrutgeren = FPA::getTipoEstruturaGerencialOrg();
        
        $estrutlegal = $this->_vinculom->getVinculosEstrutLegalDifereMovimentacao($this->_nrOrgTrabalho, $this->_mesCompetencia, self::TP_TRANSF_LEGAL, $nrtpestrutlegal, $this->_dsOrdem, $this->_dsInicio, $this->_dsFim);
        $estrutgeren = $this->_vinculom->getVinculosEstrutGerenDifereMovimentacao($this->_nrOrgTrabalho, $this->_mesCompetencia, self::TP_TRANSF_GEREN, $nrtpestrutgeren, $this->_dsOrdem, $this->_dsInicio, $this->_dsFim);
        
        if ($estrutlegal || $estrutgeren) {
            $this->msg .= "<b>Erros de movimentação </b><br>";
            $this->msg .= "<b>----------------------</b><br>";

            if($estrutlegal) {
                $this->msg .= "<b>Vinculos com estrutura legal do histórico diferente da movimentação legal: </b><br>";
                /** @var GpevinculomEntity $vinculo */
                foreach ($estrutlegal as $vinculo) {
                    $this->msg .= $vinculo->getNrvinculom() . " - " . $vinculo->getNmvinculom() . "<br>";
                }
            }

            if($estrutgeren) {
                $this->msg .= "<b>Vinculos com estrutura gerencial do histórico diferente da movimentação gerencial: </b><br>";
                /** @var GpevinculomEntity $vinculo */
                foreach ($estrutgeren as $vinculo) {
                    $this->msg .= $vinculo->getNrvinculom() . " - " . $vinculo->getNmvinculom() . "<br>";
                }
            }
            return false;
        }
        return true;
    }

    private function _verificaAssociacaoContabil($lista_calculo_folha){

        $erros = array();

        foreach ($lista_calculo_folha as $nr_calculo_folha) {
            $eventos = $this->_resumoContabil->getEventoSemAssociacaoPorSelecaoVinculos($this->_nrOrgTrabalho, $this->_nrOrgPadrao, $this->_mesCompetencia, $nr_calculo_folha, $this->_dsOrdem, $this->_dsInicio, $this->_dsFim);

            if ($eventos) {
               $erros = array_merge($erros, $eventos);
            }
        }
        if (!empty($erros)) {
            $this->msg .= "<b>Erros - Eventos sem  Associação Contábil para a estrutura legal</b><br>";
            $this->msg .= "<b>-----------------------------</b><br>";

            foreach ($erros as $evento) {
                $this->msg .= $evento['nreventom'] . " - " . $evento['nmeventoh'] . "<br>";
            }
            return false;
        }
        return true;
    }
}