<?php

namespace HCM\FPA\Calculos\Geral;
use HCM\FPA\Calculos\Geral\TeknisaRegra;

class FolhaEvaluator extends TeknisaEvaluator{

    private $_calculoRegra;
    private $_calculo;
    private $_mnemonicos;

    public function __construct($calculo_regra){
        $calculoRegra = new TeknisaRegra($calculo_regra);
        parent::__construct($calculoRegra);
        $this->_calculoRegra = $calculo_regra;

        $this->_inicializaAtributos();
        $this->_declaraRegrasFolha();
    }
    
    private function _inicializaAtributos(){
        $this->_calculo = $this->_calculoRegra->getCalculo();
    }

    private function _declaraRegrasFolha(){
        $this->_mnemonicos = array(
            'SE' => array($this->_calculoRegra, 'retornaValorPorCondicao', array('condicao', 'valorSeVerdadeiro', 'valorSeFalso')),
            'DIAS_NORM'=> array($this->_calculo, 'getDiasNormais', ''),
            'DIAS_TRAB'=> array($this->_calculo, 'getDiasTrabalhados', ''),
            'DIAS_REM'=> array($this->_calculo, 'getDiasRemuneracao', ''),
            'DIAS_FER'=> array($this->_calculo, 'getDiasFerias', ''),
            'DIAS_FER_PROX'=> array($this->_calculo, 'getDiasFeriasMesAnt', array('mes')),
            'DIAS_FER_TOTAL'=> array($this->_calculo, 'getDiasFeriasTotal', ''),
            'ATRIB_VAL'=> array($this->_calculoRegra, 'atribuiValor', array('valor')),
            'ATRIB_REF'=> array($this->_calculoRegra, 'atribuiReferencia', array('referencia')),
            'ATRIB_OCOR'=> array($this->_calculoRegra, 'atribuiOcorrencias', array('ocorrencias')),
            'VAL_EVEN'=> array($this->_calculoRegra, 'retornaValorEvento', array('nr_evento')),
            'REF_EVEN'=> array($this->_calculoRegra, 'retornaReferenciaEvento', array('nr_evento')),
            'EXISTE_EVEN'=> array($this->_calculoRegra, 'verificaExistenciaEvento', array('nr_evento')),
            'VAL_INCID'=> array($this->_calculoRegra, 'retornaValorIncidencia', array('nr_incidencia')),
            'REF_INCID'=> array($this->_calculoRegra, 'retornaReferenciaIncidencia', array('nr_incidencia')),
            'DADOS_VINC'=> array($this->_calculoRegra, 'retornaDadosVinculo', array('nomeColuna')),
            'TOTAL_VAL'=> array($this->_calculoRegra, 'totalizaValorHistorico', array('listaNrEvento', 'nrTipoMovimento', 'deslocamento', 'mesIncidencia')),
            'TOTAL_REF'=> array($this->_calculoRegra, 'totalizaReferenciaHistorico', array('listaNrEvento', 'nrTipoMovimento', 'deslocamento', 'mesIncidencia')),
            'TOTAL_PARC'=> array($this->_calculoRegra, 'totalizaOcorrenciaHistorico', array('listaNrEvento', 'nrTipoMovimento', 'deslocamento', 'mesIncidencia')),
            'DIAS_ENTRE' => array($this->_calculoRegra, "getDiasEntre", array("dataInicial", "dataFinal")),
            'IS_NULL' => array($this->_calculoRegra, "isNull", array("value")),
            'DT_COMPETEN' => array($this->_calculoRegra, "getDataCompetencia", ''),
            'TOMADOR' => array($this->_calculoRegra, "getTomador", ''),
            'DIAS_REAIS' => array($this->_calculoRegra, "getDiasCompetencia", ''),
            'DIAS_DSR'=> array($this->_calculo, 'getDiasDSR', ''),
            'DIAS_DSR_MES'=> array($this->_calculo, 'getDiasDSRmes', ''),
            'DIAS_UTEIS_MES'=> array($this->_calculo, 'getDiasDSRmesUtil', ''),
            'DIAS_MATER'=> array($this->_calculo, 'getDiasMaternidade', ''),
            'SALARIO'=> array($this->_calculo, 'getSalario', ''),
            'SALARIO_HORA'=> array($this->_calculo, 'getSalarioHora', ''),
            'JORNADA_DIA'=> array($this->_calculo, 'getJornadaDiaria', ''),
            'JORNADA_SEMANAL'=> array($this->_calculo, 'getJornadaSemanal', ''),
            'OCORRENCIA'=> array($this->_calculo, 'getOcorrencia', ''),
            'ULT_DIA_COMP'=> array($this->_calculoRegra, 'getUltimoDiaMes', ''),
            'DIAS_ATEST'=> array($this->_calculo, 'getDiasAtestados', ''),
            'DIAS_DSR_ATEST'=> array($this->_calculo, 'getDiasDsrAtest', ''),
            'DIAS_UTEIS_ATEST'=> array($this->_calculo, 'getDiasUteisAtest', ''),
            'CONV_MIN_DEC'=>array($this->_calculoRegra, 'converteHoraEmDecimal', array('ref_evento')),
            'DIAS_DSR_FER'=> array($this->_calculo, 'getDiasRsrFer', ''),
            'DIAS_ULT_FER'=> array($this->_calculo, 'getDiasUteisFer', ''),
            'DIAS_ULT'=> array($this->_calculo, 'getDiasUteis', ''),
            'IDADE_VINC'=> array($this->_calculoRegra, 'retornaIdadeVinculo', ''),
            'DIAS_ULT_SITU_FUNC'=> array($this->_calculoRegra, 'retornaDiasUltSituFunc', ''),
            'TRABALHO_EM_ANOS'=> array($this->_calculoRegra, 'retornaTrabalhoEmAnos', ''),
            'VAL_EVEN_MES'=> array($this->_calculoRegra, 'retornaValEvenOcorrencias', array('nr_evento')),
            'DIAS_SEMANA' => array($this->_calculo, "getDiasDeSemana", array("dataInicial")),
            'AVOS_DIREITO' => array($this->_calculo, "calculaAvosDireito", array("voltarMeses")),
            'DT_ULT_MOV' => array($this->_calculoRegra, 'retornaDataInicioUltMovimentacao', array('nrTipoEstrutura')),
            'TIPO_JORNADA' => array($this->_calculoRegra, "retornaTipoJornada", ''),
            'SALARIO_MINIMO' => array($this->_calculoRegra, 'retornaSalarioMinimo', ''),
            'QTDE_SEMANAS_SIT' => array($this->_calculo, 'getQtSemanas', array("nrsitufuncm")),
            'QTDE_DIAS_SIT' => array($this->_calculo, 'getQtDiasSit', array("nrsitufuncm")),
            'VALOR_REFERENCIA' => array($this->_calculo, 'getValorReferencia', array("valor")),
            'ANT_QTDE_SEMANAS_SIT' => array($this->_calculo, 'getQtSemanasAnt', array("nrsitufuncm", "meses")),
            'ANT_QTDE_DIAS_SIT' => array($this->_calculo, 'getQtDiasSitAnt', array("nrsitufuncm", "meses")),
            'ANT_VAL_EVEN'=> array($this->_calculo, 'retornaValorEventoAnt', array('nr_evento', 'valor')),
            'ANT_REF_EVEN'=> array($this->_calculo, 'retornaReferenciaEventoAnt', array('nr_evento', 'valor')),
            'ANT_DIAS_REAIS' => array($this->_calculoRegra, "getDiasCompetencia", ''),
            'ANT_DIAS_DSR_MES'=> array($this->_calculo, 'getDiasDSRmes', ''),
            'ANT_DIAS_DSR'=> array($this->_calculo, 'getDiasDSR', ''),
            'ANT_DIAS_ULT'=> array($this->_calculo, 'getDiasUteis', ''),
            'ANT_DIAS_UTEIS_ATEST'=> array($this->_calculo, 'getDiasUteisAtest', '')
        );
        
        foreach($this->_mnemonicos as $mnemonico => $value){
            $this->setObjectFunction($mnemonico, $value[0], $value[1],$value[2]);
        }
    }
    
}