<?php

namespace HCM\FPA\Controllers;

use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Response\Notification;
use Zeedhi\Framework\DTO\Response\Message;
use HCM\Util\HCMParameters;
use Doctrine\DBAL\Connection;
use Doctrine\ORM\EntityManager;
use HCM\Util\DateUtil;
use HCM\Util\Repositories;
use HCM\Geral\Repository\RateioLancamentoRepository;
use Zeedhi\Framework\DataSource\DataSet;

class RateioLancamentoController extends Crud {
    
    protected $rateioLancamentoService;
    protected $parameterBagColumns = array('P_NRORG', 'P_NRORG_PADRAO', 'P_DTMESCOMPETENC');
    public function __construct (Manager $dataSourceManager, ParameterBag $parameterBag, HCMParameters $hcmParameters, Connection $connection, EntityManager $entityManager, \HCM\FPA\Services\RateioLancamentoService $rateioLancamentoService, RateioLancamentoRepository $rateioLancamentoRepository)
    {
        $this->parameterBag = $parameterBag;
        $this->hcmParameters = $hcmParameters;
        $this->connection = $connection;
        $this->entityManager = $entityManager;
        $this->rateioLancamentoService = $rateioLancamentoService;
        $this->rateioLancamentoRepository = $rateioLancamentoRepository;
        parent::__construct($dataSourceManager);
    }

    public function getRateio(DTO\Request\Filter $request, DTO\Response $response)
    {   
       
        $this->dataSourceName = "fpa_rateio_get";
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceName);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData);
        $rows = $dataSet->getRows();
        foreach($rows as $key => &$row){
            if(isset($row['NRVINCULOEMPREG'])){
                
                $row['NRVINCULOEMPREG'] = explode(",", $row['NRVINCULOEMPREG']);
                
                foreach($row['NRVINCULOEMPREG'] as $chave => $value)
                    $row['NRVINCULOEMPREG'][$chave] = intval($value);
                $row['NMABREV'] = explode(",", $row['NMABREV']);
            }    
            if(isset($row['NRSITUFUNCM'])){
                $row['NRSITUFUNCM'] = explode(",", $row['NRSITUFUNCM']);
                
                foreach($row['NRSITUFUNCM'] as $chave => $value)
                    $row['NRSITUFUNCM'][$chave] = intval($value);
                    
                $row['NMSITUFUNCH'] = explode(",", $row['NMSITUFUNCH']);
            }    
            if(isset($row['NRTIPOVINCULOM'])){
                $row['NRTIPOVINCULOM'] = explode(",", $row['NRTIPOVINCULOM']);
                
                foreach($row['NRTIPOVINCULOM'] as $chave => $value)
                    $row['NRTIPOVINCULOM'][$chave] = intval($value);
                    
                $row['NMTIPOVINCULOH'] = explode(",", $row['NMTIPOVINCULOH']);
            }    
        }
        
        
        $response->addDataSet(new DataSet($this->dataSourceName, $rows));
    }

    public function setRateio(DTO\Request\DataSet $request, DTO\Response $response)
    {
        try {
            
            $this->dataSourceName = "fpa_rateio";
            foreach ($request->getDataSet()->getRows() as &$row) {
                $validaTamanhoNrSituFuncMeNrvincEmpreg = $this->rateioLancamentoService->validaTamanhoNrSituFuncMeNrvincEmpreg($row["NRSITUFUNCM"],$row["NRVINCULOEMPREG"]);
                if($validaTamanhoNrSituFuncMeNrvincEmpreg === 'maiorSitufunc'){
                    $response->addNotification(new Notification("Deve ser selecionado um máximo de 40 situações funcionais que não gera rateio. Favor verificar.", Notification::TYPE_ALERT));
                }
                if($validaTamanhoNrSituFuncMeNrvincEmpreg === 'maiorVinculoEmpreg'){
                    $response->addNotification(new Notification("Deve ser selecionado um máximo de 40 vínculos empregatícios. Favor verificar.", Notification::TYPE_ALERT));
                }
                
                $row["NRSITUFUNCM"] = isset($row['NRSITUFUNCM']) && is_array($row['NRSITUFUNCM']) ? implode(",", $row['NRSITUFUNCM']) : $row["NRSITUFUNCM"];
                $row["NRTIPOVINCULOM"] = isset($row['NRTIPOVINCULOM']) && is_array($row['NRTIPOVINCULOM']) ? implode(",", $row['NRTIPOVINCULOM']) : $row["NRTIPOVINCULOM"]; 
                $row["NRVINCULOEMPREG"] = isset($row['NRVINCULOEMPREG']) && is_array($row['NRVINCULOEMPREG']) ? implode(",", $row['NRVINCULOEMPREG']) : $row["NRVINCULOEMPREG"]; 
                
                $existeRateio = $this->rateioLancamentoRepository->buscaRateioExistente($row['NRORG'], $row['DTINIVIGENCIA'], $row['NRESTRUTURAM'], $row['NREVENTOM']);
                
                if($existeRateio && ($existeRateio['NRRATEIO'] != $row['NRRATEIO'] || !isset($row['NRRATEIO'])) ){
                    $response->addNotification(new Notification("Já existe um registro pra essa estrutura e evento com a mesma data de início vigência", Notification::TYPE_ALERT));
                    return;
                } 

                if($row['__is_new'] === true){
                    $row['NRRATEIO'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'FPA_RATEIO', 1, 12, true); 
                }
                
                parent::save($request, $response);
            }
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function deleteRateio(DTO\Request\DataSet $request, DTO\Response $response)
    {
        try{
            
            $this->dataSourceName = "fpa_rateio";
            $nrorg = $request->getDataSet()->getRows()[0]['NRORG'];
            $nrrateio = $request->getDataSet()->getRows()[0]['NRRATEIO'];

            $existeItens = $this->rateioLancamentoService->existeItensRateio($nrorg, $nrrateio);

            if(!$existeItens){
                parent::delete($request, $response);
            } else {
                $response->addNotification(new Notification("Existem itens relacionados a este registro, favor excluí-los.", Notification::TYPE_ALERT));
            }
        } catch(\Exception $e){
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }    
    }

    public function getRateioLancamento(DTO\Request\Filter $request, DTO\Response $response)
    {   
        $this->dataSourceName = "fpa_rateiolanc_get";
        $this->parameterBagColumns = array('P_NRORG', 'P_NRORG_PADRAO', 'P_NRRATEIO', 'P_DTMESCOMPETENC');

        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceName);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData);
        $response->addDataSet($dataSet);
    }

    public function setRateioLancamento(DTO\Request\DataSet $request, DTO\Response $response)
    {   
        try {
            $this->dataSourceName = "fpa_rateiolanc";
            foreach ($request->getDataSet()->getRows() as &$row) {
                
                /*$existeRateioLancamento = $this->rateioLancamentoRepository->buscaRateioExistenteLancamento($row['NRORG'], $row['NRRATEIO'], $row['DTLANCAMENTO']);
      
                if($existeRateioLancamento && ($existeRateioLancamento['NRRATEIOLANC'] != $row['NRRATEIOLANC'] || !isset($row['NRRATEIOLANC'])) ){
                    $response->addNotification(new Notification("Já existe um registro com essa data de lançamento", Notification::TYPE_ALERT));
                    return;
                }*/
                
                $row['VRACRESCIMO'] = empty($row['VRACRESCIMO']) ? 0 : $row['VRACRESCIMO'];
                $row['VRDESCONTO'] = empty($row['VRDESCONTO']) ? 0 : $row['VRDESCONTO'];
             
                if($row['__is_new'] === true){
                    $row['NRRATEIOLANC'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'FPA_RATEIOLANC', 1, 12, true); 
                }

                $this->rateioLancamentoService->calculaLancamento(
                    $row['NRORG'], $row['NRRATEIO'], $row['VRLANCAMENTO'], $row['VRACRESCIMO'], $row['VRDESCONTO'], $row['VRSUBTOTAL1'], $row['VRVALSUBTOTAL1'], 
                    $row['VRSUBTOTAL2'], $row['VRVALSUBTOTAL2'], $row['VRSUBTOTAL3'], $row['VRVALSUBTOTAL3'], $row['VRFINAL']
                );
                parent::save($request, $response);
            }
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function deleteRateioLancamento(DTO\Request\DataSet $request, DTO\Response $response)
    {
        try{
            $this->dataSourceName = "fpa_rateiolanc";
            
            foreach ($request->getDataSet()->getRows() as $row) {
                $existeValores = $this->rateioLancamentoService->existeValoresRateio($row['NRORG'], $row['NRRATEIO'], $row['DTLANCAMENTO']);
            }
            
            if (!$existeValores) {
                parent::delete($request, $response);
            } else {
                $response->addNotification(new Notification("Existem valores relacionados ao lançamento, favor excluí-los.", Notification::TYPE_ALERT));
            }
        } catch(\Exception $e){
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }  
    }

    public function importarCuponsOdhen(DTO\Request\DataSet $request, DTO\Response $response)
    {
        try {
            $qtdLancamentos = 0;
            
            foreach ($request->getDataSet()->getRows() as &$row) {
                $nrorg = $row['NRORG'];
                $nrrateio = $row['NRRATEIO'];
                $nrestruturam = $row['NRESTRUTURAM'];
                $cdunidade = $row['CDUNIDADE'];
                $dtinicio = $row['DTINICIO'];
                $dtfim = $row['DTFIM'];
                
                if ($nrorg != 3469) { //Atualmente só está disponivel para a organização 3469 - Pobre Juan
                    $response->addNotification(new Notification("Integração não disponível para sua organização, contate o administrador.", Notification::TYPE_ERROR));
                    return;
                }
                
                $qtdLancamentos += $this->rateioLancamentoService->importarCuponsOdhen($nrorg, $nrrateio, $nrestruturam, $cdunidade, $dtinicio, $dtfim);
            }
            
            if ($qtdLancamentos > 0) {
                $response->addNotification(new Notification("Realizada a importação de {$qtdLancamentos} lançamentos com sucesso!", Notification::TYPE_SUCCESS));
            } else {
                $response->addNotification(new Notification("Não foram encontradas vendas para a unidade e período informados.", Notification::TYPE_ERROR));
            }
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function getRateioEstrutura(DTO\Request\Filter $request, DTO\Response $response)
    {   
        $this->dataSourceName = "fpa_rateioestrut_get";
        $this->parameterBagColumns = array('P_NRORG', 'P_NRORG_PADRAO', 'P_NRRATEIO');
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceName);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData);
        $response->addDataSet($dataSet);
    }

    public function setRateioEstrutura(DTO\Request\DataSet $request, DTO\Response $response)
    {
        try {
            $this->dataSourceName = "fpa_rateioestrut";
            foreach ($request->getDataSet()->getRows() as &$row) {
                $existeRateioEstrutura = $this->rateioLancamentoRepository->buscaRateioExistenteEstrutura($row['NRORG'], $row['NRESTRUTURAM'], $row[ 'NRRATEIO']);
                
                if($existeRateioEstrutura &&  ($existeRateioEstrutura['NRRATEIOESTRUT'] != $row['NRRATEIOESTRUT'] || !isset($row['NRRATEIOESTRUT'])) ){
                    $response->addNotification(new Notification("Já existe um registro com essa estrutura", Notification::TYPE_ALERT));
                    return;
                }  
                
                if($row['__is_new'] === true){
                    $row['NRRATEIOESTRUT'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'FPA_RATEIOESTRUT', 1, 12, true); 
                }
                parent::save($request, $response);
                $this->rateioLancamentoService->calculaPorcentagem($row['NRORG'],$row['NRRATEIO'], 'FPA_RATEIOESTRUT');
            }
            
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function deleteRateioEstrutura(DTO\Request\DataSet $request, DTO\Response $response)
    {
        try{
           
            $row = $request->getDataSet()->getRows()[0];
            $this->dataSourceName = "fpa_rateioestrut";
            
            parent::delete($request, $response);
            $this->rateioLancamentoService->calculaPorcentagem($row['NRORG'],$row['NRRATEIO'], 'FPA_RATEIOESTRUT');
        } catch(\Exception $e){
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }    
    }

    public function deleteRateioEstruturaChecked(DTO\Request\DataSet $request, DTO\Response $response){
        try{
            $this->dataSourceName = 'fpa_rateioestrut';
            $row = $request->getDataSet()->getRows()[0];
            
            parent::delete($request, $response);
            $this->rateioLancamentoService->calculaPorcentagem($row['NRORG'],$row['NRRATEIO'], 'FPA_RATEIOESTRUT');
        } catch(\Exception $e){
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }    
    }

    public function getRateioOcupacao(DTO\Request\Filter $request, DTO\Response $response)
    {   
        $this->dataSourceName = "fpa_rateioocup_get";
        $this->parameterBagColumns = array('P_NRORG', 'P_NRORG_PADRAO', 'P_DTMESCOMPETENC','P_NRRATEIO');
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceName);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData);
        $response->addDataSet($dataSet);
    }

    public function setRateioOcupacao(DTO\Request\DataSet $request, DTO\Response $response)
    {
        try {
            $this->dataSourceName = "fpa_rateioocup";
            foreach ($request->getDataSet()->getRows() as &$row) {
                $row['VRFINAL'] = isset($row['VRFINAL']) ? $row['VRFINAL'] : NULL;
                $existeRateioOcupacao = $this->rateioLancamentoRepository->buscaRateioExistenteOcupacao($row['NRORG'],  $row['NRESTRUTURAM'], $row['NROCUPACAOM'], $row['NRRATEIO']);
                
                if($existeRateioOcupacao && ($existeRateioOcupacao['NRRATEIOOCUP'] != $row['NRRATEIOOCUP'] || !isset($row['NRRATEIOOCUP'])) ) {
                    $response->addNotification(new Notification("Já existe um registro pra essa ocupação e estrutura gerencial", Notification::TYPE_ALERT));
                    return;
                }  
               
                if($row['__is_new'] === true){
                    $row['NRRATEIOOCUP'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'FPA_RATEIOOCUP', 1, 12, true); 
                }

                parent::save($request, $response);
                $this->rateioLancamentoService->calculaPorcentagem($row['NRORG'],$row['NRRATEIO'], 'FPA_RATEIOOCUP');
            }
            
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }       
    }

    public function deleteRateioOcupacao(DTO\Request\DataSet $request, DTO\Response $response)
    {
        try{
            $this->dataSourceName = "fpa_rateioocup";
            $row = $request->getDataSet()->getRows()[0];
            
            parent::delete($request, $response);
            $this->rateioLancamentoService->calculaPorcentagem($row['NRORG'],$row['NRRATEIO'], 'FPA_RATEIOOCUP');
        } catch(\Exception $e){
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }      
    }

    public function getRateioValores(DTO\Request\Filter $request, DTO\Response $response)
    {   
        $this->dataSourceName = "fpa_rateioval_get";
        $this->parameterBagColumns = array('P_NRORG', 'P_NRORG_PADRAO', 'P_NRRATEIO', 'P_DTMESCOMPETENC');
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceName);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData);
        $response->addDataSet($dataSet);
    }
    
    public function replicaParametroRateioEstrutura(DTO\Request\DataSet $request, DTO\Response $response) {
        //REMOVER A ROTA DO ROUTES_FPA ANTES DE FINALIZAR
        $row = $request->getDataSet()->getRows();
        $row = $row[0];
        
        try {
            
            $retornoReplicaEstrutura = $this->rateioLancamentoService->replicaParametroRateioEstrutura($row);
            
            if($retornoReplicaEstrutura === true){
                $response->addNotification(new Notification("Estruturas importadas com sucesso", Notification::TYPE_SUCCESS));
            }else if($retornoReplicaEstrutura === false){
                $response->addNotification(new Notification("Estruturas já existentes neste rateio",Notification::TYPE_ERROR));
            }else{
                $response->addNotification(new Notification("O rateio selecionado não possui estruturas cadastradas, favor verificar.",Notification::TYPE_ERROR));
            }
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function replicaParametroRateioOcupacao(DTO\Request\DataSet $request, DTO\Response $response) {
        //REMOVER A ROTA DO ROUTES_FPA ANTES DE FINALIZAR
        $row = $request->getDataSet()->getRows();
        $row = $row[0];
        
        try {
            $retornoReplicaOcupacao = $this->rateioLancamentoService->replicaParametroRateioOcupacao($row);
            
            if($retornoReplicaOcupacao === true){
                $response->addNotification(new Notification("Ocupações importadas com sucesso", Notification::TYPE_SUCCESS));
            }else if($retornoReplicaOcupacao === false){
                $response->addNotification(new Notification("Ocupações já existentes neste rateio",Notification::TYPE_ERROR));
            }else{
                $response->addNotification(new Notification("O rateio selecionado não possui ocupações cadastradas, favor verificar.",Notification::TYPE_ERROR));
            }
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function deleteRateioValores(DTO\Request\DataSet $request, DTO\Response $response)
    {
        try{
            $this->dataSourceName = "fpa_rateioval";
            parent::delete($request, $response);
        } catch(\Exception $e){
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }      
    }
    
    public function deleteValoresGeral(DTO\Request\DataSet $request, DTO\Response $response)
    {
        try{
            $rows = $request->getDataSet()->getRows();
            if(!empty($rows)){
                $deleteValores = $this->rateioLancamentoService->deleteRateioValores($rows);
                $response->addNotification(new Notification(Notification::TYPE_SUCCESS));
            }else{
                $response->addNotification(new Notification('Não há registros para exclusão',Notification::TYPE_ALERT));
            }
            
            
        } catch(\Exception $e){
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }    
    }

    public function calculoRateioLancamento(DTO\Request\DataSet $request, DTO\Response $response)
    {
        try {
            $this->dataSourceName = "fpa_rateioval";
            
            foreach ($request->getDataSet()->getRows() as &$row) {
                
                $row['DTINICIO'] = ($row['DTINICIO'] == '' ? null : $row['DTINICIO']);
                $row['DTFIM'] = ($row['DTFIM'] == '' ? null : $row['DTFIM']);
                
                $valores = $this->rateioLancamentoService->calculoRateioLancamento($row['NRORG'], $row['NRRATEIO'], $row['DTINICIO'], $row['DTFIM'], $response);
                
                if ($valores) {
                    foreach ($valores as $valor) {
                        $row['NRORG'] = $valor['NRORG'];
                        $row['NRRATEIO'] = $valor['NRRATEIO'];
                        $row['NRESTRUTURAM'] = $valor['NRESTRUTURAM'];
                        $row['NRVINCULOM'] = $valor['NRVINCULOM'];
                        $row['DTDIALANC'] = $valor['DTDIALANC'];
                        $row['NRPESO'] = $valor['NRPESO'];
                        $row['VRVALOR'] = $valor['VRVALOR'];
                        
                        $calculoExistente = $this->rateioLancamentoRepository->buscaCalculoRateioExistente(
                            $row['NRORG'], $row['NRRATEIO'], $row['NRVINCULOM'], $row['DTDIALANC']
                        );
                                    
                        if($calculoExistente) {
                            $row['__is_new'] = false; 
                            $row['NRRATEIOVAL'] = $calculoExistente['NRRATEIOVAL'];
                        } else {
                            $row['__is_new'] = true;
                            $row['NRRATEIOVAL'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'FPA_RATEIOVAL', 1, 12, true); 
                        }
    
                        parent::save($request, $response);
                    }
                }
            
                if (is_array($valores) && count($valores) > 0) {
                    $response->addMessage(new Message("Rateio realizado com sucesso!"));
                } else {
                    $response->addMessage(new Message("Não foram encontrados itens para o período informado.", Message::TYPE_ERROR));
                }
            }
            
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }       
    }

    public function exportarLoteRateio(DTO\Request\DataSet $request, DTO\Response $response)
    {
        try {
            foreach ($request->getDataSet()->getRows() as &$row) {
                $rateio = $this->entityManager->getRepository(Repositories::FPA_RATEIO)->findOneBy(array('nrrateio' => $row['NRRATEIO'], 'nrorg' => $row['NRORG']));  
                $itens = $this->rateioLancamentoRepository->getItensValoresRateio($row['NRORG'], $row['NRRATEIO'], $row['DTINICIO'], $row['DTFIM']);
                if(empty($itens)) {
                    $response->addNotification(new Notification("Nenhum item foi encontrado para a exportação do rateio de lançamento", Notification::TYPE_ALERT));
                    return;
                }

                $this->rateioLancamentoService->exportaLote($itens, $row['NRORG'],$this->hcmParameters->getDtCompetencia(), $rateio->getNreventom());

                $response->addNotification(new Notification("Exportação do rateio de lançamento concluída", Notification::TYPE_SUCCESS));
                return;
            }
            
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }       
    }

    public function getAuditoria(){
        $auditoria = array();
        $auditoria['DTINCLUSAO'] = DateUtil::getDataAtual()->format('d/m/Y');
        $auditoria['DTULTATU'] = DateUtil::getDataAtual()->format('d/m/Y');
        $auditoria['NRORGINCLUSAO'] = $this->hcmParameters->getNrorgTrabalho();
        $auditoria['CDOPERINCLUSAO'] = $this->hcmParameters->getCdoperador();
        $auditoria['NRORGULTATU'] = $this->hcmParameters->getNrorgTrabalho();
        $auditoria['CDOPERULTATU'] = $this->hcmParameters->getCdoperador();
        return $auditoria;
    }

    protected function processConditionValue($condition) {
        return $condition['value'];
    }

    protected function populateParameterBag(FilterCriteria $filterCriteria) {

        $conditions = array();
        foreach ($filterCriteria->getConditions() as $condition) {
            $columnName = $condition['columnName'];
            if (in_array($columnName, $this->parameterBagColumns)) {
                $this->parameterBag->set($columnName, $this->processConditionValue($condition));
            } else {
                $conditions[] = $condition;
            }
        }
        $newFilterCriteria = new FilterCriteria(
            $filterCriteria->getDataSourceName(),
            $conditions,
            $filterCriteria->getPage(),
            $filterCriteria->getPageSize()
        );
        return $newFilterCriteria;
    }
    
} 
