<?php

namespace HCM\FPA\ObrigacaoLegal\CNAB\Mentore\registro;

use HCM\Util\Messages;
use HCM\Util\Repositories;
use HCM\Util\DateUtil;
use HCM\Util\FPA;
use HCM\Util\TkDI;
use HCM\Util\MsgCalculo;
use HCM\FPA\Registro\RegistroCNAB;
use HCM\FPA\ObrigacaoLegal\CNAB\Mentore\CNABMentore;

class Registro11CNABMentore extends RegistroCNAB{

    const NOME_BANCO_MENTORE = 'Banco Mentore';
    const LOTE_SERVICO_REGISTRO_0 = '0';
    const REGISTRO_11 = '11';
    const VERSAO_LAYOUT = '002';
    const CODIGO_REMESSA = '1';
    
    //HCM
    const MENSAL_HCM = 1;
    const FERIAS_HCM = 2;
    const DECIMO3SALARIO_HCM = 4;
    const ADIANTAMENTO_DECIMO3SALARIO_HCM = 22;
    const VT_HCM = 17;
    const PLR_HCM = 19;
    const BENEFICIOS_HCM = 21;
    
    //MENTORE
    const SALARIO_MENTORE = '01';
    const FERIAS_MENTORE = '04';
    const RESCISAO_MENTORE = '09';
    const VT_MENTORE = '08';
    const DECIMO3SALARIO_MENTORE = '14';
    const ADIANTAMENTO_DECIMO3SALARIO_MENTORE = '02';
    const BENEFICIOS_MENTORE = '15';
    const PLR_MENTORE = '22';
    
    private $_estrutura;
    private $_sequencia;
    private $_codConvenioBanco;
    /*Valor Descrição Layout Mentore
        1 SALÁRIO
        2 1ª PARCELA DO DECIMO
        3 2ª PARCELA DO DECIMO
        4 FÉRIAS
        5 PREMIAÇÃO
        6 COMISSAO
        7 VALE ALIMENTAÇÃO – VA
        8 VALE TRANSPORTE – VT
        9 RESCISÃO
        10 COMBUSTÍVEL
        11 OUTROS
        12 GRATIFICAÇÃO
        13 AJUDA DE CUSTO
        14 13º INTEGRAL
        15 BENEFÍCIOS
        16 DIÁRIA AVULSA
        17 REPASSE
        18 REPASSE FINAL
        19 BOLSA AUXÍLIO ESTÁGIO
        20 RECESSO REMUNERADO ESTÁGIO
        21 BOLSA ESTAGIÁRIO FINAL
        22 PARTICIPAÇÃO DE LUCRO
    */
    /*Valor Descrição HCM
        1 MENSAL
        2 FERIAS
        3 FERIAS 13 SALARIO
        4 13 SALARIO
        5 RESCISAO FERIAS VENCIDAS
        6 RESCISAO FERIAS PROPORCIONAIS
        7 FERIAS_DOBRO_RESC
        8 RESCISAO 13 SALARIO
        9 RESCISAO AVISO PREVIO
        10 ADIANTAMENTO
        11 FERIAS2
        12 FERIAS2 13 SALARIO
        13 PROVISAO FERIAS VENCIDAS
        14 PROVISAO FERIAS PROPORCIONAIS
        15 PROVISAO 13 SALARIO
        16 PROVISAO AVISO PREVIO
        17 VALE TRANSPORTE
        18 SALARIO MATERNIDADE
        19 PARTICIPACAO NOS LUCROS
        20 MEDIA MENSAL
        21 BENEFICIO
        22 ADIANTAMENTO 13 SALARIO
        100 PROCESSO TRABALHISTA
    */

    public function geraRegistro11($nrtipocarga, $nreventom){
        
        //Bloco 1 | 01-02 | Identificador registro header
        $this->atribuiDado(11, 2);
        
        //Bloco 2 | 03-05 | Versão Layout
        $this->atribuiDado('002', 3);
        
        // Bloco 3 | 06-19 | Numero Inscricao da Empresa.
        $this->atribuiDado($this->_estrutura->nrInscricao, 14, parent::CARACTER_ZERO, parent::PREENCHE_ESQUERDA);

        switch($nrtipocarga){
            //Mensal
            case self::MENSAL_HCM:
                $tipomovimentCNAB = self::SALARIO_MENTORE;
                switch($nreventom){
                    case 3050:
                        $tipomovimentCNAB = self::RESCISAO_MENTORE;
                        break;
                    default: 
                        $tipomovimentCNAB = self::SALARIO_MENTORE;
                        break;
                }
                break;
            case self::FERIAS_HCM:
                $tipomovimentCNAB = self::FERIAS_MENTORE;
                break;
            //13 Integral
            case self::DECIMO3SALARIO_HCM:
                $tipomovimentCNAB = self::DECIMO3SALARIO_MENTORE;
                break;
            case self::VT_HCM:
                $tipomovimentCNAB = self::VT_MENTORE;
                break;
            //Benefícios
            case self::BENEFICIOS_HCM:
                $tipomovimentCNAB = self::BENEFICIOS_MENTORE;
                break;
            //Participacao nos lucros
            case self::PLR_HCM:
                $tipomovimentCNAB = self::PLR_MENTORE;
                break;
            default:
                $tipomovimentCNAB = self::SALARIO_MENTORE;
                break;
        }

        // Bloco 3 | 20-21 | Numero Inscricao da Empresa.
        $this->atribuiDado($tipomovimentCNAB, 2);
        
        // Bloco 17 | 22-29 | Data da Geracao do Arquivo (DDMMAAAA).
        $this->_atribuiDataCorrente();
        
        // Bloco 18 | 30-35 | Hora da Geracao do Arquivo (HHMMSS).
        $this->_atribuiHoraCorrente();
        
        // Bloco 4 | 36-55 | Filler (Caracter branco).
        $this->atribuiDado(parent::CARACTER_BRANCO, 20, parent::CARACTER_BRANCO, parent::PREENCHE_DIREITA);

        // Bloco 19 | 56-63 | Numero Sequencial do Arquivo.
        // Evoluir um numero sequencial a cada Header de Arquivo.
        $this->atribuiDado($this->_sequencia, 8, parent::CARACTER_ZERO, parent::PREENCHE_ESQUERDA);
    }

    public function setEstrutura($estrutura){
        $this->_estrutura = $estrutura;
    }

    public function setSequencia($sequencia){
        $this->_sequencia = $sequencia;
    }

    public function setCodConvenioBanco($codConvenioBanco){
        $this->_codConvenioBanco = $codConvenioBanco;
    }
    
    private function _atribuiDataCorrente(){

        $dataAtual = DateUtil::getDataAtual();
        $dtAtual = $dataAtual->format('d/m/Y');

        $this->atribuiDado($dtAtual, 8);
    }

    private function _atribuiHoraCorrente(){

        $dataAtual = DateUtil::getDataAtual();
        $horaAtual = $dataAtual->format('H:m:s');
        $this->atribuiDado($horaAtual, 6);
    }
}