<?php

namespace HCM\FPA\ObrigacaoLegal\CNAB\Mentore\registro;

use HCM\Util\Messages;
use HCM\Util\Repositories;
use HCM\Util\DateUtil;
use HCM\Util\FPA;
use HCM\Util\TkDI;
use HCM\Util\MsgCalculo;
use HCM\FPA\Registro\RegistroCNAB;
use HCM\FPA\ObrigacaoLegal\CNAB\Mentore\CNABMentore;

class Registro21CNABMentore extends RegistroCNAB{

    const SEGMENTO_A = 'A';
    const CODIGO_CAMARA_COMPENSACAO_CC = '000';
    const FINALIDADE_CREDITOCONTA = '01';
    const TIPO_MOEDA = 'BRL';

    private $_pessoa;
    private $_vinculo;
    private $_cnpjEmpresa;

    public function geraRegistro21(){
        
        //Bloco 1 | 01-02 | Identificador registro header
        $this->atribuiDado(21, 2);
        
        //Bloco 2 | 03-13 | Numero Inscricao do Favorecido (Obrigatorio). CPF
        $this->atribuiDado($this->_pessoa->historico->getNrcpfpessoa(), 11, parent::CARACTER_BRANCO, parent::PREENCHE_DIREITA);
    
        //Bloco 3 | 14-23 | Valor do Pagamento (Obrigatorio). Salario liquido.
        $this->atribuiDado($this->_vinculo->valorPagamento, 10, parent::CARACTER_ZERO, parent::PREENCHE_ESQUERDA);
        
        //Bloco 4 | 24-30 | Identificador centro de custo
        $this->atribuiDado(parent::CARACTER_BRANCO, 7, parent::CARACTER_BRANCO, parent::PREENCHE_ESQUERDA);
        
        //Bloco 5 | 31-80 | Descrição centro de custo
        $this->atribuiDado(parent::CARACTER_BRANCO, 50, parent::CARACTER_BRANCO, parent::PREENCHE_ESQUERDA);
        
        //Espaço em Branco
        // Bloco 4 | 81-120 | Filler (Caracter branco).
        $this->atribuiDado(parent::CARACTER_BRANCO, 40, parent::CARACTER_BRANCO, parent::PREENCHE_DIREITA);

        // Bloco 19 | 56-63 | Numero Sequencial do Arquivo.
        // Evoluir um numero sequencial a cada Header de Arquivo.
        $this->atribuiDado($this->_sequencia, 8, parent::CARACTER_ZERO, parent::PREENCHE_ESQUERDA);
    }
    
    public function setPessoa($pessoa)
    {
        $this->_pessoa = $pessoa;
    }


    public function setVinculo($vinculo)
    {
        $this->_vinculo = $vinculo;
    }

    public function setCnpjEmpresa($cnpjEmpresa)
    {
        $this->_cnpjEmpresa = $cnpjEmpresa;
    }
}