<?php

namespace HCM\FPA\Services;


use Doctrine\ORM\EntityManager;
use HCM\Geral\Repository\RateioLancamentoRepository;
use HCM\FPA\Calculos\Geral\CalculoRateio;
use HCM\FPA\Services\LoteLancEventoService;
use HCM\FPA\Util\MemoriaRateio;
use HCM\Util\DateUtil;
use HCM\Util\HCMParameters;
use HCM\Util\Repositories;
use HCM\Util\TkEvaluator;
use Zeedhi\Framework\DTO\Response\Notification;

class RateioLancamentoService {
    
    protected $entityManager;
    
    public function __construct(EntityManager $entityManager, LoteLancEventoService $loteLancEventoService,  HCMParameters $hcmParameters, RateioLancamentoRepository $rateioLancamentoRepository) {
        $this->entityManager = $entityManager;
        $this->loteLancEventoService  = $loteLancEventoService;
        $this->hcmParameters = $hcmParameters;
        $this->rateioLancamentoRepository = $rateioLancamentoRepository;
    }
    
    public function existeItensRateio($nrorg, $nrrateio)
    {
        $existeItemLancamento =  $this->rateioLancamentoRepository->selectExisteItensRateio($nrorg, $nrrateio, 'FPA_RATEIOLANC');
        $existeItemEstrutura =  $this->rateioLancamentoRepository->selectExisteItensRateio($nrorg, $nrrateio, 'FPA_RATEIOESTRUT');
        $existeItemOcupacao = $this->rateioLancamentoRepository->selectExisteItensRateio($nrorg, $nrrateio, 'FPA_RATEIOOCUP');
        $existeValores =  $this->rateioLancamentoRepository->selectExisteItensRateio($nrorg, $nrrateio, 'FPA_RATEIOVAL');
        if($existeItemLancamento || $existeItemEstrutura || $existeItemOcupacao || $existeValores) {
            return true;
        }

        return false;
    }
    
    
    public function existeValoresRateio($nrorg, $nrrateio, $dtlancamento)
    {
        $existeValores =  $this->rateioLancamentoRepository->selectExisteItensRateio($nrorg, $nrrateio, 'FPA_RATEIOVAL', "AND DTDIALANC = '$dtlancamento'");
        
        if($existeValores) {
            return true;
        }

        return false;
    }

    public function callApiGetSales($cdunidade, $dtinicio, $dtfim)
    {
        try {
            $sales = array();
            
            $curl = curl_init();
            
            $urlBase = 'https://odh-sale-api.teknisa.com/pobrejuan/index.php';
            $token3469 = 'e+hE6UB4cOVyN5TWZr0pdox+u2/kNsmXEns/uooT2sY8DYAT+II9BfF0Him0PwSATiL6sHaoiCoHY++zpcFmLn4ixIe9z83IeL02a3c7gE/ecLKsrzLWkZTN1R8Zc4ShS0yN5wVyYFpBxHXu42nK1I2I9pPEZFwr59DmuzKSPg+CD2CRrDT9BqWlH7M+UhxsjhjFYq7R2fybUQ0h2jti2YmPOaFCZ82vKPs3jlwCnRYja/R3WevNy0z4wUTj9+xTFruZopONC52Ta5WcV7CMdQ==';
            $codUnidade = $cdunidade;//'0001';
            $dtInicial = $dtinicio;//'01/09/2023';
            $dtFinal = $dtfim;//'08/09/2023';
            
            curl_setopt_array($curl, array(
                CURLOPT_URL => $urlBase.'/getSales',
                CURLOPT_RETURNTRANSFER => true,
                CURLOPT_ENCODING => '',
                CURLOPT_MAXREDIRS => 10,
                CURLOPT_TIMEOUT => 0,
                CURLOPT_FOLLOWLOCATION => true,
                CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
                CURLOPT_CUSTOMREQUEST => 'POST',
                CURLOPT_POSTFIELDS => '{
                    "subsidiaryCode": "'.$codUnidade.'",
                    "dtInicial": "'.$dtInicial.'",
                    "dtFinal": "'.$dtFinal.'"
                }',
                CURLOPT_HTTPHEADER => array(
                    'Authorization: Bearer '.$token3469,
                    'Content-Type: application/json'
                ),
            ));
            
            $response = json_decode(curl_exec($curl), true);
            
            curl_close($curl);
            
            if (isset($response[0])) {
                $response = current($response);
                
                if (!isset($response['sales'])) {
                    throw new \Exception("Erro ao buscar as vendas {".json_encode($response)."}");
                }
            
                $sales = $response['sales'];
            } else if (isset($response['success'])) {
                if ($response['success'] == false) {
                    throw new \Exception($response['message']);
                } else {
                    throw new \Exception("Erro ao comunicar com a API {".$response."}");
                }
            } else {
                throw new \Exception("Erro ao comunicar com a API {".$response."}");
            }
            
            return $sales;
        } catch(\Exception $e) {
            throw $e;
        }  
    }

    public function importarCuponsOdhen($nrorg, $nrrateio, $nrestruturam, $cdunidade, $dtinicio, $dtfim)
    {
        try {
            $sales = array();
            
            $dtinicio = DateUtil::getDataDeString($dtinicio, DateUtil::FORMATO_BRASILEIRO, true);
            $dtfim = DateUtil::getDataDeString($dtfim, DateUtil::FORMATO_BRASILEIRO, true);
            
            // Como a api getSales so permite chamadas com maximo de 5 dias, faz varias chamadas caso o periodo informado na tela seja maior
            // Enquando a data inicio não for posterior à data fim faz chamada à api getSales com intervalo de 5 dias
            while (!DateUtil::comparaDatas($dtinicio, $dtfim, DateUtil::DEPOIS)) {
                $dtiniintervalo = $dtinicio;
                $dtfimintervalo = DateUtil::adicionaIntervalo($dtinicio, 4); //Adiciona 5 dias à data fim para intervalo de 5 dias
                
                // Caso ao acrescentar os dias ultrapasse a data fim, usa ela mesmo como fim do intervalo
                if (DateUtil::comparaDatas($dtfim, $dtfimintervalo)) {
                    $dtfimintervalo = $dtfim;
                }
                
                $salesReturned = $this->callApiGetSales($cdunidade, $dtiniintervalo->format(DateUtil::FORMATO_BRASILEIRO), $dtfimintervalo->format(DateUtil::FORMATO_BRASILEIRO));
                
                $sales = array_merge($sales, $salesReturned);
                
                $dtinicio = DateUtil::adicionaIntervalo($dtfimintervalo, 1); //Adiciona mais 1 dia à data inicio para o próximo intervalo
            }
            
            $qtdLancamentos = 0;
                
            foreach ($sales as $sale) {
                if ($sale['saleStatus'] == 'Concluida' && strpos(strtoupper($sale['cashierName']), 'DELIVERY') === false) {
                    $cdintegracao = $sale['cupomNumber'];
                    $dtlancamento = DateUtil::getDataDeString($sale['posOpeningDate'], 'd-m-Y H:i:s');
                    $vrlancamento = $sale['saleTotalAmount'];
                    $vracrescimo = $sale['totalAmountAddition'];
                    $vrdesconto = $sale['totalDiscountAmount'];
                    
                    $vrsubtotal1 = $vrvalsubtotal1 = $vrsubtotal2 = $vrvalsubtotal2 = $vrsubtotal3 = $vrvalsubtotal3 = $vrfinal = 0;
                    
                    $this->calculaLancamento(
                        $nrorg, $nrrateio, $vrlancamento, $vracrescimo, $vrdesconto, $vrsubtotal1, $vrvalsubtotal1, $vrsubtotal2, $vrvalsubtotal2, $vrsubtotal3, $vrvalsubtotal3, $vrfinal
                    );
                    
                    $this->addRateiolanc(
                        $nrorg, $nrrateio, $dtlancamento, $vrlancamento, $vracrescimo, $vrdesconto, $vrsubtotal1, $vrsubtotal2, $vrsubtotal3, 
                              $vrvalsubtotal1, $vrvalsubtotal2, $vrvalsubtotal3, $vrfinal, $cdintegracao
                    );
                    
                    $qtdLancamentos++;
                }
            }
            
            return $qtdLancamentos;
        } catch(\Exception $e) {
            throw $e;
        }  
    }

    public function calculaLancamento($nrorg, $nrrateio, $vrlancamento, $vracrescimo, $vrdesconto, &$vrsubtotal1, &$vrvalsubtotal1, &$vrsubtotal2, &$vrvalsubtotal2, &$vrsubtotal3, &$vrvalsubtotal3, &$vrfinal)
    {   
        $rateio = $this->entityManager->getRepository(Repositories::FPA_RATEIO)->findOneBy(array('nrrateio' => $nrrateio, 'nrorg' => $nrorg)); 
        
        $subtotal = null;
        
        if($rateio->getDsoperacao1() != null) {
            switch ($rateio->getCdvalorbase1()) {
                case 'VRACRE':
                    $vrbase1 = $vracrescimo;
                    break;
                case 'VRDESC':
                    $vrbase1 = $vrdesconto;
                    break;
                default:
                    $vrbase1 = $vrlancamento;
            }
            if(is_null($rateio->getDsregra1())) {
                $vrsubtotal1 = self::calculaSubTotal($vrbase1, $rateio->getVrtaxa1());
            } else {
                $vrsubtotal1 = self::executaRegra($rateio->getDsregra1(), $vrlancamento, $vracrescimo, $vrdesconto);
            }
            $subtotal = self::calculaVrFinalLancamento($vrbase1, $vrsubtotal1, $rateio->getDsoperacao1());
            $vrvalsubtotal1 = $subtotal;
        } else {
            $vrsubtotal1 = null;
            $vrvalsubtotal1 = null;
        }

        if($rateio->getDsoperacao2() != null) {   
            $vrparcial1 = !is_null($subtotal) ? $subtotal : $vrlancamento;
            if(is_null($rateio->getDsregra2())) {
                $vrsubtotal2 = self::calculaSubTotal($vrparcial1, $rateio->getVrtaxa2()); 
            } else {
                $vrsubtotal2 = self::executaRegra($rateio->getDsregra2(), $vrlancamento, $vracrescimo, $vrdesconto, $vrparcial1);
            }
            $subtotal = self::calculaVrFinalLancamento($vrparcial1, $vrsubtotal2, $rateio->getDsoperacao2());
            $vrvalsubtotal2 = $subtotal;
        } else {
            $vrsubtotal2 = null;
            $vrvalsubtotal2 = null;
        }

        if($rateio->getDsoperacao3() != null) {
            $vrparcial2 = !is_null($subtotal) ? $subtotal : $vrlancamento;
            if(is_null($rateio->getDsregra3())) {
                $vrsubtotal3 = self::calculaSubTotal($vrparcial2, $rateio->getVrtaxa3()); 
            } else {
                $vrsubtotal3 = self::executaRegra($rateio->getDsregra3(), $vrlancamento, $vracrescimo, $vrdesconto, $vrparcial2);
            }
            $subtotal = self::calculaVrFinalLancamento($vrparcial2, $vrsubtotal3, $rateio->getDsoperacao3());
            $vrvalsubtotal3 = $subtotal;
        } else {
            $vrsubtotal3 = null;
            $vrvalsubtotal3 = null;
        }
        
        if($subtotal === null) {
            $vrfinal = $vrlancamento;
            return;
        }

        $vrfinal = $subtotal;
        return;
    }

    private static function calculaSubTotal($vrlancamento, $taxa)
    {   
        $resultado = ($vrlancamento * ($taxa/100));
        return $resultado;
    }

    private static function calculaVrFinalLancamento($vrlancamento, $vrsubtotal, $operação)
    {   
        if($operação == 'SOMA') {
            $resultado = $vrlancamento + $vrsubtotal;
            return $resultado;
        }

        if($operação == 'SUBTRAI') {
            $resultado = $vrlancamento - $vrsubtotal;
            return $resultado;
        }

        if($operação == 'IGUAL') {
            $resultado = $vrsubtotal;
            return $resultado;
        }
    }

    private static function executaRegra($regra, $vrlancamento, $vracrescimo, $vrdesconto, $vrinicial=0)
    {
        $mnemonicos = array(
            'VRLANC' => $vrlancamento,
            'VRACRE' => $vracrescimo,
            'VRDESC' => $vrdesconto,
        );
        
        foreach ($mnemonicos as $mnemonico => $valor) {
            $valor = is_null($valor) ? 0 : $valor;
            $regra = str_replace($mnemonico, $valor, $regra);
        }
        
        $calculoRateio = new CalculoRateio();
        
        try {
            error_reporting(E_ERROR | E_PARSE);
            $resultado = $calculoRateio->executaRegra($regra);
        } catch (\Exception $e) {
            throw $e;
        }
        return $resultado;
    }

    private static function aplicaAcrescimoDesconto($vrlancamento, $vracrescimo, $vrdesconto)
    {    
        if($vracrescimo != null) {
            $vrlancamento = $vrlancamento + floatval($vracrescimo);
        }

        if($vrdesconto != null) {
            $vrlancamento = $vrlancamento - floatval($vrdesconto);
        }
        
        return $vrlancamento;
    }
    
    public function addRateiolanc($nrorg, $nrrateio, $dtlancamento, $vrlancamento, $vracrescimo, $vrdesconto, $vrsubtotal1, $vrsubtotal2, $vrsubtotal3, 
                                  $vrvalsubtotal1, $vrvalsubtotal2, $vrvalsubtotal3, $vrfinal, $cdintegracao)
    {
        $fpaRateiolanc = Repositories::FPA_RATEIOLANC;
        
        if (!is_null($cdintegracao)) {
            $rateiolanc = $this->entityManager->getRepository(Repositories::FPA_RATEIOLANC)->findOneBy(array('nrorg' => $nrorg, 'nrrateio' => $nrrateio, 'cdintegracao' => $cdintegracao));
        } else {
            $rateiolanc = $this->entityManager->getRepository(Repositories::FPA_RATEIOLANC)->findOneBy(array('nrorg' => $nrorg, 'nrrateio' => $nrrateio, 'dtlancamento' => $dtlancamento));
        }
        
        if (is_object($rateiolanc)) {
            $rateiolanc->setVrlancamento($vrlancamento);
            $rateiolanc->setVracrescimo($vracrescimo);
            $rateiolanc->setVrdesconto($vrdesconto);
            $rateiolanc->setVrsubtotal1($vrsubtotal1);
            $rateiolanc->setVrsubtotal2($vrsubtotal2);
            $rateiolanc->setVrsubtotal3($vrsubtotal3);
            $rateiolanc->setVrvalsubtotal1($vrvalsubtotal1);
            $rateiolanc->setVrvalsubtotal2($vrvalsubtotal2);
            $rateiolanc->setVrvalsubtotal3($vrvalsubtotal3);
            $rateiolanc->setVrfinal($vrfinal);
            $rateiolanc->setCdintegracao($cdintegracao);
            
            $rateiolanc->setDtultatu(DateUtil::getDataAtual());
            $rateiolanc->setCdoperultatu($this->hcmParameters->getCdoperador());
            $rateiolanc->setNrorgultatu($this->hcmParameters->getNrorgTrabalho());
        } else {
            $rateiolanc = new $fpaRateiolanc;
            $rateiolanc->setNrorg($nrorg);
            $rateiolanc->setNrrateio($nrrateio);
            $rateiolanc->setDtlancamento($dtlancamento);
            $rateiolanc->setVrlancamento($vrlancamento);
            $rateiolanc->setVracrescimo($vracrescimo);
            $rateiolanc->setVrdesconto($vrdesconto);
            $rateiolanc->setVrsubtotal1($vrsubtotal1);
            $rateiolanc->setVrsubtotal2($vrsubtotal2);
            $rateiolanc->setVrsubtotal3($vrsubtotal3);
            $rateiolanc->setVrvalsubtotal1($vrvalsubtotal1);
            $rateiolanc->setVrvalsubtotal2($vrvalsubtotal2);
            $rateiolanc->setVrvalsubtotal3($vrvalsubtotal3);
            $rateiolanc->setVrfinal($vrfinal);
            $rateiolanc->setCdintegracao($cdintegracao);
            $rateiolanc->setIdativo('S');
            
            $rateiolanc->setDtinclusao(DateUtil::getDataAtual());
            $rateiolanc->setCdoperinclusao($this->hcmParameters->getCdoperador());
            $rateiolanc->setNrorginclusao($this->hcmParameters->getNrorgTrabalho());
            $rateiolanc->setDtultatu(DateUtil::getDataAtual());
            $rateiolanc->setCdoperultatu($this->hcmParameters->getCdoperador());
            $rateiolanc->setNrorgultatu($this->hcmParameters->getNrorgTrabalho());
        }
        
        $rateiolanc->setNrrateiolanc(\HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'FPA_RATEIOLANC', 1, 12, true));
        
        $this->entityManager->persist($rateiolanc);
        $this->entityManager->flush();
        
        return $rateiolanc;
    }
    
    public function replicaParametroRateioEstrutura($row){
        $arrayEstruturasDestino[] = '';
        
        $listaNrrateioEstruturaOrigem = $this->entityManager->getRepository(Repositories::FPA_RATEIOESTRUT)->findBy(
            array('nrorg' => $row['NRORG'], 'nrrateio' =>$row['NRRATEIO_ALVO']));
            
        $listaNrrateioEstruturaDestino = $this->entityManager->getRepository(Repositories::FPA_RATEIOESTRUT)->findBy(
            array('nrorg' => $row['NRORG'], 'nrrateio' =>$row['NRRATEIO']));
        
       
        if(empty($listaNrrateioEstruturaOrigem)){
            return null;
        }
        
        $estruturasClonadas = array();
        if(!empty($listaNrrateioEstruturaDestino)){
            foreach($listaNrrateioEstruturaDestino as $listaEstruturaDestino){
                $arrayEstruturasDestino[] = $listaEstruturaDestino->getNrestruturam();
            }
        }
        foreach($listaNrrateioEstruturaOrigem as $listaEstruturaOrigem){
            if(!in_array($listaEstruturaOrigem->getNrestruturam(),$arrayEstruturasDestino)){
               array_push($estruturasClonadas,$listaEstruturaOrigem);
            }
        }
        if($estruturasClonadas){
            foreach($estruturasClonadas as $clone){
                $auditoria = $this->setAuditoria();
                $fpa_rateioestrut = Repositories::FPA_RATEIOESTRUT;
                $novaListaNrrateioEstrut = new $fpa_rateioestrut;
                $novaListaNrrateioEstrut->setNrorg($auditoria['NRORG']);
                $novaListaNrrateioEstrut->setDtinclusao($auditoria['DTINCLUSAO']);
                $novaListaNrrateioEstrut->setNrestruturam($clone->getNrestruturam());
                $novaListaNrrateioEstrut->setCdoperinclusao($auditoria['CDOPERINCLUSAO']);
                $novaListaNrrateioEstrut->setNrorginclusao($auditoria['NRORGINCLUSAO']);
                $novaListaNrrateioEstrut->setNrrateio($row['NRRATEIO']);
                $novaListaNrrateioEstrut->setNrpeso($clone->getNrpeso());
                $novaListaNrrateioEstrut->setNrrateioestrut(\HCM\Util\NovoCodigo::geraCodigo($row['NRORG'],'FPA_RATEIOESTRUT', 1, 12, true));
                $novaListaNrrateioEstrut->setIdativo('S');
                $novaListaNrrateioEstrut->setVrporcentagem($clone->getVrporcentagem());
                $this->entityManager->persist($novaListaNrrateioEstrut);
                $this->entityManager->flush();
            }
            $this->calculaPorcentagem($auditoria['NRORG'], $row['NRRATEIO'], 'FPA_RATEIOESTRUT');
            return true;
        }
        return false;
    }
    
    public function replicaParametroRateioOcupacao($row){
        
        $arrayOcupacaoDestino[] = '';
        $listaNrrateioOcupacaoOrigem = $this->entityManager->getRepository(Repositories::FPA_RATEIOOCUP)->findBy(
            array('nrorg' => $row['NRORG'], 'nrrateio' =>$row['NRRATEIO_ALVO']));
            
        $listaNrrateioOcupacaoDestino = $this->entityManager->getRepository(Repositories::FPA_RATEIOOCUP)->findBy(
            array('nrorg' => $row['NRORG'], 'nrrateio' =>$row['NRRATEIO']));
        
       
        
        if(empty($listaNrrateioOcupacaoOrigem)){
            return null;
        }
        
        $ocupacoesClonadas = array();
        if(!empty($listaNrrateioOcupacaoDestino)){
            foreach($listaNrrateioOcupacaoDestino as $listaOcupacaoDestino){
                $arrayOcupacaoDestino[] = $listaOcupacaoDestino->getNrocupacaom();
            }
        }
        foreach($listaNrrateioOcupacaoOrigem as $listaOcupacaoOrigem){
            if(!in_array($listaOcupacaoOrigem->getNrocupacaom(),$arrayOcupacaoDestino)){
               array_push($ocupacoesClonadas,$listaOcupacaoOrigem);
            }
        }
        
        if($ocupacoesClonadas){
            foreach($ocupacoesClonadas as $clone){
                $auditoria = $this->setAuditoria();
                $fpa_rateioocup = Repositories::FPA_RATEIOOCUP;
                $novaListaNrrateioOcup = new $fpa_rateioocup;
                $novaListaNrrateioOcup->setNrorg($auditoria['NRORG']);
                $novaListaNrrateioOcup->setDtinclusao($auditoria['DTINCLUSAO']);
                $novaListaNrrateioOcup->setNrestruturam($clone->getNrestruturam());
                $novaListaNrrateioOcup->setNrocupacaom($clone->getNrocupacaom());
                $novaListaNrrateioOcup->setCdoperinclusao($auditoria['CDOPERINCLUSAO']);
                $novaListaNrrateioOcup->setNrorginclusao($auditoria['NRORGINCLUSAO']);
                $novaListaNrrateioOcup->setNrrateio($row['NRRATEIO']);
                $novaListaNrrateioOcup->setNrpeso($clone->getNrpeso());
                $novaListaNrrateioOcup->setNrrateioocup(\HCM\Util\NovoCodigo::geraCodigo($row['NRORG'],'FPA_RATEIOOCUP', 1, 12, true));
                $novaListaNrrateioOcup->setIdativo('S');
                $novaListaNrrateioOcup->setVrporcentagem($clone->getVrporcentagem());
                $this->entityManager->persist($novaListaNrrateioOcup);
                $this->entityManager->flush();
            }
            $this->calculaPorcentagem($auditoria['NRORG'], $row['NRRATEIO'], 'FPA_RATEIOOCUP');
            return true;
        }
        return false;
    }
    
    
    public function setAuditoria(){
        $DTINCLUSAO = $this->hcmParameters->getDataAtual();
        $CDOPERINCLUSAO = $this->hcmParameters->getCdoperador();
        $NRORGINCLUSAO = $this->hcmParameters->getNrorgTrabalho();
        $NRORG = $this->hcmParameters->getNrorgTrabalho();
        $auditoria = [
            "DTINCLUSAO" =>  DateUtil::getDataDeString($DTINCLUSAO),
            "CDOPERINCLUSAO" => $CDOPERINCLUSAO,
            "NRORGINCLUSAO" => $NRORGINCLUSAO,
            "NRORG" => $NRORG,
            "DTULTATU" => DateUtil::getDataDeString($DTINCLUSAO),
            "CDOPERULTATU" => $CDOPERINCLUSAO,
            
            
            ]; 
        return $auditoria;
    }

    public function calculoRateioLancamento($nrorg, $nrrateio, $dtinicio, $dtfim, $response)
    {
        try {
            $valores = array();
            
            $msgRetorno = array();
            $mensagemMemoriaDeCalculo = '';
            
            $dtmescompetenc = $this->hcmParameters->getDtCompetencia();
            
            $rateio = $this->entityManager->getRepository(Repositories::FPA_RATEIO)->findOneBy(array('nrrateio' => $nrrateio, 'nrorg' => $nrorg));
            
            $lancamentos = $this->rateioLancamentoRepository->getLancamentos($nrorg, $nrrateio, $dtinicio, $dtfim);
            
            if (empty($lancamentos)) {
                $response->addNotification(new Notification("Não existem lançamentos a serem rateados", Notification::TYPE_ALERT));
                return;
            }
            
            $detalhesRateio = $this->getDetalhamentoRateio(
                $nrorg, $nrrateio, $rateio->getNrestruturam(), $rateio->getNrsitufuncm(), $rateio->getNrtipovinculom(), $rateio->getNrvinculoempreg(), $rateio->getIdngerarateiofalta(), $rateio->getIdngerarateioatest(), $response
            );
            
            foreach($lancamentos as $lancamento) {
                $valoresDia = $this->calculaRateioLancamentoDia(
                    $nrorg, $nrrateio, $rateio->getNrsitufuncm(), $rateio->getIdngerarateiofalta(), $rateio->getIdngerarateioatest(), $detalhesRateio, $lancamento['DTLANCAMENTO'], $lancamento['VRFINAL'], $response
                );
                
                if (!empty($valoresDia)) {
                    $valores = array_merge($valores, $valoresDia);
                }
            }
            
            $mensagemMemoriaDeCalculo = MemoriaRateio::getInstance()->getMemoria();
            if(!empty($mensagemMemoriaDeCalculo)) {
                MemoriaRateio::getInstance()->salvarMsgArquivo($mensagemMemoriaDeCalculo);
                $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('memoriaRateio', array(MemoriaRateio::getInstance()->getCaminhoArquivo())));
            }
            
            if(!empty($msgRetorno)) {
                $response->addDataSet((new \Zeedhi\Framework\DataSource\DataSet('msgRetorno',  $msgRetorno)));
            }
            
            return $valores;
            
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }       
    }
    
    public function getDetalhamentoRateio($nrorg, $nrrateio, $nrestrutlegal, $nrsitufuncm, $nrtipovinculom, $nrvinculoempreg, $idngerarateiofalta, $idngerarateioatest, $response)
    {
        $detalhesRateio = array();
            
        MemoriaRateio::getInstance()->inicializarDados($nrrateio, $nrestrutlegal, $nrsitufuncm, $nrtipovinculom, $nrvinculoempreg, $idngerarateiofalta, $idngerarateioatest);
        
        $rateioestrut = $this->entityManager->getRepository(Repositories::FPA_RATEIOESTRUT)->findBy(array('nrorg' => $nrorg, 'nrrateio' => $nrrateio));
        
        if(count($rateioestrut) < 1) {
            $response->addNotification(new Notification("Não foi encontrado cadastro de Estrutura para o rateio", Notification::TYPE_ALERT));
            return array();
        } else {
            foreach ($rateioestrut as $estrutura) {
                $detalhesEstrutura = array();
            
                $situFuncNaoGeraRateio = array();
                $tipoVincNaoGeraRateio = array();
                $vincEmpregNaoGeraRateio = array();
                
                $totalPesoEstrutura = 0;
                
                $estruturah = $this->entityManager->getRepository(Repositories::ESTRUTURAH)->retornaEstruturaHistorico($nrorg,$estrutura->getNrestruturam(),DateUtil::getDataDeString($this->hcmParameters->getDtCompetencia(), DateUtil::FORMATO_BRASILEIRO, true));
                
                //MemoriaRateio::getInstance()->mensagemInfoEstrutura($estrutura->getNrestruturam(), $estrutura->getVrporcentagem());
                
                $nrestrutgerenlist = $this->getEstruturasGerenciaisRelacionadas($nrorg, $estrutura->getNrestruturam());
                
                if (!empty($nrestrutgerenlist)) {
                    $ocupacoesEstrutura = array();
                    $vinculosEstrutura = array();
                    
                    $rateioocup = $this->entityManager->getRepository(Repositories::FPA_RATEIOOCUP)->findBy(array('nrorg' => $nrorg, 'nrrateio' => $nrrateio, 'nrestruturam' => $estrutura->getNrestruturam()));
                    
                    foreach ($rateioocup as $ocupacao) {
                        $detalhesOcupacao = array();
                        
                        $ocupacaoh = $this->entityManager->getRepository(Repositories::GPE_OCUPACAOH)->retornaUltimoHistorico($ocupacao->getNrocupacaom(), $nrorg, DateUtil::getDataDeString($this->hcmParameters->getDtCompetencia(), DateUtil::FORMATO_BRASILEIRO, true));
                        
                        //MemoriaRateio::getInstance()->mensagemInfoOcupacao($ocupacao->getNrestruturam(), $ocupacao->getNrocupacaom(), $ocupacao->getVrporcentagem());
                    
                        $vinculos = $this->rateioLancamentoRepository->getVinculosRateio($nrorg, $this->hcmParameters->getDtCompetencia(), $nrestrutlegal, $nrestrutgerenlist, $ocupacao->getNrocupacaom(), null/*$nrsitufuncm*/, $nrtipovinculom, $nrvinculoempreg);
                        
                        if(count($vinculos) < 1) {
                            $response->addNotification(new Notification("Não foram encontrados vínculos para rateio na estrutura ".$estrutura->getNrestruturam()." ocupação ".$ocupacao->getNrocupacaom(), Notification::TYPE_ALERT));
                            continue;
                        }
                        
                        $vinculosOcupacao = array();
                        
                        foreach ($vinculos as $vinculo) {
                            /*if(!is_null($nrsitufuncm)){
                                $rateioNrsitufuncmArray = explode(',', $nrsitufuncm);
                                if(in_array($vinculo->historico->getNrsitufuncm(), $rateioNrsitufuncmArray)){
                                    $situFuncNaoGeraRateio[$vinculo->getNrvinculom()] = $vinculo->getNmvinculom();
                                    continue;
                                }
                            }*/
                            if(!is_null($nrtipovinculom)){
                                $rateioNrtipovinculomArray = explode(',', $nrtipovinculom);
                                if(!in_array($vinculo->getNrtipovinculom(),$rateioNrtipovinculomArray)){
                                    //$tipoVincNaoGeraRateio[$vinculo->getNrvinculom()] = $vinculo->getNmvinculom();
                                    continue;
                                }
                            }
                            if(!is_null($nrvinculoempreg)){
                                $rateioNrvincempregArray = explode(',', $nrvinculoempreg);
                                if(!in_array($vinculo->getNrvinculoempreg(),$rateioNrvincempregArray)){
                                    //$vincEmpregNaoGeraRateio[$vinculo->getNrvinculom()] = $vinculo->getNmvinculom();
                                    continue;
                                }
                            }
                            
                            $vinculosOcupacao[$vinculo->getNrvinculom()] = array(
                                'NRVINCULOM' => $vinculo->getNrvinculom(),
                                'NMVINCULOM' => $vinculo->getNmvinculom(),
                                'DTADMISSAOVINC' => $vinculo->getDtadmissaovinc(),
                                'DTRESCISAOVINC' => $vinculo->getDtrescisaovinc(),
                                'NRPESO' => $ocupacao->getNrpeso(),
                            );
                            
                            $vinculosEstrutura[$vinculo->getNrvinculom()] = array(
                                'NRVINCULOM' => $vinculo->getNrvinculom(),
                                'NMVINCULOM' => $vinculo->getNmvinculom(),
                                'DTADMISSAOVINC' => $vinculo->getDtadmissaovinc(),
                                'DTRESCISAOVINC' => $vinculo->getDtrescisaovinc(),
                                'NRPESO' => $ocupacao->getNrpeso(),
                            );
                            
                            $totalPesoEstrutura += $ocupacao->getNrpeso();
                        }
                        
                        $detalhesOcupacao = array(
                            'NROCUPACAOM' => $ocupacao->getNrocupacaom(),
                            'NMOCUPACAOH' => $ocupacaoh->getNmocupacaoh(),
                            'NRPESO' => $ocupacao->getNrpeso(),
                            'VRPERCENTUAL' => $ocupacao->getVrporcentagem(),
                            'VINCULOS' => $vinculosOcupacao,
                        );
                
                        $ocupacoesEstrutura[$ocupacao->getNrocupacaom()] = $detalhesOcupacao;
                    }
                    
                    $detalhesEstrutura = array(
                        'NRESTRUTURAM' => $estrutura->getNrestruturam(),
                        'NMESTRUTURAH' => $estruturah->getNmestruturah(),
                        'VRPERCENTUAL' => $estrutura->getVrporcentagem(),
                        'NRESTRUTGERENLIST' => $nrestrutgerenlist,
                        'OCUPACOES' => $ocupacoesEstrutura,
                        'VINCULOS' => $vinculosEstrutura,
                        'NRPESOVINCULOS' => $totalPesoEstrutura,
                        'SITUFUNCNAOGERARATEIO' => $situFuncNaoGeraRateio,
                        'TIPOVINCNAOGERARATEIO' => $tipoVincNaoGeraRateio,
                        'VINCEMPREGNAOGERARATEIO' => $vincEmpregNaoGeraRateio,
                    );
                } else {
                    $response->addNotification(new Notification("Não foram encontradas estruturas gerenciais relacionadas à estrutura ".$estrutura->getNrestruturam(), Notification::TYPE_ALERT));
                }
                
                $detalhesRateio[$estrutura->getNrestruturam()] = $detalhesEstrutura;
            }
            
            return $detalhesRateio;
        }
    }
    
    /* Percorre a hierarquia descendo os níveis existentes buscando estruturas gerenciais relacionadas à estrutura passada, ou a própria se ela já for do tipo gerencial */
    public function getEstruturasGerenciaisRelacionadas($nrorg, $nrestruturam) {
        $estrutura = $this->entityManager->getRepository(Repositories::ESTRUTURAM)->findOneBy(array('nrorg' => $nrorg, 'nrestruturam' => $nrestruturam));
        if ($estrutura->getNrtipoestrutura() == $this->hcmParameters->getTipoEstruturaGerencialOrg()) {
            return array($estrutura->getNrestruturam());
        } else {
            $hierarquias = $this->entityManager->getRepository(Repositories::HIERARQUIAORG)->findBy(array('nrorg' => $nrorg, 'nrestruturamsup' => $estrutura->getNrestruturam()));
            $estrutgerenlist = array();
            foreach ($hierarquias as $hierarquia) {
                $estruturas = $this->getEstruturasGerenciaisRelacionadas($nrorg, $hierarquia->getNrestruturamrel());
                if (!empty($estruturas)) {
                    $estrutgerenlist = array_merge($estrutgerenlist, $estruturas);
                }
            }
            return $estrutgerenlist;
        }
    }
    
    public function calculaRateioLancamentoDia($nrorg, $nrrateio, $nrsitufuncm, $idngerarateiofalta, $idngerarateioatest, $detalhesRateio, $dtlancamento, $vrlancamento, $response)
    {
        $valores = array();
        
        //MemoriaRateio::getInstance()->mensagemInfoValorDia($dtlancamento, $vrlancamento);
        MemoriaRateio::getInstance()->mensagemInfoPesoDia($dtlancamento, $vrlancamento);
        
        foreach ($detalhesRateio as $nrestruturam => $detalhesEstrutura) {
                
            $vinculosRateio = array();
            $situFuncNaoGeraRateio = array();
            $faltaNaoGeraRateio = array();
            $atestadoNaoGeraRateio = array();
            
            $nmestrutura = $detalhesEstrutura['NMESTRUTURAH'];
            $percentualEstrutura = $detalhesEstrutura['VRPERCENTUAL'];
            $vinculosEstrutura = $detalhesEstrutura['VINCULOS'];
            $pesoVinculosEstrutura = $detalhesEstrutura['NRPESOVINCULOS'];
            
            $valorEstrutura = ($vrlancamento * $percentualEstrutura) / 100;
            
            foreach ($vinculosEstrutura as $vinculo) {
                $geraRateio = $this->validaVinculoGeraRateio($nrorg, $vinculo, $dtlancamento, $nrsitufuncm, $idngerarateiofalta, $idngerarateioatest, $situFuncNaoGeraRateio, $faltaNaoGeraRateio, $atestadoNaoGeraRateio);
                
                if ($geraRateio) {
                    $vinculosRateio[] = $vinculo;
                } else {
                    $pesoVinculosEstrutura -= $vinculo['NRPESO'];
                }
            }
        
            if (empty($vinculosRateio)) {
                $response->addNotification(new Notification("Não existem vínculos a serem rateados para a estrutura $nrestruturam.", Notification::TYPE_ALERT));
                continue;
            }
            
            //MemoriaRateio::getInstance()->mensagemInfoValorEstrutura($dtlancamento, $vrlancamento, $nrestruturam, $percentualEstrutura, $nmestrutura);
            MemoriaRateio::getInstance()->mensagemInfoPesoEstrutura($dtlancamento, $vrlancamento, $nrestruturam, $percentualEstrutura, $nmestrutura, count($vinculosRateio), $pesoVinculosEstrutura);
            
            foreach ($detalhesEstrutura['OCUPACOES'] as $nrocupacaom => $detalhesOcupacao) {
                
                $nmocupacao = $detalhesOcupacao['NMOCUPACAOH'];
                $pesoOcupacao = $detalhesOcupacao['NRPESO'];
                $percentualOcupacao = $detalhesOcupacao['VRPERCENTUAL'];
                $vinculosOcupacao = $detalhesOcupacao['VINCULOS'];
                
                $valorOcupacao = ($valorEstrutura * $percentualOcupacao) / 100;
                
                //MemoriaRateio::getInstance()->mensagemInfoValorOcupacao($dtlancamento, $vrlancamento, $nrestruturam, $percentualEstrutura, $nrocupacaom, $percentualOcupacao, $nmocupacao);
                MemoriaRateio::getInstance()->mensagemInfoPesoOcupacao($dtlancamento, $vrlancamento, $nrestruturam, $valorEstrutura, $nrocupacaom, $nmocupacao, $pesoOcupacao);
                
                //$valorPorVinculo = ($valorOcupacao / count($vinculosRateio));
                
                //MemoriaRateio::getInstance()->mensagemInfoValorVinculo($dtlancamento, $vrlancamento, $nrestruturam, $percentualEstrutura, $nrocupacaom, $percentualOcupacao, count($vinculosRateio));
                
                foreach ($vinculosRateio as $vinculo) {
                    if (isset($vinculosOcupacao[$vinculo['NRVINCULOM']])) {
                        $valor['NRORG'] = $nrorg;
                        $valor['NRRATEIO'] = $nrrateio;
                        $valor['NRESTRUTURAM'] = $nrestruturam;
                        $valor['NRVINCULOM'] = $vinculo['NRVINCULOM'];
                        $valor['DTDIALANC'] = DateUtil::getDataDeString($dtlancamento, DateUtil::FORMATO_BRASILEIRO_DATAHORA, true)->format('d/m/Y');
                        $valor['NRPESO'] = $vinculo['NRPESO'];
                        //$valor['VRVALOR'] = $valorPorVinculo;
                        $valor['VRVALOR'] = round($valorEstrutura * ($vinculo['NRPESO'] * 100 / $pesoVinculosEstrutura) / 100, 2);
                        $valor['EXPRESSAO'] = round($valorEstrutura,2).' * ('.$vinculo['NRPESO'].' * 100 / '.$pesoVinculosEstrutura.') / 100';
                        
                        $valores[] = $valor;
                        
                        //MemoriaRateio::getInstance()->mensagemInfoVinculo($vinculo['NRVINCULOM'], $vinculo['NMVINCULOM']);
                        MemoriaRateio::getInstance()->mensagemPesoVinculo($vinculo['NRVINCULOM'], $vinculo['NMVINCULOM'], $vinculo['NRPESO'], $valor['VRVALOR'], $valor['EXPRESSAO']);
                    }
                }
        
                MemoriaRateio::getInstance()->mensagemFimValorVinculo();
            }
        
            if (!empty($situFuncNaoGeraRateio)) {
                MemoriaRateio::getInstance()->mensagemVinculosSituFuncNaoGeraRateio($situFuncNaoGeraRateio);
            }
        
            if (!empty($faltaNaoGeraRateio)) {
                MemoriaRateio::getInstance()->mensagemVinculosFaltaNaoGeraRateio($faltaNaoGeraRateio);
            }
        
            if (!empty($atestadoNaoGeraRateio)) {
                MemoriaRateio::getInstance()->mensagemVinculosAtestadoNaoGeraRateio($atestadoNaoGeraRateio);
            }
        
            MemoriaRateio::getInstance()->mensagemFimValorEstrutura();
        }
        
        MemoriaRateio::getInstance()->mensagemFimValorDia();
        
        return $valores;
    }

    public function validaVinculoGeraRateio($nrorg, $vinculo, $dtlancamento, $nrsitufuncm, $idngerarateiofalta, $idngerarateioatest, &$situFuncNaoGeraRateio, &$faltaNaoGeraRateio, &$atestadoNaoGeraRateio)
    {
        /*Valida se o vínculo já estava admitido na data do rateio*/
        if (DateUtil::comparaDatas($dtlancamento, $vinculo['DTADMISSAOVINC'])) {
            return false;
        }
        
        /*Valida se o vínculo já estava rescindido na data do rateio*/
        if (!is_null($vinculo['DTRESCISAOVINC']) && DateUtil::comparaDatas($vinculo['DTRESCISAOVINC'], $dtlancamento)) {
            return false;
        }
        
        if (!is_null($nrsitufuncm)) {
            /*Valida situações funcionais que não geram rateio*/
            $nmsitufunch = '';
            $validaSituFunc = $this->validaVincSituFuncNaoGeraRateio($nrorg, $vinculo['NRVINCULOM'], $dtlancamento, $nrsitufuncm, $nmsitufunch);
            if (!$validaSituFunc) {
                $situFuncNaoGeraRateio[$vinculo['NRVINCULOM']] = $vinculo['NMVINCULOM'] . ($nmsitufunch == "" ? "" : " - $nmsitufunch");
                return false;
            }
        }
        
        if ($idngerarateiofalta == 'S') {
            /*Valida falta que não geram rateio*/
            $validaFalta = $this->validaVincFaltaNaoGeraRateio($nrorg, $vinculo['NRVINCULOM'], $dtlancamento);
            if (!$validaFalta) {
                $faltaNaoGeraRateio[$vinculo['NRVINCULOM']] = $vinculo['NMVINCULOM'];
                return false;
            }
        }
        
        if ($idngerarateiofalta == 'S') {
            /*Valida situações funcionais que não geram rateio*/
            $validaAtestado = $this->validaVincAtestadoNaoGeraRateio($nrorg, $vinculo['NRVINCULOM'], $dtlancamento);
            if (!$validaAtestado) {
                $atestadoNaoGeraRateio[$vinculo['NRVINCULOM']] = $vinculo['NMVINCULOM'];
                return false;
            }
        }
        
        return true;
    }

    public function validaVincSituFuncNaoGeraRateio($nrorg, $nrvinculom, $dtlancamento, $nrsitufuncm, &$nmsitufunch='')
    {
        $dtdia = DateUtil::getDataDeString($dtlancamento, DateUtil::FORMATO_BRASILEIRO_DATAHORA, true);
        
        $altesitufunc = $this->entityManager->getRepository(Repositories::GPE_ALTESITUFUNC)->retornaSituacaoFuncionalNoDia($nrorg, $nrvinculom, $dtdia);
        
        foreach ($altesitufunc as $situfunc) {
            if(!is_null($nrsitufuncm)){
                $rateioNrsitufuncmArray = explode(',', $nrsitufuncm);
                if(in_array($situfunc->getNrsitufuncm(), $rateioNrsitufuncmArray)){
                    $situfunc = $this->entityManager->getRepository(Repositories::GPE_SITUFUNCM)->retornaUltimoHistoricoCompetencia($nrorg, 0, $situfunc->getNrsitufuncm(), $dtdia);
                    if($situfunc){
                        $nmsitufunch = $situfunc->getNmsitufunch();
                    }
                    return false;
                }
            }
        }
        
        return true;
    }

    public function validaVincFaltaNaoGeraRateio($nrorg, $nrvinculom, $dtlancamento)
    {
        $dtdia = DateUtil::getDataDeString($dtlancamento, DateUtil::FORMATO_BRASILEIRO_DATAHORA, true);
        
        // Verifica falta nas ocorrências de frequencia
        //$faltas = $this->entityManager->getRepository(Repositories::PTO_PROGOCORRENCIA)->retornaOcorrenciasFalta($nrorg, $nrvinculom, $dtdia);
        $faltas = $this->rateioLancamentoRepository->getFaltasVinculo($nrorg, $nrvinculom, $dtlancamento, $dtlancamento);
        
        if (count($faltas) > 0) {
            return false;
        }
        
        // Verifica falta na situação funcional
        // 100072-Falta(Dia), 100008-Falta Justificada(Não Remunerada)
        /*$faltaNrsitufuncmArray = array(100072, 100008);
        
        $altesitufunc = $this->entityManager->getRepository(Repositories::GPE_ALTESITUFUNC)->retornaSituacaoFuncionalNoDia($nrorg, $nrvinculom, $dtdia);
        
        foreach ($altesitufunc as $situfunc) {
            if(in_array($situfunc->getNrsitufuncm(), $faltaNrsitufuncmArray)){
                return false;
            }
        }*/
        
        return true;
    }

    public function validaVincAtestadoNaoGeraRateio($nrorg, $nrvinculom, $dtlancamento)
    {
        $dtdia = DateUtil::getDataDeString($dtlancamento, DateUtil::FORMATO_BRASILEIRO_DATAHORA, true);
        
        $atestados = $this->rateioLancamentoRepository->getAtestadosVinculo($nrorg, $nrvinculom, $dtlancamento);
        
        if (count($atestados) > 0) {
            return false;
        }
        
        // 12-Atestado Médico, 15-Afastamento doença - 15 dias de responsabilidade do empregador, 100042-Atestado Médico - Covid19
        /*$atestadoNrsitufuncmArray = array(12, 15, 100042);
        
        $altesitufunc = $this->entityManager->getRepository(Repositories::GPE_ALTESITUFUNC)->retornaSituacaoFuncionalNoDia($nrorg, $nrvinculom, $dtdia);
        
        foreach ($altesitufunc as $situfunc) {
            if(in_array($situfunc->getNrsitufuncm(), $atestadoNrsitufuncmArray)){
                return false;
            }
        }*/
        
        return true;
    }

    public function validaQntdVinculosParaRateio($qntdVinculosEstrutGerenCargo, $estrutgeren, $ocupacao)
    {
        foreach ($qntdVinculosEstrutGerenCargo as $item) {
            if ($estrutgeren == $item['NRESTRUTGEREN'] && $ocupacao == $item['NRCARGO']) {
                $qntdVinculosRateio = $item['QNTDVINCULOS']; 
                return $qntdVinculosRateio;
            }
        }
    }

    public function deleteRateioValores($row){
        $nrorg = $row[0]['NRORG'];
        $nrrateio = $row[0]['NRRATEIO'];
        
        $deleteValores = $this->rateioLancamentoRepository->deleteValoresGeral($nrorg,$nrrateio);
        return true;
    }
    
    public function validaTamanhoNrSituFuncMeNrvincEmpreg($nrsitufuncm,$nrvinculoempreg){
        if(is_array($nrsitufuncm)){
            if(count($nrsitufuncm) > 40){
                return 'maiorSitufunc';
            }
        }
        if(is_array($nrvinculoempreg)){
            if(count($nrvinculoempreg) > 40){
                return 'maiorVinculoEmpreg';
            }
        }
        return 'menor';
    }

    public function calculaPorcentagem($nrorg, $nrrateio, $tabela)
    {
       if($tabela == 'FPA_RATEIOESTRUT') {
           $porcentagens = $this->rateioLancamentoRepository->selectPorcentagemEstrutura($nrorg, $nrrateio);
           foreach($porcentagens as $porcentagem) {
            $this->rateioLancamentoRepository->setPorcentagemEstrutura($nrorg, $nrrateio, $porcentagem['NRRATEIOESTRUT'],$porcentagem['PERCENT']);
           }
       } 

       if($tabela == 'FPA_RATEIOOCUP') {
           $porcentagens = $this->rateioLancamentoRepository->selectPorcentagemOcupacao($nrorg, $nrrateio);
           foreach($porcentagens as $porcentagem) {
               $this->rateioLancamentoRepository->setPorcentagemOcupacao($nrorg, $nrrateio, $porcentagem['NRRATEIOOCUP'],$porcentagem['PERCENT']);
           }
       }    
   }

    public function exportaLote($itens, $nrorg, $dtmescompetenc, $nreventom)
    {   
        foreach($itens as $item) {
            $itensLote[] = array(
                'NRVINCULOM' => $item['NRVINCULOM'],
                'NMVINCULOM' => '',
                'NREVENTOM' => $nreventom,
                'NMEVENTOH' => '',
                'VRREFFOLHALOTE' => null,
                'VREVENTOLOTE' => $item['VRVALOR']
            );
        }

        $lote = $this->loteLancEventoService->geraLoteLancEvento(
            $nrorg, $dtmescompetenc, 'FPA', 'EXPORTAÇÃO DO RATEIO DE LANÇAMENTO', 'PENDENTE', '', 1,  $itensLote
        );

    }
}