<?php

namespace HCM\FPA\Util;

use HCM\Util\Messages;
use HCM\Util\Repositories;
use HCM\Util\DateUtil;
use HCM\Util\FPA;
use HCM\FPA\Entity\FpatipocalculoEntity;

class MemoriaRateio{

    private static $instance = null;
    private static $contador = 0;
    
    private $infoRateio = null;
    private $infoEstrutura = [];
    private $infoOcupacao = [];
    private $infoValorDia = [];
    private $infoValorEstrutura = [];
    private $infoValorOcupacao = [];
    private $infoValorVinculo = [];
    private $infoVinculo = [];
    
    private $memoriaRateio = null;
    
    private $nomeArquivo = null;
    
    private $resultadoTotal;
    
    private $referencia;
    
    private $infoBenefVinc = [];
    private $infoCalculo = [];
    
    private $vinculo;
    private $dtPagamento;

    private $arquivo;
    
    
    public function inicializarDados($nrrateio, $nrestrutlegal, $nrsitufuncmlist, $nrtipovinculomlist, $nrvinculoempreglist, $idngerarateiofalta, $idngerarateioatest){
       
        $this->nrrateio = $nrrateio;
        $this->nrestrutlegal = $nrestrutlegal;
        
        $this->nrsitufuncmlist = $nrsitufuncmlist;
        $this->nrtipovinculomlist = $nrtipovinculomlist;
        $this->nrvinculoempreglist = $nrvinculoempreglist;
        
        $this->idngerarateiofalta = $idngerarateiofalta;
        $this->idngerarateioatest = $idngerarateioatest;
        
        $this->mensagemInfoRateio();
    }
    
    public static function getInstance() {
        if (self::$instance === null) {
            self::$contador++;
            self::$instance = new self();
        }

        return self::$instance;
    }

    public function clear()
    {
        self::$instance = null;
        self::$contador = 0;
    }

    public function getCaminhoArquivo(){
        return $this->arquivo;
    }
    
    public function mensagemInfoRateio(){
        $this->infoRateio =
            '_______________ ::::: Dados do Rateio ::::: _______________'.PHP_EOL.
            '* Rateio: '.$this->nrrateio.PHP_EOL.
            '* Estrutura: '.$this->nrestrutlegal.PHP_EOL.
            '-__--__--__--__--__--__--__--__--__--__--__--__--__--__--__-'.PHP_EOL.
            '* Lista de tipo de vínculo que gera rateio: '.(is_null($this->nrtipovinculomlist) ? 'Todos' : $this->nrtipovinculomlist).PHP_EOL.
            '* Lista de vínculo empregatício que gera rateio: '.(is_null($this->nrvinculoempreglist) ? 'Todos' : $this->nrvinculoempreglist).PHP_EOL.
            '* Lista de situação funcional que não gera rateio: '.$this->nrsitufuncmlist.PHP_EOL.
            '* Não gera rateio em caso de falta: '.($this->idngerarateiofalta == 'S' ? 'Sim' : 'Não').PHP_EOL.
            '* Não gera rateio em caso de atestado: '.($this->idngerarateioatest == 'S' ? 'Sim' : 'Não').PHP_EOL.
            '-__--__--__--__--__--__--__--__--__--__--__--__--__--__--__-'.PHP_EOL;
        
        $this->memoriaRateio = $this->infoRateio;
    }
    
    public function mensagemInfoEstrutura($nrestruturam, $vrporcentagem){
        $this->infoEstrutura[$nrestruturam] =
            PHP_EOL.
            '_______________ ::::: Informações de Estruturas / Ocupações ::::: _______________'.PHP_EOL.
            '* Estrutura: '.$nrestruturam.' / Percentual: '.round($vrporcentagem,2).PHP_EOL;
        
        $this->memoriaRateio .= $this->infoEstrutura[$nrestruturam];
    }
    
    public function mensagemInfoOcupacao($nrestruturam, $nrocupacaom, $vrporcentagem){
        if (!isset($this->infoOcupacao[$nrestruturam][$nrocupacaom])) {
            $this->infoOcupacao[$nrestruturam][$nrocupacaom] =
                '** Ocupação: '.$nrocupacaom.' / Percentual: '.round($vrporcentagem,2).PHP_EOL;
        }
        $this->memoriaRateio .= $this->infoOcupacao[$nrestruturam][$nrocupacaom];
    }
    
    public function mensagemInfoValorDia($dtlancamento, $vrlancamento){
        $this->infoValorDia[$dtlancamento] =
            PHP_EOL.
            '_______________ ::::: Dados do Rateio Diário ::::: _______________'.PHP_EOL.
            '* Dia '.$dtlancamento.PHP_EOL.
            '* Valor Total Dia: '.round($vrlancamento,2).PHP_EOL.
            '* Formula de Calculo: ((Valor Lançamento Dia * (%Estrutura)) * (% Ocupação)) / Qntd. Vínculos'.PHP_EOL;
        
        $this->memoriaRateio .= $this->infoValorDia[$dtlancamento];
    }
    
    public function mensagemInfoPesoDia($dtlancamento, $vrlancamento){
        $this->infoValorDia[$dtlancamento] =
            PHP_EOL.
            '_______________ ::::: Dados do Rateio Diário ::::: _______________'.PHP_EOL.
            '* Dia '.$dtlancamento.PHP_EOL.
            '* Valor Total Dia: '.round($vrlancamento,2).PHP_EOL.
            //'* Formula de Calculo: ((Valor Lançamento Dia * (%Estrutura)) * (% Ocupação)) / Qntd. Vínculos'.PHP_EOL;
            '* Formula de Calculo: Valor Estrutura * (Peso Vínculo * 100 / Peso Total Vínculos) / 100'.PHP_EOL;
        
        $this->memoriaRateio .= $this->infoValorDia[$dtlancamento];
    }
    
    public function mensagemInfoValorEstrutura($dtlancamento, $vrlancamento, $nrestruturam, $vrestrutura, $nmestruturah){
        $this->infoValorEstrutura[$dtlancamento.$nrestruturam] =
            '* ================================================================'.PHP_EOL.
            '* Estrutura: '.$nrestruturam.'-'.$nmestruturah.' / Percentual: '.round($vrestrutura,2).'%'.PHP_EOL.
            '* Valor Estrutura: '.round($vrlancamento,2).' * '.round($vrestrutura,2).'% = '.round($vrlancamento*($vrestrutura/100),2).PHP_EOL;
        
        $this->memoriaRateio .= $this->infoValorEstrutura[$dtlancamento.$nrestruturam];
    }
    
    public function mensagemInfoPesoEstrutura($dtlancamento, $vrlancamento, $nrestruturam, $vrestrutura, $nmestruturah, $qtdvinculos, $nrpesovinculos){
        $this->infoValorEstrutura[$dtlancamento.$nrestruturam] =
            '* ================================================================'.PHP_EOL.
            '* Estrutura: '.$nrestruturam.'-'.$nmestruturah.' / Percentual: '.round($vrestrutura,2).'%'.PHP_EOL.
            '* Valor Estrutura: '.round($vrlancamento,2).' * '.round($vrestrutura,2).'% = '.round($vrlancamento*($vrestrutura/100),2).PHP_EOL.
            '* Total de Vínculos: '.$qtdvinculos.PHP_EOL.
            '* Total Peso: '.$nrpesovinculos.PHP_EOL;
        
        $this->memoriaRateio .= $this->infoValorEstrutura[$dtlancamento.$nrestruturam];
    }
    
    public function mensagemInfoValorOcupacao($dtlancamento, $vrlancamento, $nrestruturam, $vrestrutura, $nrocupacaom, $vrocupacao, $nmocupacaoh){
        $this->infoValorOcupacao[$dtlancamento.$nrestruturam.$nrocupacaom] =
            '** ---------------------------------------------------------------'.PHP_EOL.
            '** Ocupação: '.$nrocupacaom.'-'.$nmocupacaoh.' / Percentual: '.round($vrocupacao,2).PHP_EOL.
            '** Valor Ocupação: '.round($vrlancamento*($vrestrutura/100),2).' * '.round($vrocupacao,2).'% = '.round($vrlancamento*($vrestrutura/100)*($vrocupacao/100),2).PHP_EOL;
        
        $this->memoriaRateio .= $this->infoValorOcupacao[$dtlancamento.$nrestruturam.$nrocupacaom];
    }
    
    public function mensagemInfoPesoOcupacao($dtlancamento, $vrlancamento, $nrestruturam, $vrestrutura, $nrocupacaom, $nmocupacaoh, $nrpesoocupacao){
        $this->infoValorOcupacao[$dtlancamento.$nrestruturam.$nrocupacaom] =
            '** ---------------------------------------------------------------'.PHP_EOL.
            '** Ocupação: '.$nrocupacaom.'-'.$nmocupacaoh.' / Peso: '.$nrpesoocupacao.PHP_EOL;
        
        $this->memoriaRateio .= $this->infoValorOcupacao[$dtlancamento.$nrestruturam.$nrocupacaom];
    }
    
    public function mensagemInfoValorVinculo($dtlancamento, $vrlancamento, $nrestruturam, $vrestrutura, $nrocupacaom, $vrocupacao, $qtdvinculos){
        $this->infoValorVinculo[$dtlancamento.$nrestruturam.$nrocupacaom] =
            '**'.PHP_EOL.
            '** Qntd. Vínculos: '.$qtdvinculos.PHP_EOL.
            '** Valor por Vínculo: (('.round($vrlancamento,2).' * ('.round($vrestrutura,2).'%)) * ('.round($vrocupacao,2).'%)) / '.$qtdvinculos.PHP_EOL.
            '** Valor por Vínculo: (('.round($vrlancamento,2).' * ('.round($vrestrutura,2).'%)) * ('.round($vrocupacao,2).'%)) / '.$qtdvinculos.PHP_EOL.
            '** Resultado: '.round((($vrlancamento*($vrestrutura/100))*($vrocupacao/100))/$qtdvinculos,2).PHP_EOL.
            '**'.PHP_EOL.
            '** Vínculos:'.PHP_EOL;
        
        $this->memoriaRateio .= $this->infoValorVinculo[$dtlancamento.$nrestruturam.$nrocupacaom];
    }
    
    public function mensagemInfoPesoVinculo($dtlancamento, $vrlancamento, $nrestruturam, $vrestrutura, $nrocupacaom, $vrocupacao, $qtdvinculos){
        $this->infoValorVinculo[$dtlancamento.$nrestruturam.$nrocupacaom] =
            '**'.PHP_EOL.
            '** Vínculos:'.PHP_EOL;
        
        $this->memoriaRateio .= $this->infoValorVinculo[$dtlancamento.$nrestruturam.$nrocupacaom];
    }
    
    public function mensagemInfoVinculo($nrvinculom, $nmvinculom, $nrpeso){
        $this->infoVinculo[$nrvinculom] = 
            '*** '.$nrvinculom.' - '.$nmvinculom.' / Peso:'.$nrpeso.PHP_EOL;
        
        $this->memoriaRateio .= $this->infoVinculo[$nrvinculom];
    }
    
    public function mensagemPesoVinculo($nrvinculom, $nmvinculom, $nrpeso, $vrvalor, $expressao){
        $this->infoVinculo[$nrvinculom] = 
            '*** '.$nrvinculom.' - '.$nmvinculom.' / Peso:'.$nrpeso.' / Valor:'.round($vrvalor,2).' ('.$expressao.')'.PHP_EOL;
        
        $this->memoriaRateio .= $this->infoVinculo[$nrvinculom];
    }
    
    public function mensagemFimValorVinculo(){
        $this->memoriaRateio .= '** ---------------------------------------------------------------'.PHP_EOL;
    }
    
    public function mensagemFimValorEstrutura(){
        $this->memoriaRateio .= '* ================================================================'.PHP_EOL;
    }
    
    public function mensagemFimValorDia(){
        $this->memoriaRateio .= '__________________________________________________________________'.PHP_EOL;
    }
    
    public function mensagemVinculosSituFuncNaoGeraRateio($arrayVinculos){
        $this->infoVinculosSitufunc = 
            '* ================================================================'.PHP_EOL.
            '* Vínculos Situação Funcional não gera rateio:'.PHP_EOL;
        foreach ($arrayVinculos as $nrvinculom => $nmvinculom) {
            $this->infoVinculosSitufunc .= 
                '** '.$nrvinculom.' - '.$nmvinculom.PHP_EOL;
        }
        $this->infoVinculosSitufunc .= 
            '* ================================================================'.PHP_EOL;
        
        $this->memoriaRateio .= $this->infoVinculosSitufunc;
    }
    
    public function mensagemVinculosFaltaNaoGeraRateio($arrayVinculos){
        $this->infoVinculosFalta = 
            '* ================================================================'.PHP_EOL.
            '* Vínculos não gera rateio por falta:'.PHP_EOL;
        foreach ($arrayVinculos as $nrvinculom => $nmvinculom) {
            $this->infoVinculosFalta .= 
                '** '.$nrvinculom.' - '.$nmvinculom.PHP_EOL;
        }
        $this->infoVinculosFalta .= 
            '* ================================================================'.PHP_EOL;
        
        $this->memoriaRateio .= $this->infoVinculosFalta;
    }
    
    public function mensagemVinculosAtestadoNaoGeraRateio($arrayVinculos){
        $this->infoVinculosAtestado = 
            '* ================================================================'.PHP_EOL.
            '* Vínculos não gera rateio por atestado:'.PHP_EOL;
        foreach ($arrayVinculos as $nrvinculom => $nmvinculom) {
            $this->infoVinculosAtestado .= 
                '** '.$nrvinculom.' - '.$nmvinculom.PHP_EOL;
        }
        $this->infoVinculosAtestado .= 
            '* ================================================================'.PHP_EOL;
        
        $this->memoriaRateio .= $this->infoVinculosAtestado;
    }
    
    public function mensagemInfoNaoGeraRateio($situFuncNaoGeraRateio,$tipoVincNaoGeraRateio,$vincEmpregNaoGeraRateio){
        if($situFuncNaoGeraRateio){
            foreach($situFuncNaoGeraRateio as $vinculo){
                $vinculoSituFuncArr[] = $vinculo;
            }
        }
        if($tipoVincNaoGeraRateio){
            foreach($tipoVincNaoGeraRateio as $tipoVinc){
                $tipoVincArr[] = $tipoVinc;
            }
        }
        if($vincEmpregNaoGeraRateio){
            foreach($vincEmpregNaoGeraRateio as $vincEmpreg){
                $vincEmpregArr[] = $vincEmpreg;
            }
        }
        $vinculoSituFuncArr[] = '';
        $tipoVincArr[] = '';
        $vincEmpregArr[] = '';
        
        $this->infoRateioNaoGeraRateio =
            '=============================================================================================================='.PHP_EOL.
            '=-=-=-=-=-=-=- VINCULOS COM SITUAÇÕES FUNCIONAIS QUE NÃO GERAM RATEIO -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-='.PHP_EOL.PHP_EOL.
            implode(PHP_EOL,$vinculoSituFuncArr).PHP_EOL.PHP_EOL.
            '=============================================================================================================='.PHP_EOL.
            '=-=-=-=-=-=-=- VINCULOS COM TIPOS DE VINCULOS QUE NÃO GERAM RATEIO -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-='.PHP_EOL.PHP_EOL.
            implode(PHP_EOL,$tipoVincArr).PHP_EOL.PHP_EOL.
            '=-=-=-=-=-=-=- VINCULOS COM VINCULOS EMPREGATICIOS QUE NÃO GERAM RATEIO -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-='.PHP_EOL.PHP_EOL.
            implode(PHP_EOL,$vincEmpregArr).PHP_EOL.PHP_EOL;
            
        return $this->infoRateioNaoGeraRateio;
    }
    
    public function getMemoria(){
        return $this->memoriaRateio;
    }
    
    public function salvarMsgArquivo($mensagemMemoriaDeCalculo){
        $this->arquivo = FPA::getTempDirectory().'/Memoria_Rateio  Execucao('.DateUtil::getDataAtual()->format('d-m-Y H:i:s').').txt';
        file_put_contents($this->arquivo, $mensagemMemoriaDeCalculo);
    }

}