<?php
/**
 * Responsável pela tela de Emissão de Documentos (Contratos/Termos) - hcm10010_ger_emissaodoc
 * Rota /hcm/backend/config/routes/routes.json /emitirDocPadrao
 * Controller Front /hcm/mobile/assets/js/controllers/EmissaoDocController.js
 * 
 * Rafael Pickbrenner
 * 13/04/2021
 * 
 */

namespace HCM\Geral\Controllers;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\Report\ReportService;
use Zeedhi\Framework\DataSource\DataSet;
use HCM\Util\HCMParameters;
use HCM\Geral\Services\EmissaoDocService;
/*use HCM\Geral\Services\VinculoServices;
use HCM\Geral\Services\RelatoriosHcmService;
use HCM\Geral\Controllers\EnviarMensagemController;*/

class EmissaoDocController {

    protected $service;
    
    protected $nrorg;
    protected $urlMidiaServer;
    protected $dtmescompetencia;
    protected $dtmescompetenciafim;
    protected $dspath;
    protected $nmdocumento;
    protected $nrvinculom_list;
    protected $nrestrutlegal_list;
    protected $nrestrutgeren_list;
    protected $nrestrutsind;
    protected $dtiniadmissao;
    protected $dtfimadmissao;
    protected $dtinirescisao;
    protected $dtfimrescisao;
    protected $parametrosAdicionais = array();
    protected $flagPDF;

    public function __construct(EmissaoDocService $service, HCMParameters $hcmParameters){
        $this->service = $service;
        //$this->hcmParameters = $hcmParameters;
        $this->nrorg         = $hcmParameters->getNrorgTrabalho();
        //$this->dtCompetencia = $hcmParameters->getDtCompetencia();
    }
    
    public function emitirDocPadrao(DTO\Request\DataSet $request, DTO\Response $response){
        try {
            $this->getParametrosTela($request->getDataSet()->getRows()[0]);
            
            $fileToDownload = $this->service->emitirDocPadrao(
                $this->nrorg,
                $this->urlMidiaServer,
                $this->dtmescompetencia,
                $this->dtmescompetenciafim,
                $this->dspath,
                $this->nmdocumento,
                $this->nrvinculom_list,
                $this->nrlotevinc_list,
                $this->nrestrutlegal_list,
                $this->nrestrutgeren_list,
                $this->nrestrutsind,
                $this->dtiniadmissao,
                $this->dtfimadmissao,
                $this->dtinirescisao,
                $this->dtfimrescisao,
                $this->parametrosAdicionais,
                $this->flagPDF);
            if($fileToDownload != null) {
                $fileContent = file_get_contents($fileToDownload[0]);
                $fileInBase64 = 'data:application/pdf;base64,'.base64_encode($fileContent);
                $fileToDownload = array('NMARQUIVO' => $fileToDownload, 'base64' => $fileInBase64);
                $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('arquivo', $fileToDownload));
            }
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    private function getParametrosTela($row) {
        $this->urlMidiaServer      = $row['urlMidiaServer'];
        $this->dtmescompetencia    = isset($row['DTMESCOMPETENCIA']   ) ? $row['DTMESCOMPETENCIA']    : null;
        $this->dtmescompetenciafim = isset($row['DTMESCOMPETENCIAFIM']) ? $row['DTMESCOMPETENCIAFIM'] : null;
        $this->dspath              = isset($row['DSPATH']             ) ? $row['DSPATH']              : null;
        $this->nmdocumento         = isset($row['NMDOCUMENTO']        ) ? $row['NMDOCUMENTO']         : null;
        $this->nrvinculom_list     = isset($row['NRVINCULOM_LIST']    ) ? $row['NRVINCULOM_LIST']     : null;
        $this->nrlotevinc_list     = isset($row['NRLOTEVINC_LIST']    ) ? $row['NRLOTEVINC_LIST']     : null;
        $this->nrestrutlegal_list  = isset($row['NRESTRUTLEGAL_LIST'] ) ? $row['NRESTRUTLEGAL_LIST']  : null;
        $this->nrestrutgeren_list  = isset($row['NRESTRUTGEREN_LIST'] ) ? $row['NRESTRUTGEREN_LIST']  : null;
        $this->nrestrutsind        = isset($row['NRESTRUTSIND']       ) ? $row['NRESTRUTSIND']        : null;
        $this->dtiniadmissao       = isset($row['DTINIADMISSAO']      ) ? $row['DTINIADMISSAO']       : null;
        $this->dtfimadmissao       = isset($row['DTFIMADMISSAO']      ) ? $row['DTFIMADMISSAO']       : null;
        $this->dtinirescisao       = isset($row['DTINIRESCISAO']      ) ? $row['DTINIRESCISAO']       : null;
        $this->dtfimrescisao       = isset($row['DTFIMRESCISAO']      ) ? $row['DTFIMRESCISAO']       : null;
        $this->flagPDF             = isset($row['CONVERTERDOCXTOPDF'] ) ? $row['CONVERTERDOCXTOPDF']  : null;
        $this->parametrosAdicionais['ADDTEMISSAO']    = isset($row['ADDTEMISSAO']   ) ? $row['ADDTEMISSAO']    : null;
        $this->parametrosAdicionais['ADDTULTIMO']     = isset($row['ADDTULTIMO']    ) ? $row['ADDTULTIMO']     : null;
        $this->parametrosAdicionais['ADDTTERLICENCA'] = isset($row['ADDTTERLICENCA']) ? $row['ADDTTERLICENCA'] : null;
        $this->parametrosAdicionais['ADDTRETORNO']    = isset($row['ADDTRETORNO']   ) ? $row['ADDTRETORNO']    : null;
        $this->parametrosAdicionais['ADDTINILICENCA'] = isset($row['ADDTINILICENCA']) ? $row['ADDTINILICENCA'] : null;
        $this->parametrosAdicionais['ADDTPAGTO']      = isset($row['ADDTPAGTO'])      ? $row['ADDTPAGTO']      : null;
        $this->parametrosAdicionais['ADDTEXAME']      = isset($row['ADDTEXAME'])      ? $row['ADDTEXAME']      : null;
        $this->parametrosAdicionais['ADHREXAME']      = isset($row['ADHREXAME'])      ? $row['ADHREXAME']      : null;
        $this->parametrosAdicionais['ADHRCOMPARECI']  = isset($row['ADHRCOMPARECI'])  ? $row['ADHRCOMPARECI']  : null;
    }
}