<?php

namespace HCM\Geral\Controllers;

use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DTO;
use HCM\Geral\Services\MunicipioService;
use HCM\Util\HCMParameters;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DataSource\ParameterBag;
use HCM\Util\Messages;

class MunicipioController extends Crud {

    protected $dataSourceName = 'municipio';
    
    protected $parameterBagColumns = array('P_NRORG', 'P_NRORG_PADRAO', 'SGESTADO', 'CDPAIS', "P_EXIBEREGISTRO");    

    public function __construct (Manager $dataSourceManager, ParameterBag $parameterBag, HCMParameters $hcmParameters, MunicipioService $municipioService){
        $this->parameterBag = $parameterBag;
        $this->hcmParameters = $hcmParameters;
    	$this->municipioService = $municipioService;
    	parent::__construct($dataSourceManager);
    }

	public function find(DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceName);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData);
        $response->addDataSet($dataSet);
    }

    public function setMunicipio(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            $isNew;
            $existPrimaryKey = false;
            
            foreach ($request->getDataSet()->getRows() as &$row) {
                $isNew = $row['__is_new'];

                $existPrimaryKey = $this->municipioService->validaRestricaoChavePrimaria(
                    $row['CDPAIS'], 
                    $row['SGESTADO'],
                    $row['CDMUNICIPIO'],
                    $row['NMMUNICIPIO'],
                    $isNew
                );

                $this->municipioService->validaCamposVaziosMunicipio($row);
            }
            if($existPrimaryKey){
                if($isNew)
                    $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message(Messages::HCM00102_GER_DUPLICATE_CODE_OR_NAME_ERROR, DTO\Response\Message::TYPE_WARNING));
                else
                    $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message(Messages::HCM00102_GER_DUPLICATE_NAME_ERROR, DTO\Response\Message::TYPE_WARNING));
            }else
                parent::save($request, $response);
        
            
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function deleteMunicipio(DTO\Request\DataSet $request, DTO\Response $response){
        parent::delete($request, $response);
    }
    
    protected function processConditionValue($condition) {
        return $condition['value'];
    }
   
    protected function populateParameterBag(FilterCriteria $filterCriteria) {

        $conditions = array();
        foreach ($filterCriteria->getConditions() as $condition) {
            $columnName = $condition['columnName'];
            if (in_array($columnName, $this->parameterBagColumns)) {
                $this->parameterBag->set($columnName, $this->processConditionValue($condition));
            } else {
                $conditions[] = $condition;
            }
        }
        $newFilterCriteria = new FilterCriteria(
            $filterCriteria->getDataSourceName(),
            $conditions,
            $filterCriteria->getPage(),
            $filterCriteria->getPageSize()
        );
        return $newFilterCriteria;
    }     
    
}