<?php

namespace HCM\Geral\Controllers;

use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Response\Notification;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Request\DataSet;
use HCM\Util\HCMParameters;
use Doctrine\ORM\EntityManager;
use HCM\Geral\Services\ParametrizacaoEsocialService;
use HCM\FPA\Services\EventoService;

class ParametrizacaoEsocialController extends Crud {

    protected $dataSourceName = 'eso_parametrizacaoEsocial_set';
    protected $parameterBagColumns = array('P_NRORG', 'P_NRORG_PADRAO','P_DTMESCOMPETENC', 'P_NREVENTOM');
    public function __construct (Manager $dataSourceManager, ParameterBag $parameterBag, HCMParameters $hcmParameters, EntityManager $entityManager, ParametrizacaoEsocialService $parametrizacaoEsocialService, 
                                 EventoService $eventoService){
        $this->parameterBag = $parameterBag;
        $this->hcmParameters = $hcmParameters;
        $this->entityManager = $entityManager; 
        $this->parametrizacaoEsocialService = $parametrizacaoEsocialService;
        $this->eventoService = $eventoService;
        
        parent::__construct($dataSourceManager);
    }

    public function setParametrizacaoEsocial(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            foreach ($request->getDataSet()->getRows() as $row) {
                $row['NRTIPOMOVIMENT'] =  $row['NRTIPOMOVIMENT']??null;
                $row['CDINCIDECP'] = $row['NRINCIDECP']??null;
                $row['CDINCIDEFGTS'] = $row['NRINCIDEFGTS']??null;
                $row['CDINCIDEIRRF'] = $row['NRINCIDEIRRF']??null;
                $row['NRTIPORUBRICA'] = $row['NRTIPORUBRICA']??null;
                $row['NREVENTOM'] = $row['NREVENTOM']??null;
                $row['DTESOCIAL'] = $row['DTESOCIAL']??null;
                $row['IDATIVO'] = $row['IDATIVO']?? 'S';

                $existeNaturezaRubricaEvento = $this->parametrizacaoEsocialService->buscaEventoComNaturezaRubrica($row['NREVENTOM']);

                if(empty($existeNaturezaRubricaEvento)) {
                    $response->addNotification(new Notification("O evento não possui a Natureza da Rubrica. Por favor, preencha este campo para prosseguir.", Notification::TYPE_ALERT));
                    return;
                }


                $existeParametrizacaoEsocial = $this->parametrizacaoEsocialService->existeParametrizacaoEsocial($row);
                if(!empty($existeParametrizacaoEsocial) && ($existeParametrizacaoEsocial->getNrrubrica() != $row['NRRUBRICA'] || !isset($row['NRRUBRICA'])) ) {
                    $response->addNotification(new Notification("Já existe uma parametrização para esse tipo de movimento", Notification::TYPE_ALERT));
                    return;
                }
               
                if($row['__is_new'] === true){
                   $row['NRRUBRICA'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgPadrao(), 'ESO_RUBRICA', 1, 12, true); 
                }

                parent::save($request, $response);
                
                $this->eventoService->verificaGeraNovoHistorico($row["NREVENTOM"]);
            }
        }catch(\Exception $e){
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function cadastroAutomaticoParametrizacaoEsocial(DTO\Request\DataSet $request, DTO\Response $response) 
    {   
        try {
            foreach ($request->getDataSet()->getRows() as $row) {
                $retorno = $this->parametrizacaoEsocialService->cadastroAutomaticoParametrizacaoEsocial($row["NREVENTOM"]);
                
                if($retorno === 'erroNaturezaRubrica') {
                    $response->addNotification(new Notification("O evento não possui a Natureza da Rubrica. Por favor, preencha este campo para prosseguir.", Notification::TYPE_ALERT));
                    return;
                }
                
                if($retorno) {
                    $this->eventoService->verificaGeraNovoHistorico($row["NREVENTOM"]);
                    $response->addNotification(new DTO\Response\Notification('cadastroAutomaticoParametrizacaoEsocial'), DTO\Response\Notification::TYPE_SUCCESS);
                    return;
                }
            }
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function deleteParametrizacaoEsocial(DTO\Request\DataSet $request, DTO\Response $response){
        try {
            $this->dataSourceName = $this->dataSourceName;
            parent::delete($request, $response);
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error(empty($error) ? $e->getMessage() : $error, empty($error) ? $e->getCode() : 5));
        }  
    }

    protected function processConditionValue($condition) {
        return $condition['value'];
    }
    protected function populateParameterBag(FilterCriteria $filterCriteria) {

        $conditions = array();
        foreach ($filterCriteria->getConditions() as $condition) {
            $columnName = $condition['columnName'];
            if (in_array($columnName, $this->parameterBagColumns)) {
                $this->parameterBag->set($columnName, $this->processConditionValue($condition));
            } else {
                $conditions[] = $condition;
            }
        }
        $newFilterCriteria = new FilterCriteria(
            $filterCriteria->getDataSourceName(),
            $conditions,
            $filterCriteria->getPage(),
            $filterCriteria->getPageSize()
        );
        return $newFilterCriteria;
    }
} 