<?php
/**
 * Created by PhpStorm.
 * User: felipesantos
 * Date: 08/08/2016
 * Time: 14:17
 */

namespace HCM\Geral\Controllers;

use Folha\Entities\FpaTipomoviment;
use HCM\Geral\Services\RelatorioCustomizavelService;
use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request;
use Doctrine\ORM\EntityManager;
use HCM\Util\HCMParameters;
use HCM\Util\Messages;
use HCM\Util\Util;
use HCM\Util\Repositories;
use HCM\Util\FPA;

class RelatorioCustomizavelController extends Crud{

    protected $dataSourceNameGet = 'getRelatCustomBeneficioNovo';
    protected $dataSourceNameGetDir = 'getRelatCustomBeneficioDiario';

    protected $parameterBagColumns = ['P_NRORG', 'P_NRORG_PADRAO', 'P_DTMESCOMPETENC', 'P_NRTIPOMOVIMENT','P_RESCISAO','P_ULTIMODIAMES','P_NRCALCULOFOLHA',
                                      'P_NRESTRUTLEGAL', 'P_NRESTRUTGEREN', 'P_NRESTRUTSIND', 'NRESTRUTURA_BENEFICIO', 'NMVINCULOM', 'NRTPBENEFICIO_LIST', 'NRBENEFICIOM_LIST',
                                      'P_NRESTRUTLEGALOPER', 'P_NRESTRUTGERENOPER', 'P_TPPERIODOBENEF', 'P_NRTIPOHIERARQORG', 'P_NRESTRUTSUPERIORM', 'LIST_NRESTRUTLEGAL',
                                      'LIST_NRESTRUTGEREN', 'LIST_NRESTRUTSIND', 'NRVINCULOM_LIST'];

    public function __construct (Manager $dataSourceManager, ParameterBag $parameterBag, HCMParameters $hcmParameters, RelatorioCustomizavelService $relatorioCustomizavelService, EntityManager $entityManager){
        $this->parameterBag = $parameterBag;
        $this->hcmParameters = $hcmParameters;
        $this->relatorioCustomizavelService = $relatorioCustomizavelService;
        $this->entityManager = $entityManager;

        parent::__construct($dataSourceManager);
    }

    public function getRelatCustomValeTransporte(DTO\Request\Filter $request, DTO\Response $response) {

        set_time_limit(0);
        

        $parameterBagColumns = array('P_NRORG_PADRAO', 'P_NRORG', 'P_DTMESCOMPETENC');
        $conditions = array();

        $nrorgPadrao = $this->hcmParameters->getNrorgPadrao();
        $nrorg = $this->hcmParameters->getNrorgTrabalho();
        $dtcompetencia = $this->hcmParameters->getDtCompetencia();
        $tipohierarquia = $condEstrutLegal = $condEstrutGeren = $condEstrutSup = null;
        $listaVinc = "";

        $filters = Util::removerCaracterePorcentagemString($request->getFilterCriteria()->getConditions());
        
        foreach($filters as $key => $condition) {
            if ($condition['columnName'] === $parameterBagColumns[0]) {
                $nrorgPadrao = $condition['value'];
            }
            else if ($condition['columnName'] === $parameterBagColumns[1]) {
                $nrorg = $condition['value'];
            }
            else if ($condition['columnName'] === $parameterBagColumns[2]) {
                $dtcompetencia = $condition['value'];
            }
            else if ($condition['columnName'] === 'NRVINCULOM') {
                $listaVinc .= empty($listaVinc) ? $condition['value'] : "," . $condition['value'];
            }
            else if ($condition['columnName'] === 'NRVINCULOM_LIST') {
                $listaVinc .= empty($listaVinc) ? implode(",", $condition['value']) : "," . implode(",", $condition['value']);
            }
            else if ($condition['columnName'] === 'NRLOTEVINC_LIST') {
                $condition = $this->adicionaVinculoPorLoteVinc($condition);
                $listaVinc .= empty($listaVinc) ? implode(",", $condition['value']) : "," . implode(",", $condition['value']);
            }
            else if ($condition['columnName'] === 'NMVINCULOM') {
                unset($filters[$key]);
            }
            else if ($condition['columnName'] === 'NRTIPOHIERARQORG') {
                $tipohierarquia = $condition['value'];
            }
            else if ($condition['columnName'] === 'LIST_NRESTRUTLEGAL') {
                $condEstrutLegal = $condition['value'];
            }
            else if ($condition['columnName'] === 'LIST_NRESTRUTGEREN') {
                $condEstrutGeren = $condition['value'];
            }
            else if ($condition['columnName'] === 'NRESTRUTSUPERIORM_LIST') {
                $condEstrutSup = $condition['value'];
            }
            else {
                $conditions[] = $condition;
            }
        }

        $listaVincArray = array_unique(explode(",", $listaVinc));
        $listaVinc = implode(",", $listaVincArray);
        $conditions = Util::createQueryFiltered($conditions);
        
        $dataSet = $this->relatorioCustomizavelService->retornaRelatCustoValeTransporte($nrorgPadrao, $nrorg, $dtcompetencia, $tipohierarquia, $condEstrutLegal, $condEstrutGeren,  $listaVinc, $conditions, $condEstrutSup);
        $response->addDataSet(new DataSet('getRelatCustomValeTransporte', $dataSet));
    }

    public function getRelatCustomBeneficio(DTO\Request\Filter $request, DTO\Response $response) {

        set_time_limit(0);
        

        $parameterBagColumns = array('P_NRORG_PADRAO', 'P_NRORG', 'P_DTMESCOMPETENC');
        $conditions = $buttonFloatingSearch = array();

        $nrorgPadrao = $this->hcmParameters->getNrorgPadrao();
        $nrorg = $this->hcmParameters->getNrorgTrabalho();
        $dtcompetencia = $this->hcmParameters->getDtCompetencia();
        $tipohierarquia = /*$listaVinc =*/ $condEstrutLegal = $condEstrutGeren = $condRescisaoVinc = $condAtivacaoBeneficio = $condAtivacaoBeneficiario = null;
        $condListaEstrutLegal = $condListaEstrutGeren = $condListaEstrutSind = $condListaBeneficio = $condListaTpBeneficio = $condListaVinculo = 'A';

        $filters = Util::removerCaracterePorcentagemString($request->getFilterCriteria()->getConditions());

        foreach($filters as $key=>$condition) {
            
            if ($condition['columnName'] === $parameterBagColumns[0]) {
                $nrorgPadrao = $condition['value'];
            }
            else if ($condition['columnName'] === $parameterBagColumns[1]) {
                $nrorg = $condition['value'];
            }
            else if ($condition['columnName'] === $parameterBagColumns[2]) {
                $dtcompetencia = $condition['value'];
            }
            /*else if ($condition['columnName'] === 'NRVINCULOM') {
                $listaVinc = $condition['value'];
            }*/
            else if ($condition['columnName'] === 'NMVINCULOM') {
                unset($filters[$key]);
            }
            else if ($condition['columnName'] === 'NRTIPOHIERARQORG') {
                $tipohierarquia = $condition['value'];
            }
            else if ($condition['columnName'] === 'NRESTRUTLEGAL') {
                $condEstrutLegal = $condition['value'];
            }
            else if ($condition['columnName'] === 'NRESTRUTGEREN') {
                $condEstrutGeren = $condition['value'];
            }
            else if ($condition['columnName'] === 'IDRESCISAO') {
                $condRescisaoVinc = $condition['value'];
            }
            else if ($condition['columnName'] === 'IDATIVBENEFICIO') {
                $condAtivacaoBeneficio = $condition['value'];
            }
            else if ($condition['columnName'] === 'IDATIVBENEFICARIO') {
                $condAtivacaoBeneficiario = $condition['value'];
            }
            else if ($condition['columnName'] === 'NRESTRUTLEGAL_LIST') {
                $condListaEstrutLegal = (is_array($condition['value']) ? '_'.implode('_',$condition['value']).'_' : 'A');
            }
            else if ($condition['columnName'] === 'NRESTRUTGEREN_LIST') {
                $condListaEstrutGeren = (is_array($condition['value']) ? '_'.implode('_',$condition['value']).'_' : 'A');
            }
            else if ($condition['columnName'] === 'NRESTRUTSIND_LIST') {
                $condListaEstrutSind = (is_array($condition['value']) ? '_'.implode('_',$condition['value']).'_' : 'A');
            }
            else if ($condition['columnName'] === 'NRBENEFICIOM_LIST') {
                $condListaBeneficio = (is_array($condition['value']) ? '_'.implode('_',$condition['value']).'_' : 'A');
            }
            else if ($condition['columnName'] === 'NRTPBENEFICIO_LIST') {
                $condListaTpBeneficio = (is_array($condition['value']) ? '_'.implode('_',$condition['value']).'_' : 'A');
            }
            else if ($condition['columnName'] === 'NRVINCULOM_LIST') {
                $condListaVinculo = (is_array($condition['value']) ? '_'.implode('_',$condition['value']).'_' : 'A');
            }
            else {
                $conditions[] = $condition;
            }
        }    
        $buttonFloatingSearch = Util::createQueryFiltered($buttonFloatingSearch);
        $nrcalculobenef = $this->hcmParameters->determinaNrCalculoFolha(FpaTipomoviment::TP_MOV_BENEFICIO);
        
        if(!$condRescisaoVinc) 
            $condRescisaoVinc = 'A';
        
        if(!$condAtivacaoBeneficio) 
            $condAtivacaoBeneficio = 'A';
        
        if(!$condAtivacaoBeneficiario) 
            $condAtivacaoBeneficiario = 'A';
        
        $conditions = Util::createQueryFiltered($conditions);
        
        $dataSet = $this->relatorioCustomizavelService->retornaRelatCustoBeneficio(
            $nrorgPadrao, $nrorg, $nrcalculobenef, $dtcompetencia, $tipohierarquia, $condEstrutLegal, $condEstrutGeren, $condRescisaoVinc, 
            $condAtivacaoBeneficio, $condAtivacaoBeneficiario, /*$listaVinc,*/ $conditions, $buttonFloatingSearch,
            $condListaEstrutLegal, $condListaEstrutGeren, $condListaEstrutSind, $condListaBeneficio, $condListaTpBeneficio, $condListaVinculo
        );
        $response->addDataSet(new DataSet('getRelatCustomBeneficio', $dataSet));
    }

    public function getRelatCustomBeneficioNovo(DTO\Request\Filter $request, DTO\Response $response) {
        set_time_limit(0);
        
        $nrtipomoviment = null;
        $nrestrutura_beneficio = null;
        $filterData = $request->getFilterCriteria();
        //$conditions = $filterData->getConditions();
        $columnsTpHierEstSup = ['NRTIPOHIERARQORG', 'NRESTRUTSUPERIORM'];
        $columnsListEsts  = ['LIST_NRESTRUTLEGAL', 'LIST_NRESTRUTGEREN', 'LIST_NRESTRUTSIND'];
        $columnsToImplode = ['NRTPBENEFICIO_LIST', 'NRBENEFICIOM_LIST'];
        $listaVinc = null;
        $conditions = Util::removerCaracterePorcentagemString($request->getFilterCriteria()->getConditions());

        foreach($conditions as $key => &$condition) {
            if($condition['columnName'] == 'P_NRTIPOMOVIMENT') {
                $nrtipomoviment = $condition['value'];
            }
            if($condition['columnName'] == 'NRESTRUTURA_BENEFICIO') {
                $nrestrutura_beneficio = $condition['value'];
            }
            // O operador recebido do front é '=' ao invés de 'IN'
            if($condition['columnName'] == 'NRVINCULOM') {
                $condition['operator'] = 'IN';
            }

            if ($condition['columnName'] === 'NRVINCULOM') {
                $listaVinc .= empty($listaVinc) ? $condition['value'] : "," . $condition['value'];
                unset($conditions[$key]);
            }
            if ($condition['columnName'] === 'NRVINCULOM_LIST') {
                $listaVinc .= empty($listaVinc) ? implode(",", $condition['value']) : "," . implode(",", $condition['value']);
                unset($conditions[$key]);
            }
            if ($condition['columnName'] === 'NRLOTEVINC_LIST') {
                $condition = $this->adicionaVinculoPorLoteVinc($condition);
                $listaVinc .= empty($listaVinc) ? implode(",", $condition['value']) : "," . implode(",", $condition['value']);
                unset($conditions[$key]);
            }

            if(in_array($condition['columnName'], $columnsToImplode)) {              
                $condition['value'] = implode(",", $condition['value']);
            }

            if(in_array($condition['columnName'], $columnsListEsts)) {

                $condition['columnName'] = 'P_'.explode('_', $condition['columnName'])[1];
                $condition['value'] = implode(",", $condition['value']);
            }

            if(in_array($condition['columnName'], $columnsTpHierEstSup)) {

                $condition['columnName'] = 'P_'.$condition['columnName'];
                $condition['value'] = implode(",", $condition['value']);
            }
        }

        $listaVincArray = array_unique(explode(",", $listaVinc));
        $listaVinc = implode(",", $listaVincArray);
        
        $filterData = new FilterCriteria(
            $filterData->getDataSourceName(),
            $conditions,
            $filterData->getPage(),
            $filterData->getPageSize()
        );
        
        $filterData->addCondition('NRVINCULOM_LIST', 'IN',  $listaVinc);
        $filterData->addCondition('P_NRESTRUTLEGALOPER', '=', FPA::getNrestrutlegalOper());
        $filterData->addCondition('P_NRESTRUTGERENOPER', '=', FPA::getNrestrutgerenOper());
        $filterData->addCondition('P_ULTIMODIAMES','=',$this->hcmParameters->getUltimoDiaMes());
        $filterData->addCondition('P_DTMESCOMPETENC','=',$this->hcmParameters->getDtCompetencia());
        $filterData->addCondition('P_NRORG_PADRAO','=',$this->hcmParameters->getNrorgPadrao());
        if($nrtipomoviment){
            $filterData->addCondition('P_NRCALCULOFOLHA','=',$this->hcmParameters->determinaNrCalculoFolha($nrtipomoviment));
            
            if($nrtipomoviment==21){
                $filterData->addCondition('P_RESCISAO','=','T');
            }
        }
         
        $nrestrutura_beneficio ? null : $filterData->addCondition('NRESTRUTURA_BENEFICIO', '=', null);
        $filterData->setDataSourceName($this->dataSourceNameGet);
        $newFilterData = $this->populateParameterBag($filterData);
        $newFilterData->setPageSize(99999999999);
        $dataSet = $this->dataSourceManager->findBy($newFilterData);

        $response->addDataSet($dataSet);
    }

    public function getRelatHistbenefvinc (DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria();
        //$conditions = $filterData->getConditions();
        $columnsTpHierEstSup = ['NRTIPOHIERARQORG', 'NRESTRUTSUPERIORM'];
        $columnsListEsts = ['LIST_NRESTRUTLEGAL', 'LIST_NRESTRUTGEREN', 'LIST_NRESTRUTSIND'];
        $columnsToImplode = ['NRBENEFICIOM_LIST'];
        $listaVinc = null;
        $conditions = Util::removerCaracterePorcentagemString($request->getFilterCriteria()->getConditions());
        
        foreach($conditions as $key => &$condition) {

            if($condition['columnName'] == 'NRVINCULOM') {
                $condition['operator'] = 'IN';
            }

            if ($condition['columnName'] === 'NRVINCULOM') {
                $listaVinc .= empty($listaVinc) ? $condition['value'] : "," . $condition['value'];
                unset($conditions[$key]);
            }
            if ($condition['columnName'] === 'NRVINCULOM_LIST') {
                $listaVinc .= empty($listaVinc) ? implode(",", $condition['value']) : "," . implode(",", $condition['value']);
                unset($conditions[$key]);
            }
            if ($condition['columnName'] === 'NRLOTEVINC_LIST') {
                $condition = $this->adicionaVinculoPorLoteVinc($condition);
                $listaVinc .= empty($listaVinc) ? implode(",", $condition['value']) : "," . implode(",", $condition['value']);
                unset($conditions[$key]);
            }

            if(in_array($condition['columnName'], $columnsToImplode)) {
                $condition['value'] = implode(",", $condition['value']);
            }

            if(in_array($condition['columnName'], $columnsListEsts)) {

                $condition['columnName'] = 'P_'.explode('_', $condition['columnName'])[1];
                $condition['value'] = implode(",", $condition['value']);
            }

            if(in_array($condition['columnName'], $columnsTpHierEstSup)) {

                $condition['columnName'] = 'P_'.$condition['columnName'];
                $condition['value'] = implode(",", $condition['value']);
            }
        }

        $listaVincArray = array_unique(explode(",", $listaVinc));
        $listaVinc = implode(",", $listaVincArray);
   

        $filterData = new FilterCriteria(
            $filterData->getDataSourceName(),
            $conditions,
            $filterData->getPage(),
            $filterData->getPageSize()
        );

        $filterData->addCondition('NRVINCULOM_LIST', 'IN',  $listaVinc);
        $filterData->setDataSourceName($this->dataSourceNameGetDir);
        $newFilterData = $this->populateParameterBag($filterData);

        $dataSet = $this->dataSourceManager->findBy($newFilterData); 
        
        $response->addDataSet($dataSet);
    }
    
    public function getRelatCustomBeneficioDirf(DTO\Request\Filter $request, DTO\Response $response) {
        set_time_limit(0);
        
        
        /*$nrorgPadrao = $this->hcmParameters->getNrorgPadrao();
        $nrorg = $this->hcmParameters->getNrorgTrabalho();
        $dtcompetencia = $this->hcmParameters->getDtCompetencia();*/

        $filters = Util::removerCaracterePorcentagemString($request->getFilterCriteria()->getConditions());
        
        $dataSet = $this->relatorioCustomizavelService->retornaRelatCustoBeneficioDirf($filters);
        
        $response->addDataSet(new DataSet('getRelatCustomBeneficioDirf', $dataSet));
    }

    public function getRelatEscala(DTO\Request\Filter $request, DTO\Response $response) {
        $parameterBagColumns = array('P_NRORG', 'P_DTMESCOMPETENC');
        
        $params = [];
        $nrorg = $nrvinculom = $dtinicontagem = $conditions = null;
        
        $filters = $request->getFilterCriteria()->getConditions();
        
        foreach($filters as $condition) {
            if ($condition['columnName'] === $parameterBagColumns[0]) {
                $nrorg = $condition['value'];
            }
            else if ($condition['columnName'] === $parameterBagColumns[1]) {
                $dtcompetencia = $condition['value'];
            }
            else if ($condition['columnName'] === 'NRVINCULOM') {
                $nrvinculom = $condition['value'];
            }
            else if ($condition['columnName'] === 'DTINICONTAGEM') {
                $dtinicontagem = $condition['value'];
            }
            else if($condition['columnName'] === 'LIST_NRESTRUTLEGAL') {
                $params['NRESTRUTLEGAL'] = $condition['value'];
            } 
            else if($condition['columnName'] === 'LIST_NRESTRUTGEREN') {
                $params['NRESTRUTGEREN'] = $condition['value'];
            }
        }
        
        Util::getValueInFilterDataSourceByName($filters, array('P_NRORG', 'P_DTMESCOMPETENC', 'DTINICONTAGEM', 'NRVINCULOM', 'LIST_NRESTRUTLEGAL', 'LIST_NRESTRUTGEREN'), true);
        
        $conditions = Util::createQueryFiltered($filters);

        $dataSet = $this->relatorioCustomizavelService->retornaRelatEscala($dtinicontagem, $nrorg, $nrvinculom, $conditions, $params);
        $response->addDataSet(new DataSet('getRelatEscala', $dataSet));
    }

    protected function processConditionValue($condition) {
        return $condition['value'];
    }
    protected function populateParameterBag(FilterCriteria $filterCriteria) {

        $conditions = array();
        foreach ($filterCriteria->getConditions() as $condition) {
            $columnName = $condition['columnName'];
            if (in_array($columnName, $this->parameterBagColumns)) {
                $this->parameterBag->set($columnName, $this->processConditionValue($condition));
            } else {
                $conditions[] = $condition;
            }
        }
        $newFilterCriteria = new FilterCriteria(
            $filterCriteria->getDataSourceName(),
            $conditions,
            $filterCriteria->getPage(),
            $filterCriteria->getPageSize()
        );


        return $newFilterCriteria;
    }

    public function adicionaVinculoPorLoteVinc($condition)
    {
        $nrvinculom = array();
        foreach($condition['value']  as $nrlotevinc ) {

            $itens_lotevinc = $this->entityManager->getRepository(Repositories::FPA_ITEMLOTEVINC)->findBy(array('nrlotevinc' => $nrlotevinc, 'nrorg' => $this->hcmParameters->getNrorgTrabalho()));

            if(!empty($itens_lotevinc)) {
                foreach( $itens_lotevinc as $item_lotevinc ) {
                    array_push( $nrvinculom, $item_lotevinc->getNrvinculom() );
                }           
            }
        }

        $condition['value'] = array();
        $condition['value'] = $nrvinculom;
        return $condition;
    }
} 