<?php

namespace HCM\Geral\Controllers;

use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request;
use HCM\Util\HCMParameters;
use Doctrine\ORM\EntityManager;
use HCM\Geral\Services\TipoAnotacaoService;

class TipoAnotacaoController extends Crud {
    
    protected $dataSourceName = 'setTipoAnotacao';
    protected $dataSourceName2 = 'tipoAnotacao_get';
    protected $dataSourceName3 = 'getTipoAnotacao';
    
    protected $parameterBagColumns = array('P_NRORG', 'P_DTMESCOMPETENC');
    public function __construct (Manager $dataSourceManager, ParameterBag $parameterBag, HCMParameters $hcmParameters, EntityManager $entityManager, TipoAnotacaoService $tipoanotacaoservice){
        $this->parameterBag           = $parameterBag;
        $this->hcmParameters          = $hcmParameters;
        $this->tipoanotacaoservice    = $tipoanotacaoservice;
        parent::__construct($dataSourceManager);
    }

    public function getTipoAnotacoes(DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceName2);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData);
        $response->addDataSet($dataSet);
    }
    
    public function getTipoAnotacoesAtivos(DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceName3);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData);
        $response->addDataSet($dataSet);
    }
    
    public function cadTipoAnotacao(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            foreach ($request->getDataSet()->getRows() as &$row) {
                $this->validarCampos($row);
                if (array_key_exists('NRTIPOANOTACAO', $row)) {
                if ($row['NRTIPOANOTACAO'] === '') { 
                    $row['NRTIPOANOTACAO'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgPadrao(), 'GPE_TIPOANOTACAO', 1, 12, true); 
                }
            } else {
                $row['NRTIPOANOTACAO'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgPadrao(), 'GPE_TIPOANOTACAO', 1, 12, true); 
            }
        }
         parent::save($request, $response);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function validarCampos($row){
        $row['NRORG']          = isset($row['NRORG'])          && !empty($row['NRORG']) ?          $row['NRORG'] :          $this->hcmParameters->getNrorgTrabalho();
        $row['IDATIVO']        = isset($row['IDATIVO'])        && !empty($row['IDATIVO']) ?        $row['IDATIVO'] : 'S';
        $row['DTINCLUSAO']     = isset($row['DTINCLUSAO'])     && !empty($row['DTINCLUSAO']) ?     $row['DTINCLUSAO'] :     $this->hcmParameters->getDataAtual(false);
        $row['DTULTATU']       = isset($row['DTULTATU'])       && !empty($row['DTULTATU']) ?       $row['DTULTATU'] :       $this->hcmParameters->getDataAtual(false);
        $row['NRORGINCLUSAO']  = isset($row['NRORGINCLUSAO'])  && !empty($row['NRORGINCLUSAO']) ?  $row['NRORGINCLUSAO'] :  $this->hcmParameters->getNrorgTrabalho();
        $row['NRORGULTATU']    = isset($row['NRORGULTATU'])    && !empty($row['NRORGULTATU']) ?    $row['NRORGULTATU'] :    $this->hcmParameters->getNrorgTrabalho();
        $row['CDOPERINCLUSAO'] = isset($row['CDOPERINCLUSAO']) && !empty($row['CDOPERINCLUSAO']) ? $row['CDOPERINCLUSAO'] : $this->hcmParameters->getCdOperador();
        $row['CDOPERULTATU']   = isset($row['CDOPERULTATU'])   && !empty($row['CDOPERULTATU']) ?   $row['CDOPERULTATU'] :   $this->hcmParameters->getCdOperador();

    }
    
    public function deleteTipoAnotacaoVinculo(DTO\Request\DataSet $request, DTO\Response $response){
        if($this->tipoanotacaoservice->validaDelete($request->getDataSet()->getRows()[0]['NRORG'],$request->getDataSet()->getRows()[0]['NRTIPOANOTACAO']))
            parent::delete($request, $response);
        else
	        $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message("Não foi possível excluir, registro em uso nas Anotações do Vínculo.", DTO\Response\Message::TYPE_ERROR));
            
    }
    

    protected function processConditionValue($condition) {
        return $condition['value'];
    }
    protected function populateParameterBag(FilterCriteria $filterCriteria) {

        $conditions = array();
        foreach ($filterCriteria->getConditions() as $condition) {
            $columnName = $condition['columnName'];
            if (in_array($columnName, $this->parameterBagColumns)) {
                $this->parameterBag->set($columnName, $this->processConditionValue($condition));
            } else {
                $conditions[] = $condition;
            }
        }
        $newFilterCriteria = new FilterCriteria(
            $filterCriteria->getDataSourceName(),
            $conditions,
            $filterCriteria->getPage(),
            $filterCriteria->getPageSize()
        );
        return $newFilterCriteria;
    }
} 