<?php

namespace HCM\Geral\Controllers;

use HCM\Util\Repositories;
use Doctrine\ORM\EntityManager;
use HCM\Geral\Services\VinculoServices;
use HCM\Geral\Services\AdmissaoService;
use HCM\Geral\Services\CrudService;
use HCM\Geral\Services\PessoaService;
use HCM\Geral\Services\EsocialService;
use HCM\Util\HCMParameters;
use Zeedhi\Framework\Controller\Crud;
use HCM\Util\DateUtil;
use HCM\Util\FPA;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request\Row;
use HCM\Geral\Services\AlteSituacaoFuncionalService;
use HCM\Geral\Services\MovimentacaoService;
use HCM\Geral\Services\AlteEscalaService;
use HCM\Geral\Services\AlteSalarioService;
use HCM\Geral\Services\AlteOcupacaoService;
use HCM\FPA\Services\FeriasService;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Request;
use HCM\Util\Util;
use Folha\Entities\FpaTipomoviment;
use HCM\Geral\Entity\GpeAltestabvincEntity;
use HCM\Geral\Services\EsoSimplificadoService;
use Setforms2\Shared\ORM\DateTime;

class VinculosController extends Crud {

    protected $dataSourceName = 'vinculom';
    protected $dataSourceNameGet = 'getVinculos';
    protected $dataSourceNameGet1 = 'getVinculoTodos';
    protected $dataSourceNameGet2 = 'gpe_vinculosAtivosCompetencia_get';
    protected $dataSourceNameGet3 = 'gpe_vinculosAtivosCalculadosVT_get';
    protected $dataSourceNameGetFicha = 'getFichaFinanceira';
    protected $dataSourceNameGet4 = 'gpe_vinculosAtivosCompetenciaComplementar_get';
    protected $dataSourceNameGet5 = 'gpe_vinculosAtivosComplementar_get';
    protected $dataSourceNameGet6 = 'gpe_loteVinculos_get';
    protected $dataSourceNameGetReciboPag = 'getVinculosReciboPagamento';
    protected $dataSourceConvocacaoTrabalhoIntermitente = 'convocacaoTrabalhoIntermitente';
    protected $parameterBagColumns = array('NRORG','P_NRORG_PADRAO','P_NRORG','P_DTMESCOMPETENC','P_RESCISAO','P_ULTIMODIAMES', 'P_VINCULOSNAOAPROV', 'P_NRESTRUTGEREN', 'P_CDRAIZCNPJEMPREG', 'P_NRESTRUTLEGALOPER', 'P_NRESTRUTGERENOPER');

    public function __construct (Manager $dataSourceManager,EntityManager $entityManager, ParameterBag $parameterBag, HCMParameters $hcmParameters, 
                                VinculoServices $vinculoServices, AdmissaoService $admissaoService, CrudService $crudService, PessoaService $pessoaService,
                                AlteSituacaoFuncionalService $altesituFunc, MovimentacaoService $movimentacaoService, ESocialService $eSocialService,
                                AlteEscalaService $alteEscalaService, AlteSalarioService $alteSalarioService, AlteOcupacaoService $alteOcupacaoService, EsoSimplificadoService $esoSimplificadoService, FeriasService $feriasService){
        
        $this->entityManager = $entityManager;
        $this->parameterBag = $parameterBag;
        parent::__construct($dataSourceManager);
        $this->hcmParameters = $hcmParameters;
        $this->vinculoServices = $vinculoServices;
        $this->admissaoService = $admissaoService;
		$this->crudService = $crudService;
		$this->pessoaService = $pessoaService;
        $this->alteSituFuncService = $altesituFunc;
        $this->movimentacaoService = $movimentacaoService;
        $this->eSocialService = $eSocialService;
        $this->alteEscalaService = $alteEscalaService;
        $this->alteSalarioService = $alteSalarioService;
        $this->alteOcupacaoService = $alteOcupacaoService;
        $this->esoSimplificadoService = $esoSimplificadoService;
        $this->feriasService = $feriasService;
    }

    public function buscarDados (DTO\Request\Filter $request, DTO\Response $response) {
        try{
            $filterData = $request->getFilterCriteria();
            $filterData->setDataSourceName($this->dataSourceName);
            $newFilterData = $this->populateParameterBag($filterData);
            $dataSet = $this->dataSourceManager->findBy($newFilterData); 
            $response->addDataSet($dataSet);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function getLoteDeVinculos(DTO\Request\Filter $request, DTO\Response $response)
    {
        try{
            $this->dataSourceNameGet = $this->dataSourceNameGet6;
            
            $filterData = $request->getFilterCriteria();
            $filterData->setDataSourceName($this->dataSourceNameGet);
            
            $filterData->addCondition('P_NRORG','=',$this->hcmParameters->getNrorgTrabalho());
            $filterData->addCondition('P_DTMESCOMPETENC','=',$this->hcmParameters->getDtCompetencia());
            $filterData->addCondition('P_CDOPERADORINCLUSAO','=',$this->hcmParameters->getCdoperador());
            $this->parameterBagColumns = ["P_NRORG", "P_DTMESCOMPETENC", "P_CDOPERADORINCLUSAO"];
            $newFilterData = $this->populateParameterBag($filterData);
            $dataSet = $this->dataSourceManager->findBy($newFilterData);

            $response->addDataSet($dataSet);
        
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function getFichaFinanceira (DTO\Request\Filter $request, DTO\Response $response) {	 
        try{	 
            $filters = $request->getFilterCriteria()->getConditions();	 
            $parameters = array();	 
            foreach ($filters as $position => $filter) {	 
                unset($filter['operator']);	 
                foreach ($filter as $key => $value) {	 
                    if ($key == 'columnName' && substr($value,0, 2) == 'P_') {	 
                        $parameters[$value] = $filter['value'];	 
                    }	 
                }	 
                	 
            }	 
            $parameters['P_NRTPMODALIDCAL'] = array_key_exists('P_NRTPMODALIDCAL', $parameters) ? $parameters['P_NRTPMODALIDCAL'] : 1;
            $parameters['P_DTMESCOMPETENC'] = array_key_exists('P_DTMESCOMPETENC', $parameters) ? $parameters['P_DTMESCOMPETENC'] : $this->hcmParameters->getDtCompetencia();
            Util::getValueInFilterDataSourceByName($filters, array('P_NRORG', 'P_NRVINCULOM', 'P_NRTPMODALIDCAL', 'P_DTMESCOMPETENC'), true);	 
            $conditions = Util::createQueryFiltered($filters);	 
            $retorno = $this->vinculoServices->getFichaFinanceira($parameters, $conditions);	 
            $response->addDataset(new \Zeedhi\Framework\DataSource\DataSet('getFichaFinanceira',$retorno));
        } catch(\Exception $e) {	 
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));	 
        }	 
    }

    public function getVinculos (DTO\Request\Filter $request, DTO\Response $response) {
        try{
            
            set_time_limit(0);
            
            
            $recisaofiltro = false;
            $filterData = $request->getFilterCriteria();
            //$filterData->setPageSize(99999);
            //A função acima ao ser descomentada removerá o limite de 9999 registros na tela, assim trazendo todos os vinculos buscados no banco
            //seria uma solução la para soluções porém o ideal é fazer paginação lá no front att:João Victor Martins Medeiros 25/04/2024 "Ta atoa ai né?"
            foreach ($filterData->getConditions() as $value) {
                if($value['columnName'] == "P_RESCISAO")
                    $recisaofiltro = true;
                if($value['columnName'] == "NRPISPASEPPES") {
                    $nrpispasValue = Util::removeOutrosCaracteresEspeciais($value['value']);
                    
                    $oldFilterDataProperties = Util::returnFilterDataProperties($filterData);
                    Util::getValueInFilterDataSourceByName($filterData, ["NRPISPASEPPES"], true);
                    $filterData = Util::createNewFilterData($oldFilterDataProperties[0], $filterData, $oldFilterDataProperties[2], $oldFilterDataProperties[3]);
                    $filterData->addCondition('NRPISPASEPPES', 'LIKE_I', $nrpispasValue);
                }
            }
            if(!$recisaofiltro)
                $filterData->addCondition('P_RESCISAO','=', 'A');
            $filterData->setDataSourceName($this->dataSourceNameGet);

            $filterData->addCondition('P_ULTIMODIAMES','=',$this->hcmParameters->getUltimoDiaMes());
            
            $filterData->addCondition('P_NRESTRUTLEGALOPER', '=', FPA::getNrestrutlegalOper());
            $filterData->addCondition('P_NRESTRUTGERENOPER', '=', FPA::getNrestrutgerenOper());

            $this->setFilterCriteriaDefaultOrderBy($filterData);
            $newFilterData = $this->populateParameterBag($filterData);
            $dataSet = $this->dataSourceManager->findBy($newFilterData);
            $response->addDataSet($dataSet);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function getVinculoTodos (DTO\Request\Filter $request, DTO\Response $response) {
        try{
            $recisaofiltro = false;
            $filterData = $request->getFilterCriteria();
            foreach ($filterData->getConditions() as $value) {
                if($value['columnName'] == "P_RESCISAO")
                    $recisaofiltro = true;
            }
            if(!$recisaofiltro)
                $filterData->addCondition('P_RESCISAO','=', 'A');
            $filterData->setDataSourceName($this->dataSourceNameGet1);

            $filterData->addCondition('P_ULTIMODIAMES','=',$this->hcmParameters->getUltimoDiaMes());
            $newFilterData = $this->populateParameterBag($filterData);
            $dataSet = $this->dataSourceManager->findBy($newFilterData); 
            $response->addDataSet($dataSet);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function getVinculosMatriculaEsocial(DTO\Request\Filter $request, DTO\Response $response){
        try{	 
            $filters = $request->getFilterCriteria()->getConditions();	 
            $parameters = array();	 
            foreach ($filters as $position => $filter) {	 
                unset($filter['operator']);	 
                foreach ($filter as $key => $value) {	 
                    if ($key == 'columnName' && substr($value,0, 2) == 'P_') {	 
                        $parameters[$value] = $filter['value'];	 
                    }	 
                }	 
                	 
            }	 
            
            $parameters['P_DTMESCOMPETENC'] = array_key_exists('P_DTMESCOMPETENC', $parameters) ? $parameters['P_DTMESCOMPETENC'] : $this->hcmParameters->getDtCompetencia();	
            $parameters['P_NRESTRUTURAM'] = array_key_exists('P_NRESTRUTURAM', $parameters) ? $parameters['P_NRESTRUTURAM'] : null;
            $parameters['P_CDRAIZCNPJEMPREG'] = array_key_exists('P_CDRAIZCNPJEMPREG', $parameters) ? $parameters['P_CDRAIZCNPJEMPREG'] : null;
            Util::getValueInFilterDataSourceByName($filters, array('P_NRORG', 'P_NRESTRUTURAM', 'P_DTMESCOMPETENC', 'P_CDRAIZCNPJEMPREG'), true);	 
            $conditions = Util::createQueryFiltered($filters);	 
            $retorno = $this->vinculoServices->getVinculosMatriculaEsocial($parameters, $conditions);	 
            $response->addDataset(new \Zeedhi\Framework\DataSource\DataSet('getVinculosMatriculaEsocial',$retorno));	 
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        } 
    }
    
    public function verificaExistenciaLancamentoFuturos(DTO\Request\DataSet $request, DTO\Response $response){
        try{
            $msg = null;
            $row = $request->getDataSet()->getRows()[0];
            $msg = $this->vinculoServices->verificaExistenciaLancamentoFuturos($row);
            
            if(!empty($msg)){
                 $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message($msg, DTO\Response\Message::TYPE_ERROR));
            }
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function getVinculoAtivosRescindidos (DTO\Request\Filter $request, DTO\Response $response) {
        try{
            $filterData = $request->getFilterCriteria();

            $filterData->addCondition('P_NRORG','=', $this->hcmParameters->getNrorgTrabalho());
            $filterData->addCondition('P_DTMESCOMPETENC','=', $this->hcmParameters->getDtCompetencia());
            $filterData->addCondition('P_NRORG_PADRAO','=', $this->hcmParameters->getNrorgPadrao());
            $filterData->addCondition('P_RESCISAO','=', 'T');


            $filterData->setDataSourceName($this->dataSourceNameGet);
            $this->setFilterCriteriaDefaultOrderBy($filterData);
            $filterData->addCondition('P_ULTIMODIAMES','=',$this->hcmParameters->getUltimoDiaMes());
            $newFilterData = $this->populateParameterBag($filterData);
            $dataSet = $this->dataSourceManager->findBy($newFilterData); 
            $response->addDataSet($dataSet);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function getVinculosReciboPagamento (DTO\Request\Filter $request, DTO\Response $response) {
        try{
            $recisaofiltro = false;
            $filterData = $request->getFilterCriteria();
            foreach ($filterData->getConditions() as $value) {
                if($value['columnName'] == "P_RESCISAO")
                    $recisaofiltro = true;
            }
            if(!$recisaofiltro)
                $filterData->addCondition('P_RESCISAO','=', 'A');
            $filterData->setDataSourceName($this->dataSourceNameGetReciboPag);
           
            $filterData->addCondition('P_ULTIMODIAMES','=',$this->hcmParameters->getUltimoDiaMes());
            $newFilterData = $this->populateParameterBag($filterData);
            $dataSet = $this->dataSourceManager->findBy($newFilterData); 
            $response->addDataSet($dataSet);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    protected function IsNullDadosReintegracao($row) {
        if (isset($row['IDTPREINTEGRACAO']) && isset($row['DTRETORNO']) && isset($row['DTEFEITOFINANCEIRO'])) {
            return false;
        }
        
        return true;
    }

    public function existeEnvioRescisaoEsocial(DTO\Request\DataSet $request, DTO\Response $response)
    {
        try {
            
            set_time_limit(0);
            
            foreach ($request->getDataSet()->getRows() as &$row) {
                $existeEnvioRescisaoEsocial = $this->esoSimplificadoService->existeEnvioRescisaoEsocial($row);
                if($existeEnvioRescisaoEsocial && $this->IsNullDadosReintegracao($row)) {
                    $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message('Não é possível alterar os dados da rescisão, pois já existe um envio referente á rescisão no ESocial.', DTO\Response\Message::TYPE_ERROR));;
                    return;
                }
                
                return false;
            }
        } catch (\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
     
    public function validaRescisao($request, $response) {
		try {
            foreach ($request->getDataSet()->getRows() as &$row) {
                $dtrescisao = $row['DTRESCISAOVINC'];
                $nrtpdemissao = $row['NRTPDEMISSAO'];
                $nrvinculom = $row['NRVINCULOM'];
                $nrpessoa = $row['NRPESSOA'];
                $nrorg = $row['NRORG'];
                $dtcompetencia = $row['DTMESCOMPETENC'];
                $cdmatriculaesocial = $row['CDMATRICULAESOCIAL'];
                $nrestrutlegal = $row['NRESTRUTLEGAL'];
                $cdmatriculaesocialante = array_key_exists('CDMATRICULAESOCIALANTE', $row) ? $row['CDMATRICULAESOCIALANTE'] : -1;
                $idtpreintegracao = $row['IDTPREINTEGRACAO'] ?? null;
		    }
		    
		    // Bloqueio rescisão
		    if ($dtrescisao) {
    		    $mesmoPeriodoAfastamento = $this->alteSituFuncService->mesmoPeriodoAfastamento($dtcompetencia, $nrvinculom, $dtrescisao);
    		    
    		    if ($mesmoPeriodoAfastamento) {
    		        return $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message($row['NMVINCULOM'].' possui período de afastamento nessa data de rescisão', DTO\Response\Message::TYPE_ERROR));
    		    }
    		    
                $periodoGozoFerias = $this->entityManager->getRepository(Repositories::FPA_FERIAS)->getPeriodoGozoFerias(
                    $nrvinculom,
                    DateUtil::getDataDeString( $dtrescisao, DateUtil::FORMATO_BRASILEIRO, true ), 
                    DateUtil::getDataDeString( $dtrescisao, DateUtil::FORMATO_BRASILEIRO, true ),
                    $this->hcmParameters->getNrorgTrabalho()
                );
                

                if ($periodoGozoFerias) {
                    $mesmoPeriodo = implode(array_reverse(explode('/', $dtrescisao))) >= implode(array_reverse(explode('/', substr($periodoGozoFerias['DTINIGOZOFERIAS'], 0,10)))) && implode(array_reverse(explode('/', $dtrescisao))) <= implode(array_reverse(explode('/', substr($periodoGozoFerias['DTFIMGOZOFERIAS'], 0,10))));
                    
                    if ( $mesmoPeriodo ) {
                        return $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message($row['NMVINCULOM'].' possui período de férias nessa data de rescisão', DTO\Response\Message::TYPE_ERROR));
                    }
                }
		    }
		    // End
		    
		    $situacao = $this->vinculoServices->verificaSituaçãoVinculoRescisao($nrorg,$nrpessoa, $dtrescisao);
		    $estab_SituFunc = $this->alteSituFuncService->verificaEstabilidade($nrorg, $dtcompetencia, $nrvinculom, $dtrescisao);
		    $alte_estab = new GpeAltestabvincEntity($this->entityManager, $this->hcmParameters);
            $estab_AlteEsta = $alte_estab->verificaEstabilidadeCalculo($nrorg, $nrvinculom, $dtrescisao);
            $estabilidade = $estab_SituFunc || $estab_AlteEsta;
            

		    $dissidio = $this->vinculoServices->verificaDissidio($nrorg, $nrvinculom, $dtcompetencia, $dtrescisao,$nrtpdemissao);
		    $matriculaesocial = $this->vinculoServices->verificaMatriculaESocial($nrorg, $nrvinculom, $cdmatriculaesocial, $dtcompetencia, $nrestrutlegal, $cdmatriculaesocialante);
		    
		    if($matriculaesocial['type'] == 'ALERT'){
		        $row['CDMATRICULAESOCIAL'] = $cdmatriculaesocialante;
		        $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message($matriculaesocial['message'], DTO\Response\Message::TYPE_ERROR));
		    }
		    
		    $retorno = array($situacao,$estabilidade,$dissidio);
		    
            if(($matriculaesocial['type'] === 'SUCESS' || $matriculaesocial['type'] === 'ALERT')){
		        if($situacao || $estabilidade || $dissidio){
		            $response->addDataset(new \Zeedhi\Framework\DataSource\DataSet('validacoes',$retorno));
		        }
		        
		        /*$dadosReintegracaoMudaram = $this->dadosReintegracaoMudaram($request, $response);*/
		        
		        $this->setVinculo($request, $response);
		        
		        /*if(!empty($idtpreintegracao) && !empty($dtrescisao) && !empty($nrtpdemissao)) {
		            $this->setReintegracao($request, $response, $dadosReintegracaoMudaram);
		        }*/
		        
		        if(array_key_exists('IDENVIAESOCIAL', $row) && $row['IDENVIAESOCIAL'] == 'S'){
	                $eventos = $row['NMEVENTO'];
		            $retornoEsocial = array();
		            foreach($eventos as $evento){
		                $row['NMEVENTO'] = $evento;
        		        $this->eSocialService->envioEsocialDescentralizado($row, $response);
        		        
        		        $dataSets = $response->getDataSets();
        		        $dataSetsResponse = array();
                        foreach($dataSets as $dataSet){
                            if($dataSet->getDataSourceName() == 'retornoEsocial'){
                                $retornoEsocial = $dataSet->getRows();
                            }else{
                                array_push($dataSetsResponse, $dataSet);        
                            }
                        }
                        $response->setDataSets($dataSetsResponse);
		           }
		           $response->addDataSet((new \Zeedhi\Framework\DataSource\DataSet('retornoEsocial', $retornoEsocial)));
    		    }
		    }else if($matriculaesocial['type'] == 'ERROR' || ($situacao || $estabilidade || $dissidio)){
		        if($matriculaesocial['type'] == 'ERROR'){
		            $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message($matriculaesocial['message'], DTO\Response\Message::TYPE_ERROR));
		        }
			    if($situacao || $estabilidade || $dissidio){
		           $response->addDataset(new \Zeedhi\Framework\DataSource\DataSet('validacoes',$retorno));
		        }
		    }
		    
		} catch (\Exception $e) {
		    if ($e->getCode() === 10) {
                $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_ERROR));
            }
			else {
                $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
 	        }
		}
	}
	
	public function validaRescisaoComBeneficio($request, $response) {
		try {
            foreach ($request->getDataSet()->getRows() as &$row) {
                $dtrescisao = $row['DTRESCISAOVINC'];
                $nrtpdemissao = $row['NRTPDEMISSAO'];
                $nrvinculom = $row['NRVINCULOM'];
                $nrpessoa = $row['NRPESSOA'];
                $nrorg = $row['NRORG'];
                $dtcompetencia = $row['DTMESCOMPETENC'];
                $cdmatriculaesocial = $row['CDMATRICULAESOCIAL'];
                $nrestrutlegal = $row['NRESTRUTLEGAL'];
                $cdmatriculaesocialante = array_key_exists('CDMATRICULAESOCIALANTE', $row) ? $row['CDMATRICULAESOCIALANTE'] : -1;
		    }
		    
		    $situacao = $this->vinculoServices->verificaSituaçãoVinculoRescisao($nrorg,$nrpessoa, $dtrescisao);
		    $estab_SituFunc = $this->alteSituFuncService->verificaEstabilidade($nrorg, $dtcompetencia, $nrvinculom, $dtrescisao);
		    $alte_estab = new GpeAltestabvincEntity($this->entityManager, $this->hcmParameters);
            $estab_AlteEsta = $alte_estab->verificaEstabilidadeCalculo($nrorg, $nrvinculom, $dtrescisao);
            $estabilidade = $estab_SituFunc || $estab_AlteEsta;
		    
		    $dissidio = $this->vinculoServices->verificaDissidio($nrorg, $nrvinculom, $dtcompetencia, $dtrescisao,$nrtpdemissao);
		    $matriculaesocial = $this->vinculoServices->verificaMatriculaESocial($nrorg, $nrvinculom, $cdmatriculaesocial, $dtcompetencia, $nrestrutlegal, $cdmatriculaesocialante);
		    
		    if($matriculaesocial['type'] == 'ALERT'){
		        $row['CDMATRICULAESOCIAL'] = $cdmatriculaesocialante;
		        $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message($matriculaesocial['message'], DTO\Response\Message::TYPE_ERROR));
		    }
		    
		    $retorno = array($situacao,$estabilidade,$dissidio);
		    
            if(($matriculaesocial['type'] === 'SUCESS' || $matriculaesocial['type'] === 'ALERT')){
		        if($situacao || $estabilidade || $dissidio){
		            $response->addDataset(new \Zeedhi\Framework\DataSource\DataSet('validacoes',$retorno));
		        } 
		        
		        //$dadosReintegracaoMudaram = $this->dadosReintegracaoMudaram($request, $response);
		        
		        $this->setVinculoComBeneficio($request, $response);
		        
		        /*if(!empty($idtpreintegracao) && !empty($dtrescisao) && !empty($nrtpdemissao)) {
		            $this->setReintegracao($request, $response, $dadosReintegracaoMudaram);
		        }*/
		        
		        if(array_key_exists('IDENVIAESOCIAL', $row) && $row['IDENVIAESOCIAL'] == 'S'){
	                $eventos = $row['NMEVENTO'];
		            $retornoEsocial = array();
		            foreach($eventos as $evento){
		                $row['NMEVENTO'] = $evento;
        		        $this->eSocialService->envioEsocialDescentralizado($row, $response);
        		        
        		        $dataSets = $response->getDataSets();
        		        $dataSetsResponse = array();
                        foreach($dataSets as $dataSet){
                            if($dataSet->getDataSourceName() == 'retornoEsocial'){
                                $retornoEsocial[$row['NMEVENTO']] = $dataSet->getRows()[0];
                            }else{
                                array_push($dataSetsResponse, $dataSet);        
                            }
                        }
                        $response->setDataSets($dataSetsResponse);
		           }
		           $response->addDataSet((new \Zeedhi\Framework\DataSource\DataSet('retornoEsocial', $retornoEsocial)));
    		    }
		    }else if($matriculaesocial['type'] == 'ERROR' || ($situacao || $estabilidade || $dissidio)){
		        if($matriculaesocial['type'] == 'ERROR'){
		            $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message($matriculaesocial['message'], DTO\Response\Message::TYPE_ERROR));
		        }
			    if($situacao || $estabilidade || $dissidio){
		           $response->addDataset(new \Zeedhi\Framework\DataSource\DataSet('validacoes',$retorno));
		        }
		    }
		    
		} catch (\Exception $e) {
		    if ($e->getCode() === 10) {
                $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_ERROR));
            }
			else {
                $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
 	        }
		}
	}

    public function associarSindicato($request, $response) {
		try {
		    $retorno = [];
		    $idSindicato = $request->getDataSet()->getRows()[0][count($request->getDataSet()->getRows()[0])-1]['associarSindicato'];
            foreach ($request->getDataSet()->getRows() as &$rows) {
                $arrayLength = count($rows);
                foreach($rows as $key=>$row){
                    if($key<$arrayLength-1){
                        $row['IDASSOCSINDICATO'] = $idSindicato;
                        $row['saveHistoryTable'] = [
                                                        "__table_competenc"    =>"DTMESCOMPETENC",
                                                        "__table_history_name" =>"gpe_vinculoh",
                                                        "__table_history_pk"   =>"NRVINCULOH",
                                                        "__table_master_pk"    =>"NRVINCULOM",
                                                   ];
                        
                        $row["__columns_to_return"] = ["NRVINCULOM", "NRVINCULOH"];                                                   
                        
            		    $newDataSet = new \Zeedhi\Framework\DataSource\DataSet($request->getDataSet()->getDataSourceName(), [0=>$row]);
        	        	$newRequest = new \Zeedhi\Framework\DTO\Request\DataSet($newDataSet, $request->getMethod(), $request->getRoutePath(), $request->getUserId());
        		        $this->setVinculo($newRequest, $response);
                    }
                }
            }

		} catch (\Exception $e) {
		    if ($e->getCode() === 10) {
                $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_ERROR));
            }
			else {
                $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
 	        }
		}
	}
    
    protected function atualizarOutrasDatas($nrorg, $nrvinculom, $dtadmissaoanterior, $dtadmissao) {
        try {
            $this->entityManager->beginTransaction();
            
            
            $this->feriasService->alteraPeriodoFerias($nrorg, $nrvinculom, $dtadmissaoanterior, $dtadmissao);
            $this->movimentacaoService->alteraDataIniMovimentacao($nrorg, $nrvinculom, $dtadmissaoanterior, $dtadmissao);
            $this->alteEscalaService->alteraDataIniEscala($nrorg, $nrvinculom, $dtadmissaoanterior, $dtadmissao);
            $this->alteSituFuncService->alteraDataIniSituFunc($nrorg, $nrvinculom, $dtadmissaoanterior, $dtadmissao);
            $this->alteSalarioService->alteraDataAlteSalario($nrorg, $nrvinculom, $dtadmissaoanterior, $dtadmissao);
            $this->alteOcupacaoService->alteraDataIniOcupacao($nrorg, $nrvinculom, $dtadmissaoanterior, $dtadmissao);
            
            $this->entityManager->flush();
            $this->entityManager->commit();
            
        } catch (\Exception $e) {
            $this->entityManager->rollback();
            throw $e;
        }
            
    }

    function novoHistVinculoh($nrorg, $nrvinculom, $dtadmissaoHistObj, $dtbaseferias, $dtfimcontrdetermin) {
        try {
            $vinculohDtAdm = $this->entityManager->getRepository(Repositories::GPE_VINCULOH)->retornaHistoricoCompetencia($nrorg, $nrvinculom, $dtadmissaoHistObj);
            $dtbaseferias  = DateUtil::getDataDeString($dtbaseferias, DateUtil::FORMATO_BRASILEIRO, true);
            $dtfimcontrdetermin = DateUtil::getDataDeString($dtfimcontrdetermin, DateUtil::FORMATO_BRASILEIRO, true);

            if(!$vinculohDtAdm) {
                $vinculohComp  = $this->entityManager->getRepository(Repositories::GPE_VINCULOH)->retornaUltimoHistoricoCompetencia($nrorg, $nrvinculom, $this->hcmParameters->getDtCompetencia());
                $vinculohDtAdm = clone $vinculohComp;
                
                $vinculohDtAdm->setDtmescompetenc($dtadmissaoHistObj);
                $vinculohDtAdm->setDtbaseferias($dtbaseferias);
                $vinculohDtAdm->setDtfimcontrdetermin($dtfimcontrdetermin);
                $vinculohDtAdm->setNrvinculoh(\HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'GPE_VINCULOH', 1, 12, true));

                $vinculohDtAdm->setDtinclusao(DateUtil::getDataAtual());
                $vinculohDtAdm->setCdoperinclusao($this->hcmParameters->getCdoperador());
                $vinculohDtAdm->setNrorginclusao($this->hcmParameters->getNrorgTrabalho());
                $vinculohDtAdm->setDtultatu(DateUtil::getDataAtual());
                $vinculohDtAdm->setCdoperultatu($this->hcmParameters->getCdoperador());
                $vinculohDtAdm->setNrorgultatu($this->hcmParameters->getNrorgTrabalho());

                $this->entityManager->persist($vinculohDtAdm);
                $this->entityManager->flush($vinculohDtAdm);
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }

    function novoHistPessoah($nrorg, $nrpessoa, $dtadmissaoHistObj) {
        try {
            $pessoahDtAdm = $this->entityManager->getRepository(Repositories::GPE_PESSOAH)->findOneBy(['nrpessoa' => $nrpessoa, 'nrorg' => $nrorg, 'dtmescompetenc' => $dtadmissaoHistObj]);

            if(!$pessoahDtAdm) {

                $pessoahComp  = $this->entityManager->getRepository(Repositories::GPE_PESSOAH)->retornaHistoricoAtualPorPessoa($nrorg, $nrpessoa, $this->hcmParameters->getDtCompetencia());
                $pessoahDtAdm = clone $pessoahComp;
                
                $pessoahDtAdm->setDtmescompetenc($dtadmissaoHistObj);
                $pessoahDtAdm->setNrpessoah(\HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'GPE_PESSOAH', 1, 12, true));

                $pessoahDtAdm->setDtinclusao(DateUtil::getDataAtual());
                $pessoahDtAdm->setCdoperinclusao($this->hcmParameters->getCdoperador());
                $pessoahDtAdm->setNrorginclusao($this->hcmParameters->getNrorgTrabalho());
                $pessoahDtAdm->setDtultatu(DateUtil::getDataAtual());
                $pessoahDtAdm->setCdoperultatu($this->hcmParameters->getCdoperador());
                $pessoahDtAdm->setNrorgultatu($this->hcmParameters->getNrorgTrabalho());

                $this->entityManager->persist($pessoahDtAdm);
                $this->entityManager->flush($pessoahDtAdm);
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function setVinculo($request, $response) {
		try {
            foreach ($request->getDataSet()->getRows() as &$row) {
                $dtrescisao = $row['DTRESCISAOVINC'];
                $nrvinculom = $row['NRVINCULOM'];
                $nrpessoa = $row['NRPESSOA'];
                $nrorg = $row['NRORG'];
                $nrEstruturaLegal = $row['NRESTRUTLEGAL'];
    		    $nrEstruturaGerencial = $row['NRESTRUTGEREN'];
    		    $nrtpdemissao = $row['NRTPDEMISSAO'];
                $dtadmissao = $row['DTADMISSAOVINC'];
                $dtreincid = $row['DTRETORNO'];
                $dtbaseferias = $row['DTBASEFERIAS'];
                $dtfimcontrdetermin = $row['DTFIMCONTRDETERMIN'];
                $dtadmissaoanterior = array_key_exists('DTADMISSAOVINCANTERIOR', $row) ? $row['DTADMISSAOVINCANTERIOR'] : null;
                $dtadmissaoalterada = array_key_exists('DTADMISSAOALTERADA', $row) ? $row['DTADMISSAOALTERADA'] : null;
	        }
	        
	        $mensagem = null;
            $dtcompetencia = $this->hcmParameters->getDtCompetencia();
		    $gpevinculom = $this->entityManager->getRepository(Repositories::GPE_VINCULOM)->findOneBy(array("nrvinculom" => $nrvinculom,"nrorg" => $nrorg));
            if($gpevinculom->getDtretorno() && $dtreincid){
                $acao = "ALTERA";
                $this->movimentacaoService->manipulaMovimentacao($nrorg, $nrvinculom, $nrEstruturaLegal, null, $dtreincid, $dtcompetencia, $acao, 25);
            }else if(!$gpevinculom->getDtretorno() && $dtreincid){
                $acao = "INSERE";
                $this->movimentacaoService->manipulaMovimentacao($nrorg, $nrvinculom, $nrEstruturaLegal, null, $dtreincid, $dtcompetencia, $acao, 25);
            }else if($gpevinculom->getDtretorno() && !$dtreincid){
                $acao = "ALTERA";
                $this->movimentacaoService->manipulaMovimentacao($nrorg, $nrvinculom, $nrEstruturaLegal, null, $dtreincid, $dtcompetencia, $acao, 25);
            }
            
            if($gpevinculom->getDtrescisaovinc() && $dtrescisao){
                $acao = "ALTERA";
                $this->movimentacaoService->manipulaMovimentacao($nrorg, $nrvinculom, $nrEstruturaLegal, null, $dtrescisao, $dtcompetencia, $acao, $nrtpdemissao);
            }else if(!$gpevinculom->getDtrescisaovinc() && $dtrescisao){
                $acao = "INSERE";
                $this->movimentacaoService->manipulaMovimentacao($nrorg, $nrvinculom, $nrEstruturaLegal, null, $dtrescisao, $dtcompetencia, $acao, $nrtpdemissao);
            }else if($gpevinculom->getDtrescisaovinc() && !$dtrescisao){
                $acao = "ALTERA";
                $this->movimentacaoService->manipulaMovimentacao($nrorg, $nrvinculom, $nrEstruturaLegal, null, $dtrescisao, $dtcompetencia, $acao, $nrtpdemissao);
            }
            $acao = "ALTERA";
            if(!$gpevinculom->getDtrescisaovinc() && !$dtrescisao){
                $acao = null;
            }
            
            if(!empty($dtrescisao)){
                //Se existir data de recisão adicione mais um dia na data. 
                $dtReferenciaRescisao = clone DateUtil::getDataDeString($dtrescisao, DateUtil::FORMATO_BRASILEIRO, true);
                $this->alteEscalaService->alteraDataFimUltimaAlteracao($nrorg, $nrvinculom, $dtReferenciaRescisao->add(new \DateInterval('P1D')));
            } else {
                $this->alteEscalaService->alteraDataFimUltimaAlteracao($nrorg, $nrvinculom, $dtrescisao);
            }
            
            $nr = $this->alteSituFuncService->manipulaSituacaoFuncional($dtrescisao, $nrorg, $nrvinculom, $dtadmissao, $acao, null, 1, $gpevinculom->getDtrescisaovinc(), $nrtpdemissao);//vai retornar o nr e poe na row
			
			if($dtadmissaoalterada == true){
			    $this->atualizarOutrasDatas($nrorg, $nrvinculom, $dtadmissaoanterior, $dtadmissao);

                $data = explode("/", $dtadmissao);
                list($dia, $mes, $ano) = $data;

                $dtadmissaoHist = "$ano-$mes-01";
                $dtadmissaoHistObj = new DateTime($dtadmissaoHist);

                $this->novoHistVinculoh($nrorg, $nrvinculom, $dtadmissaoHistObj, $dtbaseferias, $dtfimcontrdetermin); 
                $this->novoHistPessoah($nrorg, $nrpessoa, $dtadmissaoHistObj);
			}
			
			$this->dataSourceName = 'gpe_vinculom';
			$this->dataSourceNameToSave = $this->crudService->getDataSourceNameToSaveByName($this->dataSourceName);
			$this->dataSourceName = $this->dataSourceNameToSave ? $this->dataSourceNameToSave : $this->dataSourceName;
			$rows = $this->crudService->getRowsToSave($request, $this->dataSourceName);
			
			$this->crudService->save($this->dataSourceName, $rows);
			
			$request = $this->crudService->buildDataSetIfRow($request, $this->dataSourceName);
			
			$currentRow = current($request->getDataSet()->getRows());
			
			if($nr){
			    $currentRow['NRSITUFUNCM'] = $nr;
			} else {
			    $ultsitufunc = $this->alteSituFuncService->retornaUltimaSituacaoPorDatainicio($nrorg, $nrvinculom, $this->hcmParameters->getUltimoDiaMes());
			    if(is_object($ultsitufunc)){
			        $currentRow['NRSITUFUNCM'] = $ultsitufunc->getNrsitufuncm(); //pega a ultima situação antes do demitido
			    }else{
			        throw new \Exception('Alteração de situação funcional não encontrada para o vínculo', 10);
			    }
			}
            
			if(isset($currentRow['saveHistoryTable'])) {
                if (!$currentRow['__is_new']) {
                    if (!$this->checkHistory($response, $currentRow)) {
                        // is new history
                        $currentRow['__is_new'] = true;
                    }
                }
                
                if($row['NRVINCULOM']){
    		        $nrorg = $row['NRORG'];
        		    $nrvinculom = $row['NRVINCULOM'];
        		    $competencia = $row['DTMESCOMPETENC'];
        		    $competencia = DateUtil::getDataDeString( substr($competencia, 0, 10) );
        		    $competencia = DateUtil::truncateData( $competencia );
        		    
        		    $vinculo = $this->entityManager->getRepository(Repositories::GPE_VINCULOH)->findOneBy(array('nrvinculom' => $nrvinculom, 'nrorg' => $nrorg, 'dtmescompetenc' => $competencia ));
    		        
    		        if($vinculo && $vinculo->getNrvinculoh() != $row['NRVINCULOH']){
    		            $mensagem = "Operação bloqueada! Favor atualizar os dados da tela antes de prosseguir com a respectiva atualização.";
    		        }
		        }
                
                if( !$mensagem ){
    				$tableMasterPK = strtoupper($currentRow['saveHistoryTable']['__table_master_pk']);
    				$newCode = array_filter($this->crudService->getColumnsToReturn(), function ($element) use ($tableMasterPK) {
    					return $element[$tableMasterPK];
    				});
    
    				$this->dataSourceNameToSave = $this->crudService->getDataSourceNameToSaveByName($currentRow['saveHistoryTable']['__table_history_name']);
    				$this->dataSourceName = $this->dataSourceNameToSave ? $this->dataSourceNameToSave : $this->dataSourceName;
    				$currentRow[strtoupper($currentRow['saveHistoryTable']['__table_master_pk'])] = intval($newCode[0][strtoupper($currentRow['saveHistoryTable']['__table_master_pk'])]);
                    
                    // set history properties
                    $currentRow['__new_code_pk'] = $currentRow['saveHistoryTable']['__table_history_name'];
                    $currentRow['__table_pk'] = $currentRow['saveHistoryTable']['__table_history_pk'];
                    
                    
    	        	$newDataSet = new DataSet($this->dataSourceName, array($currentRow));
    	        	$newRequest = new \Zeedhi\Framework\DTO\Request\DataSet($newDataSet,
    	            $request->getMethod(), $request->getRoutePath(), $request->getUserId());
    				$rows = $this->crudService->getRowsToSave($newRequest, $this->dataSourceName);
    				
    				$this->crudService->save($this->dataSourceName, $rows);
                } else {
                    $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message($mensagem, DTO\Response\Message::TYPE_ERROR));
                }
            }
			$response->addDataset(new DataSet('columnsToReturn', $this->crudService->getColumnsToReturn()));
			
		} catch (\Exception $e) {
		    if ($e->getCode() === 10) {
                $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_ERROR));
            }
			else {
                $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
 	        }
		}
	}
	
	public function verificaExistenciaVinculoNovo(DTO\Request\DataSet $request, DTO\Response $response) {
	    try{
            $msg = null;
            $row = $request->getDataSet()->getRows()[0];
            $msg = $this->vinculoServices->verificaExistenciaVinculoNovo($row);

            if(!empty($msg)){
                 $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message($msg, DTO\Response\Message::TYPE_ERROR));
            }
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
	}
    
    public function setReintegracao(DTO\Request\DataSet $request, DTO\Response $response) {
		try {
            foreach ($request->getDataSet()->getRows() as &$row) {
                $nrvinculom = $row['NRVINCULOM'];
                $nrorg = $row['NRORG'];
                $dtretorno = $row['DTRETORNO'];
                $dtrescisao = $row['DTRESCISAOVINC'];
                $cdmatricula = $row['CDMATRICULA'];
                $dtadmissao = $row['DTADMISSAOVINC'];
                $dtefeitofinanceiro = $row['DTEFEITOFINANCEIRO'];
	        }
	        $nrVinculomNovoVinculo = null;
	        $nrVinculomNovoVinculo = $this->vinculoServices->setReplicarVinculo($nrvinculom, $nrorg, $dtrescisao, $dtadmissao, $dtefeitofinanceiro, $dtretorno);
	        $dtMesCompetencToReturn = DateUtil::setDay($dtefeitofinanceiro, 1)->format('d/m/Y');
		    $response->addDataset(new DataSet('vinculoReintegrado', [0 => "O vínculo foi reintegrado na competência {$dtMesCompetencToReturn}, o número do novo vínculo é {$nrVinculomNovoVinculo}."]));
	        
		} catch (\Exception $e) {
		    if ($e->getCode() === 10) {
                $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_ERROR));
            }
			else {
                $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
 	        }
		}
	}
	
	public function dadosReintegracaoMudaram($request, $response) {
		try {
            foreach ($request->getDataSet()->getRows() as &$row) {
                $nrvinculom = $row['NRVINCULOM'];
                $nrorg = $row['NRORG'];
                $idtpreintegracao = $row['IDTPREINTEGRACAO'] ?? null;
                $nrprocjud = $row['NRPROCJUD'] ?? null;
                $nrleianistia = $row['NRLEIANISTIA'] ?? null;
                $idpagtojuizo = $row['IDPAGTOJUIZO'] ?? null;
                $dtretorno = $row['DTRETORNO'] ?? null;
                $dtefeitofinanceiro = $row['DTEFEITOFINANCEIRO'] ?? null;
                $obsreintegracao = $row['OBSREINTEGRACAO'] ?? null;
	        }
	        $vinculoAtual = $this->entityManager->getRepository(Repositories::GPE_VINCULOM)->findOneBy(array("nrvinculom" => $nrvinculom, "nrorg" => $nrorg));
	        
	        $paraRetornar = [];
	        
	        if($dtretorno != $vinculoAtual->getDtretorno()) $paraRetornar['DTRETORNO'] = $vinculoAtual->getDtretorno();
	        if($idtpreintegracao != $vinculoAtual->getIdtpreintegracao()) $paraRetornar['IDTPREINTEGRACAO'] = $vinculoAtual->getIdtpreintegracao();
	        if($nrprocjud != $vinculoAtual->getNrprocjud()) $paraRetornar['NRPROCJUD'] = $vinculoAtual->getNrprocjud();
	        if($nrleianistia != $vinculoAtual->getNrleianistia()) $paraRetornar['NRLEIANISTIA'] = $vinculoAtual->getNrleianistia();
	        if($idpagtojuizo != $vinculoAtual->getIdpagtojuizo()) $paraRetornar['IDPAGTOJUIZO'] = $vinculoAtual->getIdpagtojuizo();
	        if($dtefeitofinanceiro != $vinculoAtual->getDtefeitofinanceiro()) $paraRetornar['DTEFEITOFINANCEIRO'] = $vinculoAtual->getDtefeitofinanceiro();
	        if($obsreintegracao != $vinculoAtual->getObsreintegracao()) $paraRetornar['OBSREINTEGRACAO'] = $vinculoAtual->getObsreintegracao();
            
            return $paraRetornar;
			
		} catch (\Exception $e) {
		    if ($e->getCode() === 10) {
                $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_ERROR));
            }
			else {
                $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
 	        }
		}
	}
	
	public function setVinculoComBeneficio($request, $response) {
		try {
            foreach ($request->getDataSet()->getRows() as &$row) {
                $dtrescisao = $row['DTRESCISAOVINC'];
                $nrvinculom = $row['NRVINCULOM'];
                $nrorg = $row['NRORG'];
                $nrEstruturaLegal = $row['NRESTRUTLEGAL'];
    		    $nrEstruturaGerencial = $row['NRESTRUTGEREN'];
    		    $nrtpdemissao = $row['NRTPDEMISSAO'];
                $dtadmissao = $row['DTADMISSAOVINC'];
                $dtreincid = $row['DTRETORNO'];
                $dtadmissaoanterior = array_key_exists('DTADMISSAOVINCANTERIOR', $row) ? $row['DTADMISSAOVINCANTERIOR'] : null;
                $dtadmissaoalterada = array_key_exists('DTADMISSAOALTERADA', $row) ? $row['DTADMISSAOALTERADA'] : null;

	        }
            $dtcompetencia = $this->hcmParameters->getDtCompetencia();
		    $gpevinculom = $this->entityManager->getRepository(Repositories::GPE_VINCULOM)->findOneBy(array("nrvinculom" => $nrvinculom,"nrorg" => $nrorg));
            if($gpevinculom->getDtretorno() && $dtreincid){
                $acao = "ALTERA";
                $this->movimentacaoService->manipulaMovimentacao($nrorg, $nrvinculom, $nrEstruturaLegal, null, $dtreincid, $dtcompetencia, $acao, 25);
            }else if(!$gpevinculom->getDtretorno() && $dtreincid){
                $acao = "INSERE";
                $this->movimentacaoService->manipulaMovimentacao($nrorg, $nrvinculom, $nrEstruturaLegal, null, $dtreincid, $dtcompetencia, $acao, 25);
            }else if($gpevinculom->getDtretorno() && !$dtreincid){
                $acao = "ALTERA";
                $this->movimentacaoService->manipulaMovimentacao($nrorg, $nrvinculom, $nrEstruturaLegal, null, $dtreincid, $dtcompetencia, $acao, 25);
            }
            
            if($gpevinculom->getDtrescisaovinc() && $dtrescisao){
                $acao = "ALTERA";
                $this->movimentacaoService->manipulaMovimentacao($nrorg, $nrvinculom, $nrEstruturaLegal, null, $dtrescisao, $dtcompetencia, $acao, $nrtpdemissao);
            }else if(!$gpevinculom->getDtrescisaovinc() && $dtrescisao){
                $acao = "INSERE";
                $this->movimentacaoService->manipulaMovimentacao($nrorg, $nrvinculom, $nrEstruturaLegal, null, $dtrescisao, $dtcompetencia, $acao, $nrtpdemissao);
            }else if($gpevinculom->getDtrescisaovinc() && !$dtrescisao){
                $acao = "ALTERA";
                $this->movimentacaoService->manipulaMovimentacao($nrorg, $nrvinculom, $nrEstruturaLegal, null, $dtrescisao, $dtcompetencia, $acao, $nrtpdemissao);
            }
            $acao = "ALTERA";
            if(!$gpevinculom->getDtrescisaovinc() && !$dtrescisao){
                $acao = null;
            }
            
            $this->alteEscalaService->alteraDataFimUltimaAlteracao($nrorg, $nrvinculom, $dtrescisao);

            $nr = $this->alteSituFuncService->manipulaSituacaoFuncional($dtrescisao, $nrorg, $nrvinculom, $dtadmissao, $acao, null, 1, $gpevinculom->getDtrescisaovinc(), $nrtpdemissao);//vai retornar o nr e poe na row
			
			if($dtadmissaoalterada == true){
			    $this->atualizarOutrasDatas($nrorg, $nrvinculom, $dtadmissaoanterior, $dtadmissao);
			}
			
			$this->dataSourceName = 'gpe_vinculom';
			$this->dataSourceNameToSave = $this->crudService->getDataSourceNameToSaveByName($this->dataSourceName);
			$this->dataSourceName = $this->dataSourceNameToSave ? $this->dataSourceNameToSave : $this->dataSourceName;
			$rows = $this->crudService->getRowsToSave($request, $this->dataSourceName);
			
			$this->crudService->save($this->dataSourceName, $rows);
			
			$request = $this->crudService->buildDataSetIfRow($request, $this->dataSourceName);
			
			$currentRow = current($request->getDataSet()->getRows());
			
			if($nr){
			    $currentRow['NRSITUFUNCM'] = $nr;
			} else {
			    $currentRow['NRSITUFUNCM'] = $this->alteSituFuncService->retornaUltimaSituacaoPorDatainicio($nrorg, $nrvinculom, $this->hcmParameters->getUltimoDiaMes())->getNrsitufuncm(); //pega a ultima situação antes do demitido
			}
            
            $this->vinculoServices->alteraBeneficiários($gpevinculom->getNrvinculom(), $gpevinculom->getDtrescisaovinc(), $dtrescisao);
            
			if(isset($currentRow['saveHistoryTable'])) {

                if (!$currentRow['__is_new']) {
                    if (!$this->checkHistory($response, $currentRow)) {
                        // is new history
                        $currentRow['__is_new'] = true;
                    }
                }

				$tableMasterPK = strtoupper($currentRow['saveHistoryTable']['__table_master_pk']);
				$newCode = array_filter($this->crudService->getColumnsToReturn(), function ($element) use ($tableMasterPK) {
					return $element[$tableMasterPK];
				});

				$this->dataSourceNameToSave = $this->crudService->getDataSourceNameToSaveByName($currentRow['saveHistoryTable']['__table_history_name']);
				$this->dataSourceName = $this->dataSourceNameToSave ? $this->dataSourceNameToSave : $this->dataSourceName;
				$currentRow[strtoupper($currentRow['saveHistoryTable']['__table_master_pk'])] = intval($newCode[0][strtoupper($currentRow['saveHistoryTable']['__table_master_pk'])]);
                
                // set history properties
                $currentRow['__new_code_pk'] = $currentRow['saveHistoryTable']['__table_history_name'];
                $currentRow['__table_pk'] = $currentRow['saveHistoryTable']['__table_history_pk'];
                
                

	        	$newDataSet = new DataSet($this->dataSourceName, array($currentRow));
	        	$newRequest = new \Zeedhi\Framework\DTO\Request\DataSet($newDataSet,
	            $request->getMethod(), $request->getRoutePath(), $request->getUserId());
				$rows = $this->crudService->getRowsToSave($newRequest, $this->dataSourceName);
				$this->crudService->save($this->dataSourceName, $rows);
            }
			$response->addDataset(new DataSet('columnsToReturn', $this->crudService->getColumnsToReturn()));
			
		} catch (\Exception $e) {
		    if ($e->getCode() === 10) {
                $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_ERROR));
            }
			else {
                $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
 	        }
		}
	}
	
	public function cancelaAdmissao(DTO\Request\DataSet $request, DTO\Response $response){
        try {
            foreach($request->getDataSet()->getRows() as $row){
                $this->vinculoServices->cancelaAdmissao($row);
            }
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('delete', array(true)));
        } catch (\Exception $e) {
            if ($e->getCode() === 10) {
                $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_ERROR));
            }else{
                $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode()));
            }  
        }
    }
	
    public function getVinculosAtivosCompetencia(DTO\Request\Filter $request, DTO\Response $response)
    {
        try{/*
            $nrestrutlegal = null; $nrestrutgeren = null; $nrtipovinculom = null;
            $filters = $request->getFilterCriteria()->getConditions();
            
            foreach ($filters as $filter) {
                if($filter['columnName'] === 'NRESTRUTLEGAL'){
                    $nrestrutlegal = $filter['value'];
                }
                if($filter['columnName'] === 'NRESTRUTGEREN'){
                    $nrestrutgeren = $filter['value'];
                }
                if($filter['columnName'] === 'NRTIPOVINCULOM'){
                    $nrtipovinculom = $filter['value'];
                }
            }
            $nrorg = $this->hcmParameters->getNrorgTrabalho();
            $dtmescompetenc = $this->hcmParameters->getDtCompetencia();

            $result = $this->vinculoServices->retornaVinculosAtivosCompetencia($nrorg, $dtmescompetenc, $nrestrutlegal, $nrestrutgeren, $nrtipovinculom);
            
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('getVinculosAtivosCompetencia', $result));
            */
            $this->dataSourceNameGet = $this->dataSourceNameGet2;
            
            $filterData = $request->getFilterCriteria();
            $filterData->setDataSourceName($this->dataSourceNameGet);
            
            $filterData->addCondition('P_NRORG','=',$this->hcmParameters->getNrorgTrabalho());
            $filterData->addCondition('P_DTMESCOMPETENCIA','=',$this->hcmParameters->getDtCompetencia());
            
            $filterData->addCondition('P_NRESTRUTLEGALOPER', '=', FPA::getNrestrutlegalOper());
            $filterData->addCondition('P_NRESTRUTGERENOPER', '=', FPA::getNrestrutgerenOper());
            
            $filterData->addCondition('P_NRTIPOESTRUTGEREN', '=', $this->hcmParameters->getTipoEstruturaGerencialOrg());
            $filterData->addCondition('P_NRTIPOESTRUTOMADOR', '=', $this->hcmParameters->getTipoEstruturaTomadorOrg());
            $filterData->setPageSize(99999);
            $this->parameterBagColumns = ["P_NRORG", "P_DTMESCOMPETENCIA", "P_TODOSRESCINDIDOS", "P_NRESTRUTLEGALOPER", "P_NRESTRUTGERENOPER", "P_NRTIPOESTRUTGEREN", "P_NRTIPOESTRUTOMADOR"];
            
            $newFilterData = $this->populateParameterBag($filterData);
            
            $dataSet = $this->dataSourceManager->findBy($newFilterData);
            
            $response->addDataSet($dataSet);
        
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function getVinculosAtivosCompetenciaComplementar(DTO\Request\Filter $request, DTO\Response $response)
    {
        try{
            $this->dataSourceNameGet = $this->dataSourceNameGet4;
            
            $filterData = $request->getFilterCriteria();
            $filterData->setDataSourceName($this->dataSourceNameGet);
            
            $filterData->addCondition('P_NRORG','=',$this->hcmParameters->getNrorgTrabalho());
            $filterData->addCondition('P_DTMESCOMPETENCIA','=',$this->hcmParameters->getDtCompetencia());
            $filterData->addCondition('P_NRTIPOESTRUTLEGAL','=',$this->hcmParameters->getTipoEstruturaLegalOrg());
            
            $this->parameterBagColumns = ["P_NRORG", "P_DTMESCOMPETENCIA", "P_NRTIPOESTRUTLEGAL", "P_TODOSRESCINDIDOS"];
            $newFilterData = $this->populateParameterBag($filterData);
            $dataSet = $this->dataSourceManager->findBy($newFilterData);
            $response->addDataSet($dataSet);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function getVinculosAtivosComplementar(DTO\Request\Filter $request, DTO\Response $response)
    {
        try{
            $this->dataSourceNameGet = $this->dataSourceNameGet5;
            
            $filterData = $request->getFilterCriteria();
            $filterData->setDataSourceName($this->dataSourceNameGet);
            
            $filterData->addCondition('P_NRORG','=',$this->hcmParameters->getNrorgTrabalho());
            $filterData->addCondition('P_DTMESCOMPETENCIA','=',$this->hcmParameters->getDtCompetencia());
            
            $this->parameterBagColumns = ["P_NRORG", "P_DTMESCOMPETENCIA", "P_TODOSRESCINDIDOS"];
            $newFilterData = $this->populateParameterBag($filterData);
            $dataSet = $this->dataSourceManager->findBy($newFilterData);
            $response->addDataSet($dataSet);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function getVinculosAtivosCalculadosVT(DTO\Request\Filter $request, DTO\Response $response)
    {
        try{
            $this->dataSourceNameGet = $this->dataSourceNameGet3;
            $filterData = $request->getFilterCriteria();
            $filterData->setDataSourceName($this->dataSourceNameGet);
            
            $filterData->addCondition('P_NRORG','=',$this->hcmParameters->getNrorgTrabalho());
            $filterData->addCondition('P_DTMESCOMPETENCIA','=',$this->hcmParameters->getDtCompetencia());
            $filterData->addCondition('P_NRCALCULOVT','=',$this->hcmParameters->determinaNrCalculoFolha(FpaTipomoviment::TP_MOV_VALE_TRANSPORTE));
            
            $this->parameterBagColumns = ["P_NRORG", "P_DTMESCOMPETENCIA", "P_TODOSRESCINDIDOS", "P_NRCALCULOVT"];
            $newFilterData = $this->populateParameterBag($filterData);
            $dataSet = $this->dataSourceManager->findBy($newFilterData);
            $response->addDataSet($dataSet);
        
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function getVinculoNr ($request, $response) {
        $dataset = $request->getDataSet();
        $dataset = $dataset->getRows();
        $dataset = $dataset[0];
            
        $vinculosAtivos = $this->vinculoServices->getVinculoNr($dataset);
        $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('vinculosAtivos', $vinculosAtivos));
    }    

    public function retornaVinculosCompetencia(DTO\Request\Filter $request, DTO\Response $response)
    {
        try{
            $nrorg = $this->hcmParameters->getNrorgTrabalho();
            $dtmescompetenc = $this->hcmParameters->getDtCompetencia();

            $result = $this->vinculoServices->retornaVinculosCompetencia($nrorg, $dtmescompetenc);

            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('getVinculos', $result));
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function retornaRescinditosCompetencia(DTO\Request\Filter $request, DTO\Response $response)
    {
        try{
            $todosRescindidos = null;
            $movTransfSaida = null;
            $nrEstrutura = null; $nrTipoVinculom = null; $carregaHistorico = null; $nrVinculoEmpreg = null;
            $filters = $request->getFilterCriteria()->getConditions();
            $filtro = array();
            $nrorg = $this->hcmParameters->getNrorgTrabalho();
            $nrOrgPadrao = $this->hcmParameters->getNrorgPadrao();
            $dtmescompetenc = $this->hcmParameters->getDtCompetencia();
            
            foreach ($filters as $key=>$filter) {
                
                if($filter['columnName'] === 'P_NRORG'){
                    $nrorg = $filter['value'];
                }
                if($filter['columnName'] === 'P_NRORG_PADRAO'){
                    $nrOrgPadrao = $filter['value'];
                }
                if($filter['columnName'] === 'P_TODOSRESCINDIDOS'){
                    $todosRescindidos = true;
                }
                //Considera também como rescindido os que tiverem movimentação de transferência de saída
                if($filter['columnName'] === 'P_MOVTRANSFSAIDA'){
                    $movTransfSaida = true;
                }
                
                if($filter['columnName'] === 'NRESTRUTLEGAL'){
                    $nrEstrutura = $filter['value'];
                    $filtro[] = $filter;
                    unset($filters[$key]);
                } else if($filter['columnName'] === 'NRESTRUTGEREN'){
                    $nrEstrutura = $filter['value'];
                    $filtro[] = $filter;
                    unset($filters[$key]);
                } else if($filter['columnName'] === 'NRESTRUTSIND'){
                    $filtro[] = $filter;
                    unset($filters[$key]);
                } else if($filter['columnName'] === 'NRTIPOHIERARQORG'){
                    $filtro[] = $filter;
                    unset($filters[$key]);
                } else if($filter['columnName'] === 'NRESTRUTSUPERIORM'){
                    $filtro[] = $filter;
                    unset($filters[$key]);
                } else if($filter['columnName'] === 'NRTIPOVINCULOM' and $filter['operator'] != 'IN'){
                    $nrTipoVinculom = $filter['value'];
                     unset($filters[$key]);
                } else if($filter['columnName'] === 'NRVINCULOEMPREG'){
                    $nrVinculoEmpreg = $filter['value'];
                     unset($filters[$key]);
                } else if($filter['columnName'] === 'NRTPDEMISSAO'){
                    $filter['columnName'] = 'GPE_VINCULOM.NRTPDEMISSAO';
                    $filtro[] = $filter;
                } else if($filter['columnName'] === 'NRMOTIVORESC'){
                    $filter['columnName'] = 'GPE_VINCULOM.NRMOTIVORESC';
                    $filtro[] = $filter;
                } else {
                    $filtro[] = $filter;
                }
                
                //Flag para identificar que a tela de entrevista de desligamento esta sendo usada.
                if($filter['columnName'] == 'P_ENTREVISTADEDESLIGAMENTO'){
                    $entrevistadeDesligamento = true;
                    unset($filters[$key]);
                }
                
            }
            //Passar vazio os filtros quando for usado a tela de entrevista de desligamento, o filtro sera feito no repository.
            $nrEstrutura = $entrevistadeDesligamento ? null : $nrEstrutura;
            
            Util::getValueInFilterDataSourceByName($filters, array('P_TODOSRESCINDIDOS','P_MOVTRANSFSAIDA'), true);
            $conditions = Util::createQueryFiltered($filters);
            $result = $this->vinculoServices->retornaRescinditosCompetencia($nrorg, $nrOrgPadrao, $dtmescompetenc, $nrEstrutura, $nrTipoVinculom, $nrVinculoEmpreg, $carregaHistorico, $filtro,$todosRescindidos, null, null, null, null, $conditions, $movTransfSaida, $entrevistadeDesligamento);

            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('retornaRescinditosCompetencia', $result));
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function retornaRescindidosGeral(DTO\Request\Filter $request, DTO\Response $response)
    {
        set_time_limit(0);
        
        try{
            $ds_inicio = null;$ds_fim = null;$dtini = null;$dtfim = null;$dsOrdem = null; 
            $nrEstrutura = null; $carregaHistorico = null; $nrTipoVinculom = null;
            $nrVinculoEmpreg = null; $nrEstrutSind = null;
            $dtmescompetenc = DateUtil::getDataDeString($this->hcmParameters->getDtCompetencia(), DateUtil::FORMATO_BRASILEIRO, true);
            
            $filters = $request->getFilterCriteria()->getConditions();
            foreach ($filters as $key => &$filter) {
                if($filter['columnName'] === 'NRESTRUTLEGAL'){
                    $dsOrdem = 'ESTRUTURA';
                    $nrEstrutura = $filter['value'];
                    
                    // exlcui posição do array pois o filtro pela estrutura é realizado na própria query
                    unset($filters[$key]);
                }
            }
            Util::getValueInFilterDataSourceByName($filters, array('NRTIPOVINCULOM'), true);
            
            $conditions = Util::createQueryFiltered($filters);
            
            $result = $this->vinculoServices->retornaRescindidosGeral($dtmescompetenc, $ds_inicio, $ds_fim, $dtini, $dtfim, $dsOrdem, $nrEstrutura, $carregaHistorico, $nrTipoVinculom,$nrVinculoEmpreg, $nrEstrutSind, $conditions);
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('retornaRescindidosGeral', $result));
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function setSeguroVinculo($request, $response) {
		try {
            foreach ($request->getDataSet()->getRows() as &$row) {
                $nrvinculom = $row['NRVINCULOM'];
                $idsegdesempvinc = $row['IDSEGDESEMPVINC'];
                $result = $this->vinculoServices->setSeguroVinculo($nrvinculom, $idsegdesempvinc);
	        }

            $response->addNotification(new DTO\Response\Notification('gerarCargaInicial', DTO\Response\Notification::TYPE_SUCCESS));
			
		} catch (\Exception $e) {
		    if ($e->getCode() === 10) {
                $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_ERROR));
            }
			else {
                $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
 	        }
		}
	}
    
    public function checkHistory($response, $row) {
        $condition = array(
            array(
                "columnName" => "NRORG",
                "operator" => "=",
                "value" => $this->hcmParameters->getNrorgTrabalho()
            ),
            array(
                "columnName" => strtoupper($row['saveHistoryTable']['__table_master_pk']),
                "operator" => "=",
                "value" => $row[strtoupper($row['saveHistoryTable']['__table_master_pk'])]
            ),
            array(
                "columnName" => strtoupper($row['saveHistoryTable']['__table_competenc']),
                "operator" => "=",
                "value" => $row[strtoupper($row['saveHistoryTable']['__table_competenc'])]
            )
        );

        $filterCriteria = new FilterCriteria($row['saveHistoryTable']['__table_history_name'], $condition);
        $newFilterCriteria = $this->crudService->populateParameterBag($filterCriteria, array());
        $this->crudService->find($response, $newFilterCriteria);

        if (count($response->getDataSets()[0]->getRows()) > 0) {
            return true;
        } else {
            return false;
        }
    }
    
    /**
     * Retorna os históricos posteriores do vínculo
     */
    public function getVinculoHistoricoPosterior(DTO\Request $request, DTO\Response $response) {
        $row            = $request->getDataSet()->getRows()[0];
        $nrorg          = $this->hcmParameters->getNrorgTrabalho();
        $nrvinculom     = $row[0];
        $dtmescompetenc = $row[1];
        $historicos     = $this->vinculoServices->retornaHistoricosPosteriores($nrorg, $nrvinculom, $dtmescompetenc);

        $retorno = array();
        
        /*monta uma lista de históricos posteriores à competência logada do operador caso estes existam*/
        foreach($historicos as $historico)
            $retorno[] = $historico->getNrvinculoh();

        $response->addDataset(new \Zeedhi\Framework\DataSource\DataSet('historicos', $retorno));
    }
    
    /**
     * Atualiza os históricos posteriores do vínculo
     */
    public function setVinculoHistoricoPosterior(DTO\Request $request, DTO\Response $response) {
        $nrhistoricos   = $request->getDataSet()->getRows()[0][0];
        $nrvinculom     = $request->getDataSet()->getRows()[0][1][0];
        $dtmescompetenc = $request->getDataSet()->getRows()[0][1][1];
        $nomeTela = $request->getDataSet()->getRows()[0][1][2];
        $nrorg          = $this->hcmParameters->getNrorgTrabalho();
        
        $historicoAtual = $this->vinculoServices->retornaUltimoHistoricoCompetencia($nrorg, $nrvinculom, $dtmescompetenc);

        $dados['NRSITUFUNCM'  ] = $nomeTela === 'situacaoFuncional' || $nomeTela === 'vinculo' ? $historicoAtual->getNrsitufuncm() : null;
        $dados['NRCARGO'      ] = $nomeTela === 'ocupacao' || $nomeTela === 'vinculo' ? $historicoAtual->getNrcargo() : null;
        $dados['NRFUNCAO'     ] = $nomeTela === 'ocupacao' || $nomeTela === 'vinculo' ? $historicoAtual->getNrfuncao() : null;
        $dados['NRESCALATRABM'] = $nomeTela === 'escala' || $nomeTela === 'vinculo'? $historicoAtual->getNrescalatrabm() : null;
        $dados['NRESTRUTLEGAL'] = $nomeTela === 'movimentacao LEGAL' || $nomeTela === 'vinculo' ? $historicoAtual->getNrestrutlegal() : null;
        $dados['NRESTRUTGEREN'] = $nomeTela === 'movimentacao GERENCIAL' || $nomeTela === 'vinculo'? $historicoAtual->getNrestrutgeren() : null;
        $dados['NRESTRUTSIND' ] = $nomeTela === 'movimentacao SINDICAL' || $nomeTela === 'vinculo' ? $historicoAtual->getNrestrutsind() : null;
        $dados['NRAGENTENOCIVO'] = $nomeTela === 'vinculo' ? $historicoAtual->getNragentenocivo() : null;
        $dados['CDMATRICULAESOCIAL'] = $historicoAtual->getCdmatriculaesocial();

        $compAlteradas = $this->vinculoServices->atualizaHistoricosVinculo($nrorg, $nrhistoricos,
            $dados['NRSITUFUNCM'  ],
            $dados['NRCARGO'      ],
            $dados['NRFUNCAO'     ],
            $dados['NRESCALATRABM'],
            $dados['NRESTRUTLEGAL'],
            $dados['NRESTRUTGEREN'],
            $dados['NRESTRUTSIND' ],
            $dados['CDMATRICULAESOCIAL'],
            $dados['NRAGENTENOCIVO']);

        $response->addMessage(new DTO\Response\Message($compAlteradas, DTO\Response\Message::TYPE_MESSAGE));
    }
    
    public function getConvocacaoTrabalhoIntermitente (DTO\Request\Filter $request, DTO\Response $response){
        try {
            $filterData = $request->getFilterCriteria();
            $filterData->setDataSourceName($this->dataSourceConvocacaoTrabalhoIntermitente);
            $newFilterData = $this->populateParameterBag($filterData);
            $dataSet = $this->dataSourceManager->findBy($newFilterData); 
            $response->addDataSet($dataSet);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function setConvocacaoTrabalhoIntermitente (DTO\Request\DataSet $request, DTO\Response $response){
        try {
            $this->dataSourceName = "convocacaoTrabalhoIntermitente";
            foreach ($request->getDataSet()->getRows() as &$row) {
                $existeMesmoPeriodo = $this->vinculoServices->existeMesmoPeriodoConvocacaoTrabalhoIntermitente($row);
                if( $existeMesmoPeriodo == true ){
                    $response->addNotification(new DTO\Response\Notification('Existe convocação nesse período', DTO\Response\Notification::TYPE_ERROR));
                    return -1;
                }
                if (array_key_exists('NRCONVOCACAO', $row)) {
                    if ($row['NRCONVOCACAO'] === '') {
                        $row['NRCONVOCACAO'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'GPE_CONVOCACAO', 1, 12, true); 
                    }
                    $result = $this->vinculoServices->updateConvocacaoTrabalhoIntermitente($row);
                    $response->addNotification(new DTO\Response\Notification('update', DTO\Response\Notification::TYPE_SUCCESS));
                } else {
                    $row['NRCONVOCACAO'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'GPE_CONVOCACAO', 1, 12, true);
                    parent::save($request, $response);
                }
                $this->vinculoServices->updateComposicaoValeTransporte($row, "update");
            }
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function setCadastroAutomaticoConvocacaoTrabalhoIntermitente (DTO\Request\DataSet $request, DTO\Response $response){
        try {
            foreach ($request->getDataSet()->getRows() as &$row) {
                $alertas = array();
                foreach( $row["NRVINCULOM"] as $vinculo ){
                    $parameters = array( 'NRORG'           => $row['NRORG'],
                                         'NRVINCULOM'      => $vinculo,
                                         'DSCONVOCACAO'    => array_key_exists('DSCONVOCACAO', $row) ? $row['DSCONVOCACAO'] : null,
                                         'DTINICONVOCACAO' => $row['DTINICONVOCACAO'],
                                         'DTFIMCONVOCACAO' => $row['DTFIMCONVOCACAO'],
                                         'DTINCLUSAO'      => $row['DTINCLUSAO'],
                                         'DTULTATU'        => $row['DTULTATU'],
                                         'NRORGINCLUSAO'   => $row['NRORGINCLUSAO'],
                                         'CDOPERINCLUSAO'  => $row['CDOPERINCLUSAO'],
                                         'NRORGULTATU'     => $row['NRORGULTATU'],
                                         'CDOPERULTATU'    => $row['CDOPERULTATU'],
                                         'PERIODOANTIGO'   => 0,
                                         'NRDIASAPURVALES' => 0);
                    $existeMesmoPeriodo = $this->vinculoServices->existeMesmoPeriodoConvocacaoTrabalhoIntermitente($parameters);
                    if( $existeMesmoPeriodo == true ){
                        $alertas[] = array(
                            "DSALERTA" => 'Já existe convocação nesse período para o vínculo'.' '.$vinculo
                        );
                    } else {
                        $parameters['NRCONVOCACAO'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'GPE_CONVOCACAO', 1, 12, true);
                        $this->vinculoServices->setCadastroAutomaticoConvocacaoTrabalhoIntermitente($parameters);
                    }
                }
                 if( empty($alertas) ){
                    $response->addNotification(new DTO\Response\Notification("Cadastro automático realizado com sucesso!", DTO\Response\Notification::TYPE_SUCCESS));
                } else {
                    $response->addNotification(new DTO\Response\Notification("Cadastro automático realizado com alertas!", DTO\Response\Notification::TYPE_ALERT));
                    $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('alertasCadastroAutomatico', $alertas));
                }
            }
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function deleteConvocacaoTrabalhoIntermitente (DTO\Request\DataSet $request, DTO\Response $response){
        try{
            $this->dataSourceName = "convocacaoTrabalhoIntermitente";
            parent::delete($request, $response);
            $this->vinculoServices->updateComposicaoValeTransporte($request->getDataSet()->getRows()[0], "delete");
        } catch(\Exception $e){
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }    
    }
    
    public function getSituacaoVinculoEsocial(DTO\Request\Filter $request, DTO\Response $response) {	 
        try{	 
            $filters = $request->getFilterCriteria()->getConditions();
            
            $parameters = array();	 
            foreach ($filters as $position => $filter) {	 
                unset($filter['operator']);	 
                foreach ($filter as $key => $value) {	 
                    if ($key == 'columnName' && substr($value,0, 2) == 'P_') {	 
                        $parameters[$value] = $filter['value'];	 
                    }	 
                }	 
            }
            
            Util::getValueInFilterDataSourceByName($filters, array('P_NRORG', 'P_DTMESCOMPETENC', 'P_NRVINCULOM'), true);	 
            $conditions = Util::createQueryFiltered($filters);	 
            $retorno = $this->vinculoServices->getSituacaoVinculoEsocial($parameters, $conditions);	 
            $response->addDataset(new \Zeedhi\Framework\DataSource\DataSet('getSituacaoVinculoEsocial',$retorno));
        } catch(\Exception $e) {	 
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));	 
        }	 
    }

    public function setFilterCriteriaDefaultOrderBy(FilterCriteria $filterData) {
        if (empty($filterData->getOrderBy())) {
            $filterData->addOrderBy('NMVINCULOM', 'ASC');
        }
    }
    
    protected function processConditionValue($condition) {
        return $condition['value'];
    }
    protected function populateParameterBag(FilterCriteria $filterCriteria) {

        $conditions = array();
        foreach ($filterCriteria->getConditions() as $condition) {
            $columnName = $condition['columnName'];
            if (in_array($columnName, $this->parameterBagColumns)) {
                $this->parameterBag->set($columnName, $this->processConditionValue($condition));
            } else {
                $conditions[] = $condition;
            }
        }

        $newFilterCriteria = new FilterCriteria(
            $filterCriteria->getDataSourceName(),
            $conditions,
            $filterCriteria->getPage(),
            $filterCriteria->getPageSize()
        );

        Util::copyOrderByClauseOnFilters($filterCriteria, $newFilterCriteria);

        return $newFilterCriteria;
    }
    
    public function verificaQuadroVagas($arrayDados){
        $verificacaoQuadroVagas = $this->vinculoServices->getVerificacaoQuadroVagasImportacao($this->hcmParameters->getNrorgTrabalho(), $arrayDados['NRESTRUTGEREN'], $arrayDados['NROCUPACAOM'], $arrayDados['NRESCALATRABM'], $arrayDados['NRESTRUTLEGAL'], $arrayDados['NRESTRUTSIND']);
        
        if ($verificacaoQuadroVagas["hasLimiteQuadroVagas"]) {
            $mensagemVerificacao = $verificacaoQuadroVagas["mensagemVerificacao"];
            
            //Remove o trecho "Deseja Prosseguir?" da mensagem retornada caso seja do tipo 'A' (alerta).
            if($verificacaoQuadroVagas["tipoValidacao"] === "A")
            {
                $msg_parts = explode(" Deseja prosseguir?", $verificacaoQuadroVagas["mensagemVerificacao"]);
                $mensagemVerificacao = $msg_parts[count($msg_parts)-2];
                $mensagemVerificacao = 'Operação bloqueada. ' . $mensagemVerificacao;
            }
            return $mensagemVerificacao;
        }
    }

    public function importarPlanilhaVinculo(DTO\Request\DataSet $request, DTO\Response $response) {
        set_time_limit(0); 
        
        try{
            $row = $request->getDataSet()->getRows()[0];
            
            $dataReturn = $this->vinculoServices->validaPlanilhaVinculo($row); 
            
            if(!empty($dataReturn['impVinculos'])) {
                foreach($dataReturn['impVinculos'] as $nrRow => &$arrayDados) {
                    
                    $arrayMsgError = [];
                    $arrayDados['NRROW'] = $nrRow;
                    $arrayDados['CDMUNICIPIO_END']   = $arrayDados['CDMUNICIPIO_END'] ?? null;
                    $arrayDados['NEWDTMESCOMPETENC'] = str_split(DateUtil::getPrimeiroDiaDoMes(DateUtil::getDataDeString($arrayDados['DTADMISSAOVINC']))->__toString(), 10);
                    $arrayDados['NEWDTMESCOMPETENC'] = Util::datadeString($arrayDados['NEWDTMESCOMPETENC'][0], true);

                    $detailMsg = ' Erro na linha '.$nrRow.'. Vinculo '.$arrayDados['NRVINCULOM'].' - '.$arrayDados['NMVINCULOM'];
                    
                    
                    $resultQuadroVagas = $this->verificaQuadroVagas($arrayDados);
                    
                    if(!empty($resultQuadroVagas))
                    {
                        $dataReturn['messages'][] = ["msg" => $resultQuadroVagas . " Linha: $nrRow, Nome do Vínculo: " . $arrayDados['NMVINCULOM']];
                        continue;
                    }
                    
                    try {
                        $this->admissaoService->insereVinculoPlanilha($arrayDados, $arrayMsgError);

                        if(!empty($arrayMsgError)) {
                            foreach($arrayMsgError as &$valueMsg) {
                                if (is_string($valueMsg) && strpos($valueMsg, '<br>') !== false) {
                                    $valueMsg = str_replace('<br>', PHP_EOL, $valueMsg);
                                }
                                $valueMsg .= $detailMsg;
                                $dataReturn['messages'][] = ['msg' => $valueMsg];
                            }
                        }
                        
                    } catch(\Exception $e) {
                        if(!empty($arrayMsgError)) {
                            foreach($arrayMsgError as &$valueMsg) {
                                if (is_string($valueMsg) && strpos($valueMsg, '<br>') !== false) {
                                    $valueMsg = str_replace('<br>', PHP_EOL, $valueMsg);
                                }
                                $valueMsg .= $detailMsg;
                                $dataReturn['messages'][] = ['msg' => $valueMsg];
                            }
                        }
                        $dataReturn['messages'][] = ['msg' => 'Ocorreu um erro e não foi possível importar o vínculo. Erro: '.$e->getMessage().$detailMsg];
                    }
                }
            }

            if(empty($dataReturn['messages'])){
                $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message("Importação concluida com sucesso.", \Zeedhi\Framework\DTO\Response\Message::TYPE_MESSAGE));
            } else {
                $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message("Operação concluida. Existem erros na importação", \Zeedhi\Framework\DTO\Response\Message::TYPE_WARNING));
                $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('msg', $dataReturn['messages']));
            }

        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
}