<?php
namespace HCM\Geral\Repository\Esocial;

use Doctrine\DBAL\Connection;
use Doctrine\ORM\EntityManager;
use HCM\Util\HCMParameters;

class S1280Repository {
    protected  $connection;

    public function __construct(EntityManager $entityManager, HCMParameters $hcmParameters) {
        $this->connection = $entityManager->getConnection();
        $this->entityManager = $entityManager;
        $this->hcmParameters = $hcmParameters;
    }
    
    public function getItensParaEnvioS1280($params) {
        try {
            // Só envia itens se for CNPJ Matriz ou se for outro tipo de inscrição
            $idMatriz = isset($params['IDMATRIZ']) ? $params['IDMATRIZ'] : 'S';
            if ($idMatriz == 'S') {
                $sql = <<<SQL
SELECT DISTINCT EM.NRESTRUTURAM as NRCHAVE,
       DECODE(CF.NRTIPOMOVIMENT,4,'4','1') as NRTABELA,
       'I' as IDOPERACAO,
       CF.DTOCORRENCIA as DTMESCOMPETENC
  FROM ESTRUTURAM EM
  JOIN ESTRUTURAH EH ON EH.NRORG = EM.NRORG
                    AND EH.NRESTRUTURAM = EM.NRESTRUTURAM
                    AND EH.DTMESCOMPETENC = (SELECT * FROM(SELECT H1.DTMESCOMPETENC FROM ESTRUTURAH H1
                                              WHERE H1.NRESTRUTURAM = EH.NRESTRUTURAM
                                                AND H1.NRORG = EH.NRORG
                                                AND H1.DTMESCOMPETENC <= :P_DTMESCOMPETENC
                                                ORDER BY H1.DTMESCOMPETENC DESC)WHERE ROWNUM = 1)
  JOIN FPA_CALCULOFOLHA CF ON CF.NRORG = EM.NRORG
                          AND CF.DTOCORRENCIA = :P_DTMESCOMPETENC
                          AND (CF.NRTIPOMOVIMENT IN (1,10,19) OR (CF.NRTIPOMOVIMENT = 4 AND TO_CHAR(CF.DTOCORRENCIA,'MM') = '12'))
                          AND CF.NRTPMODALIDCAL = 1
  JOIN CLASSTRIBUT CT ON CT.NRCLASSTRIBUT = EH.NRCLASSTRIBUT
 WHERE EM.NRORG = :P_NRORG
   AND EM.NRTIPOESTRUTURA = :P_NRTIPOESTRUTLEGAL
   AND (EM.NRESTRUTURAM = :P_NRESTRUTURAM OR :P_NRESTRUTURAM IS NULL)
   /*Empresas substituida o parcialmente substrituida, com valor de aliquota de desoneracao ou optante do simples*/
   AND (CT.CDESOCIAL IN ('01','03','12') OR EH.VRALIQDESON IS NOT NULL OR EH.IDOPTSIMPLES = 'S')
   AND SUBSTR(REPLACE(REPLACE(REPLACE(EH.CDCNPJESTRUT,'.',''),'/',''),'-',''),9,4) = '0001'
 ORDER BY 1,2
SQL;
                $params = array(
                    "P_NRORG" => $this->hcmParameters->getNrorgTrabalho(),
                    "P_DTMESCOMPETENC" => $this->hcmParameters->getDtCompetencia(),
                    "P_NRTIPOESTRUTLEGAL" => $this->hcmParameters->getTipoEstruturaLegalOrg(),
                    "P_NRESTRUTURAM" => isset($params['NRESTRUTURAM']) ? $params['NRESTRUTURAM'] : null,
                );
                return $this->connection->fetchAll($sql, $params);
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }

 
    public function getDadosS1280($parametros) {
        try {
            $dados = $this->getDadosPrincipais(
                $parametros['NRCHAVE'], $parametros['NRTABELA'], $parametros['DTMESCOMPETENC']
            );
            return $dados;
        } catch(\Exception $e) {
            throw $e;
        }
    }
    
    public function getDadosPrincipais($nrestruturam, $nrtabela, $dtocorrencia) {
        $sql = <<<SQL
SELECT DISTINCT EMH.NRESTRUTURAM CHAVE,
       /*ideEvento*/
       DECODE(CF.NRTIPOMOVIMENT,1,1,10,1,19,1,4,2) as indApuracao,
       TO_CHAR(CF.DTOCORRENCIA, DECODE(CF.NRTIPOMOVIMENT,4,'YYYY','YYYY-MM')) as perApur,
       /*ideEmpregador*/
       DECODE(PAR.CDTIPOINSCRICAO,'CNPJ','1','CPF','2','') TPINSC,
       CASE WHEN PAR.CDTIPOINSCRICAO = 'CPF' OR EMH.CDNATUJURI IN ('1015','1040','1074','1163') THEN PAR.NRINSCRICAOPARC ELSE SUBSTR(PAR.NRINSCRICAOPARC,1,8) END NRINSC,
       /*infoSubstPatr*/
       DECODE(EMH.IDOPTSIMPLES,'S','',DECODE(CT.CDESOCIAL,'01','1','2')) AS indSubstPatr,
       DECODE(EMH.IDOPTSIMPLES,'S','',EMH.VRALIQDESON) AS percRedContrib,
       /*infoSubstPatrOpPort*/
       /*infoAtivConcom*/
       DECODE(EMH.IDOPTSIMPLES,'S',EMH.VRALIQDESON,'') AS fatorMes,
       DECODE(EMH.IDOPTSIMPLES,'S',EMH.VRALIQDESON,'') AS fator13
  FROM ESTRUTURAH EMH
  JOIN PARCNEGOCIO PAR ON PAR.NRPARCNEGOCIO = EMH.NRPARCNEGOCIO
                      AND PAR.NRORG = EMH.NRORG
  JOIN CLASSTRIBUT CT ON CT.NRCLASSTRIBUT = EMH.NRCLASSTRIBUT
  /*Movimento*/
  LEFT JOIN FPA_CALCULOFOLHA CF ON CF.NRORG = EMH.NRORG
                               AND CF.NRTIPOMOVIMENT = :P_NRTABELA
                               AND CF.DTOCORRENCIA = :P_DTOCORRENCIA
                               AND CF.NRTPMODALIDCAL = 1
                               AND CF.NROCORRECAL = 1
 WHERE EMH.NRORG = :P_NRORG
   AND EMH.NRESTRUTURAM = :P_NRESTRUTURAM
   AND EMH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM ESTRUTURAH
                              WHERE ESTRUTURAH.DTMESCOMPETENC <= :P_DTMESCOMPETENC
                                AND ESTRUTURAH.NRESTRUTURAM = EMH.NRESTRUTURAM
                                AND ESTRUTURAH.NRORG = EMH.NRORG)
SQL;
        $params = array(
            "P_NRORG" => $this->hcmParameters->getNrorgTrabalho(),
            "P_DTMESCOMPETENC" => $this->hcmParameters->getDtCompetencia(),
            "P_NRESTRUTURAM" => $nrestruturam,
            "P_NRTABELA" => $nrtabela,
            "P_DTOCORRENCIA" => $dtocorrencia,
        );
        return $this->connection->fetchAssoc($sql, $params);
    }
    

}