<?php
namespace HCM\Geral\Repository\Esocial;

use Doctrine\DBAL\Connection;
use Doctrine\ORM\EntityManager;
use HCM\Util\HCMParameters;

class S2230Repository {

    protected  $connection;

    public function __construct(EntityManager $entityManager, HCMParameters $hcmParameters) {
        $this->connection = $entityManager->getConnection();
        $this->entityManager = $entityManager;
        $this->hcmParameters = $hcmParameters;
    }

    public function getItensParaEnvioS2230($params) : array 
    { 
      $itensParaEnvio = [];

      $params = array(
        "P_NRORG" => $this->hcmParameters->getNrorgTrabalho(),
        "P_DTMESCOMPETENC" => $this->hcmParameters->getDtCompetencia(),
        "P_INIVALIDPRODREST" => $this->hcmParameters->getEsoIniValidNaoPerProdRest(),
        "P_INIVALIDPRODUCAO" => $this->hcmParameters->getEsoIniValidNaoPerProducao(),
        "P_NRTPAMBIENTE" => $params['NRTPAMBIENTE'],
        "P_CDCNPJEMPREGADOR" => isset($params['CDCNPJEMPREGADOR']) ? $params['CDCNPJEMPREGADOR'] : null,
        "P_NRVINCULOM" => isset($params['NRVINCULOM']) ? $params['NRVINCULOM'] : null,
        "P_NRALTESITUFUNC" => isset($params['NRALTESITUFUNC']) ? $params['NRALTESITUFUNC'] : null,
        "P_NRGOZOFERIAS" => isset($params['NRGOZOFERIAS']) ? $params['NRGOZOFERIAS'] : null,
        "P_DTREFNAOPERIOD" => isset($params['DTREFNAOPERIOD']) ? $params['DTREFNAOPERIOD'] : null,
    );

      $itensParaEnvio = array_merge($itensParaEnvio, $this->consultaAfastamentosDataInicialCompetencia($params));
      $itensParaEnvio = array_merge($itensParaEnvio, $this->consultaAfastamentosDataInicialEFinalCompetencia($params));
      $itensParaEnvio = array_merge($itensParaEnvio, $this->consultaAfastamentosDataInicialAnteriorEFinalNaCompetencia($params));
      $itensParaEnvio = array_merge($itensParaEnvio, $this->consultaAfastamentosFinalCompetenciaVinculosIniciaramAfastados($params));
      $itensParaEnvio = array_merge($itensParaEnvio, $this->consultaAfastamentosDataInicialEFinalAnteriorCompetencia($params));
      $itensParaEnvio = array_merge($itensParaEnvio, $this->consultaGozoFerias($params));
      return $itensParaEnvio;
    }

    /**
     * Consulta afastamentos com data inicial na competência e sem data final ou data final posterior a competência
     * 
     * @param array $params Parâmetros da consulta.
     * @return array Resultado da consulta.
     */
    private function consultaAfastamentosDataInicialCompetencia(array $params) : array
    {
      $sql = "
      SELECT ASF.NRALTESITUFUNC as NRCHAVE,
             'I' as IDOPERACAO,
             NULL as NRTABELA,
             ASF.DTINISITUFUNC as DTMESCOMPETENC,
             ASF.NRVINCULOM
        FROM GPE_ALTESITUFUNC ASF
        JOIN GPE_VINCULOM VM ON VM.NRORG = ASF.NRORG AND ASF.NRVINCULOM = VM.NRVINCULOM
        JOIN GPE_VINCULOH VH ON VH.NRORG = VM.NRORG
                            AND VH.NRVINCULOM = VM.NRVINCULOM
                            AND VH.DTMESCOMPETENC = (SELECT * FROM(SELECT H1.DTMESCOMPETENC FROM GPE_VINCULOH H1
                                                      WHERE H1.NRVINCULOM = VH.NRVINCULOM
                                                        AND H1.NRORG = VH.NRORG
                                                        AND H1.DTMESCOMPETENC >= ADD_MONTHS(TRUNC(TO_DATE(:P_DTMESCOMPETENC),'YEAR'),-12)
                                                        AND H1.DTMESCOMPETENC <= :P_DTMESCOMPETENC
                                                        ORDER BY H1.DTMESCOMPETENC DESC)WHERE ROWNUM = 1)
         LEFT JOIN ESTRUTURAH EH ON EH.NRORG = VH.NRORG
                                AND EH.NRESTRUTURAM = VH.NRESTRUTLEGAL
                                AND EH.DTMESCOMPETENC = (SELECT * FROM(SELECT H1.DTMESCOMPETENC FROM ESTRUTURAH H1
                                                           WHERE H1.NRORG = EH.NRORG
                                                             AND H1.NRESTRUTURAM = EH.NRESTRUTURAM
                                                             AND H1.DTMESCOMPETENC <= :P_DTMESCOMPETENC
                                                             ORDER BY H1.DTMESCOMPETENC DESC)WHERE ROWNUM = 1)
        /*Situação Funcional*/
        JOIN GPE_SITUFUNCM SF ON SF.NRSITUFUNCM = ASF.NRSITUFUNCM
        JOIN GPE_SITUFUNCH SFH ON SFH.NRSITUFUNCM = SF.NRSITUFUNCM
                              AND SFH.DTEFETIVA = (SELECT MAX(DTEFETIVA) FROM GPE_SITUFUNCH
                                                    WHERE GPE_SITUFUNCH.DTEFETIVA <= :P_DTMESCOMPETENC
                                                      AND GPE_SITUFUNCH.NRSITUFUNCM = SFH.NRSITUFUNCM
                                                      AND GPE_SITUFUNCH.NRORG = SFH.NRORG)
       WHERE ASF.NRORG = :P_NRORG
         AND ASF.DTINISITUFUNC >= TO_DATE(DECODE(:P_NRTPAMBIENTE,1,:P_INIVALIDPRODUCAO,:P_INIVALIDPRODREST))
         AND ASF.DTINISITUFUNC BETWEEN :P_DTMESCOMPETENC AND LAST_DAY(:P_DTMESCOMPETENC)
         AND (ASF.DTFIMSITUFUNC IS NULL OR ASF.DTFIMSITUFUNC > LAST_DAY(:P_DTMESCOMPETENC))
         AND VM.DTADMISSAOVINC <= LAST_DAY(:P_DTMESCOMPETENC)
         AND SUBSTR(REPLACE(REPLACE(REPLACE(EH.CDCNPJESTRUT,'.'),'/'),'-'),1,8) = SUBSTR(REPLACE(REPLACE(REPLACE(:P_CDCNPJEMPREGADOR,'.'),'/'),'-'),1,8)
         AND (SFH.IDAFASTSITUFUNC = 'S' /*Situacao funcional que gere afastamento*/
              OR SF.NRSITUFUNCM = 12 OR SF.NRSITUFUNCM = 100012 OR SF.NRSITUFUNCM = 15 OR SF.NRSITUFUNCM = 16) /*Atestados médico e odontológico, Afastamentos 15 dias empregador*/
         AND SF.NRSITUFUNCM <> 13 /*Demitito eh parametrizado como gera afastamento mas nao gera o evento*/
         AND (ASF.NRALTESITUFUNC = :P_NRALTESITUFUNC OR :P_NRALTESITUFUNC IS NULL)
         AND (ASF.NRVINCULOM = :P_NRVINCULOM OR :P_NRVINCULOM IS NULL)
         AND (SFH.IDENVIAESOCIAL = 'S') /*verifica se a situação funcional está marcada para ser enviada*/
         AND (ASF.DTINISITUFUNC >= :P_DTREFNAOPERIOD OR :P_DTREFNAOPERIOD IS NULL)
        ";

        return $this->connection->fetchAll($sql, $params);
    }

    /**
     * Consulta afastamentos com data inicial e data final na competência*
     * 
     * @param array $params Parâmetros da consulta.
     * @return array Resultado da consulta.
     */
    private function consultaAfastamentosDataInicialEFinalCompetencia(array $params) : array
    {
      $sql = "
      SELECT ASF.NRALTESITUFUNC as NRCHAVE,
            'I' as IDOPERACAO,
            NULL as NRTABELA,
            ASF.DTINISITUFUNC as DTMESCOMPETENC,
            ASF.NRVINCULOM
        FROM GPE_ALTESITUFUNC ASF
        JOIN GPE_VINCULOM VM ON VM.NRORG = ASF.NRORG AND ASF.NRVINCULOM = VM.NRVINCULOM
        JOIN GPE_VINCULOH VH ON VH.NRORG = VM.NRORG
                            AND VH.NRVINCULOM = VM.NRVINCULOM
                            AND VH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM GPE_VINCULOH H1
                                                      WHERE H1.NRVINCULOM = VH.NRVINCULOM
                                                        AND H1.NRORG = VH.NRORG
                                                        AND H1.DTMESCOMPETENC >= ADD_MONTHS(TRUNC(TO_DATE(:P_DTMESCOMPETENC),'YEAR'),-12)
                                                        AND H1.DTMESCOMPETENC <= :P_DTMESCOMPETENC)
        LEFT JOIN ESTRUTURAH EH ON EH.NRORG = VH.NRORG
                                AND EH.NRESTRUTURAM = VH.NRESTRUTLEGAL
                                AND EH.DTMESCOMPETENC = (SELECT MAX(H1.DTMESCOMPETENC) FROM ESTRUTURAH H1
                                                          WHERE H1.NRORG = EH.NRORG
                                                            AND H1.NRESTRUTURAM = EH.NRESTRUTURAM
                                                            AND H1.DTMESCOMPETENC <= :P_DTMESCOMPETENC)
        /*Situação Funcional*/
        JOIN GPE_SITUFUNCM SF ON SF.NRSITUFUNCM = ASF.NRSITUFUNCM
        JOIN GPE_SITUFUNCH SFH ON SFH.NRSITUFUNCM = SF.NRSITUFUNCM
                              AND SFH.DTEFETIVA = (SELECT MAX(DTEFETIVA) FROM GPE_SITUFUNCH
                                                    WHERE GPE_SITUFUNCH.DTEFETIVA <= :P_DTMESCOMPETENC
                                                      AND GPE_SITUFUNCH.NRSITUFUNCM = SFH.NRSITUFUNCM
                                                      AND GPE_SITUFUNCH.NRORG = SFH.NRORG)
      WHERE ASF.NRORG = :P_NRORG
        AND ASF.DTINISITUFUNC >= TO_DATE(DECODE(:P_NRTPAMBIENTE,1,:P_INIVALIDPRODUCAO,:P_INIVALIDPRODREST))
        AND ASF.DTINISITUFUNC BETWEEN :P_DTMESCOMPETENC AND LAST_DAY(:P_DTMESCOMPETENC)
        AND ASF.DTFIMSITUFUNC BETWEEN :P_DTMESCOMPETENC AND LAST_DAY(:P_DTMESCOMPETENC)
        AND VM.DTADMISSAOVINC <= LAST_DAY(:P_DTMESCOMPETENC)
        AND SUBSTR(REPLACE(REPLACE(REPLACE(EH.CDCNPJESTRUT,'.'),'/'),'-'),1,8) = SUBSTR(REPLACE(REPLACE(REPLACE(:P_CDCNPJEMPREGADOR,'.'),'/'),'-'),1,8)
        AND (SFH.IDAFASTSITUFUNC = 'S' /*Situacao funcional que gere afastamento*/
              OR SF.NRSITUFUNCM = 12 OR SF.NRSITUFUNCM = 100012 OR SF.NRSITUFUNCM = 15 OR SF.NRSITUFUNCM = 16) /*Atestados médico e odontológico, Afastamentos 15 dias empregador*/
        AND SF.NRSITUFUNCM <> 13 /*Demitito eh parametrizado como gera afastamento mas nao gera o evento*/
        AND (ASF.NRALTESITUFUNC = :P_NRALTESITUFUNC OR :P_NRALTESITUFUNC IS NULL)
        AND (ASF.NRVINCULOM = :P_NRVINCULOM OR :P_NRVINCULOM IS NULL)
        AND (SFH.IDENVIAESOCIAL = 'S') /*verifica se a situação funcional está marcada para ser enviada*/
        AND (ASF.DTINISITUFUNC >= :P_DTREFNAOPERIOD OR :P_DTREFNAOPERIOD IS NULL)
      ";

      return $this->connection->fetchAll($sql, $params);
    }

     /**
     * Consulta afastamentos com data inicial anterior à competência e data final na competência*
     * 
     * @param array $params Parâmetros da consulta.
     * @return array Resultado da consulta.
     */
    private function consultaAfastamentosDataInicialAnteriorEFinalNaCompetencia(array $params) : array
    {         
      $sql = "
      SELECT ASF.NRALTESITUFUNC as NRCHAVE,
             'I' as IDOPERACAO,
             NULL as NRTABELA,
             ASF.DTFIMSITUFUNC as DTMESCOMPETENC,
             ASF.NRVINCULOM
        FROM GPE_ALTESITUFUNC ASF
        JOIN GPE_VINCULOM VM ON VM.NRORG = ASF.NRORG AND ASF.NRVINCULOM = VM.NRVINCULOM
        JOIN GPE_VINCULOH VH ON VH.NRORG = VM.NRORG
                            AND VH.NRVINCULOM = VM.NRVINCULOM
                            AND VH.DTMESCOMPETENC = (SELECT * FROM(SELECT H1.DTMESCOMPETENC FROM GPE_VINCULOH H1
                                                      WHERE H1.NRVINCULOM = VH.NRVINCULOM
                                                        AND H1.NRORG = VH.NRORG
                                                        AND H1.DTMESCOMPETENC >= ADD_MONTHS(TRUNC(TO_DATE(:P_DTMESCOMPETENC),'YEAR'),-12)
                                                        AND H1.DTMESCOMPETENC <= :P_DTMESCOMPETENC
                                                        ORDER BY H1.DTMESCOMPETENC DESC)WHERE ROWNUM = 1)
         LEFT JOIN ESTRUTURAH EH ON EH.NRORG = VH.NRORG
                                AND EH.NRESTRUTURAM = VH.NRESTRUTLEGAL
                                AND EH.DTMESCOMPETENC = (SELECT * FROM(SELECT H1.DTMESCOMPETENC FROM ESTRUTURAH H1
                                                           WHERE H1.NRORG = EH.NRORG
                                                             AND H1.NRESTRUTURAM = EH.NRESTRUTURAM
                                                             AND H1.DTMESCOMPETENC <= :P_DTMESCOMPETENC
                                                             ORDER BY H1.DTMESCOMPETENC DESC)WHERE ROWNUM = 1)
        /*Situação Funcional*/
        JOIN GPE_SITUFUNCM SF ON SF.NRSITUFUNCM = ASF.NRSITUFUNCM
        JOIN GPE_SITUFUNCH SFH ON SFH.NRSITUFUNCM = SF.NRSITUFUNCM
                              AND SFH.DTEFETIVA = (SELECT MAX(DTEFETIVA) FROM GPE_SITUFUNCH
                                                    WHERE GPE_SITUFUNCH.DTEFETIVA <= :P_DTMESCOMPETENC
                                                      AND GPE_SITUFUNCH.NRSITUFUNCM = SFH.NRSITUFUNCM
                                                      AND GPE_SITUFUNCH.NRORG = SFH.NRORG)
       WHERE ASF.NRORG = :P_NRORG
         AND ASF.DTINISITUFUNC >= TO_DATE(DECODE(:P_NRTPAMBIENTE,1,:P_INIVALIDPRODUCAO,:P_INIVALIDPRODREST))
         AND ASF.DTINISITUFUNC < :P_DTMESCOMPETENC
         AND ASF.DTFIMSITUFUNC BETWEEN :P_DTMESCOMPETENC AND LAST_DAY(:P_DTMESCOMPETENC)
         AND VM.DTADMISSAOVINC <= LAST_DAY(:P_DTMESCOMPETENC)
         AND SUBSTR(REPLACE(REPLACE(REPLACE(EH.CDCNPJESTRUT,'.'),'/'),'-'),1,8) = SUBSTR(REPLACE(REPLACE(REPLACE(:P_CDCNPJEMPREGADOR,'.'),'/'),'-'),1,8)
         AND (SFH.IDAFASTSITUFUNC = 'S' /*Situacao funcional que gere afastamento*/
              OR SF.NRSITUFUNCM = 12 OR SF.NRSITUFUNCM = 100012 OR SF.NRSITUFUNCM = 15 OR SF.NRSITUFUNCM = 16) /*Atestados médico e odontológico, Afastamentos 15 dias empregador*/
         AND SF.NRSITUFUNCM <> 13 /*Demitito eh parametrizado como gera afastamento mas nao gera o evento*/
         AND (ASF.NRALTESITUFUNC = :P_NRALTESITUFUNC OR :P_NRALTESITUFUNC IS NULL)
         AND (ASF.NRVINCULOM = :P_NRVINCULOM OR :P_NRVINCULOM IS NULL)
         AND (SFH.IDENVIAESOCIAL = 'S') /*verifica se a situação funcional está marcada para ser enviada*/
         AND (ASF.DTFIMSITUFUNC >= :P_DTREFNAOPERIOD OR :P_DTREFNAOPERIOD IS NULL)
        ";

      return $this->connection->fetchAll($sql, $params);
    }

    /**
     * Consulta afastamentos com data final na competência para vínculos que iniciaram o eSocial afastados*
     * 
     * @param array $params Parâmetros da consulta.
     * @return array Resultado da consulta.
     */
    private function consultaAfastamentosFinalCompetenciaVinculosIniciaramAfastados(array $params) : array
    {
      $sql = "
      SELECT ASF.NRALTESITUFUNC as NRCHAVE,
            'I' as IDOPERACAO,
            NULL as NRTABELA,
            ASF.DTFIMSITUFUNC as DTMESCOMPETENC,
            ASF.NRVINCULOM
        FROM GPE_ALTESITUFUNC ASF
        JOIN GPE_VINCULOM VM ON VM.NRORG = ASF.NRORG AND ASF.NRVINCULOM = VM.NRVINCULOM
        JOIN GPE_VINCULOH VH ON VH.NRORG = VM.NRORG
                            AND VH.NRVINCULOM = VM.NRVINCULOM
                            AND VH.DTMESCOMPETENC = (SELECT * FROM(SELECT H1.DTMESCOMPETENC FROM GPE_VINCULOH H1
                                                      WHERE H1.NRVINCULOM = VH.NRVINCULOM
                                                        AND H1.NRORG = VH.NRORG
                                                        AND H1.DTMESCOMPETENC >= ADD_MONTHS(TRUNC(TO_DATE(:P_DTMESCOMPETENC),'YEAR'),-12)
                                                        AND H1.DTMESCOMPETENC <= :P_DTMESCOMPETENC
                                                        ORDER BY H1.DTMESCOMPETENC DESC)WHERE ROWNUM = 1)
        LEFT JOIN ESTRUTURAH EH ON EH.NRORG = VH.NRORG
                                AND EH.NRESTRUTURAM = VH.NRESTRUTLEGAL
                                AND VH.DTMESCOMPETENC = (SELECT * FROM(SELECT H1.DTMESCOMPETENC FROM GPE_VINCULOH H1
                                                      WHERE H1.NRVINCULOM = VH.NRVINCULOM
                                                        AND H1.NRORG = VH.NRORG
                                                        AND H1.DTMESCOMPETENC >= ADD_MONTHS(TRUNC(TO_DATE(:P_DTMESCOMPETENC),'YEAR'),-12)
                                                        AND H1.DTMESCOMPETENC <= :P_DTMESCOMPETENC
                                                        ORDER BY H1.DTMESCOMPETENC DESC)WHERE ROWNUM = 1)
        /*Situação Funcional*/
        JOIN GPE_SITUFUNCM SF ON SF.NRSITUFUNCM = ASF.NRSITUFUNCM
        JOIN GPE_SITUFUNCH SFH ON SFH.NRSITUFUNCM = SF.NRSITUFUNCM
                              AND SFH.DTEFETIVA = (SELECT MAX(DTEFETIVA) FROM GPE_SITUFUNCH
                                                    WHERE GPE_SITUFUNCH.DTEFETIVA <= :P_DTMESCOMPETENC
                                                      AND GPE_SITUFUNCH.NRSITUFUNCM = SFH.NRSITUFUNCM
                                                      AND GPE_SITUFUNCH.NRORG = SFH.NRORG)
      WHERE ASF.NRORG = :P_NRORG
        AND ASF.DTINISITUFUNC < TO_DATE(DECODE(:P_NRTPAMBIENTE,1,:P_INIVALIDPRODUCAO,:P_INIVALIDPRODREST))
        AND ASF.DTFIMSITUFUNC BETWEEN :P_DTMESCOMPETENC AND LAST_DAY(:P_DTMESCOMPETENC)
        AND ASF.DTFIMSITUFUNC >= TO_DATE(DECODE(:P_NRTPAMBIENTE,1,:P_INIVALIDPRODUCAO,:P_INIVALIDPRODREST))
        AND VM.DTADMISSAOVINC <= LAST_DAY(:P_DTMESCOMPETENC)
        AND SUBSTR(REPLACE(REPLACE(REPLACE(EH.CDCNPJESTRUT,'.'),'/'),'-'),1,8) = SUBSTR(REPLACE(REPLACE(REPLACE(:P_CDCNPJEMPREGADOR,'.'),'/'),'-'),1,8)
        AND (SFH.IDAFASTSITUFUNC = 'S' /*Situacao funcional que gere afastamento*/
              OR SF.NRSITUFUNCM = 12 OR SF.NRSITUFUNCM = 100012 OR SF.NRSITUFUNCM = 15 OR SF.NRSITUFUNCM = 16) /*Atestados médico e odontológico, Afastamentos 15 dias empregador*/
        AND SF.NRSITUFUNCM <> 13 /*Demitito eh parametrizado como gera afastamento mas nao gera o evento*/
        AND (ASF.NRALTESITUFUNC = :P_NRALTESITUFUNC OR :P_NRALTESITUFUNC IS NULL)
        AND (ASF.NRVINCULOM = :P_NRVINCULOM OR :P_NRVINCULOM IS NULL)
        AND (SFH.IDENVIAESOCIAL = 'S') /*verifica se a situação funcional está marcada para ser enviada*/
        AND (ASF.DTFIMSITUFUNC >= :P_DTREFNAOPERIOD OR :P_DTREFNAOPERIOD IS NULL)
        ";
        return $this->connection->fetchAll($sql, $params);
    }
    
    /**
     * Consulta afastamentos com data inicial e final anterior à competência*
     * 
     * @param array $params Parâmetros da consulta.
     * @return array Resultado da consulta.
     */
    private function consultaAfastamentosDataInicialEFinalAnteriorCompetencia(array $params) : array
    {
      $sql = "
      SELECT ASF.NRALTESITUFUNC as NRCHAVE,
             'I' as IDOPERACAO,
             NULL as NRTABELA,
             ASF.DTFIMSITUFUNC as DTMESCOMPETENC,
             ASF.NRVINCULOM,
             ASF.NRALTESITUFUNC as ALTESITUDESCENT
        FROM GPE_ALTESITUFUNC ASF
        JOIN GPE_VINCULOM VM ON VM.NRORG = ASF.NRORG AND ASF.NRVINCULOM = VM.NRVINCULOM
        JOIN GPE_VINCULOH VH ON VH.NRORG = VM.NRORG
                            AND VH.NRVINCULOM = VM.NRVINCULOM
                            AND VH.DTMESCOMPETENC = (SELECT * FROM(SELECT H1.DTMESCOMPETENC FROM GPE_VINCULOH H1
                                                      WHERE H1.NRVINCULOM = VH.NRVINCULOM
                                                        AND H1.NRORG = VH.NRORG
                                                        AND H1.DTMESCOMPETENC >= ADD_MONTHS(TRUNC(TO_DATE(:P_DTMESCOMPETENC),'YEAR'),-12)
                                                        AND H1.DTMESCOMPETENC <= :P_DTMESCOMPETENC
                                                        ORDER BY H1.DTMESCOMPETENC DESC)WHERE ROWNUM = 1)
         LEFT JOIN ESTRUTURAH EH ON EH.NRORG = VH.NRORG
                                AND EH.NRESTRUTURAM = VH.NRESTRUTLEGAL
                                AND EH.DTMESCOMPETENC = (SELECT * FROM(SELECT H1.DTMESCOMPETENC FROM ESTRUTURAH H1
                                                           WHERE H1.NRORG = EH.NRORG
                                                             AND H1.NRESTRUTURAM = EH.NRESTRUTURAM
                                                             AND H1.DTMESCOMPETENC <= :P_DTMESCOMPETENC
                                                             ORDER BY H1.DTMESCOMPETENC DESC)WHERE ROWNUM = 1)
        /*Situação Funcional*/
        JOIN GPE_SITUFUNCM SF ON SF.NRSITUFUNCM = ASF.NRSITUFUNCM
        JOIN GPE_SITUFUNCH SFH ON SFH.NRSITUFUNCM = SF.NRSITUFUNCM
                              AND SFH.DTEFETIVA = (SELECT MAX(DTEFETIVA) FROM GPE_SITUFUNCH
                                                    WHERE GPE_SITUFUNCH.DTEFETIVA <= :P_DTMESCOMPETENC
                                                      AND GPE_SITUFUNCH.NRSITUFUNCM = SFH.NRSITUFUNCM
                                                      AND GPE_SITUFUNCH.NRORG = SFH.NRORG)
       WHERE ASF.NRORG = :P_NRORG
         AND ASF.DTINISITUFUNC >= TO_DATE(DECODE(:P_NRTPAMBIENTE,1,:P_INIVALIDPRODUCAO,:P_INIVALIDPRODREST))
         AND ASF.DTINISITUFUNC < :P_DTMESCOMPETENC
         AND ASF.DTFIMSITUFUNC < :P_DTMESCOMPETENC
         AND VM.DTADMISSAOVINC <= LAST_DAY(:P_DTMESCOMPETENC)
         AND SUBSTR(REPLACE(REPLACE(REPLACE(EH.CDCNPJESTRUT,'.'),'/'),'-'),1,8) = SUBSTR(REPLACE(REPLACE(REPLACE(:P_CDCNPJEMPREGADOR,'.'),'/'),'-'),1,8)
         AND (SFH.IDAFASTSITUFUNC = 'S' /*Situacao funcional que gere afastamento*/
              OR SF.NRSITUFUNCM = 12 OR SF.NRSITUFUNCM = 100012 OR SF.NRSITUFUNCM = 15 OR SF.NRSITUFUNCM = 16) /*Atestados médico e odontológico, Afastamentos 15 dias empregador*/
         AND SF.NRSITUFUNCM <> 13 /*Demitito eh parametrizado como gera afastamento mas nao gera o evento*/
         AND (ASF.NRALTESITUFUNC = :P_NRALTESITUFUNC)
         AND (ASF.NRVINCULOM = :P_NRVINCULOM OR :P_NRVINCULOM IS NULL)
         AND (SFH.IDENVIAESOCIAL = 'S') /*verifica se a situação funcional está marcada para ser enviada*/
         AND (ASF.DTFIMSITUFUNC >= :P_DTREFNAOPERIOD OR :P_DTREFNAOPERIOD IS NULL)
        ";
        return $this->connection->fetchAll($sql, $params);
    }

    /**
     * Consulta gozo de férias como afastamento*
     * *Não considera IDENVIAESOCIAL pois gozo de férias ou recesso é sempre obrigatório o envio para o eSocial*
     * 
     * @param array $params Parâmetros da consulta.
     * @return array Resultado da consulta.
     */
    private function consultaGozoFerias(array $params) : array
    {
      $sql = "
      SELECT GF.NRGOZOFERIAS as NRCHAVE,
            'I' as IDOPERACAO,
            2 as NRTABELA,
            GF.DTINIGOZOFERIAS as DTMESCOMPETENC,
            FE.NRVINCULOM
        FROM FPA_GOZOFERIAS GF
        JOIN FPA_FERIAS FE ON FE.NRORG = GF.NRORG
                          AND FE.NRFERIAS = GF.NRFERIAS
        JOIN GPE_VINCULOM VM ON FE.NRORG = VM.NRORG
                            AND FE.NRVINCULOM = VM.NRVINCULOM
        JOIN GPE_VINCULOH VH ON VH.NRORG = VM.NRORG
                            AND VH.NRVINCULOM = VM.NRVINCULOM
                            AND VH.DTMESCOMPETENC = (SELECT * FROM(SELECT H1.DTMESCOMPETENC FROM GPE_VINCULOH H1
                                                      WHERE H1.NRVINCULOM = VH.NRVINCULOM
                                                        AND H1.NRORG = VH.NRORG
                                                        AND H1.DTMESCOMPETENC >= ADD_MONTHS(TRUNC(TO_DATE(:P_DTMESCOMPETENC),'YEAR'),-12)
                                                        AND H1.DTMESCOMPETENC <= :P_DTMESCOMPETENC
                                                        ORDER BY H1.DTMESCOMPETENC DESC)WHERE ROWNUM = 1)
        LEFT JOIN ESTRUTURAH EH ON EH.NRORG = VH.NRORG
                                AND EH.NRESTRUTURAM = VH.NRESTRUTLEGAL
                                AND EH.DTMESCOMPETENC = (SELECT * FROM(SELECT H1.DTMESCOMPETENC FROM ESTRUTURAH H1
                                                          WHERE H1.NRORG = EH.NRORG
                                                            AND H1.NRESTRUTURAM = EH.NRESTRUTURAM
                                                            AND H1.DTMESCOMPETENC <= :P_DTMESCOMPETENC
                                                            ORDER BY H1.DTMESCOMPETENC DESC)WHERE ROWNUM = 1)
      WHERE GF.NRORG = :P_NRORG
        AND (GF.DTINIGOZOFERIAS BETWEEN :P_DTMESCOMPETENC AND LAST_DAY(:P_DTMESCOMPETENC) OR
              GF.DTFIMGOZOFERIAS BETWEEN :P_DTMESCOMPETENC AND LAST_DAY(:P_DTMESCOMPETENC))
        AND VM.DTADMISSAOVINC <= LAST_DAY(:P_DTMESCOMPETENC)
        AND SUBSTR(REPLACE(REPLACE(REPLACE(EH.CDCNPJESTRUT,'.'),'/'),'-'),1,8) = SUBSTR(REPLACE(REPLACE(REPLACE(:P_CDCNPJEMPREGADOR,'.'),'/'),'-'),1,8)
        AND (GF.NRGOZOFERIAS = :P_NRGOZOFERIAS OR :P_NRGOZOFERIAS IS NULL)
        AND (FE.NRVINCULOM = :P_NRVINCULOM OR :P_NRVINCULOM IS NULL)
        AND  EXISTS (SELECT 1 
                      FROM FPA_ITECALCFOLHA ITC
                      JOIN FPA_CALCULOFOLHA CFL ON CFL.NRORG = ITC.NRORG
                                                AND CFL.NRCALCULOFOLHA = ITC.NRCALCULOFOLHA
                                                AND CFL.NRTIPOMOVIMENT = 2
                                                AND CFL.NRTPMODALIDCAL = 1
                  LEFT JOIN ESO_PARAMETRO ESOPARAMS ON ESOPARAMS.NRESTRUTURAM = VH.NRESTRUTLEGAL
                                                  AND ESOPARAMS.NRORG = VH.NRORG 
                      WHERE ITC.NRVINCULOM = VH.NRVINCULOM
                        AND ITC.NRORG = VH.NRORG
                        AND ITC.NREVENTOM = '3030'
                        AND CFL.DTOCORRENCIA = (SELECT MAX(CF.DTOCORRENCIA)
                                                  FROM FPA_ITECALCFOLHA IT
                                                  JOIN FPA_CALCULOFOLHA CF ON CF.NRORG = IT.NRORG
                                                                          AND CF.NRCALCULOFOLHA = IT.NRCALCULOFOLHA
                                                                          AND CF.NRTIPOMOVIMENT = 2
                                                                          AND CF.NRTPMODALIDCAL = 1
                                                WHERE IT.NRORG      = ITC.NRORG
                                                  AND IT.NRVINCULOM = ITC.NRVINCULOM
                                                  AND IT.NREVENTOM = '3030')   
                        AND((ESOPARAMS.IDAPROVACALCFERIAS = 'S'AND EXISTS(SELECT 1
                                                                          FROM FPA_ITECALCFOLHA ITFEC 
                                                                        WHERE ITFEC.NRORG = ITC.NRORG
                                                                          AND ITFEC.NRVINCULOM = ITC.NRVINCULOM
                                                                          AND ITFEC.NREVENTOM = 3035
                                                                          AND ITFEC.NRCALCULOFOLHA = CFL.NRCALCULOFOLHA)) OR NVL(ESOPARAMS.IDAPROVACALCFERIAS, 'N') = 'N'))
        AND (GF.DTINIGOZOFERIAS >= :P_DTREFNAOPERIOD OR :P_DTREFNAOPERIOD IS NULL)
      ORDER BY 4,5";
    
        return $this->connection->fetchAll($sql, $params);
    }
    
    public function getDadosS2230($parametros) {
        try {
            if ($parametros['NRTABELA'] == 2) {
                $dados = $this->getDadosGozoFerias($parametros['NRCHAVE'], $parametros['NRESTRUTURAM'], $parametros['NRTPAMBIENTE']);
            } else {
                $dados = $this->getDadosAltesitufunc($parametros['NRCHAVE'], $parametros['NRESTRUTURAM'], $parametros['NRTPAMBIENTE']);
            }
            return $dados;
        } catch(\Exception $e) {
            throw $e;
        }
    }

    public function getDadosAltesitufunc($nrchave, $nrestruturam, $nrtpambiente) {
        $sql = <<<SQL
/*getDadosS2230_Altesitufunc*/
SELECT ASF.NRALTESITUFUNC CHAVE,
       --ideEmpregador
       DECODE(PAR.CDTIPOINSCRICAO,'CNPJ','1','CPF','2','') TPINSC,
       CASE WHEN PAR.CDTIPOINSCRICAO = 'CPF' OR EMH.CDNATUJURI IN ('1015','1040','1074','1163') THEN PAR.NRINSCRICAOPARC ELSE SUBSTR(PAR.NRINSCRICAOPARC,1,8) END NRINSC,
       --ideVinculo
       REPLACE(REPLACE(PH.NRCPFPESSOA,'.'),'-') AS cpfTrab,
       CASE WHEN VM.NRTIPOVINCULOM IN (2,3,10) THEN DECODE(NP.NRENVIO,NULL,NULL,NVL(VH.CDMATRICULAESOCIAL, VM.NRVINCULOM)) ELSE NVL(VH.CDMATRICULAESOCIAL, VM.NRVINCULOM) END AS matricula,
       CASE WHEN VM.NRTIPOVINCULOM IN (2,3,10) THEN DECODE(NP.NRENVIO,NULL,CE.CDESOCIAL,NULL) ELSE '' END codCateg,
       --infoAfastamento
       --iniAfastamento
       CASE WHEN ASF.DTINISITUFUNC >= TO_DATE(DECODE(:P_NRTPAMBIENTE, 1, DECODE(EMP.NRGRUPOESOCIAL, 3, :P_INIVALIDADEAMBNS, :P_INIVALIDADEAMBP), 2, :P_INIVALIDADEAMBPR)) AND (ASF.DTINISITUFUNC >= :P_DTMESCOMPETENC OR (ASF.DTINISITUFUNC < :P_DTMESCOMPETENC AND ASF.DTFIMSITUFUNC < :P_DTMESCOMPETENC)) THEN TO_CHAR(ASF.DTINISITUFUNC, 'YYYY-MM-DD') ELSE '' END AS dtIniAfast,
       CASE WHEN ASF.DTINISITUFUNC >= TO_DATE(DECODE(:P_NRTPAMBIENTE, 1, DECODE(EMP.NRGRUPOESOCIAL, 3, :P_INIVALIDADEAMBNS, :P_INIVALIDADEAMBP), 2, :P_INIVALIDADEAMBPR)) AND (ASF.DTINISITUFUNC >= :P_DTMESCOMPETENC OR (ASF.DTINISITUFUNC < :P_DTMESCOMPETENC AND ASF.DTFIMSITUFUNC < :P_DTMESCOMPETENC)) THEN MA.CDESOCIAL ELSE '' END AS codMotAfast,
       CASE WHEN ASF.DTINISITUFUNC >= TO_DATE(DECODE(:P_NRTPAMBIENTE, 1, DECODE(EMP.NRGRUPOESOCIAL, 3, :P_INIVALIDADEAMBNS, :P_INIVALIDADEAMBP), 2, :P_INIVALIDADEAMBPR)) AND (ASF.DTINISITUFUNC >= :P_DTMESCOMPETENC OR (ASF.DTINISITUFUNC < :P_DTMESCOMPETENC AND ASF.DTFIMSITUFUNC < :P_DTMESCOMPETENC)) THEN DECODE(MAANT.CDESOCIAL, NULL,'N', 'S') ELSE '' END AS infoMesmoMtv,  
       NULL AS tpAcidTransito,
       CASE WHEN ASF.DTINISITUFUNC >= TO_DATE(DECODE(:P_NRTPAMBIENTE, 1, DECODE(EMP.NRGRUPOESOCIAL, 3, :P_INIVALIDADEAMBNS, :P_INIVALIDADEAMBP), 2, :P_INIVALIDADEAMBPR)) AND (ASF.DTINISITUFUNC >= :P_DTMESCOMPETENC OR (ASF.DTINISITUFUNC < :P_DTMESCOMPETENC AND ASF.DTFIMSITUFUNC < :P_DTMESCOMPETENC)) THEN ASF.DSOBSALTSITFUNC ELSE '' END AS observacao,
       --perAquis
       '' dtInicio,
       '' dtFim,
       --fimAfastamento
       CASE WHEN ASF.DTFIMSITUFUNC >= TO_DATE(DECODE(:P_NRTPAMBIENTE, 1, DECODE(EMP.NRGRUPOESOCIAL, 3, :P_INIVALIDADEAMBNS, :P_INIVALIDADEAMBP), 2, :P_INIVALIDADEAMBPR)) AND ASF.DTFIMSITUFUNC <= LAST_DAY(:P_DTMESCOMPETENC) THEN TO_CHAR(ASF.DTFIMSITUFUNC, 'YYYY-MM-DD') ELSE '' END AS dtTermAfast,
       ASF.NRORG, ASF.NRALTESITUFUNC as NRCHAVE, 'AFASTAMENTO' as DSORIGEM
  FROM GPE_ALTESITUFUNC ASF
  JOIN GPE_VINCULOM VM ON VM.NRORG = ASF.NRORG
                      AND VM.NRVINCULOM = ASF.NRVINCULOM
  /*Dados do Empregado - Vínculo*/
  JOIN GPE_VINCULOH VH ON VH.NRVINCULOM = VM.NRVINCULOM
                      AND VH.NRORG      = VM.NRORG
                      AND VH.DTMESCOMPETENC = (SELECT * FROM(SELECT GPE_VINCULOH.DTMESCOMPETENC FROM GPE_VINCULOH
                                                WHERE GPE_VINCULOH.DTMESCOMPETENC <= :P_DTMESCOMPETENC
                                                  AND GPE_VINCULOH.DTMESCOMPETENC >= ADD_MONTHS(TRUNC(TO_DATE(:P_DTMESCOMPETENC),'YEAR'),-12)
                                                  AND GPE_VINCULOH.NRVINCULOM = VM.NRVINCULOM
                                                  AND GPE_VINCULOH.NRORG = VM.NRORG
                                                  ORDER BY GPE_VINCULOH.DTMESCOMPETENC DESC)WHERE ROWNUM = 1)   
   /*Categoria Trabalhador*/    
  JOIN GPE_VINCULOEMPREG VE ON VE.NRVINCULOEMPREG = VM.NRVINCULOEMPREG                           
  LEFT JOIN GPE_CATEGTRABESOC CE ON CE.NRCATEGESOC = VE.NRCATEGESOC  
  /*Dados do Empregado - Pessoa*/                               
  JOIN GPE_PESSOA PE ON PE.NRPESSOA = VM.NRPESSOA
                    AND PE.NRORG    = VM.NRORG
  JOIN GPE_PESSOAH PH ON PH.NRPESSOA = PE.NRPESSOA
                     AND PH.NRORG    = PE.NRORG
                     AND PH.DTMESCOMPETENC = (SELECT * FROM(SELECT GPE_PESSOAH.DTMESCOMPETENC FROM GPE_PESSOAH
                                               WHERE GPE_PESSOAH.DTMESCOMPETENC <= :P_DTMESCOMPETENC
                                                 AND GPE_PESSOAH.NRPESSOA = PE.NRPESSOA
                                                 AND GPE_PESSOAH.NRORG = PE.NRORG
                                                 ORDER BY GPE_PESSOAH.DTMESCOMPETENC DESC)WHERE ROWNUM = 1)
  JOIN PARCNEGOCIO PN ON PN.NRPARCNEGOCIO = PE.NRPARCNEGOCIO
                     AND PN.NRORG = PE.NRORG
  /*Empregador*/
  JOIN ESTRUTURAM EMP ON EMP.NRORG = :P_NRORG
                     AND EMP.NRESTRUTURAM = :P_NRESTRUTURAM
  JOIN ESTRUTURAH EMH ON EMH.NRORG = EMP.NRORG
                     AND EMH.NRESTRUTURAM = EMP.NRESTRUTURAM
                     AND EMH.DTMESCOMPETENC = (SELECT * FROM(SELECT ESTRUTURAH.DTMESCOMPETENC FROM ESTRUTURAH
                                                WHERE DTMESCOMPETENC <= :P_DTMESCOMPETENC
                                                  AND NRESTRUTURAM = EMH.NRESTRUTURAM
                                                  AND NRORG = EMH.NRORG
                                                  ORDER BY ESTRUTURAH.DTMESCOMPETENC DESC)WHERE ROWNUM = 1)
  JOIN PARCNEGOCIO PAR ON PAR.NRPARCNEGOCIO = EMH.NRPARCNEGOCIO
                      AND PAR.NRORG = EMH.NRORG
  /*Situação Funcional*/
  JOIN GPE_SITUFUNCM SF ON SF.NRSITUFUNCM = ASF.NRSITUFUNCM
  JOIN GPE_SITUFUNCH SFH ON SFH.NRSITUFUNCM = SF.NRSITUFUNCM
                        AND SFH.DTEFETIVA = (SELECT MAX(DTEFETIVA) FROM GPE_SITUFUNCH
                                              WHERE GPE_SITUFUNCH.DTEFETIVA <= :P_DTMESCOMPETENC
                                                AND GPE_SITUFUNCH.NRSITUFUNCM = SFH.NRSITUFUNCM
                                                AND GPE_SITUFUNCH.NRORG = SFH.NRORG)
  LEFT JOIN GPE_MOTIVOAFASTA MA ON MA.NRMOTIVOAFASTA = ASF.NRMOTIVOAFASTA
                               AND (MA.NRORG = ASF.NRORG OR MA.NRORG = 0)
  LEFT JOIN GPE_ALTESITUFUNC ASFANT ON ASFANT.NRVINCULOM = VM.NRVINCULOM
                                   AND ASFANT.NRORG = VM.NRORG
                                   AND ASFANT.DTFIMSITUFUNC BETWEEN ADD_MONTHS(ASF.DTINISITUFUNC, -2) AND ASF.DTINISITUFUNC
                                   AND ASFANT.DTFIMSITUFUNC = (SELECT MAX(ASFANT1.DTFIMSITUFUNC) FROM GPE_ALTESITUFUNC ASFANT1
                                                                WHERE ASFANT1.NRVINCULOM = ASFANT.NRVINCULOM
                                                                  AND ASFANT1.NRORG = ASFANT.NRORG
                                                                  AND ASFANT1.DTFIMSITUFUNC BETWEEN ADD_MONTHS(ASF.DTINISITUFUNC, -2) AND ASF.DTINISITUFUNC)
  LEFT JOIN GPE_SITUFUNCM SFANT ON SFANT.NRSITUFUNCM = ASFANT.NRSITUFUNCM
  LEFT JOIN GPE_SITUFUNCH SFHANT ON SFHANT.NRSITUFUNCM = SFANT.NRSITUFUNCM
                                AND SFHANT.DTEFETIVA = (SELECT MAX(DTEFETIVA) FROM GPE_SITUFUNCH
                                                         WHERE GPE_SITUFUNCH.DTEFETIVA <= :P_DTMESCOMPETENC
                                                           AND GPE_SITUFUNCH.NRSITUFUNCM = SFHANT.NRSITUFUNCM
                                                           AND GPE_SITUFUNCH.NRORG = SFHANT.NRORG)
                                AND (SFHANT.IDAFASTSITUFUNC = 'S' /*Situacao funcional que gere afastamento*/
                                    OR SFANT.NRSITUFUNCM = 12 OR SFANT.NRSITUFUNCM = 100012 OR SFANT.NRSITUFUNCM = 15 OR SFANT.NRSITUFUNCM = 16) /*Atestados médico e odontológico, Afastamentos 15 dias empregador*/
                                AND SFANT.NRSITUFUNCM <> 13 /*Demitito eh parametrizado como gera afastamento mas nao gera o evento*/
  LEFT JOIN GPE_MOTIVOAFASTA MAANT ON MAANT.NRMOTIVOAFASTA = ASFANT.NRMOTIVOAFASTA
                                  AND (MAANT.NRORG = ASFANT.NRORG OR MAANT.NRORG = 0)
                                  AND (MAANT.CDESOCIAL = '01' OR MAANT.CDESOCIAL = '03')
                                  AND MAANT.CDESOCIAL = MA.CDESOCIAL
  /*Envio do S2300 na versão S-1.0*/
  LEFT JOIN ESO_NAOPERIODICO NP ON NP.NRORG = VM.NRORG
                               AND NP.NRVINCULOM = VM.NRVINCULOM
                               AND NP.NMEVENTO = 'S-2300'
                               AND NP.IDATIVO = 'S'
                               AND (NP.IDSTATUS = 'C' OR NP.IDSTATUS = 'A')
                               AND NP.NRENVIOEXCL IS NULL
                               AND NP.NRTPAMBIENTE = :P_NRTPAMBIENTE
 WHERE ASF.NRORG = :P_NRORG
   AND ASF.NRALTESITUFUNC = :P_NRCHAVE
   AND (TO_CHAR(ASF.DTFIMSITUFUNC, 'YYYY-MM') = TO_CHAR(TO_DATE(:P_DTMESCOMPETENC),'YYYY-MM') OR TO_CHAR(ASF.DTINISITUFUNC, 'YYYY-MM') = TO_CHAR(TO_DATE(:P_DTMESCOMPETENC),'YYYY-MM') OR (ASF.DTINISITUFUNC < :P_DTMESCOMPETENC AND ASF.DTFIMSITUFUNC < :P_DTMESCOMPETENC))
SQL;
        $params = array(
            "P_NRORG" => $this->hcmParameters->getNrorgTrabalho(),
            "P_DTMESCOMPETENC" => $this->hcmParameters->getDtCompetencia(),
            "P_INIVALIDADEAMBP" => $this->hcmParameters->getEsoIniValidNaoPerProducao(),
            "P_INIVALIDADEAMBPR" => $this->hcmParameters->getEsoIniValidNaoPerProdRest(),
            "P_INIVALIDADEAMBNS" => $this->hcmParameters->getEsoIniValidNaoPerNacioSimples(),
            "P_NRCHAVE" => $nrchave,
            "P_NRESTRUTURAM" => $nrestruturam,
            "P_NRTPAMBIENTE" => $nrtpambiente
        );
        return $this->connection->fetchAssoc($sql, $params);
    }
    
    public function getDadosGozoFerias($nrchave, $nrestruturam, $nrtpambiente) {
        $sql = <<<SQL
/*getDadosS2230_GozoFerias*/
SELECT GF.NRGOZOFERIAS CHAVE,
       --ideEmpregador
       DECODE(PAR.CDTIPOINSCRICAO,'CNPJ','1','CPF','2','') TPINSC,
       CASE WHEN PAR.CDTIPOINSCRICAO = 'CPF' OR EMH.CDNATUJURI IN ('1015','1040','1074','1163') THEN PAR.NRINSCRICAOPARC ELSE SUBSTR(PAR.NRINSCRICAOPARC,1,8) END NRINSC,
       --ideVinculo
       REPLACE(REPLACE(PH.NRCPFPESSOA,'.'),'-') AS cpfTrab,
       CASE WHEN VM.NRTIPOVINCULOM IN (2,3,10) THEN DECODE(NP.NRENVIO,NULL,NULL,NVL(VH.CDMATRICULAESOCIAL, VM.NRVINCULOM)) ELSE NVL(VH.CDMATRICULAESOCIAL, VM.NRVINCULOM) END AS matricula,
       CASE WHEN VM.NRTIPOVINCULOM IN (2,3,10) THEN DECODE(NP.NRENVIO,NULL,CE.CDESOCIAL,NULL) ELSE '' END codCateg,
       --infoAfastamento
       --iniAfastamento
       CASE WHEN GF.DTINIGOZOFERIAS >= TO_DATE(DECODE(:P_NRTPAMBIENTE, 1, DECODE(EMP.NRGRUPOESOCIAL, 3, :P_INIVALIDADEAMBNS, :P_INIVALIDADEAMBP), 2, :P_INIVALIDADEAMBPR)) THEN TO_CHAR(GF.DTINIGOZOFERIAS, 'YYYY-MM-DD') ELSE NULL END AS dtIniAfast,
       CASE WHEN GF.DTINIGOZOFERIAS >= TO_DATE(DECODE(:P_NRTPAMBIENTE, 1, DECODE(EMP.NRGRUPOESOCIAL, 3, :P_INIVALIDADEAMBNS, :P_INIVALIDADEAMBP), 2, :P_INIVALIDADEAMBPR)) THEN '15' ELSE '' END AS codMotAfast, /*Gozo de férias ou recesso - Afastamento temporário para o gozo de férias ou recesso*/
       '' AS infoMesmoMtv,  
       '' AS tpAcidTransito,
       '' AS observacao,
       --perAquis
       CASE WHEN CE.CDESOCIAL IN ('101','102','103','104','105','106','107','108','111','301','302','303','304','306','307','309','310','312','410') OR VM.NRTIPOVINCULOM = 1 THEN TO_CHAR(FE.DTINIAQUISICAO, 'YYYY-MM-DD') ELSE NULL END dtInicio,
       CASE WHEN CE.CDESOCIAL IN ('101','102','103','104','105','106','107','108','111','301','302','303','304','306','307','309','310','312','410') OR VM.NRTIPOVINCULOM = 1 THEN TO_CHAR(FE.DTFIMAQUISICAO, 'YYYY-MM-DD') ELSE NULL END dtFim,
       --fimAfastamento
       TO_CHAR(GF.DTFIMGOZOFERIAS, 'YYYY-MM-DD') AS dtTermAfast,
       GF.NRORG, GF.NRGOZOFERIAS as NRCHAVE, 'FERIAS' as DSORIGEM
  FROM FPA_GOZOFERIAS GF
  JOIN FPA_FERIAS FE ON FE.NRORG = GF.NRORG
                    AND FE.NRFERIAS = GF.NRFERIAS
  JOIN GPE_VINCULOM VM ON VM.NRORG = FE.NRORG
                      AND VM.NRVINCULOM = FE.NRVINCULOM
  /*Dados do Empregado - Vínculo*/
  JOIN GPE_VINCULOH VH ON VH.NRVINCULOM = VM.NRVINCULOM
                      AND VH.NRORG      = VM.NRORG
                      AND VH.DTMESCOMPETENC = (SELECT * FROM(SELECT GPE_VINCULOH.DTMESCOMPETENC FROM GPE_VINCULOH
                                                WHERE GPE_VINCULOH.DTMESCOMPETENC <= :P_DTMESCOMPETENC
                                                  AND GPE_VINCULOH.DTMESCOMPETENC >= ADD_MONTHS(TRUNC(TO_DATE(:P_DTMESCOMPETENC),'YEAR'),-12)
                                                  AND GPE_VINCULOH.NRVINCULOM = VM.NRVINCULOM
                                                  AND GPE_VINCULOH.NRORG = VM.NRORG
                                                  ORDER BY GPE_VINCULOH.DTMESCOMPETENC DESC)WHERE ROWNUM = 1)
  /*Categoria Trabalhador*/    
  JOIN GPE_VINCULOEMPREG VE ON VE.NRVINCULOEMPREG = VM.NRVINCULOEMPREG                           
  LEFT JOIN GPE_CATEGTRABESOC CE ON CE.NRCATEGESOC = VE.NRCATEGESOC  
  /*Dados do Empregado - Pessoa*/                               
  JOIN GPE_PESSOA PE ON PE.NRPESSOA = VM.NRPESSOA
                    AND PE.NRORG    = VM.NRORG
  JOIN GPE_PESSOAH PH ON PH.NRPESSOA = PE.NRPESSOA
                     AND PH.NRORG    = PE.NRORG
                     AND PH.DTMESCOMPETENC = (SELECT * FROM(SELECT GPE_PESSOAH.DTMESCOMPETENC FROM GPE_PESSOAH
                                               WHERE GPE_PESSOAH.DTMESCOMPETENC <= :P_DTMESCOMPETENC
                                                 AND GPE_PESSOAH.NRPESSOA = PE.NRPESSOA
                                                 AND GPE_PESSOAH.NRORG = PE.NRORG
                                                 ORDER BY GPE_PESSOAH.DTMESCOMPETENC DESC)WHERE ROWNUM = 1)
  JOIN PARCNEGOCIO PN ON PN.NRPARCNEGOCIO = PE.NRPARCNEGOCIO
                     AND PN.NRORG = PE.NRORG  
  /*Empregador*/
  JOIN ESTRUTURAM EMP ON EMP.NRORG = :P_NRORG
                     AND EMP.NRESTRUTURAM = :P_NRESTRUTURAM
  JOIN ESTRUTURAH EMH ON EMH.NRORG = EMP.NRORG
                     AND EMH.NRESTRUTURAM = EMP.NRESTRUTURAM
                     AND EMH.DTMESCOMPETENC = (SELECT * FROM(SELECT ESTRUTURAH.DTMESCOMPETENC FROM ESTRUTURAH
                                                WHERE DTMESCOMPETENC <= :P_DTMESCOMPETENC
                                                  AND NRESTRUTURAM = EMH.NRESTRUTURAM
                                                  AND NRORG = EMH.NRORG
                                                  ORDER BY ESTRUTURAH.DTMESCOMPETENC DESC)WHERE ROWNUM = 1)
  JOIN PARCNEGOCIO PAR ON PAR.NRPARCNEGOCIO = EMH.NRPARCNEGOCIO
                      AND PAR.NRORG = EMH.NRORG
  /*Envio do S2300 na versão S-1.0*/
  LEFT JOIN ESO_NAOPERIODICO NP ON NP.NRORG = VM.NRORG
                               AND NP.NRVINCULOM = VM.NRVINCULOM
                               AND NP.NMEVENTO = 'S-2300'
                               AND NP.IDATIVO = 'S'
                               AND (NP.IDSTATUS = 'C' OR NP.IDSTATUS = 'A')
                               AND NP.NRENVIOEXCL IS NULL
                               AND NP.NRTPAMBIENTE = :P_NRTPAMBIENTE
 WHERE GF.NRORG = :P_NRORG
   AND GF.NRGOZOFERIAS = :P_NRCHAVE
SQL;
        $params = array(
            "P_NRORG" => $this->hcmParameters->getNrorgTrabalho(),
            "P_DTMESCOMPETENC" => $this->hcmParameters->getDtCompetencia(),
            "P_INIVALIDADEAMBP" => $this->hcmParameters->getEsoIniValidNaoPerProducao(),
            "P_INIVALIDADEAMBPR" => $this->hcmParameters->getEsoIniValidNaoPerProdRest(),
            "P_INIVALIDADEAMBNS" => $this->hcmParameters->getEsoIniValidNaoPerNacioSimples(),
            "P_NRCHAVE" => $nrchave,
            "P_NRESTRUTURAM" => $nrestruturam,
            "P_NRTPAMBIENTE" => $nrtpambiente,
        );
        return $this->connection->fetchAssoc($sql, $params);
    }

}