<?php
namespace HCM\Geral\Repository\Esocial;

use Doctrine\DBAL\Connection;
use Doctrine\ORM\EntityManager;
use HCM\Util\HCMParameters;

class S3500Repository {

    protected  $connection;

    public function __construct(EntityManager $entityManager, HCMParameters $hcmParameters) {
        $this->connection = $entityManager->getConnection();
        $this->entityManager = $entityManager;
        $this->hcmParameters = $hcmParameters;
    }

    public function getDadosS3500($parametros) {
        try {
            $dados = $this->getDadosPrincipais($parametros['NRENVIO'], $parametros['NRCHAVE'], $parametros['NRESTRUTURAM'], $parametros['NRENVIOORIG'], $parametros['NRCHAVEORIG']);
            return $dados;
        } catch(\Exception $e) {
            throw $e;
        }
    }
    
    public function getDadosPrincipais($nrenvio, $nrchave, $nrestruturam, $nrenvioorig, $nrchaveorig) {
        $sql = <<<SQL
/*getDadosS3500*/
SELECT EC.NRCHAVE CHAVE,
       --ideEmpregador
       DECODE(PAR.CDTIPOINSCRICAO,'CNPJ','1','CPF','2','') TPINSC,
       CASE WHEN PAR.CDTIPOINSCRICAO = 'CPF' OR EMH.CDNATUJURI IN ('1015','1040','1074','1163') THEN PAR.NRINSCRICAOPARC ELSE SUBSTR(PAR.NRINSCRICAOPARC,1,8) END NRINSC,   
       --infoExclusao
       EC.NMEVENTO tpEvento,
       EC.NRRECIBO nrRecEvt,
       --ideProcTrab
       REPLACE(REPLACE(PR.CDPROCESSTRAB,'.',''),'-','') nrProcTrab,
       DECODE(EC.NMEVENTO, 'S-2500', REPLACE(REPLACE(PH.NRCPFPESSOA,'.'),'-'), NULL) cpfTrab,
       DECODE(EC.NMEVENTO, 'S-2501', TO_CHAR(PR.DTPERPGTO, 'YYYY-MM'), NULL) perApurPgto
  FROM ESO_PERIODICO EC
  /*Empregador*/
  JOIN ESTRUTURAH EMH ON EMH.NRORG = :P_NRORG
                     AND EMH.NRESTRUTURAM = :P_NRESTRUTURAM
                     AND EMH.DTMESCOMPETENC = (SELECT * FROM(SELECT ESTRUTURAH.DTMESCOMPETENC FROM ESTRUTURAH
                                               WHERE ESTRUTURAH.DTMESCOMPETENC <= :P_DTMESCOMPETENC
                                                 AND ESTRUTURAH.NRESTRUTURAM = EMH.NRESTRUTURAM
                                                 AND ESTRUTURAH.NRORG = EMH.NRORG
                                                 ORDER BY ESTRUTURAH.DTMESCOMPETENC DESC)WHERE ROWNUM = 1)
  JOIN PARCNEGOCIO PAR ON PAR.NRPARCNEGOCIO = EMH.NRPARCNEGOCIO
                      AND PAR.NRORG = EMH.NRORG
  /*S-2500*/
  LEFT JOIN GPE_PESSOAH PH ON PH.NRORG = EC.NRORG
                          AND PH.NRPESSOA = EC.NRPESSOA
                          AND PH.DTMESCOMPETENC = (SELECT * FROM(SELECT GPE_PESSOAH.DTMESCOMPETENC FROM GPE_PESSOAH
                                                    WHERE GPE_PESSOAH.NRORG = PH.NRORG
                                                      AND GPE_PESSOAH.NRPESSOA = PH.NRPESSOA
                                                      AND GPE_PESSOAH.DTMESCOMPETENC <= :P_DTMESCOMPETENC
                                                      ORDER BY GPE_PESSOAH.DTMESCOMPETENC DESC)WHERE ROWNUM = 1)
  LEFT JOIN GPE_VINCULOM VM ON VM.NRORG = PH.NRORG
                           AND VM.NRPESSOA = PH.NRPESSOA
  LEFT JOIN GPE_INFOPROCTRAB IP ON IP.NRORG = VM.NRORG
                               AND IP.NRVINCULOM = VM.NRVINCULOM
  /*S-2501*/
  LEFT JOIN GPE_PROCESSTRAB PR ON PR.NRORG = EC.NRORG
                              AND PR.NRPROCESSTRAB = NVL(IP.NRPROCESSTRAB,EC.NRCHAVE)
 WHERE EC.NRORG = :P_NRORG
   AND EC.NRENVIO = :P_NRENVIOORIG
SQL;
        $params = array(
            "P_NRORG" => $this->hcmParameters->getNrorgTrabalho(),
            "P_DTMESCOMPETENC" => $this->hcmParameters->getDtCompetencia(),
            "P_NRESTRUTURAM" => $nrestruturam,
            "P_NRENVIOORIG" => $nrenvioorig,
            "P_NRPROCESSTRAB" => $nrchaveorig,
        );
        return $this->connection->fetchAssoc($sql, $params);
    }

}