<?php

namespace HCM\Geral\Services;

use Doctrine\ORM\EntityManager;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\DTO;
use Teknisa\Libs\Util\Environment;
use Zeedhi\Framework\DataSource\Manager\Doctrine\NameProvider;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\Manager\SQL\ManagerImpl;
use HCM\Util\HCMParameters;
use \HCM\FPA\Services\RegraService;
use \HCM\FPA\Services\IncidenciaEventoService;
use \HCM\FPA\Services\EventoService;
use \HCM\Geral\Services\ParametroEstruturaService;
use HCM\Geral\Services\ParametrizacaoEsocialService;
use HCM\Util\Repositories;

class CrudService {
    
    protected $dataSourceManager;
    protected $environment;
    protected $nameProvider;
    protected $parameterBag;
    protected $hcmParameters;
    private $columnsToReturn = array();

    public function __construct(ManagerImpl $dataSourceManager, Environment $environment, 
                                NameProvider $nameProvider, ParameterBag $parameterBag, HCMParameters $hcmParameters, RegraService $regraService, IncidenciaEventoService $incidenciaEventoService, EventoService $eventoService, ParametroEstruturaService $parametroEstruturaService, ParametrizacaoEsocialService $parametrizacaoEsocialService, EntityManager $entityManager) {
        $this->dataSourceManager = $dataSourceManager;
        $this->environment = $environment;
        $this->nameProvider = $nameProvider;
        $this->parameterBag = $parameterBag;
        $this->hcmParameters = $hcmParameters;
        $this->incidenciaEventoService = $incidenciaEventoService;
        $this->regraService = $regraService;
        $this->eventoService = $eventoService;
        $this->parametroEstruturaService = $parametroEstruturaService;
        $this->parametrizacaoEsocialService = $parametrizacaoEsocialService;
        $this->entityManager = $entityManager;
    }

    public function getRowsToSave($request, $dataSourceName) {
        $rows = array();

        $dataSource = $this->nameProvider->getDataSourceByName($dataSourceName);
        $request = $this->buildDataSetIfRow($request, $dataSourceName);
        $currentTime = $this->environment->getCurrentTime()->format("d/m/Y H:i:s");

        $this->columnsToReturn = array();
        foreach ($request->getDataSet()->getRows() as $row) {
            if (is_object($row))
                $row = (array) $row;

            $row['NRORG'] = (isset($row['NRORG']) && !empty($row['NRORG'])) ? $row['NRORG'] : $this->hcmParameters->getNrorgTrabalho();
            $row['IDATIVO'] = isset($row['IDATIVO']) ? ($row['IDATIVO'] ?: 'S') : 'S';

            /**
             * @todo Refatorar esta função
             */
            if ($row['__is_new']) {
                $row['DTULTATU'] = $currentTime;
                $row['NRORGULTATU'] = $this->hcmParameters->getNrorgTrabalho();
                $row['CDOPERULTATU'] = $this->environment->getCdOperador();
                $row['DTINCLUSAO'] = $currentTime;
                $row['NRORGINCLUSAO'] = $this->hcmParameters->getNrorgTrabalho();
                $row['CDOPERINCLUSAO'] = $this->environment->getCdOperador();

                if(!empty($row['__new_code_pk'])) {
                    $nrorg = $this->hcmParameters->getNrorgTrabalho();

                    if (isset($row['__nrorgpadrao']))
                        $nrorg = $row['__nrorgpadrao'];

                    $castNumber = empty($row['__cast_number']) ? false : $row['__cast_number'];
                    $newCodeKey = empty($row['__using_unit']) ? $row['__new_code_pk'] : ($row['__new_code_pk'].$this->environment->getCdFilial());
                    $newCodeKey = strtoupper($newCodeKey);
                    $length = isset($row['__new_code_length']) ? $row['__new_code_length'] : 10;
                    $interval = isset($row['__new_code_interval']) ? $row['__new_code_interval'] : 1;
                    
                    $row[strtoupper($row['__table_pk'])] = \HCM\Util\NovoCodigo::geraCodigo($nrorg, $newCodeKey, 1, 12, true);
                }
                
                $row = $this->getRowWithDefaultColumnsProperties($row, $dataSource);
                $this->validatePrimaryKeys($row, $dataSourceName);
            } else {
                $row['DTULTATU'] = $currentTime;
                $row['NRORGULTATU'] = $this->hcmParameters->getNrorgTrabalho();
                $row['CDOPERULTATU'] = $this->environment->getCdOperador();
                $row = $this->getOthersFields($dataSourceName, $dataSource->getPrimaryKeyColumns(), $row);
            }

            if (isset($row['__fieldsToValidateDuplication'])) {
                $this->validateDuplicatedFields($row, $dataSourceName);
            }

            $rows[] = $row;

            $this->buildColumnsToReturn($row);
        }
        return $rows;
    }

    public function getConditionsByColumns(array $columns, array $row, $isNew = false) {
        $conditions = array();

        foreach ($columns as $column) {
            if ($isNew && !isset($row[$column])) {
                throw new \Exception("Column $column cannot be null.", 1);
            }

            $conditions[] = array(
                'columnName' => $column,
                'operator' => '=',
                'value' => $row[$column],
            );
        }

        return $conditions;
    }

    public function getOthersFields($dataSourceName, $primaryKeys, $row) {
        $conditions = $this->getConditionsByColumns($primaryKeys, $row);
        $dataSet = $this->getFilteredDataSet($dataSourceName, $conditions);
        
        $dataSetRows = $dataSet->getRows();
        
        if (count($dataSetRows) !== 1)
            throw new \Exception('Cannot find row to update.', 1);

        return array_merge($dataSetRows[0], $row);
    }

    public function buildColumnsToReturn($row) {
        if (array_key_exists('__columns_to_return', $row)) {
            foreach ($row['__columns_to_return'] as $column) {
                $columnToReturn[$column] = $row[$column];
            }

            $this->addColumnToReturn($columnToReturn);
        }
    }

    public function addColumnToReturn(array $value) {
        $this->columnsToReturn[] = $value;
    }

    public function getColumnsToReturn() {
        return $this->columnsToReturn;
    }

    public function getRowsToDelete($request, $dataSourceName){
        $rows = array();

        $dataSource = $this->nameProvider->getDataSourceByName($dataSourceName);
        $request = $this->buildDataSetIfRow($request, $dataSourceName);
        $primaryKeys = $dataSource->getPrimaryKeyColumns();

        foreach ($request->getDataSet()->getRows() as $row) {
            $rowToDelete = array();

            if(!isset($row['NRORG']))
                $row['NRORG'] = $this->hcmParameters->getNrorgTrabalho();

            foreach ($primaryKeys as $primaryKey)
                $rowToDelete[$primaryKey] = $row[$primaryKey];

            $rowToDelete['__is_new'] = false;
            $rows[] = $rowToDelete;
        }

        return $rows;
    }

    public function save($dataSourceName, $rows) {
        $dataSet = new DataSet($dataSourceName, $rows);
        $this->dataSourceManager->persist($dataSet);
    }

    public function find($response, $filterCriteria) {
        $dataSet = $this->dataSourceManager->findBy($filterCriteria);
        $response->addDataSet($dataSet);
    }

    public function delete($response, $dataSet) {
        $this->dataSourceManager->delete($dataSet);
        $response->addMessage(new Message("Deletado com sucesso."));
    }

    public function buildDataSetIfRow(DTO\Request $request, $dataSourceName) {
        if ($request instanceof DTO\Request\Row) {
            $dataSet = new DataSet($dataSourceName, array((array)$request->getRow()));
            $request = new DTO\Request\DataSet($dataSet, $request->getMethod(), $request->getRoutePath(), $request->getUserId());
        }

        return $request;
    }

    public function validatePrimaryKeys($row, $dataSourceName) {
        $primaryKeys = $this->nameProvider->getDataSourceByName($dataSourceName)->getPrimaryKeyColumns();
        $conditions = $this->getConditionsByColumns($primaryKeys, $row, true);

        $dataSet = $this->getFilteredDataSet($dataSourceName, $conditions);
        $test = $dataSet->getRows();

        if (!empty($test)){
            throw new \Exception("Impossível adicionar. Registro já existe.", 1);
        }
    }

    private function getRowWithDefaultColumnsProperties($row, $dataSource) {
        
        if (method_exists($dataSource, 'getColumnsProperties')) {
            $columnsProperties = $dataSource->getColumnsProperties();

            if (isset($columnsProperties)) {
                foreach ($columnsProperties as $columnProperty) {
                    if (!array_key_exists($columnProperty['COLUMN_NAME'], $row)) {
                        if ($columnProperty['NULLABLE'] == 'Y') {
                            $row[$columnProperty['COLUMN_NAME']] = $this->getDataDefaultValue($columnProperty);
                        } else if ($columnProperty['COLUMN_NAME'] == 'CDFILIAL') {
                            $row[$columnProperty['COLUMN_NAME']] = $this->environment->getCdFilial();
                        } else if ($columnProperty['NULLABLE'] == 'N') {
                            if (isset($columnProperty['DATA_DEFAULT'])){
                                $row[$columnProperty['COLUMN_NAME']] = $this->getDataDefaultValue($columnProperty);
                            } else {
                                throw new \Exception("Column ".$columnProperty['COLUMN_NAME']." cannot be null and does not have a default value! Set a value for this column.", 1);
                            }
                        }
                    }
                }
            }
        }
        
        if($row['saveHistoryTable']['__table_history_name'] === "fpa_eventoh" && !$row['NRORDIMPRESEVEN']){
            $row['NRORDIMPRESEVEN'] = $row['NREVENTOM'];
        }

        return $row;
    }

    private function getDataDefaultValue($columnProperty) {
        if ($columnProperty['DATA_DEFAULT'] == null) {
            return null;
        } else {
            switch ($columnProperty['DATA_TYPE']) {
                case 'DATE':
                    return $this->environment->getCurrentTime()->format("d/m/Y H:i:s");
                    break;
                case 'NUMBER':
                    return intval($columnProperty['DATA_DEFAULT']);
                    break;
                case 'FLOAT':
                    return floatval($columnProperty['DATA_DEFAULT']);
                    break;
                default:
                    return $columnProperty['DATA_DEFAULT'];
                    break;
            }
        }
    }

    public function getDataSourceNameToSaveByName($name){
        $dataSource = $this->nameProvider->getDataSourceByName($name);

        return strtolower($dataSource->getTableName());
    }

    /**
     * @param FilterCriteria $filterCriteria
     * @return FilterCriteria
     */
    public function getFilterCriteriaWithNrOrg($filterCriteria) {
        $conditions = $filterCriteria->getConditions();

        foreach ($conditions as &$condition) {
            if ($condition['value'] === 'FILIAL_LOGADA') {
                $condition['value'] = $this->environment->getCdFilial();
            } else if ($condition['value'] === 'OPERADOR_LOGADO') {
                $condition['value'] = $this->environment->getCdOperador();
            }
        }

        $newFilterCriteria = new FilterCriteria($filterCriteria->getDataSourceName(), $conditions,
            $filterCriteria->getPage(), $filterCriteria->getPageSize());

        $this->keepOrderBy($filterCriteria, $newFilterCriteria);
        $nrorgFilter = array_filter($conditions, function($item) {
            return strtoupper($item['columnName']) == 'NRORG';
        });

        if (empty($nrorgFilter)) {
            $newFilterCriteria->addCondition('NRORG', '=', $this->hcmParameters->getNrorgTrabalho());
        }

        return $newFilterCriteria;
    }

    protected function processConditionValue($condition) {
        return $condition['value'];
    }

    public function populateParameterBag(FilterCriteria $filterCriteria, $parameterBagColumns) {
        $conditions = array();

        foreach ($filterCriteria->getConditions() as $condition) {
            $columnName = $condition['columnName'];
            if (in_array($columnName, $parameterBagColumns)) {
                $this->parameterBag->set($columnName, $this->processConditionValue($condition));
            } else {
                $conditions[] = $condition;
            }
        }

        $newFilterCriteria = new FilterCriteria(
            $filterCriteria->getDataSourceName(),
            $conditions,
            $filterCriteria->getPage(),
            $filterCriteria->getPageSize()
        );

        $this->keepOrderBy($filterCriteria, $newFilterCriteria);

        return $newFilterCriteria;
    }

    public function getFilteredDataSet($dataSourceName, $conditions, $page = null, $pageSize = null) {
        $filterCriteria = new FilterCriteria($dataSourceName, $conditions, $page, $pageSize);
        //$filterCriteria = $this->getFilterCriteriaWithNrOrg($filterCriteria); // quando organização for chave ela virá em $conditions
        $filterCriteria = $this->getPopulatedFilterCriteria($filterCriteria, array());
        $dataSet = $this->dataSourceManager->findBy($filterCriteria);

        return $dataSet;
    }

    public function getEnvironment() {
        return $this->environment;
    }

    /**
     * Keep old filter order by to new filter
     * @param  FilterCriteria $oldFilterCriteria
     * @param  FilterCriteria $newFilterCriteria
     */
    public function keepOrderBy(FilterCriteria $oldFilterCriteria, FilterCriteria $newFilterCriteria) {
        $orderBy = $oldFilterCriteria->getOrderBy();

        if (!empty($orderBy)) {
            foreach ($orderBy as $key => $value) {
                $newFilterCriteria->addOrderBy($key, $value);
            }
        }
    }

    /**
     * @param FilterCriteria $filterCriteria
     * @param $parameterBagColumns
     * @return FilterCriteria
     */
    public function getPopulatedFilterCriteria($filterCriteria, $parameterBagColumns) {
        $dataSourceName = $filterCriteria->getDataSourceName();
        $query = $this->nameProvider->getDataSourceByName($dataSourceName)->getQuery();

        if ($query) {
            if (empty($parameterBagColumns)) {
                $parameterBagColumns = $this->buildParamBag($filterCriteria, $query);
            }

            $filterCriteria = $this->populateParameterBag($filterCriteria, $parameterBagColumns);
        }

        return $filterCriteria;
    }
    
    public function atualizaDemaisInformacoes($row, $newCode, $prevInfo){
        if($row["__new_code_pk"] === 'fpa_eventom' || $row["__new_code_pk"] === 'fpa_eventoh'){
            $eventoh = $prevInfo;
            $parametros = ['NREVENTOM' => $row['NREVENTOM'], 'NRORG' => $row['NRORG']];

            if(($row['NRINCIDCP'] == 2 || $row['NRINCIDFGTS'] == 2 || $row['NRINCIDIRRF'] == 2 || $row['IDREPDECTERC'] == 'S' || $row['IDREPFERIAS'] == 'S' || $row['IDCOMPSALFER'] == 'S' || $row['IDCOMPSALPROV'] == 'S'
                || $row['IDINCIDEPERIC'] == 'S' || $row['IDINCIDEINSALUB'] == 'S' || $row['IDINCIDEPENSALIM'] == 'S' || $row['IDINCIDEPENSALIMFE'] == 'S' || $row['IDINCIDEDSRHORAEXT'] == 'S' || $row['IDINCIDEDSRADICNOT'] == 'S'
                || $row['IDINCIDECSIND'] == 'S' || $row['IDSALARIOFAMILIA'] == 'S' || $row['IDDEDUCAOINSS'] == 'S') || $row['__is_new'] == false){

                $this->incidenciaEventoService->atualizaIncidencias($row['NRINCIDCP'], $row['NRINCIDFGTS'], $row['NRINCIDIRRF'], $row['IDREPDECTERC'], $row['IDREPFERIAS'], $row['IDCOMPSALFER'], $row['IDCOMPSALPROV'], $row['IDINCIDEPERIC'], 
                                                                    $row['IDINCIDEINSALUB'], $row['IDINCIDEPENSALIM'], $row['IDINCIDEPENSALIMFE'], $row['IDINCIDEDSRHORAEXT'], $row['IDINCIDEDSRADICNOT'], $row['IDINCIDECSIND'], 
                                                                    $row['IDSALARIOFAMILIA'], $row['IDDEDUCAOINSS'],$row['NREVENTOM'], $row['NREVENTOH'], $row['IDTIPOEVENTO'], $newCode[0]['NREVENTOM'], $row['__is_new'], $eventoh);
            }
            if( ($row['IDTIPOEVENTO'] == 'PROVENTO' || $row['IDTIPOEVENTO'] == 'DESCONTO') && $row['IDTIPOOCORREVEN'] == 'EVENTUAL' && $this->hcmParameters->operadorEstaEmOrganizacaoCliente()){
                $this->regraService->insereRegrasEventoProventoEventual($row['NREVENTOM'], $row['NMEVENTOH'], $row['IDUNIDREFEREVEN'], $row['VRPERCENTUALEVENTO'],
                                                                       $row['IDTIPOEVENTO'], $row['IDTIPOOCORREVEN']);
            }

            if (isset($row['NRNATURUBR']) && $row['NRNATURUBR'] !== "" && $row['__is_new'] === true) {
                $this->parametrizacaoEsocialService->cadastroAutomaticoParametrizacaoEsocial($row['NREVENTOM']);
            }

            
            if($row['IDPROPORCEVENTO'] == 'S' && $this->hcmParameters->operadorEstaEmOrganizacaoCliente() && !$this->regraService->temRegraNoEvento($parametros)) { 
                $this->regraService->insereRegraProporcionalizaEvento($row['NREVENTOM'], $row['NMEVENTOH']);
            } else if ($row['IDPROPORCEVENTO'] == 'N') {
                $this->regraService->deleteRegraPropor($row['NREVENTOM'], $row['NRORG']);
            }
            
            if($row['IDINCIDEPENSALIMFE'] == 'S' && $this->hcmParameters->operadorEstaEmOrganizacaoCliente() && !$this->regraService->temRegraNoEvento($parametros)) {
                $this->regraService->insideParaPensãoAlimenEmFerias($row['NREVENTOM'], $row['NMEVENTOH']);
            } else if ($row['IDINCIDEPENSALIMFE'] == 'N') {
                $this->regraService->deleteRegraAlim($row['NREVENTOM'], $row['NRORG']);
            }
            
        } else if($row["__new_code_pk"] === 'fpa_paramfolham' || $row["__new_code_pk"] === 'fpa_paramfolhah'){
            if($row['__is_new']){
                $nrparamestrutm = array_key_exists('NRPARAMESTRUTM', $row) ? $row['NRPARAMESTRUTM'] : null;
                $nrparamfolham = $newCode[0]['NRPARAMFOLHAM'];
                $nrestruturam = array_key_exists('NRESTRUTURAM', $row) ? $row['NRESTRUTURAM'] : null;
                    
                $this->parametroEstruturaService->salvaParametrosEstruturasNaoPreenchidos($nrestruturam, $nrparamfolham);
            }
        } else if($row["__new_code_pk"] === 'fpa_paramferiasm' || $row["__new_code_pk"] === 'fpa_paramferiash'){
            if($row['__is_new']){
                $nrparamestrutm = array_key_exists('NRPARAMESTRUTM', $row) ? $row['NRPARAMESTRUTM'] : null;
                $nrparamferiasm = $newCode[0]['NRPARAMFERIASM'];
                $nrestruturam = array_key_exists('NRESTRUTURAM', $row) ? $row['NRESTRUTURAM'] : null;
                
                $this->parametroEstruturaService->relacionaParametroFeriasParametroEstrutura($nrestruturam, $nrparamestrutm, $nrparamferiasm);
            }
        }
    }
    
    public function getInformacoesPrevias($row){
        
        //se o registro em questão for um evento e já tiver um NREVENTOM, significa que é uma edição e os dados da tabela histórico do registro 
        //são salvos para utilização posterior, para o cadastro de incidências
        if($row["__new_code_pk"] === 'fpa_eventom' || $row["__new_code_pk"] === 'fpa_eventoh'){ 
            return $this->eventoService->getEventoh($row['NREVENTOM'], $row['NREVENTOH']);
        }
        
    }

    /**
     * @param FilterCriteria $filterCriteria
     * @param $query
     * @return array
     */
    private function buildParamBag($filterCriteria, $query) {
        $parameterBagColumns = array();
        $query = strtoupper($query);

        foreach ($filterCriteria->getConditions() as $condition) {
            $columnName = strtoupper($condition['columnName']);

            if (strpos($query, ':'.$columnName)) {
                $parameterBagColumns[] = $columnName;
            }
        }

        return $parameterBagColumns;
    }

    private function validateDuplicatedFields($row, $dataSourceName){
        $fieldsToValidate = $row['__fieldsToValidateDuplication'];

        foreach($fieldsToValidate as $field) {
            $fieldName = $field['name'];
            $conditions = [
                array(
                    "columnName" => $fieldName,
                    "operator"   => '=',
                    "value"      => $row[$fieldName]
                ),
                array(
                    "columnName" => "NRORG",
                    "operator"   => '=',
                    "value"      => $this->hcmParameters->getNrorgTrabalho()
                )
            ];

            $filterCriteria = new FilterCriteria($dataSourceName, $conditions);

            $newFilterCriteria = $this->getPopulatedFilterCriteria($filterCriteria, array());

            $dataSet = $this->dataSourceManager->findBy($newFilterCriteria);

            if(current($dataSet->getRows())) {
                $errorMsg = "O valor informado para o campo de ".$field['description']." já foi cadastrado.";
                throw new \Exception($errorMsg, 1);
            }
        }
    }
    
    
    public function verificaVigenciaEnvio($row){
        if(array_key_exists('IDVERIFICAVIGENCIAESO', $row) && $row['IDVERIFICAVIGENCIAESO'] == 'S'){
            $row['NRORG'] = isset($row['NRORG']) && !empty($row['NRORG']) ? $row['NRORG'] : $this->hcmParameters->getNrorgTrabalho();
            $envios = $this->entityManager->getRepository(Repositories::ESO_ENVIOCADASTROS)->getEnviosPorNomeEventoChave($row['NRORG'], $row['NMEVENTO'], $row['NRCHAVE'], 1);
            
            if(empty($envios)){
                return true;
            }else{
                return false;
            }
        }else{
            return true;
        }
    }
}