<?php

namespace HCM\PTO\Controllers;

use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use HCM\PTO\Services\MarcacaoPontoService;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request\Row;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Request;
use HCM\Util\HCMParameters;
use HCM\Util\DateUtil;
use HCM\Util\Messages;
use HCM\Util\Util;
use HCM\Util\FPA;
use HCM\PTO\Logic\AFD\Parametros;
use HCM\PTO\Logic\AFD\Portaria671\Exportador;
use HCM\Util\MsgProcesso;

class MarcacaoPontoController extends Crud {

    protected $parameterBagColumns = array('P_NRORG' , 'P_NRORG_PADRAO', 'P_DTMESCOMPETENC', 'NRPERIODOAPURACAO', 'NRVINCULOM_LIST', 'NRESTRUTLEGAL_LIST', 'NRESTRUTGEREN_LIST', 'P_CONDICAO', 'P_VALOR', 'P_TIPOOCORRENCIA', 'NRCOLETOR_LIST', 'NRTIPOVINCULOM_LIST', 'NRSITUFUNCM_LIST');
    
    protected const COLLUMNS_TO_BE_ALTERED_BEFORE_APPLY_FILTER = ['NRVINCULOM_LIST', 'NRESTRUTLEGAL_LIST', 'NRESTRUTGEREN_LIST', 'P_TIPOOCORRENCIA', 'NRCOLETOR_LIST', 'NRTIPOVINCULOM_LIST', 'NRSITUFUNCM_LIST', 'NRCOLETOR_LIST'];
    
    public function __construct (Manager $dataSourceManager, ParameterBag $parameterBag, HCMParameters $hcmParameters, MarcacaoPontoService $marcacaoPontoService){
        $this->parameterBag = $parameterBag;
        $this->hcmParameters   = $hcmParameters;
        $this->marcacaoPontoService = $marcacaoPontoService;
        parent::__construct($dataSourceManager);
    }

    public function getMarcacaoPonto (DTO\Request\Filter $request, DTO\Response $response) {
        try{
            $this->dataSourceName = "getMarcacaoPonto";
            $filterData = $request->getFilterCriteria();
            $filterData->setDataSourceName($this->dataSourceName);
            $filterData = Util::createNewFilterColumnConditionsToCommaImploded($filterData, self::COLLUMNS_TO_BE_ALTERED_BEFORE_APPLY_FILTER);
            $newFilterData = $this->populateParameterBag($filterData);
            $dataSet = $this->dataSourceManager->findBy($newFilterData); 
            $response->addDataSet($dataSet);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function getMarcacaoPontoIntervalo (DTO\Request\Filter $request, DTO\Response $response) {
        try{
            $this->dataSourceName = "getMarcacaoPontoIntervalo";
            $filterData = $request->getFilterCriteria();
            $filterData->setDataSourceName($this->dataSourceName);
            $newFilterData = $this->populateParameterBag($filterData);
            $dataSet = $this->dataSourceManager->findBy($newFilterData); 
            $response->addDataSet($dataSet);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function getMarcacaoPontoEspelho (DTO\Request\Filter $request, DTO\Response $response) {
        try{
            $filterData = $request->getFilterCriteria();

            $newFilter = array();
            
            foreach($filterData->getConditions() as $filter) {
                if(is_array($filter['value'])) {
                    $newFilter[$filter['columnName']] = "'" . implode("', '", $filter['value']) . "'";
                } else {
                    $newFilter[$filter['columnName']] = $filter['value'];
                }
            }
            
            $dataset = $this->marcacaoPontoService->getMarcacaoPontoEspelho($newFilter);
            
            $response->addDataSet(new DataSet('getMarcacaoPontoEspelho', $dataset));
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function getMarcacaoPontoOcorrencias (DTO\Request\Filter $request, DTO\Response $response) {
        try{
            $this->dataSourceName = "getMarcacaoPontoOcorrencias";
            $filterData = $request->getFilterCriteria();
            $filterData->setDataSourceName($this->dataSourceName);
            $filterData = Util::createNewFilterColumnConditionsToCommaImploded($filterData, self::COLLUMNS_TO_BE_ALTERED_BEFORE_APPLY_FILTER);
            $newFilterData = $this->populateParameterBag($filterData);
            $dataSet = $this->dataSourceManager->findBy($newFilterData); 
            $response->addDataSet($dataSet);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function setMarcacaoPonto (DTO\Request\DataSet $request, DTO\Response $response) {
        
        try{
            $this->dataSourceName = "GpeMarcacaoponto_setMarcacaoPonto";
            foreach($request->getDataSet()->getRows() as &$row) {
                if(array_key_exists('NRMARCACAOPONTO', $row)){
                    if(empty($row['NRMARCACAOPONTO'])){
                        $row['NRMARCACAOPONTO'] = \HCM\Util\NovoCodigo::geraCodigo($row['NRORG'], 'GPE_MARCACAOPONTO', 1, 12, true);
                    }
                } else {
                    $row['NRMARCACAOPONTO'] = \HCM\Util\NovoCodigo::geraCodigo($row['NRORG'], 'GPE_MARCACAOPONTO', 1, 12, true);
                }
                if(array_key_exists('IDATIVO', $row)){
                    if($row['IDATIVO'] === ''){
                    
                    $row['IDATIVO'] = 'S';
                    }
                } else {
                    $row['IDATIVO'] = 'S';
                }
                
                // Trata caso de data e hora da marcação serem salvos em campos separados
                if(!empty($row['DTHRMARCACAOPONTO_DATA'])){
                    $horaMarcacaoPonto              = !empty($row['DTHRMARCACAOPONTO_HORA']) ? $row['DTHRMARCACAOPONTO_HORA'] . ':00' : '00:00:00';
                    $dataHoraMarcacaoPontoString    = $row['DTHRMARCACAOPONTO_DATA'] . ' ' . $horaMarcacaoPonto;
                    $dataHoraMarcacaoPonto          = DateUtil::getDataDeString($dataHoraMarcacaoPontoString, DateUtil::FORMATO_DATAHORA);
                    if (is_null($dataHoraMarcacaoPonto)) {
                        throw new \Exception(Messages::HCM07003_PTO_SAVE_MARC_DATA_HORA_MARCACAO_ILLEGAL);
                    }
                    $row['DTHRMARCACAOPONTO']       = $dataHoraMarcacaoPonto->format(DateUtil::FORMATO_DATAHORA);
                }
                unset($row['DTHRMARCACAOPONTO_HORA']);
                unset($row['DTHRMARCACAOPONTO_DATA']);
                
                // Trata caso de data e hora ajustadas da marcação serem salvos em campos separados
                if(!empty($row['DTHRAJUSTADA_DATA'])){
                    $horaMarcacaoPontoAjust             = !empty($row['DTHRAJUSTADA_HORA']) ? $row['DTHRAJUSTADA_HORA'] . ':00' : '00:00:00';
                    $dataHoraMarcacaoPontoAjustString   = $row['DTHRAJUSTADA_DATA'] . ' ' . $horaMarcacaoPontoAjust;
                    $dataHoraMarcacaoPontoAjust         = DateUtil::getDataDeString($dataHoraMarcacaoPontoAjustString, DateUtil::FORMATO_DATAHORA);
                    $row['DTHRAJUSTADA']                = $dataHoraMarcacaoPontoAjust->format(DateUtil::FORMATO_DATAHORA);
                }
                unset($row['DTHRAJUSTADA_HORA']);
                unset($row['DTHRAJUSTADA_DATA']);
            }
            parent::save($request, $response);
            
            // Atualizar ocorrência de almoço, caso exista?
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
     public function delete(DTO\Request\DataSet $request, DTO\Response $response){
        try{
            $this->dataSourceName = "GpeMarcacaoponto_setMarcacaoPonto";
            parent::delete($request, $response);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function importaArquivoParaMarcacao(DTO\Request\DataSet $request, DTO\Response $response){
        try{
            $row = $request->getDataSet()->getRows();
            $row = $row[0];
            $row['NRORG'] = $this->hcmParameters->getNrorgTrabalho();
            $row['NRORG_PADRAO'] = $this->hcmParameters->getNrorgPadrao();
            $row['MESCOMPETENCIA'] = $this->hcmParameters->getDtCompetencia();
            foreach ($row['DSPATH'] as $arquivo) {
                $row['DSPATH'] = [$arquivo];
                $mensagem = $this->marcacaoPontoService->importaArquivoParaMarcacao($row);    
                // $mensagem = $this->marcacaoPontoService->importaArquivoParaMarcacao_og($row);    
            }
            if(array_key_exists('erro', $mensagem)){
                $message = new \Zeedhi\Framework\DTO\Response\Message($mensagem['erro'], 'E');
                $response->addMessage($message);
            }
            if($mensagem['sucesso']){
                $message = new \Zeedhi\Framework\DTO\Response\Message($mensagem['sucesso'], 'S');
                $response->addMessage($message);
            }
            
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function geraArquivoAFD(DTO\Request\Row $request, DTO\Response $response) {
        try{
            $row = $request->getRow()->getArrayCopy();
            $exportador = $this->criaExportadorFromRow($row);
            $nomeArquivo = $exportador->gerarNomeArquivo();
            file_put_contents(Util::getTmpFileFullPath($nomeArquivo), $exportador->gerarConteudoArquivo());
            $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message('Exportação concluída com sucesso.', 'S'));
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('data', ['file' => Util::getTmpFileFullPath($nomeArquivo)]));
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function setMarcacoesCadastroAutomatico(DTO\Request\Row $request, DTO\Response $response) {
        try{
            $row = $request->getRow();
            
            $diaInicio  = isset($row['MARC_DAYS']) ? $row['MARC_DAYS']['formattedStart'] : null;
            $diaFim     = isset($row['MARC_DAYS']) ? $row['MARC_DAYS']['formattedEnd'] : null;
            
            if (is_null($diaInicio) || is_null($diaFim)) {
                throw new \Exception(Messages::HCM07003_CADAUTO_UNSUFICCIENT_ITEMS, Messages::ALERT_MESSAGE_CODE);
            }
            
            $nrvinculomList     = $row['NRVINCULOM_LIST'] ?? null;
            $nrestrutlegalList  = $row['NRESTRUTLEGAL_LIST'] ?? null;
            $nrestrutgerenList  = $row['NRESTRUTGEREN_LIST'] ?? null;
            $conformeHorarioBase  = !empty($row['IDCONFHORARIOBASE']) ? $row['IDCONFHORARIOBASE'] == 'S' : false;
            
            $marcacaoList = $row['MARCACAO_LIST'];
            
            $result = $this->marcacaoPontoService->setMarcacoesCadastroAutomatico($diaInicio, $diaFim, $marcacaoList, $nrestrutlegalList, $nrestrutgerenList, $nrvinculomList, $conformeHorarioBase);

            $msg = [];
            $msgProcesso = MsgProcesso::getInstance();
            if(!empty($msgProcesso->getMessages())) {
                foreach($msgProcesso->getMessages() as $mensagem) {
                    $msg[] = array('msg' => $mensagem['MSG']);
                }
            }

            $dataSet = array(
                'success' => $result,
                'msg'     => $msg
            );
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('data', $dataSet));
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function criaExportadorFromRow(array $row): Exportador {
        $parametros = new Parametros(
            $this->hcmParameters->getNrorgTrabalho(),
            $this->hcmParameters->getNrorgPadrao(),
            $this->hcmParameters->getDtCompetencia(),
            $row['NRPERIODOAPURACAO'],
            $row['DTINICIALAPURACAOFILTRO'] ?? null,
            $row['DTFIMAPURACAOFILTRO'] ?? null,
            $row['NRESTRUTURAMEMPREGADOR'],
            $row['NRVINCULOM_LIST'] ?? [],
            $row['NRTIPOVINCULOM_LIST'] ?? [],
            $row['NRESTRUTLEGAL_LIST'] ?? [],
            $row['NRESTRUTGEREN_LIST'] ?? []
        );
        return new Exportador($parametros);
    }

    protected function processConditionValue($condition) {
        return $condition['value'];
    }
    protected function populateParameterBag(FilterCriteria $filterCriteria) {

        $conditions = array();
        foreach ($filterCriteria->getConditions() as $condition) {
            $columnName = $condition['columnName'];
            if (in_array($columnName, $this->parameterBagColumns)) {
                $this->parameterBag->set($columnName, $this->processConditionValue($condition));
            } else {
                $conditions[] = $condition;
            }
        }

        $this->parameterBag->set('NRORG', $this->hcmParameters->getNrorgTrabalho());  
    

        $newFilterCriteria = new FilterCriteria(
            $filterCriteria->getDataSourceName(),
            $conditions,
            $filterCriteria->getPage(),
            $filterCriteria->getPageSize()
        );


        return $newFilterCriteria;
    }

    public function importarAFDdireto(DTO\Request\DataSet $request, DTO\Response $response){ 
        try{
            $dataReturn = [];
            $row = $request->getDataSet()->getRows()[0];
            $nrpontoacesso = $row['NRPONTOACESSO'];
            $params = [
                'requestType' => 'Row',
                'row' => [
                    'NRORG'          => $this->hcmParameters->getNrorgTrabalho(),
                    'NRORG_PADRAO'   => $this->hcmParameters->getNrorgPadrao(),
                    'MESCOMPETENCIA' => $this->hcmParameters->getDtCompetencia(),
                    'DTINIFILTRO'    => $row['DTINIFILTRO'],
                    'NRPONTOACESSO'  => []
                ]
            ];

            foreach($nrpontoacesso as $index => $value) {
                $params['row']['NRPONTOACESSO'][$value] = [
                    'DSPONTOACESSO' => $row['DSPONTOACESSO'][$index],
                    'IPHOSTREP' => $row['IPHOSTREP'][$index],
                    'DSLOGINREP' => $row['DSLOGINREP'][$index],
                    'DSSENHAREP' => $row['DSSENHAREP'][$index],
                ];
            }
            
            $result = $this->marcacaoPontoService->importarAFDdireto($params);

            if(is_array($result) && array_key_exists('dataset', $result)){
                if(array_key_exists('loginREP', $result['dataset'])){

                    foreach($result['dataset']['loginREP'] as $index => $arrData) {

                        if(array_key_exists('erro', $arrData['result'])){
                            $dataReturn[]['msg'] = "Ocorreu um erro na sincronização com o ponto de acesso $index - ".$arrData['dspontoacesso'].". Erro: ".$arrData['result']['erro'];
                        }
                        if(array_key_exists('sucesso', $arrData['result'])){
                            $dataReturn[]['msg'] = "Sincronização realizada com o ponto de acesso $index - ".$arrData['dspontoacesso'].". Resultado: ".$arrData['result']['sucesso'];
                        }
                    }
                }
            }
            $response->addDataSet(new DataSet('importarAFDdireto', $dataReturn));

        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
}