<?php
namespace HCM\PTO\Services;

use HCM\PTO\Repository\GerarLoteEventosRepository;
use Doctrine\ORM\EntityManager;
use HCM\FPA\Services\LoteLancEventoService;   
use HCM\FPA\Services\ItemLoteLancService;
use HCM\Util\DateUtil;
use HCM\Util\Util;
use HCM\Util\Repositories;

class GerarLoteEventosService {

    /**
     * @var gerarLoteEventosRepository
     */
    protected $gerarLoteEventosRepository;
    protected $loteLancEventoService;
    
    public static $dslotelancTemplate               = 'SITUAÇÃO DE FREQUÊNCIA | ${TIPO_VINCULO} | ${HORA_GERACAO} | ${DSFORMGERACAOEVENTO} | ${DSORDEM} | ${DSINICIO} | ${DSFIM} | [${DATA_INICIO} - ${DATA_FIM}]';
    public static $dslotelancTemplateSemFim         = 'SITUAÇÃO DE FREQUÊNCIA | ${TIPO_VINCULO} | ${HORA_GERACAO} | ${DSFORMGERACAOEVENTO} | ${DSORDEM} | ${DSINICIO} | [${DATA_INICIO} - ${DATA_FIM}]';
    public static $dslotelancTemplateSemCriterio    = 'SITUAÇÃO DE FREQUÊNCIA | ${TIPO_VINCULO} | ${HORA_GERACAO} | ${DSFORMGERACAOEVENTO} | ${DSORDEM} | [${DATA_INICIO} - ${DATA_FIM}]';

    const DSLOTEFREQ = 'SITUAÇÃO DE FREQUÊNCIA | [HORA_GERACAO] | [DATA_INICIO] - [DATA_FIM]';

    const OBSLOTEFREQ = <<<END
Critérios Utilizados
Nome do formulário: [DSFORMGERACAOEVENTO]
Critério: [DSORDEM]
Período Apuração: [PERIODO_APURACAO]
Critério 1: [DSINICIO]
Critério 2: [DSFIM]
Data Inicial: [DATA_INICIO]
Data Final: [DATA_FIM]
Gerado em: [HORA_GERACAO]
END;

    public function __construct(EntityManager $entityManager, GerarLoteEventosRepository $gerarLoteEventosRepository, LoteLancEventoService $loteLancEventoService, ItemLoteLancService $itemLoteLancService) {
        $this->entityManager = $entityManager;
        $this->gerarLoteEventosRepository = $gerarLoteEventosRepository;
        $this->loteLancEventoService = $loteLancEventoService;
        $this->itemLoteLancService = $itemLoteLancService;
    }

    /**
     * @return int
     */    
    public function getResumoPorSelecao ($criterio, $inicio, $fim, $dtCompetencia, $nrTipoVinculo, $nrPeriodoApuracao, $dtPeriodoIni, $dtPeriodoFim, $nrOrgPadrao, $nrOrg, $lote = null) {
        $result = $this->gerarLoteEventosRepository->getResumoPorSelecao($criterio, $inicio, $fim, $dtCompetencia, $nrTipoVinculo, $nrPeriodoApuracao, $dtPeriodoIni, $dtPeriodoFim, $nrOrgPadrao, $nrOrg, $lote);
        return $result;
    }

    public function getResumoParaLote ($criterio, $inicio, $fim, $dtCompetencia, $nrTipoVinculo, $nrPeriodoApuracao, $dtPeriodoIni, $dtPeriodoFim, $nrOrgPadrao, $nrOrg) {
        $result = $this->gerarLoteEventosRepository->getResumoParaLote($criterio, $inicio, $fim, $dtCompetencia, $nrTipoVinculo, $nrPeriodoApuracao, $dtPeriodoIni, $dtPeriodoFim, $nrOrgPadrao, $nrOrg);
        return $result;
    }
    public function geraLote($criterio, $inicio, $fim, $dtCompetencia, $nrTipoVinculo, $nrPeriodoApuracao, $dtPeriodoIni, $dtPeriodoFim, $nrOrgPadrao, $nrOrg, $nmTipoVinculo, $dsFormGeracaoEvento, $dsOrdem) {
        set_time_limit(0);
        error_reporting(E_ERROR | E_PARSE);
        $msg = array();
        $resumos = $this->getResumoPorSelecao($criterio, $inicio, $fim, $dtCompetencia, $nrTipoVinculo, $nrPeriodoApuracao, $dtPeriodoIni, $dtPeriodoFim, $nrOrgPadrao, $nrOrg, 'S');
        $resumosLote = $this->getResumoParaLote($criterio, $inicio, $fim, $dtCompetencia, $nrTipoVinculo, $nrPeriodoApuracao, $dtPeriodoIni, $dtPeriodoFim, $nrOrgPadrao, $nrOrg);

        if(!empty($resumosLote)){
            foreach ($resumosLote as $item) {
                $disponibilidade = true;
                if (isset($nrTipoVinculo)) {
                    $bloq = $this->itemLoteLancService->verificaEventoBloqueado($nrOrg, $item['NREVENTOM'], $nrTipoVinculo);
                    if ($bloq !== null){
                        if ($bloq['NREVENTOMPARA'] === null) {
                            $this->msg[] = array('msg' => 'Operação bloqueada. O evento '.$item['NREVENTOM'].' não pode ser lançado para o vínculo "'.$item['NRVINCULOM'].' - '.$item['NMVINCULOM']. '"!');
                            $disponibilidade = false;
                        } else {
                            $msg[] = array('msg' => 'Operação bloqueada.  O evento '.$item['NREVENTOM'].' não pode ser lançado para o vínculo "'.$item['NRVINCULOM'].' - '.$item['NMVINCULOM'].'" e foi ajustado para o evento '.$bloq['NREVENTOMPARA'].'!');
                            $disponibilidade = true;
                            $item['NREVENTOM'] = $bloq['NREVENTOMPARA'];
                        }
                    } else {
                        $disponibilidade = true;
                    }
                }
                
                if ($disponibilidade === true) {
                    $nrsitufrequencia = '';
                    
                    $info = (is_null($item['QTREFAPURSITUFREQ'])) ? 0 : $item['QTREFAPURSITUFREQ'];
                    
                    $ref = $this->gerarLoteEventosRepository->getIdmediavalref($item['NREVENTOM'], $dtCompetencia);
                    
                    $info = DateUtil::getHoraStringDeFloat($info);
                    $info = DateUtil::getHoraFloatDeString($info);
                    if ($ref == 'REFERENCIA') {
                        $vrRefFolhaLote = $info;
                        $vrRefEvento = '';
                    } else {
                        $vrRefFolhaLote = '';
                        $vrRefEvento = $info;
                    }
                    
                    $itensLote[] = array(
                        'NRVINCULOM' => $item['NRVINCULOM'],
                        'NMVINCULOM' => '',
                        'NREVENTOM' => $item['NREVENTOM'],
                        'NMEVENTOH' => '',
                        'VRREFFOLHALOTE' => $vrRefFolhaLote,
                        'VREVENTOLOTE' => $vrRefEvento
                    );
                }
            }
            
            $this->loteLancEventoService->setOrigemLoteLanc('PTO');
            
            // Gera o nome do lote a partir do template definido
            $dslotelanc = $this->montaNomeLote($inicio, $fim, $nmTipoVinculo, $dsFormGeracaoEvento, $dsOrdem, $dtPeriodoIni, $dtPeriodoFim, $nrOrg);

            $origem = 'RESUMOSITUFREQ';

            $nrlote = $this->loteLancEventoService->geraLoteLancEvento($nrOrg, $dtCompetencia, $origem, $dslotelanc, 'PENDENTE', null, 1, $itensLote);
        } else {
            $nrlote = null;
        }

        try {
            $this->entityManager->getConnection()->beginTransaction();
            
            if(!empty($resumos)){                
                foreach ($resumos as $item) {
                    $resumoSituFreq = $this->entityManager->getRepository('\Folha\Entities\PtoResumositufreq')
                        ->findOneBy(array('nrresumositufreq' => $item['NRRESUMOSITUFREQ'], 'nrorg' => $item['NRORG']));
                    $resumoSituFreq->setNrlotelancevento($nrlote);
                    $resumoSituFreq->persist();
                }
            }

            $this->entityManager->flush();
            $this->entityManager->getConnection()->commit();
            
            $retorno = array($nrlote, $msg);
            return $retorno;
        } catch (\Exception $e) {
            $this->entityManager->getConnection()->rollBack();
            throw $e;
        }
    }

    /**
     * Cria um lote de eventos de folha de acordo com calculos de frequencia
     * @param string $nrFormGeracaoEvento
     * @return array
     */
    public function novoGeraLote(
        string $nrFormGeracaoEvento
    ): array {
        set_time_limit(0);
        error_reporting(E_ERROR | E_PARSE);

        $nrOrg = Util::getHCMParameters()->getNrorgTrabalho();
        $dtCompetencia = Util::getHCMParameters()->getDtCompetencia();
        $msg = array();
        $totalizadores = $this->gerarLoteEventosRepository->getTotalizadoresEventos($nrFormGeracaoEvento);

        foreach ($totalizadores as $totalizador) {

            /* Analise bloqueio de eventos */

            $bloq = $this->itemLoteLancService->verificaEventoBloqueado($nrOrg, $totalizador['NREVENTOM'], $totalizador['NRTIPOVINCULOM']);
            if ($bloq !== null) {
                if ($bloq['NREVENTOMPARA'] === null) {
                    $msg[] = array('msg' => 'Operação bloqueada. O evento '.$totalizador['NREVENTOM'].' não pode ser lançado para o vínculo "'.$totalizador['NRVINCULOM'].' - '.$totalizador['NMVINCULOM']. '"!');
                    $disponibilidade = false;
                } else {
                    $msg[] = array('msg' => 'Operação bloqueada.  O evento '.$totalizador['NREVENTOM'].' não pode ser lançado para o vínculo "'.$totalizador['NRVINCULOM'].' - '.$totalizador['NMVINCULOM'].'" e foi ajustado para o evento '.$bloq['NREVENTOMPARA'].'!');
                    $disponibilidade = true;
                    $totalizador['NREVENTOM'] = $bloq['NREVENTOMPARA'];
                }

                continue;
            }

            /* Definicao de referencia/valor */
                    
            $info = (is_null($totalizador['QTREFAPURSITUFREQ'])) ? 0 : $totalizador['QTREFAPURSITUFREQ'];
            
            $ref = $this->gerarLoteEventosRepository->getIdmediavalref($totalizador['NREVENTOM'], $dtCompetencia);
            
            $info = DateUtil::getHoraStringDeFloat($info);
            $info = DateUtil::getHoraFloatDeString($info);
            if ($ref == 'REFERENCIA') {
                $vrRefFolhaLote = $info;
                $vrRefEvento = '';
            } else {
                $vrRefFolhaLote = '';
                $vrRefEvento = $info;
            }

            /* Item que sera adicionado no novo lote */
            
            $itensLote[] = array(
                'NRVINCULOM' => $totalizador['NRVINCULOM'],
                'NMVINCULOM' => '',
                'NREVENTOM' => $totalizador['NREVENTOM'],
                'NMEVENTOH' => '',
                'VRREFFOLHALOTE' => $vrRefFolhaLote,
                'VREVENTOLOTE' => $vrRefEvento
            );
        }

        /* Parametrizacoes da criacao do lote */

        $this->loteLancEventoService->setOrigemLoteLanc('PTO');

        $dslotelanc = $this->montaNovoNomeLote($nrFormGeracaoEvento);
        $obslotelanc = $this->montaNovaObservacaoLote($nrFormGeracaoEvento);
        $origem = 'RESUMOSITUFREQ';
        $nrLote = $this->loteLancEventoService->geraLoteLancEvento($nrOrg, $dtCompetencia, $origem, $dslotelanc, 'PENDENTE', $obslotelanc, 1, $itensLote);

        /* Atualizacao dos resumos com o numero do novo lote */

        if(empty($nrLote)){
            return [$nrLote, $msg]; 
        }

        $nrResumos = array_reduce(
            $totalizadores,
            function ($acumulador, $totalizador) {
                return array_merge($acumulador, explode(',', $totalizador['LISTA_NRRESUMOSITUFREQ'] ?? ''));
            },
            []
        );

        $this->atualizaResumosNovoLote(
            $nrResumos, $nrLote
        );

        return [$nrLote, $msg];
    }
    
    /**
     *  Rotina responsável por montar o nome do lote a ser gerado pelo frequência
     * 
     *  @param  string  $dsInicio
     *  @param  string  $dsFim
     *  @param  string  $nmTipoVinculo
     *  @param  string  $dsFormGeracaoEvento
     *  @param  string  $dsOrdem
     *  @param  string  $dtPeriodoIni
     *  @param  string  $dtPeriodoFim
     * 
     *  @return string  $dslotelanc
     * 
     */
    public function montaNomeLote($dsInicio, $dsFim, $nmTipoVinculo, $dsFormGeracaoEvento, $dsOrdem, $dtPeriodoIni, $dtPeriodoFim, $nrOrg) {
        $nomeInicio = $dsInicio;
        $nomeFim    = $dsFim;
        $template   = self::$dslotelancTemplate;
        
        switch ($dsOrdem) {
            case 'ESTRUTURA':
                $estruturamInicio = $this->entityManager->getRepository(Repositories::ESTRUTURAM)->findOneBy(array('nrorg' => $nrOrg, 'nrestruturam' => $dsInicio));
                $estruturamFim    = $this->entityManager->getRepository(Repositories::ESTRUTURAM)->findOneBy(array('nrorg' => $nrOrg, 'nrestruturam' => $dsFim));
                
                $nomeInicio = !empty($nomeInicio)   && !empty($estruturamInicio)    ? $estruturamInicio->getNmestruturam()  : '';
                $nomeFim    = !empty($nomeFim)      && !empty($estruturamFim)       ? $estruturamFim->getNmestruturam()     : '';
                
                if (empty($nomeFim)) {
                    $template = self::$dslotelancTemplateSemFim;
                }
                
                break;
            case 'TODOS':
                $template = self::$dslotelancTemplateSemCriterio;
                break;
        }
        
        $dslotelanc = Util::stringApplyTemplate(
            $template,
            array(
                '${TIPO_VINCULO}'           => $nmTipoVinculo,
                '${HORA_GERACAO}'           => date('H:i:s'),
                '${DSFORMGERACAOEVENTO}'    => $dsFormGeracaoEvento,
                '${DSORDEM}'                => $dsOrdem,
                '${DSINICIO}'               => $nomeInicio,
                '${DSFIM}'                  => $nomeFim,
                '${DATA_INICIO}'            => substr($dtPeriodoIni, 0, 10),
                '${DATA_FIM}'               => substr($dtPeriodoFim, 0, 10),
            )
        );
        
        return $dslotelanc;
    }

    /**
     * Monta o nome do novo lote a ser gerado acordo com os criterios do formulario
     * @param string $nrFormGeracaoEvento
     * @return string
     */
    public function montaNovoNomeLote(
        string $nrFormGeracaoEvento
    ): string {
        $formGeracao = $this->entityManager->getRepository(Repositories::PTO_FORMGERACAOEVENTO)->findOneBy(
            ['nrorg' => Util::getHCMParameters()->getNrorgTrabalho(), 'nrformgeracaoevento' => $nrFormGeracaoEvento]
        );

        if (empty($formGeracao)) {
            return '';
        }

        return Util::stringApplyTemplate(
            self::DSLOTEFREQ,
            [
                '[HORA_GERACAO]' => date('H:i:s'),
                '[DATA_INICIO]' => $formGeracao->getDtinicial()->format(DateUtil::FORMATO_DATA),
                '[DATA_FIM]' => !empty($formGeracao->getDtfinal()) ? $formGeracao->getDtfinal()->format(DateUtil::FORMATO_DATA) : '',
            ]
        );
    }

    /**
     * Monta as observacoes do novo lote a ser gerado acordo com os criterios do formulario
     * @param string $nrFormGeracaoEvento
     * @return string
     */
    public function montaNovaObservacaoLote(
        string $nrFormGeracaoEvento
    ): string {
        $formGeracao = $this->entityManager->getRepository(Repositories::PTO_FORMGERACAOEVENTO)->findOneBy(
            ['nrorg' => Util::getHCMParameters()->getNrorgTrabalho(), 'nrformgeracaoevento' => $nrFormGeracaoEvento]
        );

        if (empty($formGeracao)) {
            return '';
        }

        return Util::stringApplyTemplate(
            self::OBSLOTEFREQ,
            [
                '[DSFORMGERACAOEVENTO]' => $formGeracao->getDsformgeracaoevento(),
                '[DSORDEM]' => $formGeracao->getDsordem(),
                '[PERIODO_APURACAO]' => $formGeracao->getNrperiodoapuracao(),
                '[DSINICIO]' => $formGeracao->getDsinicio(),
                '[DSFIM]' => $formGeracao->getDsfim(),
                '[DATA_INICIO]' => $formGeracao->getDtinicial()->format(DateUtil::FORMATO_DATA),
                '[DATA_FIM]' => !empty($formGeracao->getDtfinal()) ? $formGeracao->getDtfinal()->format(DateUtil::FORMATO_DATA) : '',
                '[HORA_GERACAO]' => date('H:i:s'),
            ]
        );
    }

    /**
     * Atualiza os resumos de frequencia informados atualizando o numero do lote
     * @param array $nrResumos
     * @param string $nrLote
     * @return void
     */
    public function atualizaResumosNovoLote(
        array $nrResumos,
        string $nrLote
    ): void {
        $nrOrg = Util::getHCMParameters()->getNrorgTrabalho();

        try {
            $this->entityManager->getConnection()->beginTransaction();
            
            foreach ($nrResumos as $nrResumo) {
                $resumoSituFreq = $this->entityManager->getRepository('\Folha\Entities\PtoResumositufreq')
                    ->findOneBy(array('nrresumositufreq' => $nrResumo, 'nrorg' => $nrOrg));
                $resumoSituFreq->setNrlotelancevento($nrLote);
                $resumoSituFreq->persist();
            }

            $this->entityManager->flush();
            $this->entityManager->getConnection()->commit();
        } catch (\Exception $e) {
            $this->entityManager->getConnection()->rollBack();
            throw $e;
        }
    }

} 