<?php
namespace HCM\SST\Controllers;

use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request;
use HCM\Util\HCMParameters;
use HCM\SST\Services\GheHomogenExposicaoService;
use Zeedhi\Framework\DTO\Response\Notification;

class GheHomogenExposicaoController extends Crud {

    protected $dataSourceName            = 'sst_gherisco';
    protected $dataSourceName_get        = 'sst_gherisco_get';
    protected $dataSourceClassFrequencia = 'sst_classfrequencia';
    protected $dataSourceClassEfeito     = 'sst_classefeito';
    protected $dataSourceClassNivelAcao  = 'sst_classnivelacao';
    protected $dataSourceClassRisco      = 'sst_classrisco';

    protected $parameterBagColumns = array('P_NRORG', 'P_NRORG_PADRAO', 'P_DTMESCOMPETENC');

    public function __construct (Manager $dataSourceManager, ParameterBag $parameterBag, HCMParameters $hcmParameters, GheHomogenExposicaoService $gheHomogenExposicaoService){
        $this->parameterBag          = $parameterBag;
        $this->hcmParameters         = $hcmParameters;
        $this->gheHomogenExposicaoService = $gheHomogenExposicaoService;
        parent::__construct($dataSourceManager);
    }

    public function getGheHomogenExposicao (DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria();
        $filterData->addCondition('P_NRORG_PADRAO','=',$this->hcmParameters->getNrorgPadrao());
        $filterData->setDataSourceName($this->dataSourceName_get);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData);
        $rows = $dataSet->getRows();
        
        foreach($rows as $key => &$row){
            $row['LIST_NREPI'] = explode(",", $row['NREPI_LIST']);
            
            foreach($row['LIST_NREPI'] as $chave => $value)
                $row['LIST_NREPI'][$chave] = intval($value);

            $row['LIST_NMEPI'] = explode(",", $row['NMEPI_LIST']);
        }

        $response->addDataSet(new DataSet($this->dataSourceName_get, $rows));
    }

    public function setGheHomogenExposicao (DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            foreach ($request->getDataSet()->getRows() as &$row) {
                if (array_key_exists('NRGHERISCO', $row)) {
                    if ($row['NRGHERISCO'] === '') {
                        $row['NRGHERISCO'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'SST_GHERISCO', 1, 12, true);
                    }
                } else {
                    $row['NRGHERISCO'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'SST_GHERISCO', 1, 12, true);
                }
            }
            $row["NRSETOR"]        = isset($row["NRSETOR"])         ? $row["NRSETOR"]                   : null;
            $row["NRSITUGERADORA"] = isset($row["NRSITUGERADORA"])  ? $row["NRSITUGERADORA"]            : null;
            $row["NRCLASSRISCO"]   = isset($row["NRCLASSRISCO"])    ? $row["NRCLASSRISCO"]              : null;
            $row["NRFREQUENCIA"]   = isset($row["NRFREQUENCIA"])    ? $row["NRFREQUENCIA"]              : null;
            $row["NRNIVELACAO"]    = isset($row["NRNIVELACAO"])     ? $row["NRNIVELACAO"]               : null;
            $row["NRCLASSEFEITO"]  = isset($row["NRCLASSEFEITO"])   ? $row["NRCLASSEFEITO"]             : null;
            $row["NREPI_LIST"]     = !empty($row["LIST_NREPI"])     ? implode( ',', $row["LIST_NREPI"]) : null;
            $row["PERCINSALPERIC"] = isset($row["PERCINSALPERIC"])  ? $row["PERCINSALPERIC"]            : 0;

            parent::save($request, $response);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function deleteGheHomogenExposicao(DTO\Request\DataSet $request, DTO\Response $response){
        parent::delete($request, $response);
    }

    public function setCadastroAutomaticoGHE(DTO\Request\DataSet $request, DTO\Response $response)
    {
        try {
            foreach ($request->getDataSet()->getRows() as $row) {
                $this->gheHomogenExposicaoService->setCadastroAutomaticoGHE($row);
            }

        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function getClassFrequencia(DTO\Request\Filter $request, DTO\Response $response)
    {
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceClassFrequencia);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData);
        $response->addDataSet($dataSet);
    }

    public function getClassEfeito(DTO\Request\Filter $request, DTO\Response $response)
    {
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceClassEfeito);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData);
        $response->addDataSet($dataSet);
    }

    public function getClassNivelAcao(DTO\Request\Filter $request, DTO\Response $response)
    {
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceClassNivelAcao);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData);
        $response->addDataSet($dataSet);
    }

    public function getClassRisco(DTO\Request\Filter $request, DTO\Response $response)
    {
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceClassRisco);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData);
        $response->addDataSet($dataSet);
    }

    protected function processConditionValue($condition) {
        return $condition['value'];
    }
    protected function populateParameterBag(FilterCriteria $filterCriteria) {

        $conditions = array();
        foreach ($filterCriteria->getConditions() as $condition) {
            $columnName = $condition['columnName'];
            if (in_array($columnName, $this->parameterBagColumns)) {
                $this->parameterBag->set($columnName, $this->processConditionValue($condition));
            } else {
                $conditions[] = $condition;
            }
        }
        $newFilterCriteria = new FilterCriteria(
            $filterCriteria->getDataSourceName(),
            $conditions,
            $filterCriteria->getPage(),
            $filterCriteria->getPageSize()
        );
        return $newFilterCriteria;
    }
} 