<?php

namespace HCM\SST\Services;

use Doctrine\ORM\EntityManager;
use HCM\Util\HCMParameters;
use HCM\Util\Messages;
use HCM\Util\Repositories;
use Zeedhi\Framework\DTO\Response\Notification;
use Zeedhi\Framework\DTO\Response\Error;
use HCM\SST\Repository\HistoricoepiRepository;

class HistoricoEpiService {

    protected $entityManager;
    protected $hcmParameters;
    protected $historicoepiRepository;
    
    public function __construct(EntityManager $entityManager, HCMParameters $hcmParameters, HistoricoepiRepository $historicoepiRepository) {
        $this->entityManager          = $entityManager;
        $this->hcmParameters          = $hcmParameters;
        $this->historicoepiRepository = $historicoepiRepository;
    }
    
    public function validaCamposVazios(&$row){
        $row['NRPARCNEGOCIO']    = (!isset($row['NRPARCNEGOCIO'])    || empty($row['NRPARCNEGOCIO']))     ?  NULL : $row['NRPARCNEGOCIO'];
        $row['NRMOTIVODEVEPI']   = (!isset($row['NRMOTIVODEVEPI'])   || empty($row['NRMOTIVODEVEPI']))    ?  NULL : $row['NRMOTIVODEVEPI'];
        $row['QTFORNECIDAEPI']   = (!isset($row['QTFORNECIDAEPI'])   || empty($row['QTFORNECIDAEPI']))    ?  NULL : $row['QTFORNECIDAEPI'];
        $row['NRCTRLAPROVEPI']   = (!isset($row['NRCTRLAPROVEPI'])   || empty($row['NRCTRLAPROVEPI']))    ?  NULL : $row['NRCTRLAPROVEPI'];
        $row['DSMODELOEPI']      = (!isset($row['DSMODELOEPI'])      || empty($row['DSMODELOEPI']))       ?  NULL : $row['DSMODELOEPI'];
        $row['IDPROTECAOEFICAZ'] = (!isset($row['IDPROTECAOEFICAZ']) || empty($row['IDPROTECAOEFICAZ']))  ?  NULL : $row['IDPROTECAOEFICAZ'];
        $row['IDCTRLENTREGA']    = (!isset($row['IDCTRLENTREGA'])    || empty($row['IDCTRLENTREGA']))     ?  NULL : $row['IDCTRLENTREGA'];
        $row['DTENTREGAEPI']     = (!isset($row['DTENTREGAEPI'])     || empty($row['DTENTREGAEPI']))      ?  NULL : $row['DTENTREGAEPI'];
        $row['DTEFETIVA']        = (!isset($row['DTEFETIVA'])        || empty($row['DTEFETIVA']))         ?  NULL : $row['DTEFETIVA'];
        $row['DTDEVOLUCAO']      = (!isset($row['DTDEVOLUCAO'])      || empty($row['DTDEVOLUCAO']))       ?  NULL : $row['DTDEVOLUCAO'];
    }
    
    public function setAssinarAutomaticamenteService( $row ) {

        $nrvinculom         = isset( $row["NRVINCULOM_LIST"] ) && !empty( $row["NRVINCULOM_LIST"] ) ? $row["NRVINCULOM_LIST"] : null;
        $nrlotevinc_list    = isset( $row["NRLOTEVINC_LIST"] ) && !empty( $row["NRLOTEVINC_LIST"] ) ? $row["NRLOTEVINC_LIST"] : null;      
        $nrepi              = isset( $row["NREPI_LIST"     ] ) && !empty( $row["NREPI_LIST"     ] ) ? $row["NREPI_LIST"     ] : null;
        $nrorg              = $row["NRORG"             ];
        $cdoperultatu       = $row['CDOPERULTATU'      ];
        $dtefetiva_ini      = $row["DTINIDEVOLUCAO"    ];
        $dtefetiva_fim      = $row["DTFIMDEVOLUCAO"    ];
        $idfichaepiassinada = $row["IDFICHAEPIASSINADA"];

        if ( is_null($nrvinculom) && !is_null($nrlotevinc_list) ) {
            $nrvinculom = array();
            foreach( $nrlotevinc_list as $nrlotevinc ) {

                $itens_lotevinc = $this->entityManager->getRepository(Repositories::FPA_ITEMLOTEVINC)->findBy(array('nrlotevinc' => $nrlotevinc, 'nrorg' => $nrorg));

                if(!empty($itens_lotevinc)) {
                    foreach( $itens_lotevinc as $item_lotevinc ) {
                        array_push( $nrvinculom, $item_lotevinc->getNrvinculom() );
                    }           
                }
            }
        }
        
        $this->historicoepiRepository->setAssinarAutomaticamente( $nrorg, $nrvinculom, $nrepi, $dtefetiva_ini, $dtefetiva_fim, $idfichaepiassinada, $cdoperultatu );
    }

    public function setInlineHistoricoEpi($rows)
    {
        try {
            foreach ($rows as &$row) {
                $cdOperultatu = $this->hcmParameters->getCdoperador();
                if(isset($row['NRHISTORICOEPI'])){
                    $this->historicoepiRepository->setInlineHistoricoEpi($row['NRORG'], $row['NRHISTORICOEPI'], $row ["QTFORNECIDAEPI_inline"], $cdOperultatu);
                }
            }
            return;
        } catch (Exception $e) {
            throw $e;    
        }
    }
}