<?php

namespace HCM\TRE\Controllers;

use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request;
use HCM\Util\HCMParameters;

class ProdutoController extends Crud {

    protected $dataSourceName = 'produto';
    
    protected $parameterBagColumns = array('P_NRORG', 'P_NRORG_PADRAO');
    
    public function __construct (Manager $dataSourceManager, ParameterBag $parameterBag, HCMParameters $hcmParameters){
        $this->parameterBag = $parameterBag;
        $this->hcmParameters = $hcmParameters;
        parent::__construct($dataSourceManager);
    }    
    
    public function setProduto(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            foreach ($request->getDataSet()->getRows() as &$row) {
                if (array_key_exists('CDPRODUTO', $row)) {
                    if ($row['CDPRODUTO'] === '') {
                        $row['CDPRODUTO'] = \HCM\Util\NovoCodigo::geraCodigo( $row['NRORG'], 'PRODUTO', 1, 12, true);
                    }
                } else {
                    $row['CDPRODUTO'] = \HCM\Util\NovoCodigo::geraCodigo( $row['NRORG'], 'PRODUTO', 1, 12, true);
                }
                $row["IDCOBTXSERV"]  = "S";
			    $row["CDNVPRODUTO"]  = 1;
			    $row["CDARVPROD"]  = 1;
			    $row["IDTPPRODDEBCONS"]  = "N";
			    $row["IDPRODCUSFIXO"]  = "N";
			    $row["IDVDANALISE"]  = "1";
			    $row["IDCONTROLAREFIL"]  = "N";
			    $row["IDTEOROBRIG"]  = "N";
			    $row["IDGERRELEASEMETA"]  = "N";
			    $row["IDALTERANUTR"]  = "N";
			    $row["IDTEMNUTRIENTE"]  = "N";
			    $row["IDGERACOMISOREV"]  = "S";
			    $row["IDGERREVPROCFA"]  = "N";
			    $row["IDPRODUSOCONS"]  = "N";
			    $row["IDTIPOPRODOREV"]  = "P";
			    $row["IDIMCOMPOSPRORE"]  = "N";
			    $row["IDFATURAPRODORE"]  = "S";
			    $row["IDRETISEMESTORE"]  = "N";
			    $row["IDTIPORETIORE"]  = "C";
			    $row["IDORIGPRODPR"]  = "T";
			    $row["IDTIPOMEDCASVND"]  = "00";
			    $row["IDPADEMEDCASVND"]  = "0";
			    $row["IDUNPORCAOVND"]  = "0";
			    $row["IDPRODCONTRSUBV"]  = "N";
			    $row["IDSITUPROD"]  = "L";
			    $row["IDPRODPROPTERC"]  = "P";
			    $row["IDARREDTRUNC"]  = "T";
			    $row["IDTIPOCOMPPROD"]  = "0";
			    $row["IDSEGMENTOPROD"]  = "OUTR";
            }
            
            parent::save($request, $response);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function deleteProduto(DTO\Request\DataSet $request, DTO\Response $response){
        try{
            parent::delete($request, $response);
        } catch(\Exception $e){
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }    
    }
    
    protected function processConditionValue($condition) {
        return $condition['value'];
    }
    protected function populateParameterBag(FilterCriteria $filterCriteria) {

        $conditions = array();
        foreach ($filterCriteria->getConditions() as $condition) {
            $columnName = $condition['columnName'];
            if (in_array($columnName, $this->parameterBagColumns)) {
                $this->parameterBag->set($columnName, $this->processConditionValue($condition));
            } else {
                $conditions[] = $condition;
            }
        }
        $newFilterCriteria = new FilterCriteria(
            $filterCriteria->getDataSourceName(),
            $conditions,
            $filterCriteria->getPage(),
            $filterCriteria->getPageSize()
        );
        return $newFilterCriteria;
    }
} 