<?php
namespace HCM\VAB\Repository;

use Doctrine\DBAL\Connection;
use Doctrine\ORM\EntityManager;
use Folha\Entities\VabBeneficiario;
use Folha\Entities\GpeVinculom;
use HCM\Util\Messages;
use HCM\Util\DateUtil;
use HCM\Util\Repositories;
use HCM\Util\MsgCalculo;
use Geral\Util\NovoCodigo;
use HCM\Util\HCMParameters;
use HCM\Util\Valor;
use HCM\Util\FPA;

class CalculoValeTransporteRepository {

    protected  $connection;
 
    public function __construct(Connection $connection, EntityManager $entityManager, HCMParameters $hcmParameters) {
        $this->connection = $connection;
        $this->entityManager = $entityManager;
        $this->hcmParameters = $hcmParameters;
    }
 
    public function buscaVinculos_Mensal($inicial, $final, $competencia, $ordem, $dataInicial, $dataFinal, $row)
    {
        try {
            if($competencia instanceof \DateTime){
                $dtmescompetencia = $competencia;
            }else{
                $dtmescompetencia = DateUtil::getDataDeString($competencia, DateUtil::FORMATO_BRASILEIRO, true);
            }
            
            $this->dataInicial = DateUtil::getDataDeString($dataInicial, DateUtil::FORMATO_BRASILEIRO, true);
            $this->dataFinal = DateUtil::getDataDeString($dataFinal, DateUtil::FORMATO_BRASILEIRO, true);
            $this->row = $row;
            
            //18/09/20 alteração para olhar vínculos ativos na competência do operador e não na competência + 1 mês ($dtiniperiodo) como era feito
            $inicompetencia = $dtmescompetencia->format('d/m/Y');
            $fimcompetencia = $dtmescompetencia->format('t/m/Y');

            $dtmescompetencia->add(new \DateInterval('P1M'));
            $dtiniperiodo = $dtmescompetencia->format('d/m/Y');
            $dtfinalvale = $this->dataFinal->format('d/m/Y');
            $dtinicialvale = $this->dataInicial->format('d/m/Y');

            
            // Filtra somente vínculos que possuam histórico
            $vinculosComHistorico = " AND EXISTS (SELECT 1 FROM GPE_VINCULOH WHERE NRVINCULOM = e.NRVINCULOM AND NRORG = e.NRORG and DTMESCOMPETENC <> '01/01/3000') ";
            
            // Filtra somente vínculos que não estejam em situação funcional de afastamento (exceto atestado médico)
            $cond_afastados =
                "AND EXISTS (SELECT * FROM VAB_LINHAVINC VLV 
                                    WHERE VLV.NRVINCULOM = e.NRVINCULOM 
                                    AND VLV.NRORG = e.NRORG
                                    AND VLV.DTINILINHAVINC <= '$dtfinalvale'
                                    AND (VLV.DTFIMLINHAVINC >='$dtinicialvale' OR VLV.DTFIMLINHAVINC IS NULL)
                                ) 
                AND NOT EXISTS (SELECT * FROM GPE_ALTESITUFUNC
                                   WHERE NRORG         = e.NRORG
                                     AND NRVINCULOM    = e.NRVINCULOM
                                     AND NRSITUFUNCM IN (SELECT GPE_SITUFUNCH.NRSITUFUNCM
                                                           FROM GPE_SITUFUNCH
                                                          WHERE GPE_SITUFUNCH.IDGERVALETRANSP = 'N'
                                                            AND GPE_SITUFUNCH.NRSITUFUNCM NOT IN (12, 15, 16, 13)
                                                            AND GPE_SITUFUNCH.DTEFETIVA = ( SELECT * FROM 
                                                                                                ( SELECT DTEFETIVA DTEFETIVA
                                                                                                    FROM GPE_SITUFUNCH MAX_SITUFUNCH
                                                                                                    WHERE MAX_SITUFUNCH.DTEFETIVA <= TO_DATE('$dtfinalvale','DD/MM/YYYY')
                                                                                                    AND MAX_SITUFUNCH.NRSITUFUNCM = GPE_SITUFUNCH.NRSITUFUNCM
                                                                                                ORDER BY DTEFETIVA DESC
                                                                                                )
                                                                                            WHERE ROWNUM = 1))
                                     AND DTINISITUFUNC = ( SELECT * FROM 
                                                            ( SELECT DTINISITUFUNC
                                                                FROM GPE_ALTESITUFUNC B
                                                                WHERE B.NRORG= e.NRORG 
                                                                AND B.NRVINCULOM= e.NRVINCULOM 
                                                                AND DTINISITUFUNC <= TO_DATE('$dtinicialvale','DD/MM/YYYY')
                                                            ORDER BY DTINISITUFUNC DESC
                                                            )
                                                        WHERE ROWNUM = 1)
                                     AND (DTFIMSITUFUNC > TO_DATE('$dtfinalvale','DD/MM/YYYY') OR DTFIMSITUFUNC IS NULL))";
            switch($ordem)
            {
                case 'ALFABETICA':
                {
                    $param = array(':P_INICIO' => $inicial,
                        ':P_FIM' => $final,
                        ':P_NRORG' => $this->hcmParameters->getNrorgTrabalho());

                    //Nome Inicial
                    $whereinivinculo = "NRORG = :P_NRORG AND NRVINCULOM = :P_INICIO"; /** @var GpeVinculom $objinivinculo */
                    $objinivinculo = GpeVinculom::getObjectsByWhere($whereinivinculo, $param);

                    //Nome Final
                    $wherefimvinculo = "NRORG = :P_NRORG AND NRVINCULOM = :P_FIM"; /** @var GpeVinculom $objfimvinculo */
                    $objfimvinculo = GpeVinculom::getObjectsByWhere($wherefimvinculo, $param);

                    $nomeInicio = $objinivinculo->getNmvinculom();
                    $nomeFinal = $objfimvinculo->getNmvinculom();

                    if($nomeFinal<$nomeInicio){
                        $aux = $nomeFinal;
                        $nomeFinal = $nomeInicio;
                        $nomeInicio = $aux;
                    }

                    $param = array( ':P_INICIO' => $nomeInicio,
                                    ':P_FIM' => $nomeFinal,
                                    ':P_NRORG' => $this->hcmParameters->getNrorgTrabalho());
                    //$where = "(DTRESCISAOVINC > '$dtiniperiodo' OR DTRESCISAOVINC IS NULL) AND DTADMISSAOVINC <= '$dtiniperiodo' AND NMVINCULOM BETWEEN :P_INICIO AND :P_FIM AND NRORG = :P_NRORG";
                    $where = "(DTRESCISAOVINC >= '$inicompetencia' OR DTRESCISAOVINC IS NULL) AND DTADMISSAOVINC <= '$fimcompetencia' AND NMVINCULOM BETWEEN :P_INICIO AND :P_FIM AND NRORG = :P_NRORG";
                    $msgerro = 'Não foram encontrados vínculos para os nomes informados';
                    break;
                }
                case 'MATRICULA':{
                    $param = array(
                        'P_INICIO' => $inicial,
                        'P_FIM' => $final,
                        'P_NRORG' => $this->hcmParameters->getNrorgTrabalho());
					//$where = "(DTRESCISAOVINC > '$dtiniperiodo' OR DTRESCISAOVINC IS NULL) AND DTADMISSAOVINC <= '$dtiniperiodo' AND CDMATRICULA BETWEEN :P_INICIO AND :P_FIM AND NRORG = :P_NRORG";
					$where = "(DTRESCISAOVINC >= '$inicompetencia' OR DTRESCISAOVINC IS NULL) AND DTADMISSAOVINC <= '$fimcompetencia' AND CDMATRICULA BETWEEN :P_INICIO AND :P_FIM AND NRORG = :P_NRORG";
                    $msgerro = 'Não foram encontrados vínculos para as matrículas informadas';
                    break;
                }
                case 'DATA_ADMISSAO':{
                    $param = array(':P_NRORG' => $this->hcmParameters->getNrorgTrabalho());
                    //$where = "(DTRESCISAOVINC > '$dtiniperiodo' OR DTRESCISAOVINC IS NULL) AND DTADMISSAOVINC BETWEEN '$inicial' AND '$final' AND NRORG = :P_NRORG";
                    $where = "(DTRESCISAOVINC >= '$inicompetencia' OR DTRESCISAOVINC IS NULL) AND DTADMISSAOVINC BETWEEN '$inicial' AND '$final' AND NRORG = :P_NRORG";
                    $msgerro = 'Não foram encontrados vínculos para as datas informadas';
                    break;
                }
                case 'ESTRUTURA':{
                    $param = array( 'P_NRORG' => $this->hcmParameters->getNrorgTrabalho(),
                                    'P_NRESTRUTLEGAL' =>$inicial,
                                    'P_NRESTRUTGEREN' =>$final);
                    //$where = "NRORG = :P_NRORG AND (DTRESCISAOVINC > '$dtiniperiodo' OR DTRESCISAOVINC IS NULL) AND DTADMISSAOVINC <= '$dtiniperiodo' AND NRVINCULOM IN ( SELECT NRVINCULOM FROM GPE_VINCULOH WHERE NRORG = :P_NRORG AND (NRESTRUTGEREN = :P_NRESTRUTGEREN OR :P_NRESTRUTGEREN IS NULL) AND NRESTRUTLEGAL = :P_NRESTRUTLEGAL AND DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC)DTMESCOMPETENC FROM GPE_VINCULOH MAX_VINCULOH WHERE MAX_VINCULOH.NRVINCULOM = GPE_VINCULOH.NRVINCULOM AND MAX_VINCULOH.NRORG = GPE_VINCULOH.NRORG))";
                    $where = "NRORG = :P_NRORG AND (DTRESCISAOVINC >= '$inicompetencia' OR DTRESCISAOVINC IS NULL) AND DTADMISSAOVINC <= '$fimcompetencia' AND NRVINCULOM IN ( SELECT NRVINCULOM FROM GPE_VINCULOH WHERE NRORG = :P_NRORG AND (NRESTRUTGEREN = :P_NRESTRUTGEREN OR :P_NRESTRUTGEREN IS NULL) AND NRESTRUTLEGAL = :P_NRESTRUTLEGAL AND DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC)DTMESCOMPETENC FROM GPE_VINCULOH MAX_VINCULOH WHERE MAX_VINCULOH.NRVINCULOM = GPE_VINCULOH.NRVINCULOM AND MAX_VINCULOH.NRORG = GPE_VINCULOH.NRORG))";
                    $msgerro = 'Não foram encontrados vínculos para as estruturas informadas';
                    break;
                }
                case 'LISTA_VINCULO':
                    $param = array('P_NRORG' => $this->hcmParameters->getNrorgTrabalho());
                    //$where = "NRORG = :P_NRORG AND (DTRESCISAOVINC > '$dtiniperiodo' OR DTRESCISAOVINC IS NULL) AND DTADMISSAOVINC <= '$dtiniperiodo' AND NRVINCULOM IN ($inicial)";
                    $listaVinculo = isset($this->row['NRVINCULOM_LIST']) && is_array($this->row['NRVINCULOM_LIST']) && count($this->row['NRVINCULOM_LIST']) ? implode(",", $this->row['NRVINCULOM_LIST']) : null;
                    $where = "NRORG = :P_NRORG AND (DTRESCISAOVINC >= '$inicompetencia' OR DTRESCISAOVINC IS NULL) AND DTADMISSAOVINC <= '$fimcompetencia' AND NRVINCULOM IN ($listaVinculo)";
                    $msgerro = 'Não foram encontrados vínculos para a lista informada';
                    break;
                case 'LISTA_ESTRUTURA':
                    $listaEstrutLegal = isset($this->row['LIST_NRESTRUTLEGAL']) && is_array($this->row['LIST_NRESTRUTLEGAL']) && count($this->row['LIST_NRESTRUTLEGAL']) ? implode(",", $this->row['LIST_NRESTRUTLEGAL']) : null;
                    $listaEstrutGeren = isset($this->row['LIST_NRESTRUTGEREN']) && is_array($this->row['LIST_NRESTRUTGEREN']) && count($this->row['LIST_NRESTRUTGEREN']) ? implode(",", $this->row['LIST_NRESTRUTGEREN']) : 1;
                    
                    
                    
                    $inicial = !empty($this->row['DSINICIOVALE1']) ? $this->row['DSINICIOVALE1'] : null;
                    $final = !empty($this->row['DSFIMVALE1']) ? $this->row['DSFIMVALE1'] : null;
                    
                    $param = array( 'P_NRORG' => $this->hcmParameters->getNrorgTrabalho());
                    if(isset($inicial) && isset($final)){
                        $where = "NRORG = :P_NRORG AND (DTRESCISAOVINC >= '$inicompetencia' OR DTRESCISAOVINC IS NULL) AND DTADMISSAOVINC <= '$fimcompetencia' AND NRVINCULOM IN ( SELECT NRVINCULOM FROM GPE_VINCULOH WHERE NRORG = :P_NRORG AND ( NRESTRUTGEREN IN($listaEstrutGeren) OR  1 IN($listaEstrutGeren)  ) AND NRESTRUTLEGAL IN($listaEstrutLegal) AND DTMESCOMPETENC = ( SELECT * FROM ( SELECT DTMESCOMPETENC DTMESCOMPETENC FROM GPE_VINCULOH MAX_VINCULOH WHERE MAX_VINCULOH.NRORG = GPE_VINCULOH.NRORG AND MAX_VINCULOH.NRVINCULOM = GPE_VINCULOH.NRVINCULOM ORDER BY DTMESCOMPETENC DESC)WHERE ROWNUM = 1)) AND (DTRESCISAOVINC >= '$inicompetencia' OR DTRESCISAOVINC IS NULL) AND DTADMISSAOVINC BETWEEN '$inicial' AND '$final' AND NRORG = :P_NRORG";
                    } else if(isset($inicial) && !isset($final)){
                        $where = "NRORG = :P_NRORG AND (DTRESCISAOVINC >= '$inicompetencia' OR DTRESCISAOVINC IS NULL) AND DTADMISSAOVINC <= '$fimcompetencia' AND NRVINCULOM IN ( SELECT NRVINCULOM FROM GPE_VINCULOH WHERE NRORG = :P_NRORG AND ( NRESTRUTGEREN IN($listaEstrutGeren) OR  1 IN($listaEstrutGeren)  ) AND NRESTRUTLEGAL IN($listaEstrutLegal) AND DTMESCOMPETENC = ( SELECT * FROM ( SELECT DTMESCOMPETENC DTMESCOMPETENC FROM GPE_VINCULOH MAX_VINCULOH WHERE MAX_VINCULOH.NRORG = GPE_VINCULOH.NRORG AND MAX_VINCULOH.NRVINCULOM = GPE_VINCULOH.NRVINCULOM ORDER BY DTMESCOMPETENC DESC)WHERE ROWNUM = 1)) AND (DTRESCISAOVINC >= '$inicompetencia' OR DTRESCISAOVINC IS NULL) AND DTADMISSAOVINC >= '$inicial'AND NRORG = :P_NRORG";
                    } else if(!isset($inicial) && isset($final)){
                        $where = "NRORG = :P_NRORG AND (DTRESCISAOVINC >= '$inicompetencia' OR DTRESCISAOVINC IS NULL) AND DTADMISSAOVINC <= '$fimcompetencia' AND NRVINCULOM IN ( SELECT NRVINCULOM FROM GPE_VINCULOH WHERE NRORG = :P_NRORG AND ( NRESTRUTGEREN IN($listaEstrutGeren) OR  1 IN($listaEstrutGeren)  ) AND NRESTRUTLEGAL IN($listaEstrutLegal) AND DTMESCOMPETENC = ( SELECT * FROM ( SELECT DTMESCOMPETENC DTMESCOMPETENC FROM GPE_VINCULOH MAX_VINCULOH WHERE MAX_VINCULOH.NRORG = GPE_VINCULOH.NRORG AND MAX_VINCULOH.NRVINCULOM = GPE_VINCULOH.NRVINCULOM ORDER BY DTMESCOMPETENC DESC)WHERE ROWNUM = 1)) AND (DTRESCISAOVINC >= '$inicompetencia' OR DTRESCISAOVINC IS NULL) AND DTADMISSAOVINC <= '$final' AND NRORG = :P_NRORG";
                    } else{
                        $where = "NRORG = :P_NRORG AND (DTRESCISAOVINC >= '$inicompetencia' OR DTRESCISAOVINC IS NULL) AND DTADMISSAOVINC <= '$fimcompetencia' AND NRVINCULOM IN ( SELECT NRVINCULOM FROM GPE_VINCULOH WHERE NRORG = :P_NRORG AND ( NRESTRUTGEREN IN($listaEstrutGeren) OR  1 IN($listaEstrutGeren)  ) AND NRESTRUTLEGAL IN($listaEstrutLegal) AND DTMESCOMPETENC = ( SELECT * FROM ( SELECT DTMESCOMPETENC DTMESCOMPETENC FROM GPE_VINCULOH MAX_VINCULOH WHERE MAX_VINCULOH.NRORG = GPE_VINCULOH.NRORG AND MAX_VINCULOH.NRVINCULOM = GPE_VINCULOH.NRVINCULOM ORDER BY DTMESCOMPETENC DESC)WHERE ROWNUM = 1))";
                    }                
                    //$where = "NRORG = :P_NRORG AND (DTRESCISAOVINC > '$dtiniperiodo' OR DTRESCISAOVINC IS NULL) AND DTADMISSAOVINC <= '$dtiniperiodo' AND NRVINCULOM IN ( SELECT NRVINCULOM FROM GPE_VINCULOH WHERE NRORG = :P_NRORG AND ( NRESTRUTGEREN IN($listaEstrutGeren) OR  1 IN($listaEstrutGeren)  ) AND NRESTRUTLEGAL IN($listaEstrutLegal) AND DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC)DTMESCOMPETENC FROM GPE_VINCULOH MAX_VINCULOH WHERE MAX_VINCULOH.NRVINCULOM = GPE_VINCULOH.NRVINCULOM AND MAX_VINCULOH.NRORG = GPE_VINCULOH.NRORG))";
                    
                    $msgerro = 'Não foram encontrados vínculos para as estruturas informadas';
                    break;
                case 'LISTA_NRESTRUTSIND':
                    $estrutLegal = isset($this->row['DSINICIOVALE']) && !empty($this->row['DSINICIOVALE']) ? $this->row['DSINICIOVALE'] : null;
                    $listaEstrutSind = isset($this->row['LIST_NRESTRUTSIND']) && is_array($this->row['LIST_NRESTRUTSIND']) && count($this->row['LIST_NRESTRUTSIND']) ? implode(",", $this->row['LIST_NRESTRUTSIND']) : 1;
                    
                    $inicial = !empty($this->row['DSINICIOVALE1']) ? $this->row['DSINICIOVALE1'] : null;
                    $final = !empty($this->row['DSFIMVALE1']) ? $this->row['DSFIMVALE1'] : null;
                    
                    if(isset($inicial) && isset($final)){
                        $where = "NRORG = :P_NRORG AND (DTRESCISAOVINC >= '$inicompetencia' OR DTRESCISAOVINC IS NULL) AND DTADMISSAOVINC <= '$fimcompetencia' AND NRVINCULOM IN ( SELECT NRVINCULOM FROM GPE_VINCULOH WHERE NRORG = :P_NRORG AND ( NRESTRUTSIND IN($listaEstrutSind) ) AND NRESTRUTLEGAL IN ($estrutLegal) AND DTMESCOMPETENC = ( SELECT * FROM ( SELECT DTMESCOMPETENC DTMESCOMPETENC FROM GPE_VINCULOH MAX_VINCULOH WHERE MAX_VINCULOH.NRORG = GPE_VINCULOH.NRORG AND MAX_VINCULOH.NRVINCULOM = GPE_VINCULOH.NRVINCULOM ORDER BY DTMESCOMPETENC DESC)WHERE ROWNUM = 1)) AND (DTRESCISAOVINC >= '$inicompetencia' OR DTRESCISAOVINC IS NULL) AND DTADMISSAOVINC BETWEEN '$inicial' AND '$final' AND NRORG = :P_NRORG";
                    } else if(isset($inicial) && !isset($final)){
                        $where = "NRORG = :P_NRORG AND (DTRESCISAOVINC >= '$inicompetencia' OR DTRESCISAOVINC IS NULL) AND DTADMISSAOVINC <= '$fimcompetencia' AND NRVINCULOM IN ( SELECT NRVINCULOM FROM GPE_VINCULOH WHERE NRORG = :P_NRORG AND ( NRESTRUTSIND IN($listaEstrutSind) ) AND NRESTRUTLEGAL IN ($estrutLegal) AND DTMESCOMPETENC = ( SELECT * FROM ( SELECT DTMESCOMPETENC DTMESCOMPETENC FROM GPE_VINCULOH MAX_VINCULOH WHERE MAX_VINCULOH.NRORG = GPE_VINCULOH.NRORG AND MAX_VINCULOH.NRVINCULOM = GPE_VINCULOH.NRVINCULOM ORDER BY DTMESCOMPETENC DESC)WHERE ROWNUM = 1)) AND (DTRESCISAOVINC >= '$inicompetencia' OR DTRESCISAOVINC IS NULL) AND DTADMISSAOVINC >= '$inicial' AND NRORG = :P_NRORG";   
                    } else if(!isset($inicial) && isset($final)){
                        $where = "NRORG = :P_NRORG AND (DTRESCISAOVINC >= '$inicompetencia' OR DTRESCISAOVINC IS NULL) AND DTADMISSAOVINC <= '$fimcompetencia' AND NRVINCULOM IN ( SELECT NRVINCULOM FROM GPE_VINCULOH WHERE NRORG = :P_NRORG AND ( NRESTRUTSIND IN($listaEstrutSind) ) AND NRESTRUTLEGAL IN ($estrutLegal) AND DTMESCOMPETENC = ( SELECT * FROM ( SELECT DTMESCOMPETENC DTMESCOMPETENC FROM GPE_VINCULOH MAX_VINCULOH WHERE MAX_VINCULOH.NRORG = GPE_VINCULOH.NRORG AND MAX_VINCULOH.NRVINCULOM = GPE_VINCULOH.NRVINCULOM ORDER BY DTMESCOMPETENC DESC)WHERE ROWNUM = 1)) AND (DTRESCISAOVINC >= '$inicompetencia' OR DTRESCISAOVINC IS NULL) AND DTADMISSAOVINC <= '$final' AND NRORG = :P_NRORG";
                    } else{
                        $where = "NRORG = :P_NRORG AND (DTRESCISAOVINC >= '$inicompetencia' OR DTRESCISAOVINC IS NULL) AND DTADMISSAOVINC <= '$fimcompetencia' AND NRVINCULOM IN ( SELECT NRVINCULOM FROM GPE_VINCULOH WHERE NRORG = :P_NRORG AND ( NRESTRUTSIND IN($listaEstrutSind) ) AND NRESTRUTLEGAL IN ($estrutLegal) AND DTMESCOMPETENC = ( SELECT * FROM ( SELECT DTMESCOMPETENC DTMESCOMPETENC FROM GPE_VINCULOH MAX_VINCULOH WHERE MAX_VINCULOH.NRORG = GPE_VINCULOH.NRORG AND MAX_VINCULOH.NRVINCULOM = GPE_VINCULOH.NRVINCULOM ORDER BY DTMESCOMPETENC DESC)WHERE ROWNUM = 1))";
                    }
                    $param = array( 'P_NRORG' => $this->hcmParameters->getNrorgTrabalho());
                                    
                    //$where = "NRORG = :P_NRORG AND (DTRESCISAOVINC > '$dtiniperiodo' OR DTRESCISAOVINC IS NULL) AND DTADMISSAOVINC <= '$dtiniperiodo' AND NRVINCULOM IN ( SELECT NRVINCULOM FROM GPE_VINCULOH WHERE NRORG = :P_NRORG AND ( NRESTRUTSIND IN($listaEstrutSind) ) AND NRESTRUTLEGAL = $estrutLegal AND DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC)DTMESCOMPETENC FROM GPE_VINCULOH MAX_VINCULOH WHERE MAX_VINCULOH.NRVINCULOM = GPE_VINCULOH.NRVINCULOM AND MAX_VINCULOH.NRORG = GPE_VINCULOH.NRORG))";
                    $msgerro = 'Não foram encontrados vínculos para as estruturas informadas';
                    break;
                    
                case 'LISTA_TOMADOR':
                    $listaEstrutSup = isset($this->row['LIST_NRESTRUTURAMSUP']) && is_array($this->row['LIST_NRESTRUTURAMSUP']) && count($this->row['LIST_NRESTRUTURAMSUP']) ? implode(",", $this->row['LIST_NRESTRUTURAMSUP']) : null;
                    $listaEstrutGeren = isset($this->row['LIST_NRESTRUTGEREN']) && is_array($this->row['LIST_NRESTRUTGEREN']) && count($this->row['LIST_NRESTRUTGEREN']) ? implode(",", $this->row['LIST_NRESTRUTGEREN']) : 1;
                    
                    $inicial = !empty($this->row['DSINICIOVALE1']) ? $this->row['DSINICIOVALE1'] : null;
                    $final = !empty($this->row['DSFIMVALE1']) ? $this->row['DSFIMVALE1'] : null;
                    $param = array( 'P_NRORG' => $this->hcmParameters->getNrorgTrabalho());
                    
                    if(isset($listaEstrutSup) && (isset($listaEstrutGeren) && $listaEstrutGeren != 1)){
                        if(isset($inicial) && isset($final)){
                            $where = "NRORG = :P_NRORG AND(DTRESCISAOVINC >= '$inicompetencia' OR DTRESCISAOVINC IS NULL) AND DTADMISSAOVINC <= '$fimcompetencia' AND NRVINCULOM IN ( SELECT NRVINCULOM FROM GPE_VINCULOH WHERE NRORG = :P_NRORG AND ( NRESTRUTGEREN IN($listaEstrutGeren)) AND DTMESCOMPETENC = ( SELECT * FROM ( SELECT DTMESCOMPETENC DTMESCOMPETENC FROM GPE_VINCULOH MAX_VINCULOH WHERE MAX_VINCULOH.NRORG = GPE_VINCULOH.NRORG AND MAX_VINCULOH.NRVINCULOM = GPE_VINCULOH.NRVINCULOM ORDER BY DTMESCOMPETENC DESC)WHERE ROWNUM = 1) AND EXISTS (SELECT 1 FROM ESTRUTURAM EMA WHERE EMA.NRORG = :P_NRORG AND EMA.NRESTRUTURAM = (SELECT H.NRESTRUTURAMSUP FROM HIERARQUIAORG H WHERE H.NRORG = :P_NRORG AND H.NRESTRUTURAMREL = GPE_VINCULOH.NRESTRUTGEREN AND H.NRESTRUTURAMSUP IN ($listaEstrutSup) AND H.NRTIPOHIERARQORG = (SELECT NRTIPOHIERARQORG FROM TIPOHIERARQORG WHERE NRORG IN (0, :P_NRORG) AND NRTIPOESTRUTSUP = (SELECT NVL(NRTPESTRUTOMADOR, 7) FROM GPE_PARAMORG WHERE NRORG = :P_NRORG) AND NRTIPOESTRUTREL = (SELECT NRTPESTRUTGEREN FROM GPE_PARAMORG WHERE NRORG = :P_NRORG)) AND H.DTINIVIGENCIA = (SELECT MAX(DTINIVIGENCIA) FROM HIERARQUIAORG WHERE DTINIVIGENCIA <= '$fimcompetencia' AND NRORG = :P_NRORG AND NRESTRUTURAMREL = GPE_VINCULOH.NRESTRUTGEREN AND NRTIPOHIERARQORG = H.NRTIPOHIERARQORG)))) AND (DTRESCISAOVINC >= '$inicompetencia' OR DTRESCISAOVINC IS NULL) AND DTADMISSAOVINC BETWEEN '$inicial' AND '$final' AND NRORG = :P_NRORG";
                        } else if(isset($inicial) && !isset($final)){
                            $where = "NRORG = :P_NRORG AND(DTRESCISAOVINC >= '$inicompetencia' OR DTRESCISAOVINC IS NULL) AND DTADMISSAOVINC <= '$fimcompetencia' AND NRVINCULOM IN ( SELECT NRVINCULOM FROM GPE_VINCULOH WHERE NRORG = :P_NRORG AND ( NRESTRUTGEREN IN($listaEstrutGeren)) AND DTMESCOMPETENC = ( SELECT * FROM ( SELECT DTMESCOMPETENC DTMESCOMPETENC FROM GPE_VINCULOH MAX_VINCULOH WHERE MAX_VINCULOH.NRORG = GPE_VINCULOH.NRORG AND MAX_VINCULOH.NRVINCULOM = GPE_VINCULOH.NRVINCULOM ORDER BY DTMESCOMPETENC DESC)WHERE ROWNUM = 1) AND EXISTS (SELECT 1 FROM ESTRUTURAM EMA WHERE EMA.NRORG = :P_NRORG AND EMA.NRESTRUTURAM = (SELECT H.NRESTRUTURAMSUP FROM HIERARQUIAORG H WHERE H.NRORG = :P_NRORG AND H.NRESTRUTURAMREL = GPE_VINCULOH.NRESTRUTGEREN AND H.NRESTRUTURAMSUP IN ($listaEstrutSup) AND H.NRTIPOHIERARQORG = (SELECT NRTIPOHIERARQORG FROM TIPOHIERARQORG WHERE NRORG IN (0, :P_NRORG) AND NRTIPOESTRUTSUP = (SELECT NVL(NRTPESTRUTOMADOR, 7) FROM GPE_PARAMORG WHERE NRORG = :P_NRORG) AND NRTIPOESTRUTREL = (SELECT NRTPESTRUTGEREN FROM GPE_PARAMORG WHERE NRORG = :P_NRORG)) AND H.DTINIVIGENCIA = (SELECT MAX(DTINIVIGENCIA) FROM HIERARQUIAORG WHERE DTINIVIGENCIA <= '$fimcompetencia' AND NRORG = :P_NRORG AND NRESTRUTURAMREL = GPE_VINCULOH.NRESTRUTGEREN AND NRTIPOHIERARQORG = H.NRTIPOHIERARQORG)))) AND (DTRESCISAOVINC >= '$inicompetencia' OR DTRESCISAOVINC IS NULL) AND DTADMISSAOVINC >= '$inicial' AND NRORG = :P_NRORG";
                        } else if(!isset($inicial) && isset($final)){
                           $where = "NRORG = :P_NRORG AND(DTRESCISAOVINC >= '$inicompetencia' OR DTRESCISAOVINC IS NULL) AND DTADMISSAOVINC <= '$fimcompetencia' AND NRVINCULOM IN ( SELECT NRVINCULOM FROM GPE_VINCULOH WHERE NRORG = :P_NRORG AND ( NRESTRUTGEREN IN($listaEstrutGeren)) AND DTMESCOMPETENC = ( SELECT * FROM ( SELECT DTMESCOMPETENC DTMESCOMPETENC FROM GPE_VINCULOH MAX_VINCULOH WHERE MAX_VINCULOH.NRORG = GPE_VINCULOH.NRORG AND MAX_VINCULOH.NRVINCULOM = GPE_VINCULOH.NRVINCULOM ORDER BY DTMESCOMPETENC DESC)WHERE ROWNUM = 1) AND EXISTS (SELECT 1 FROM ESTRUTURAM EMA WHERE EMA.NRORG = :P_NRORG AND EMA.NRESTRUTURAM = (SELECT H.NRESTRUTURAMSUP FROM HIERARQUIAORG H WHERE H.NRORG = :P_NRORG AND H.NRESTRUTURAMREL = GPE_VINCULOH.NRESTRUTGEREN AND H.NRESTRUTURAMSUP IN ($listaEstrutSup) AND H.NRTIPOHIERARQORG = (SELECT NRTIPOHIERARQORG FROM TIPOHIERARQORG WHERE NRORG IN (0, :P_NRORG) AND NRTIPOESTRUTSUP = (SELECT NVL(NRTPESTRUTOMADOR, 7) FROM GPE_PARAMORG WHERE NRORG = :P_NRORG) AND NRTIPOESTRUTREL = (SELECT NRTPESTRUTGEREN FROM GPE_PARAMORG WHERE NRORG = :P_NRORG)) AND H.DTINIVIGENCIA = (SELECT MAX(DTINIVIGENCIA) FROM HIERARQUIAORG WHERE DTINIVIGENCIA <= '$fimcompetencia' AND NRORG = :P_NRORG AND NRESTRUTURAMREL = GPE_VINCULOH.NRESTRUTGEREN AND NRTIPOHIERARQORG = H.NRTIPOHIERARQORG)))) AND (DTRESCISAOVINC >= '$inicompetencia' OR DTRESCISAOVINC IS NULL) AND DTADMISSAOVINC <= '$final' AND NRORG = :P_NRORG"; 
                        } else {
                            $where = "NRORG = :P_NRORG AND(DTRESCISAOVINC >= '$inicompetencia' OR DTRESCISAOVINC IS NULL) AND DTADMISSAOVINC <= '$fimcompetencia' AND NRVINCULOM IN ( SELECT NRVINCULOM FROM GPE_VINCULOH WHERE NRORG = :P_NRORG AND ( NRESTRUTGEREN IN($listaEstrutGeren)) AND DTMESCOMPETENC = ( SELECT * FROM ( SELECT DTMESCOMPETENC DTMESCOMPETENC FROM GPE_VINCULOH MAX_VINCULOH WHERE MAX_VINCULOH.NRORG = GPE_VINCULOH.NRORG AND MAX_VINCULOH.NRVINCULOM = GPE_VINCULOH.NRVINCULOM ORDER BY DTMESCOMPETENC DESC)WHERE ROWNUM = 1) AND EXISTS (SELECT 1 FROM ESTRUTURAM EMA WHERE EMA.NRORG = :P_NRORG AND EMA.NRESTRUTURAM = (SELECT H.NRESTRUTURAMSUP FROM HIERARQUIAORG H WHERE H.NRORG = :P_NRORG AND H.NRESTRUTURAMREL = GPE_VINCULOH.NRESTRUTGEREN AND H.NRESTRUTURAMSUP IN ($listaEstrutSup) AND H.NRTIPOHIERARQORG = (SELECT NRTIPOHIERARQORG FROM TIPOHIERARQORG WHERE NRORG IN (0, :P_NRORG) AND NRTIPOESTRUTSUP = (SELECT NVL(NRTPESTRUTOMADOR, 7) FROM GPE_PARAMORG WHERE NRORG = :P_NRORG) AND NRTIPOESTRUTREL = (SELECT NRTPESTRUTGEREN FROM GPE_PARAMORG WHERE NRORG = :P_NRORG)) AND H.DTINIVIGENCIA = (SELECT MAX(DTINIVIGENCIA) FROM HIERARQUIAORG WHERE DTINIVIGENCIA <= '$fimcompetencia' AND NRORG = :P_NRORG AND NRESTRUTURAMREL = GPE_VINCULOH.NRESTRUTGEREN AND NRTIPOHIERARQORG = H.NRTIPOHIERARQORG)))) AND (DTRESCISAOVINC >= '$inicompetencia' OR DTRESCISAOVINC IS NULL) AND NRORG = :P_NRORG";
                        }
                    } else{
                        if(isset($inicial) && isset($final)){
                            $where = "NRORG = :P_NRORG AND (DTRESCISAOVINC >= '$inicompetencia' OR DTRESCISAOVINC IS NULL) AND DTADMISSAOVINC <= '$fimcompetencia' AND NRVINCULOM IN ( SELECT NRVINCULOM FROM GPE_VINCULOH WHERE NRORG = :P_NRORG AND DTMESCOMPETENC = ( SELECT * FROM ( SELECT DTMESCOMPETENC DTMESCOMPETENC FROM GPE_VINCULOH MAX_VINCULOH WHERE MAX_VINCULOH.NRORG = GPE_VINCULOH.NRORG AND MAX_VINCULOH.NRVINCULOM = GPE_VINCULOH.NRVINCULOM ORDER BY DTMESCOMPETENC DESC)WHERE ROWNUM = 1) AND EXISTS (SELECT 1 FROM ESTRUTURAM EMA WHERE EMA.NRORG = :P_NRORG AND EMA.NRESTRUTURAM = (SELECT H.NRESTRUTURAMSUP FROM HIERARQUIAORG H WHERE H.NRORG = :P_NRORG AND H.NRESTRUTURAMREL = GPE_VINCULOH.NRESTRUTGEREN AND H.NRESTRUTURAMSUP IN ($listaEstrutSup) AND H.NRTIPOHIERARQORG = (SELECT NRTIPOHIERARQORG FROM TIPOHIERARQORG WHERE NRORG IN (0, :P_NRORG) AND NRTIPOESTRUTSUP = (SELECT NVL(NRTPESTRUTOMADOR, 7) FROM GPE_PARAMORG WHERE NRORG = :P_NRORG) AND NRTIPOESTRUTREL = (SELECT NRTPESTRUTGEREN FROM GPE_PARAMORG WHERE NRORG = :P_NRORG)) AND H.DTINIVIGENCIA = (SELECT MAX(DTINIVIGENCIA) FROM HIERARQUIAORG WHERE DTINIVIGENCIA <= '$fimcompetencia' AND NRORG = :P_NRORG AND NRESTRUTURAMREL = GPE_VINCULOH.NRESTRUTGEREN AND NRTIPOHIERARQORG = H.NRTIPOHIERARQORG)))) AND (DTRESCISAOVINC >= '$inicompetencia' OR DTRESCISAOVINC IS NULL) AND DTADMISSAOVINC BETWEEN '$inicial' AND '$final' AND NRORG = :P_NRORG";
                        } else if(isset($inicial) && !isset($final)){
                            $where = "NRORG = :P_NRORG AND (DTRESCISAOVINC >= '$inicompetencia' OR DTRESCISAOVINC IS NULL) AND DTADMISSAOVINC <= '$fimcompetencia' AND NRVINCULOM IN ( SELECT NRVINCULOM FROM GPE_VINCULOH WHERE NRORG = :P_NRORG AND DTMESCOMPETENC = ( SELECT * FROM ( SELECT DTMESCOMPETENC DTMESCOMPETENC FROM GPE_VINCULOH MAX_VINCULOH WHERE MAX_VINCULOH.NRORG = GPE_VINCULOH.NRORG AND MAX_VINCULOH.NRVINCULOM = GPE_VINCULOH.NRVINCULOM ORDER BY DTMESCOMPETENC DESC)WHERE ROWNUM = 1) AND EXISTS (SELECT 1 FROM ESTRUTURAM EMA WHERE EMA.NRORG = :P_NRORG AND EMA.NRESTRUTURAM = (SELECT H.NRESTRUTURAMSUP FROM HIERARQUIAORG H WHERE H.NRORG = :P_NRORG AND H.NRESTRUTURAMREL = GPE_VINCULOH.NRESTRUTGEREN AND H.NRESTRUTURAMSUP IN ($listaEstrutSup) AND H.NRTIPOHIERARQORG = (SELECT NRTIPOHIERARQORG FROM TIPOHIERARQORG WHERE NRORG IN (0, :P_NRORG) AND NRTIPOESTRUTSUP = (SELECT NVL(NRTPESTRUTOMADOR, 7) FROM GPE_PARAMORG WHERE NRORG = :P_NRORG) AND NRTIPOESTRUTREL = (SELECT NRTPESTRUTGEREN FROM GPE_PARAMORG WHERE NRORG = :P_NRORG)) AND H.DTINIVIGENCIA = (SELECT MAX(DTINIVIGENCIA) FROM HIERARQUIAORG WHERE DTINIVIGENCIA <= '$fimcompetencia' AND NRORG = :P_NRORG AND NRESTRUTURAMREL = GPE_VINCULOH.NRESTRUTGEREN AND NRTIPOHIERARQORG = H.NRTIPOHIERARQORG)))) AND (DTRESCISAOVINC >= '$inicompetencia' OR DTRESCISAOVINC IS NULL) AND DTADMISSAOVINC >= '$inicial' AND NRORG = :P_NRORG";
                        } else if(!isset($inicial) && isset($final)){
                            $where = "NRORG = :P_NRORG AND (DTRESCISAOVINC >= '$inicompetencia' OR DTRESCISAOVINC IS NULL) AND DTADMISSAOVINC <= '$fimcompetencia' AND NRVINCULOM IN ( SELECT NRVINCULOM FROM GPE_VINCULOH WHERE NRORG = :P_NRORG AND DTMESCOMPETENC = ( SELECT * FROM ( SELECT DTMESCOMPETENC DTMESCOMPETENC FROM GPE_VINCULOH MAX_VINCULOH WHERE MAX_VINCULOH.NRORG = GPE_VINCULOH.NRORG AND MAX_VINCULOH.NRVINCULOM = GPE_VINCULOH.NRVINCULOM ORDER BY DTMESCOMPETENC DESC)WHERE ROWNUM = 1) AND EXISTS (SELECT 1 FROM ESTRUTURAM EMA WHERE EMA.NRORG = :P_NRORG AND EMA.NRESTRUTURAM = (SELECT H.NRESTRUTURAMSUP FROM HIERARQUIAORG H WHERE H.NRORG = :P_NRORG AND H.NRESTRUTURAMREL = GPE_VINCULOH.NRESTRUTGEREN AND H.NRESTRUTURAMSUP IN ($listaEstrutSup) AND H.NRTIPOHIERARQORG = (SELECT NRTIPOHIERARQORG FROM TIPOHIERARQORG WHERE NRORG IN (0, :P_NRORG) AND NRTIPOESTRUTSUP = (SELECT NVL(NRTPESTRUTOMADOR, 7) FROM GPE_PARAMORG WHERE NRORG = :P_NRORG) AND NRTIPOESTRUTREL = (SELECT NRTPESTRUTGEREN FROM GPE_PARAMORG WHERE NRORG = :P_NRORG)) AND H.DTINIVIGENCIA = (SELECT MAX(DTINIVIGENCIA) FROM HIERARQUIAORG WHERE DTINIVIGENCIA <= '$fimcompetencia' AND NRORG = :P_NRORG AND NRESTRUTURAMREL = GPE_VINCULOH.NRESTRUTGEREN AND NRTIPOHIERARQORG = H.NRTIPOHIERARQORG)))) AND (DTRESCISAOVINC >= '$inicompetencia' OR DTRESCISAOVINC IS NULL) AND DTADMISSAOVINC <= '$final' AND NRORG = :P_NRORG";
                        } else {
                            $where = "NRORG = :P_NRORG AND (DTRESCISAOVINC >= '$inicompetencia' OR DTRESCISAOVINC IS NULL) AND DTADMISSAOVINC <= '$fimcompetencia' AND NRVINCULOM IN ( SELECT NRVINCULOM FROM GPE_VINCULOH WHERE NRORG = :P_NRORG AND DTMESCOMPETENC = ( SELECT * FROM ( SELECT DTMESCOMPETENC DTMESCOMPETENC FROM GPE_VINCULOH MAX_VINCULOH WHERE MAX_VINCULOH.NRORG = GPE_VINCULOH.NRORG AND MAX_VINCULOH.NRVINCULOM = GPE_VINCULOH.NRVINCULOM ORDER BY DTMESCOMPETENC DESC)WHERE ROWNUM = 1) AND EXISTS (SELECT 1 FROM ESTRUTURAM EMA WHERE EMA.NRORG = :P_NRORG AND EMA.NRESTRUTURAM = (SELECT H.NRESTRUTURAMSUP FROM HIERARQUIAORG H WHERE H.NRORG = :P_NRORG AND H.NRESTRUTURAMREL = GPE_VINCULOH.NRESTRUTGEREN AND H.NRESTRUTURAMSUP IN ($listaEstrutSup) AND H.NRTIPOHIERARQORG = (SELECT NRTIPOHIERARQORG FROM TIPOHIERARQORG WHERE NRORG IN (0, :P_NRORG) AND NRTIPOESTRUTSUP = (SELECT NVL(NRTPESTRUTOMADOR, 7) FROM GPE_PARAMORG WHERE NRORG = :P_NRORG) AND NRTIPOESTRUTREL = (SELECT NRTPESTRUTGEREN FROM GPE_PARAMORG WHERE NRORG = :P_NRORG)) AND H.DTINIVIGENCIA = (SELECT MAX(DTINIVIGENCIA) FROM HIERARQUIAORG WHERE DTINIVIGENCIA <= '$fimcompetencia' AND NRORG = :P_NRORG AND NRESTRUTURAMREL = GPE_VINCULOH.NRESTRUTGEREN AND NRTIPOHIERARQORG = H.NRTIPOHIERARQORG)))) AND (DTRESCISAOVINC >= '$inicompetencia' OR DTRESCISAOVINC IS NULL) AND NRORG = :P_NRORG";
                        }
                    }
                    
                    $msgerro = 'Não foram encontrados vínculos para as estruturas informadas';
                    break;
                case 'LISTA_LOTE':
                    $param = array('P_NRORG' => $this->hcmParameters->getNrorgTrabalho());
                    $listaLote = isset($this->row['NRLOTEVINC_LIST']) && is_array($this->row['NRLOTEVINC_LIST']) && count($this->row['NRLOTEVINC_LIST']) ? implode(",", $this->row['NRLOTEVINC_LIST']) : null;

                    $where = "NRORG = :P_NRORG AND (DTRESCISAOVINC >= '$inicompetencia' OR DTRESCISAOVINC IS NULL) AND DTADMISSAOVINC <= '$fimcompetencia' AND NRVINCULOM IN (SELECT NRVINCULOM FROM FPA_ITEMLOTEVINC WHERE NRORG = :P_NRORG AND NRLOTEVINC IN($listaLote))";
                    $msgerro = 'Não foram encontrados vínculos para a lista informada';
                    break;
                case 'LISTA_HIERARQUIA':
                    $param = array('P_NRORG' => $this->hcmParameters->getNrorgTrabalho());
                    
                    $tipoHierarquia = isset($this->row['DSINICIOVALE']) && !empty($this->row['DSINICIOVALE']) ? $this->row['DSINICIOVALE'] : null;
                    
                    if(empty($tipoHierarquia)){
                        isset($this->row['NRTIPOHIERARQORG']) && !empty($this->row['NRTIPOHIERARQORG']) ? $this->row['NRTIPOHIERARQORG'] : null;
                    }
                    
                    $listaEstrutSup = isset($this->row['NRESTRUTURAMSUP']) && is_array($this->row['NRESTRUTURAMSUP']) && count($this->row['NRESTRUTURAMSUP']) ? implode(",", $this->row['NRESTRUTURAMSUP']) : 1;
                    
                    $inicial = !empty($this->row['DSINICIOVALE1']) ? $this->row['DSINICIOVALE1'] : null;
                    $final = !empty($this->row['DSFIMVALE1']) ? $this->row['DSFIMVALE1'] : null;
                    
                    if(isset($inicial) && isset($final)){
                        $where = "NRORG = :P_NRORG AND( DTRESCISAOVINC >= '$inicompetencia' OR DTRESCISAOVINC IS NULL ) AND DTADMISSAOVINC <= '$fimcompetencia' AND NRVINCULOM IN ( SELECT NRVINCULOM FROM GPE_VINCULOH WHERE NRORG = :P_NRORG AND DTMESCOMPETENC = ( SELECT * FROM ( SELECT DTMESCOMPETENC DTMESCOMPETENC FROM GPE_VINCULOH MAX_VINCULOH WHERE MAX_VINCULOH.NRORG = GPE_VINCULOH.NRORG AND MAX_VINCULOH.NRVINCULOM = GPE_VINCULOH.NRVINCULOM ORDER BY DTMESCOMPETENC DESC)WHERE ROWNUM = 1) AND EXISTS ( SELECT 1 FROM ESTRUTURAM EMA WHERE EMA.NRORG = :P_NRORG AND EMA.NRESTRUTURAM = ( SELECT H.NRESTRUTURAMSUP FROM HIERARQUIAORG H WHERE H.NRORG = :P_NRORG AND H.NRESTRUTURAMREL = GPE_VINCULOH.NRESTRUTGEREN AND H.NRESTRUTURAMSUP IN ($listaEstrutSup) AND H.NRTIPOHIERARQORG = '$tipoHierarquia' AND H.DTINIVIGENCIA = ( SELECT MAX(DTINIVIGENCIA) FROM HIERARQUIAORG WHERE DTINIVIGENCIA <= '$fimcompetencia' AND NRORG = :P_NRORG AND NRESTRUTURAMREL = GPE_VINCULOH.NRESTRUTGEREN AND NRTIPOHIERARQORG = H.NRTIPOHIERARQORG ) ) ) ) AND ( DTRESCISAOVINC >= '$inicompetencia' OR DTRESCISAOVINC IS NULL ) AND DTADMISSAOVINC BETWEEN '$inicial' AND '$final' AND NRORG = :P_NRORG";
                    }
                    else if(isset($inicial) && !isset($final)){
                        $where = "NRORG = :P_NRORG AND( DTRESCISAOVINC >= '$inicompetencia' OR DTRESCISAOVINC IS NULL ) AND DTADMISSAOVINC <= '$fimcompetencia' AND NRVINCULOM IN ( SELECT NRVINCULOM FROM GPE_VINCULOH WHERE NRORG = :P_NRORG AND DTMESCOMPETENC = ( SELECT * FROM ( SELECT DTMESCOMPETENC DTMESCOMPETENC FROM GPE_VINCULOH MAX_VINCULOH WHERE MAX_VINCULOH.NRORG = GPE_VINCULOH.NRORG AND MAX_VINCULOH.NRVINCULOM = GPE_VINCULOH.NRVINCULOM ORDER BY DTMESCOMPETENC DESC)WHERE ROWNUM = 1) AND EXISTS ( SELECT 1 FROM ESTRUTURAM EMA WHERE EMA.NRORG = :P_NRORG AND EMA.NRESTRUTURAM = ( SELECT H.NRESTRUTURAMSUP FROM HIERARQUIAORG H WHERE H.NRORG = :P_NRORG AND H.NRESTRUTURAMREL = GPE_VINCULOH.NRESTRUTGEREN AND H.NRESTRUTURAMSUP IN ($listaEstrutSup) AND H.NRTIPOHIERARQORG = '$tipoHierarquia' AND H.DTINIVIGENCIA = ( SELECT MAX(DTINIVIGENCIA) FROM HIERARQUIAORG WHERE DTINIVIGENCIA <= '$fimcompetencia' AND NRORG = :P_NRORG AND NRESTRUTURAMREL = GPE_VINCULOH.NRESTRUTGEREN AND NRTIPOHIERARQORG = H.NRTIPOHIERARQORG ) ) ) ) AND ( DTRESCISAOVINC >= '$inicompetencia' OR DTRESCISAOVINC IS NULL ) AND DTADMISSAOVINC >= '$inicial' AND NRORG = :P_NRORG";
                    }
                    else if(!isset($inicial) && isset($final)){
                        $where = "NRORG = :P_NRORG AND( DTRESCISAOVINC >= '$inicompetencia' OR DTRESCISAOVINC IS NULL ) AND DTADMISSAOVINC <= '$fimcompetencia' AND NRVINCULOM IN ( SELECT NRVINCULOM FROM GPE_VINCULOH WHERE NRORG = :P_NRORG AND DTMESCOMPETENC = ( SELECT * FROM ( SELECT DTMESCOMPETENC DTMESCOMPETENC FROM GPE_VINCULOH MAX_VINCULOH WHERE MAX_VINCULOH.NRORG = GPE_VINCULOH.NRORG AND MAX_VINCULOH.NRVINCULOM = GPE_VINCULOH.NRVINCULOM ORDER BY DTMESCOMPETENC DESC)WHERE ROWNUM = 1) AND EXISTS ( SELECT 1 FROM ESTRUTURAM EMA WHERE EMA.NRORG = :P_NRORG AND EMA.NRESTRUTURAM = ( SELECT H.NRESTRUTURAMSUP FROM HIERARQUIAORG H WHERE H.NRORG = :P_NRORG AND H.NRESTRUTURAMREL = GPE_VINCULOH.NRESTRUTGEREN AND H.NRESTRUTURAMSUP IN ($listaEstrutSup) AND H.NRTIPOHIERARQORG = '$tipoHierarquia' AND H.DTINIVIGENCIA = ( SELECT MAX(DTINIVIGENCIA) FROM HIERARQUIAORG WHERE DTINIVIGENCIA <= '$fimcompetencia' AND NRORG = :P_NRORG AND NRESTRUTURAMREL = GPE_VINCULOH.NRESTRUTGEREN AND NRTIPOHIERARQORG = H.NRTIPOHIERARQORG ) ) ) ) AND ( DTRESCISAOVINC >= '$inicompetencia' OR DTRESCISAOVINC IS NULL ) AND DTADMISSAOVINC <= '$final' AND NRORG = :P_NRORG";
                    }
                    else {
                        $where = "NRORG = :P_NRORG AND( DTRESCISAOVINC >= '$inicompetencia' OR DTRESCISAOVINC IS NULL ) AND DTADMISSAOVINC <= '$fimcompetencia' AND NRVINCULOM IN ( SELECT NRVINCULOM FROM GPE_VINCULOH WHERE NRORG = :P_NRORG AND DTMESCOMPETENC = ( SELECT * FROM ( SELECT DTMESCOMPETENC DTMESCOMPETENC FROM GPE_VINCULOH MAX_VINCULOH WHERE MAX_VINCULOH.NRORG = GPE_VINCULOH.NRORG AND MAX_VINCULOH.NRVINCULOM = GPE_VINCULOH.NRVINCULOM ORDER BY DTMESCOMPETENC DESC)WHERE ROWNUM = 1) AND EXISTS ( SELECT 1 FROM ESTRUTURAM EMA WHERE EMA.NRORG = :P_NRORG AND EMA.NRESTRUTURAM = ( SELECT H.NRESTRUTURAMSUP FROM HIERARQUIAORG H WHERE H.NRORG = :P_NRORG AND H.NRESTRUTURAMREL = GPE_VINCULOH.NRESTRUTGEREN AND H.NRESTRUTURAMSUP IN ($listaEstrutSup) AND H.NRTIPOHIERARQORG = '$tipoHierarquia' AND H.DTINIVIGENCIA = ( SELECT MAX(DTINIVIGENCIA) FROM HIERARQUIAORG WHERE DTINIVIGENCIA <= '$fimcompetencia' AND NRORG = :P_NRORG AND NRESTRUTURAMREL = GPE_VINCULOH.NRESTRUTGEREN AND NRTIPOHIERARQORG = H.NRTIPOHIERARQORG ) ) ) ) AND ( DTRESCISAOVINC >= '$inicompetencia' OR DTRESCISAOVINC IS NULL ) AND NRORG = :P_NRORG";
                    }
                    
                    $msgerro = 'Não foram encontrados vínculos para as estruturas superiores informadas';
                    break;
                default: {
                    return null;
                }
            }
            
            $where .= $vinculosComHistorico . $cond_afastados;

            $result = GpeVinculom::getObjectsByWhere($where, $param);
            
            if ($result) {
                return $result;
            } else {
                throw new \Exception($msgerro, Messages::ERROR_MESSAGE_CODE);
            }

        } catch (\Exception $e) {
            throw $e;
        }
    }
    

    public function getBeneficiarioPorVinculo($nrorg, $nrvinculom, $dtmescompetenc, $tpperiodo, $nrtpbeneficio=null, $nrestruturam=null){
        $condTpbeneficio = $nrtpbeneficio ? "AND BH.NRTPBENEFICIO = $nrtpbeneficio" : "";
        $condEstrutura = $nrestruturam ? "AND BH.NRESTRUTURAM = $nrestruturam" : "";
        $where =
            "NRVINCULOM = $nrvinculom AND
            NRORG = $nrorg AND
            NRBENEFICIOM IN (SELECT BM.NRBENEFICIOM
                               FROM VAB_BENEFICIOM BM
                               JOIN VAB_BENEFICIOH BH ON BH.NRORG = BM.NRORG
                                                     AND BH.NRBENEFICIOM = BM.NRBENEFICIOM
                                                     AND BH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM VAB_BENEFICIOH
                                                                               WHERE NRORG = BH.NRORG
                                                                                 AND NRBENEFICIOM = BH.NRBENEFICIOM
                                                                                 AND DTMESCOMPETENC <= '$dtmescompetenc')
                              WHERE BM.NRORG = $nrorg
                                AND BM.IDATIVO = 'S'
                                AND BH.TPPERIODOBENEF = '$tpperiodo'
                                $condTpbeneficio
                                $condEstrutura)";

        $beneficiarios = VabBeneficiario::getObjectsByWhere($where, array(), 'NRBENEFICIOM ASC, NREVENTOM ASC');
        return $beneficiarios;
    }
    
    public function getBeneficioPorVinculo($dtmescompetenc, $nrorg, $nrbeneficiom){
        $query = <<<SQL
    SELECT
        BNM.*,
        BNH.*
    FROM
        VAB_BENEFICIOM BNM 
    INNER JOIN
        VAB_BENEFICIOH BNH 
            ON BNH.NRBENEFICIOM = BNM.NRBENEFICIOM 
            AND(
                BNM.NRORG = BNH.NRORG 
            ) 
            AND BNH.DTMESCOMPETENC =(
                    SELECT
                        MAX(BNEH.DTMESCOMPETENC) 
                FROM
                    VAB_BENEFICIOH BNEH 
                WHERE
                    BNEH.NRORG = BNH.NRORG 
                    AND BNEH.NRBENEFICIOM = BNH.NRBENEFICIOM 
                    AND BNEH.DTMESCOMPETENC <= :DTMESCOMPETENC
            )        
    WHERE 
        BNM.NRBENEFICIOM = :NRBENEFICIOM
        AND BNM.NRORG = :NRORG
SQL;

        $retorno = $this->connection->fetchAll($query,[ 
                                                    "DTMESCOMPETENC" => $dtmescompetenc,
                                                    "NRORG"         => $nrorg,
                                                    "NRBENEFICIOM"  => $nrbeneficiom,
                                                   ]
                                           );
        return $retorno[0];
    }
    
    public function validaLiquidoBeneficioOcorrencia1($nrtipomoviment, $nrcalculofolha, $nrvinculom, $dtmescompetencia, $nrorg, $nreventom, $dtpagamento, $nrocorrencia){
        $query = <<<SQL
    SELECT
        ITEC.*,
        CF.NROCORRECAL,
        CF.NRTIPOMOVIMENT
    FROM
        FPA_ITECALCFOLHA ITEC
        JOIN FPA_CALCULOFOLHA CF ON CF.NRCALCULOFOLHA = ITEC.NRCALCULOFOLHA
        AND CF.NRORG = ITEC.NRORG
        AND CF.NROCORRECAL = 1
        AND CF.NRTIPOMOVIMENT = :P_NRTPMOVIMENT
    WHERE
            ITEC.NRORG = :P_NRORG
        AND ITEC.NREVENTOM = :P_NREVENTOM
        AND ITEC.DTOCORRENCIA = :P_DTPAGAMENTO
        AND ITEC.NRCALCULOFOLHA = :P_NRCALCULOFOLHA
        AND ITEC.NRVINCULOM = :P_NRVINCULOM
SQL;

        $retorno = $this->connection->fetchAll($query,[ 
                                                    'P_NRORG'            => $nrorg,
                                                    'P_NRVINCULOM'       => $nrvinculom,
                                                    'P_NRCALCULOFOLHA'   => $nrcalculofolha,
                                                    'P_NREVENTOM'        => $nreventom,
                                                    'P_DTPAGAMENTO'      => $dtpagamento,
                                                    'P_NRTPMOVIMENT'     => $nrtipomoviment,
                                                   ]
                                           );
        return $retorno[0];
    }
    
    public function getBeneficioDescontoSind($dtmescompetenc, $nrorg, $nrTpBeneficio, $nrEstrutSind){
        $query = <<<SQL
    SELECT
        BCCT.*
    
    FROM 
        FPA_CADASTROCCT CCT 

    JOIN
        VAB_BENEFICIOCCT BCCT
            ON BCCT.NRCADASTROCCT = CCT.NRCADASTROCCT
            AND BCCT.NRORG = CCT.NRORG
    
    WHERE 
        BCCT.NRTPBENEFICIO = :NRTPBENEFICIO
        AND CCT.NRESTRUTURAM = :NRESTRUTURASIND
        AND CCT.NRORG = :NRORG
        AND CCT.IDATIVO = 'S'
        AND CCT.DTANOBASECCT = (SELECT MAX(DTANOBASECCT) FROM FPA_CADASTROCCT CCT2
                        WHERE CCT2.IDATIVO = 'S'
                        AND CCT2.NRORG = CCT.NRORG
                        AND CCT2.NRESTRUTURAM = CCT.NRESTRUTURAM
                        AND CCT2.DTANOBASECCT <= :DTMESCOMPETENC
                       )
SQL;
        $params = [ 
                    "DTMESCOMPETENC"  => $dtmescompetenc,
                    "NRORG"           => $nrorg,
                    "NRTPBENEFICIO"   => intval($nrTpBeneficio),
                    "NRESTRUTURASIND" => intval($nrEstrutSind),
                  ];
        
        $retorno = $this->connection->fetchAll($query, $params);
        
        if(is_array($retorno) && isset($retorno[0])) $retorno = $retorno[0];
        else $retorno = null;

        return $retorno;
    }
    
    public function retrocederCalculos_Mensal($nrcalculobenef, $nrvinculom, $dtmescompetenc, $nrorg, $eventos, $dtpagamento){
        
        $qtdeDeletados = 0;
        
        $IN = " AND NREVENTOM IN (";

        foreach($eventos as $key=>$value){
            $row = $this->entityManager->getRepository(Repositories::FPA_ITECALCFOLHA)->findOneBy( 
                [ 
                 'nrcalculofolha' => $nrcalculobenef,
                 'nrvinculom'     => $nrvinculom,
                 'nreventom'      => $value,
                 'nrorg'          => $nrorg,
                 'dtocorrencia'   => Dateutil::getDataDeString($dtpagamento, DateUtil::FORMATO_BRASILEIRO, true),
                ]
            );
            
            if($row){
                if($row->getNrlotelancevento())
                    MsgCalculo::getInstance()->adicionaMensagem('Evento '.$value.' para o vínculo '.$nrvinculom.' já exportado para folha. ', MsgCalculo::ALERTA);
                else{
                    $IN .= $value.', ';
                    $qtdeDeletados++;
                }    
            }        

        }

        $virgula = strripos($IN, ',');
        
        if($virgula){ 
            $IN = substr($IN, 0, -2);
            $IN .= ")";
        }else
            $IN .= 'null)';
        
        
        // Retrocesso na tabela de itens de cálculo
        $query = $this->entityManager->getConnection()->prepare("
            DELETE 
            FROM FPA_ITECALCFOLHA
            WHERE 
                NRORG = :NRORG
                AND NRVINCULOM = :NRVINCULOM 
                AND DTOCORRENCIA = :DTOCORRENCIA
                AND NRCALCULOFOLHA = :NRCALCULOFOLHA".$IN
        );

        $query->execute(
            [
                "NRVINCULOM"     => $nrvinculom,
                "DTOCORRENCIA"   => $dtpagamento,
                "NRORG"          => $nrorg,
                "NRCALCULOFOLHA" => $nrcalculobenef,
            ]
        );
        
        // Retrocesso na tabela VAB_APURABENEFICIO
        $query = $this->entityManager->getConnection()->prepare("
            DELETE 
            FROM VAB_APURABENEFICIO
            WHERE 
                NRORG = :NRORG
                AND NRVINCULOM = :NRVINCULOM 
                AND NRCALCULOFOLHA = :NRCALCULOFOLHA".$IN
        );

        $query->execute(
            [
                "NRVINCULOM"     => $nrvinculom,
                "NRORG"          => $nrorg,
                "NRCALCULOFOLHA" => $nrcalculobenef,
            ]
        );


        return $qtdeDeletados;

    }

    public function retrocederCalculos_Diario($nrcalculobenef, $nrvinculom, $dtmescompetenc, $nrorg, $eventos, $dtpagamento){
        
        $qtdeDeletados = 0;
        
        $IN = " AND NREVENTOM IN (";

        foreach($eventos as $key=>$value){
            
            $row = $this->entityManager->getRepository(Repositories::VAB_HISTBENEFVINC)->findOneBy( 
                [ 
                 'nrcalculofolha' => $nrcalculobenef,
                 'dtmescompetenc' => Dateutil::getDataDeString($this->hcmParameters->getDtCompetencia(), DateUtil::FORMATO_BRASILEIRO, true),
                 'nrvinculom'     => $nrvinculom,
                 'nreventom'      => $value,
                 'nrorg'          => $this->hcmParameters->getNrorgTrabalho(),
                 'dtpgtobenef'    => Dateutil::getDataDeString($dtpagamento, DateUtil::FORMATO_BRASILEIRO, true)
                ]
            );

            if($row){
                $IN .= $value.', ';
                $qtdeDeletados++;
            }        
        }

        $virgula = strripos($IN, ',');
        
        if($virgula){ 
            $IN = substr($IN, 0, -2);
            $IN .= ")";
        }else
            $IN .= 'null)';
        
        
        // Retrocesso na tabela de VAB_HISTBENEFVINC
        $query = $this->entityManager->getConnection()->prepare("
            DELETE 
            FROM VAB_HISTBENEFVINC
            WHERE 
                NRORG = :NRORG
                AND NRVINCULOM = :NRVINCULOM 
                AND DTMESCOMPETENC = :DTCOMPETENC
                AND DTPGTOBENEF = :DTOCORRENCIA
                AND NRCALCULOFOLHA = :NRCALCULOFOLHA".$IN
                
        );

        $query->execute(
            [
                "NRVINCULOM"     => $nrvinculom,
                "DTOCORRENCIA"   => $dtpagamento,
                "DTCOMPETENC"   => $dtmescompetenc,
                "NRORG"          => $nrorg,
                "NRCALCULOFOLHA" => $nrcalculobenef,
            ]
        );

        return $qtdeDeletados;

    }
    
    public function persistirCalculoDIRF_Mensal($nrcalculobenef, $nrbeneficiom, $nrvinculom, $nrPessoaBeneficiario, $proporcao, $nrEventoValorEvento, $tipoEvento){
        
        $row_fpaIteCalcFolha = null;
        
        $row_fpaIteCalcFolha = $this->entityManager->getRepository(Repositories::FPA_ITECALCFOLHA)->findOneBy( 
            [ 
             'nrcalculofolha' => $nrcalculobenef,
             'nrvinculom'     => $nrvinculom,
             'nreventom'      => key($nrEventoValorEvento),
             'nrorg'          => $this->hcmParameters->getNrorgTrabalho(),
             'dtocorrencia'   => Dateutil::getDataDeString($this->hcmParameters->getDtCompetencia(), DateUtil::FORMATO_BRASILEIRO, true),
            ]
        );


        if (
                ( !$row_fpaIteCalcFolha || !$row_fpaIteCalcFolha->getNrlotelancevento() ) &&
                count($nrEventoValorEvento) > 0 && ($nrEventoValorEvento[key($nrEventoValorEvento)] >= 0) 
            
           ) {
               
            $this->deleteVabApuraBeneficio($nrcalculobenef, $nrvinculom, key($nrEventoValorEvento), $nrPessoaBeneficiario, $tipoEvento, $nrbeneficiom);
            
            $vlrEvento = $nrEventoValorEvento[key($nrEventoValorEvento)];

            $eventoh = $this->entityManager->getRepository(Repositories::FPA_EVENTOM)->retornaEvento(
                key($nrEventoValorEvento),
                $this->hcmParameters->getNrorgTrabalho(), 
                $this->hcmParameters->getNrorgPadrao(), 
                $this->hcmParameters->getDtCompetencia() 
            );                    

             $vlrEvento = Valor::arredonda(
                $vlrEvento,
                $eventoh->historico->getIdtipoarredonda(), 
                $eventoh->historico->getNrdigarredonda(),
                $eventoh->historico->getVrlimitearred()
            );

            $query = $this->entityManager->getConnection()->prepare("
                INSERT INTO VAB_APURABENEFICIO (          
                  NRAPURABENEFICIO,
                  NRCALCULOFOLHA,
                  NRVINCULOM,
                  NRPESSOA,
                  NRBENEFICIOM,
                  DSTIPOBENEFICIO,
                  NREVENTOM,
                  VRAPURABENEFICIO,
                  NRORG,
                  IDATIVO,
                  DTINCLUSAO,
                  DTULTATU,
                  NRORGINCLUSAO,
                  NRORGULTATU,
                  CDOPERINCLUSAO,
                  CDOPERULTATU              
                ) 
                
                VALUES (
                  :P_NRAPURABENEFICIO,
                  :P_NRCALCULOFOLHA,
                  :P_NRVINCULOM,
                  :P_NRPESSOA,
                  :P_NRBENEFICIOM,
                  :P_DSTIPOBENEFICIO,
                  :P_NREVENTOM,
                  :P_VRAPURABENEFICIO,
                  :P_NRORG,
                  :P_IDATIVO,
                  :P_DTINCLUSAO,
                  :P_DTULTATU,
                  :P_NRORGINCLUSAO,
                  :P_NRORGULTATU,
                  :P_CDOPERINCLUSAO,
                  :P_CDOPERULTATU              
                )"
            );
            $query->execute(
                array(
                    "P_NRAPURABENEFICIO" => NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'VAB_APURABENEFICIO', 1, 22, true),
                    "P_NRCALCULOFOLHA"   => $nrcalculobenef,
                    "P_NRVINCULOM"       => $nrvinculom,
                    "P_NRPESSOA"         => $nrPessoaBeneficiario,
                    "P_NRBENEFICIOM"     => $nrbeneficiom,
                    "P_DSTIPOBENEFICIO"  => $tipoEvento,
                    "P_NREVENTOM"        => key($nrEventoValorEvento),
                    "P_VRAPURABENEFICIO" => $vlrEvento,
                    "P_NRORG"            => $this->hcmParameters->getNrorgTrabalho(),
                    "P_IDATIVO"          => "S",
                    "P_DTINCLUSAO"       => DateUtil::getDataAtual()->format('d/m/Y h:m:s'),
                    "P_DTULTATU"         => DateUtil::getDataAtual()->format('d/m/Y h:m:s'),
                    "P_NRORGINCLUSAO"    => $this->hcmParameters->getNrorgTrabalho(),
                    "P_NRORGULTATU"      => $this->hcmParameters->getNrorgTrabalho(),
                    "P_CDOPERINCLUSAO"   => $this->hcmParameters->getCdoperador(),
                    "P_CDOPERULTATU"     => $this->hcmParameters->getCdoperador()
                )
            );       

        }

    }
    
    public function deleteVabApuraBeneficio($nrcalculobenef, $nrvinculom, $nrEvento, $nrPessoaBeneficiario, $tipoEvento, $nrbeneficiom){
        $query = $this->entityManager->getConnection()->prepare("
            DELETE FROM VAB_APURABENEFICIO
            WHERE 
                NRORG = :P_NRORG 
                AND NRCALCULOFOLHA = :P_NRCALCULOFOLHA 
                AND NRVINCULOM = :P_NRVINCULOM 
                AND NRPESSOA = :P_NRPESSOA
                AND NREVENTOM = :P_NREVENTOM
                AND DSTIPOBENEFICIO = :P_DSTIPOBENEFICIO
                AND NRBENEFICIOM = :P_NRBENEFICIOM"
        );
        $query->execute(
            array(
                "P_NRCALCULOFOLHA"   => $nrcalculobenef,
                "P_NRVINCULOM"       => $nrvinculom,
                "P_NREVENTOM"        => $nrEvento,
                "P_DSTIPOBENEFICIO"  => $tipoEvento,
                "P_NRORG"            => $this->hcmParameters->getNrorgTrabalho(),
                "P_NRPESSOA"         => $nrPessoaBeneficiario,
                "P_NRBENEFICIOM"     => $nrbeneficiom
            )
        );          
    }
    
    public function persistirHistorico_Diario($nrcalculobenef, $nrbeneficiom, $nrvinculom, $nrEvento, $valorEvento, $arrayDiasCalculados, $dtIni, $dtFim, $dtpagamento, $vlrTotalBenef, $qtdeBeneficio, $idcalccomplementar = null, $valorAdescontar = null){
            
            $query = $this->entityManager->getConnection()->prepare("
                INSERT INTO VAB_HISTBENEFVINC 
                (
                    NRHISTBENEFVINC,
                    NRBENEFICIOM,
                    NRVINCULOM,
                    NRCALCULOFOLHA,
                    NREVENTOM,
                    NRORG,
                    DTMESCOMPETENC,
                    DTINIAPURBENEF,
                    DTFINAPURBENEF,
                    DTPGTOBENEF,
                    QTAPURBENEF,
                    QTDIASBENEF,
                    QTDIASAFASTA,
                    QTDIASFALTAS,
                    QTDIASFERIAS,
                    QTDIASAVISO,
                    QTDIASDSR,
                    VRBENEFICIO,
                    VRDEDUCAOBENEF,
                    VRTOTALBENEF,
                    IDATIVO,
                    NRORGINCLUSAO,
                    DTINCLUSAO,
                    CDOPERINCLUSAO,
                    NRORGULTATU,
                    DTULTATU,
                    CDOPERULTATU,
                    QTDIASFERIADO,
                    QTDIASTRABPERIODO,           
                    QTDIASATESTADO,
                    IDCALCCOMPLEMENTAR
                )

                VALUES (
                    :P_NRHISTBENEFVINC,
                    :P_NRBENEFICIOM,
                    :P_NRVINCULOM,
                    :P_NRCALCULOFOLHA,
                    :P_NREVENTOM,
                    :P_NRORG,
                    :P_DTMESCOMPETENC,
                    :P_DTINIAPURBENEF,
                    :P_DTFINAPURBENEF,
                    :P_DTPGTOBENEF,
                    :P_QTAPURBENEF,
                    :P_QTDIASBENEF,
                    :P_QTDIASAFASTA,
                    :P_QTDIASFALTAS,
                    :P_QTDIASFERIAS,
                    :P_QTDIASAVISO,
                    :P_QTDIASDSR,
                    :P_VRBENEFICIO,
                    :P_VRDEDUCAOBENEF,
                    :P_VRTOTALBENEF,
                    :P_IDATIVO,
                    :P_NRORGINCLUSAO,
                    :P_DTINCLUSAO,
                    :P_CDOPERINCLUSAO,
                    :P_NRORGULTATU,
                    :P_DTULTATU,
                    :P_CDOPERULTATU,
                    :P_QTDIASFERIADO,
                    :P_QTDIASTRABPERIODO,           
                    :P_QTDIASATESTADO,
                    :P_IDCALCCOMPLEMENTAR
                )"
            );

            $query->execute(
                array(
                    "P_NRHISTBENEFVINC"   => intval(NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'VAB_HISTBENEFVINC', 1, 22, true)),
                    "P_NRBENEFICIOM"      => intval($nrbeneficiom),
                    "P_NRVINCULOM"        => intval($nrvinculom),
                    "P_NRCALCULOFOLHA"    => intval($nrcalculobenef),
                    "P_NREVENTOM"         => intval($nrEvento),
                    "P_NRORG"             => intval($this->hcmParameters->getNrorgTrabalho()),
                    
                    "P_DTMESCOMPETENC"    => $this->hcmParameters->getDtCompetencia(),
                    "P_DTINIAPURBENEF"    => substr($dtIni, 0, 10),
                    "P_DTFINAPURBENEF"    => substr($dtFim, 0, 10),
                    "P_DTPGTOBENEF"       => substr($dtpagamento, 0, 10),
                    
                    "P_QTAPURBENEF"       => $qtdeBeneficio,
                    "P_QTDIASBENEF"       => intval($arrayDiasCalculados['DIASCONSIDERACAO']),
                    "P_QTDIASAFASTA"      => intval($arrayDiasCalculados['DIASAFASTAMENTO']),
                    "P_QTDIASFALTAS"      => intval($arrayDiasCalculados['DIASFALTAS']),
                    "P_QTDIASFERIAS"      => intval($arrayDiasCalculados['DIASFERIAS']),
                    "P_QTDIASAVISO"       => intval($arrayDiasCalculados['DIASAVISO']),
                    "P_QTDIASDSR"         => intval($arrayDiasCalculados['DIASDSR']),
                    "P_QTDIASATESTADO"    => intval($arrayDiasCalculados['DIASATESTADO']),
                    "P_VRBENEFICIO"       => $valorEvento,
                    "P_VRDEDUCAOBENEF"    => $valorAdescontar,
                    "P_VRTOTALBENEF"      => $vlrTotalBenef,
                    "P_IDATIVO"           => "S",
                    "P_NRORGINCLUSAO"     => intval($this->hcmParameters->getNrorgTrabalho()),
                    "P_DTINCLUSAO"        => DateUtil::getDataAtual()->format('d/m/Y H:i:s'),
                    "P_CDOPERINCLUSAO"    => intval($this->hcmParameters->getCdoperador()),
                    "P_NRORGULTATU"       => intval($this->hcmParameters->getNrorgTrabalho()),
                    "P_DTULTATU"          => DateUtil::getDataAtual()->format('d/m/Y H:i:s'),
                    "P_CDOPERULTATU"      => intval($this->hcmParameters->getCdoperador()),
                    "P_QTDIASFERIADO"     => intval($arrayDiasCalculados['DIASFERIADO']),
                    "P_QTDIASTRABPERIODO" => intval($arrayDiasCalculados['DIASDETRABALHONOPERIODO']),
                    "P_IDCALCCOMPLEMENTAR"=> $idcalccomplementar,
                )
            );

    }

}