/* -------------------------------------------------------------------------
* Criao da coluna TXPROPFAS_BLOB, necessria na converso LONG RAW -> CLOB
*/ -------------------------------------------------------------------------

BEGIN
	EXECUTE IMMEDIATE 'ALTER TABLE TXTPROPFAS ADD TXPROPFAS_BLOB BLOB';
EXCEPTION
	WHEN OTHERS THEN
		NULL;
END;
/

/* -----------------------------------------------------------------------------
* Converte coluna TXPROPFAS_CLOB em formato LONG RAW e atualiza coluna TXPROPFAS.
*/ -----------------------------------------------------------------------------

CREATE OR REPLACE 
PROCEDURE SET_TXPROPFAS_LR
(P_NRPROPOSFAS			IN TXTPROPFAS.NRPROPOSFAS%TYPE,
 P_NRVERPROPFAS		IN TXTPROPFAS.NRVERPROPFAS%TYPE,
 P_CDTXTPROPFAS		IN TXTPROPFAS.CDTXTPROPFAS%TYPE,
 P_IDTIPOTXTPROPFAS	IN TXTPROPFAS.IDTIPOTXTPROPFAS%TYPE,
 P_NRORG 				IN TXTPROPFAS.NRORG%TYPE)
 
AS
	-- variables for convertion CLOB -> BLOB
	POS PLS_INTEGER := 1;
	BUFFER RAW( 32767 );
	V_CLOB_VALUE CLOB;
	V_BLOB_VALUE BLOB;
	LOB_LEN PLS_INTEGER;

	-- variables for convertion BLOB -> LONG RAW
	V_LENGTH NUMBER;
	V_TEXT_AMOUNT NUMBER;
	V_TEXT_POS NUMBER;
	V_RAW_VALUE LONG RAW;
	V_RAW_TEMP LONG RAW;
BEGIN
	-- Convert CLOB to BLOB
	SELECT TXPROPFAS_CLOB
	  INTO V_CLOB_VALUE
	  FROM TXTPROPFAS
	 WHERE NRPROPOSFAS = P_NRPROPOSFAS 
	   AND NRVERPROPFAS = P_NRVERPROPFAS    
	   AND CDTXTPROPFAS = P_CDTXTPROPFAS 	
	   AND IDTIPOTXTPROPFAS = P_IDTIPOTXTPROPFAS
	   AND NRORG = P_NRORG;
	   
	LOB_LEN := DBMS_LOB.GETLENGTH( V_CLOB_VALUE );
	DBMS_LOB.CREATETEMPORARY( V_BLOB_VALUE, TRUE );
	DBMS_LOB.OPEN( V_BLOB_VALUE, DBMS_LOB.LOB_READWRITE );

	LOOP
		BUFFER := UTL_RAW.CAST_TO_RAW( DBMS_LOB.SUBSTR( V_CLOB_VALUE, 16000, POS ) );

		IF UTL_RAW.LENGTH( BUFFER ) > 0 THEN
			DBMS_LOB.WRITEAPPEND( V_BLOB_VALUE, UTL_RAW.LENGTH( BUFFER ), BUFFER );
		END IF;

		POS := POS + 16000;
		EXIT WHEN POS > LOB_LEN;
	END LOOP;

	-- find the length of the blob (bytes)
	V_LENGTH := DBMS_LOB.GETLENGTH(V_BLOB_VALUE);

	-- Set the amount (bytes) to be converted equal to the length of the blob.
	V_TEXT_AMOUNT := V_LENGTH;

	--Convert blob to long raw
	V_TEXT_POS := 1;
	WHILE V_TEXT_POS <= V_LENGTH LOOP
		DBMS_LOB.READ(V_BLOB_VALUE, V_TEXT_AMOUNT, V_TEXT_POS, V_RAW_TEMP);
		V_RAW_VALUE := V_RAW_VALUE||V_RAW_TEMP;
		V_TEXT_POS := V_TEXT_POS + V_TEXT_AMOUNT; 
	END LOOP;

	UPDATE TXTPROPFAS
	   SET TXPROPFAS = V_RAW_VALUE
	 WHERE NRPROPOSFAS = P_NRPROPOSFAS 
	   AND NRVERPROPFAS = P_NRVERPROPFAS    
	   AND CDTXTPROPFAS = P_CDTXTPROPFAS 	
	   AND IDTIPOTXTPROPFAS = P_IDTIPOTXTPROPFAS
	   AND NRORG = P_NRORG;
	   
	COMMIT;

END;
/

/* -----------------------------------------------------------------
* Retorna valor recebido em formato BLOB, convertido em formato CLOB
*/ -----------------------------------------------------------------

CREATE OR REPLACE FUNCTION GET_TXPROPFAS_CLOB (blob_in IN BLOB)
RETURN CLOB
AS
	v_clob CLOB;
	v_varchar VARCHAR2(32767);
	v_start PLS_INTEGER := 1;
	v_buffer PLS_INTEGER := 32767;
BEGIN
	DBMS_LOB.CREATETEMPORARY(v_clob, TRUE);

	FOR i IN 1..CEIL(DBMS_LOB.GETLENGTH(blob_in) / v_buffer)
	LOOP

		v_varchar := UTL_RAW.CAST_TO_VARCHAR2(DBMS_LOB.SUBSTR(blob_in, v_buffer, v_start));

		DBMS_LOB.WRITEAPPEND(v_clob, LENGTH(v_varchar), v_varchar);
		v_start := v_start + v_buffer;
	END LOOP;
	
	RETURN v_clob;
	
END GET_TXPROPFAS_CLOB;
/

/* ------------------------------------------------------------------------------------
* Converte coluna TXPROPFAS (LONG RAW) em formato CLOB e atualiza coluna TXPROPFAS_CLOB
*/ ------------------------------------------------------------------------------------

CREATE OR REPLACE 
PROCEDURE SET_TXPROPFAS_CLOB
(P_NRPROPOSFAS 		IN TXTPROPFAS.NRPROPOSFAS%TYPE,
 P_NRVERPROPFAS		IN TXTPROPFAS.NRVERPROPFAS%TYPE,
 P_CDTXTPROPFAS		IN TXTPROPFAS.CDTXTPROPFAS%TYPE,
 P_IDTIPOTXTPROPFAS	IN TXTPROPFAS.IDTIPOTXTPROPFAS%TYPE,
 P_NRORG 				IN TXTPROPFAS.NRORG%TYPE)
 
AS

BEGIN
	UPDATE TXTPROPFAS 
		SET TXPROPFAS_BLOB = (SELECT TO_LOB(TXPROPFAS) 
										FROM TXTPROPFAS 
									  WHERE NRPROPOSFAS = P_NRPROPOSFAS 
										 AND NRVERPROPFAS = P_NRVERPROPFAS
										 AND CDTXTPROPFAS = P_CDTXTPROPFAS
										 AND IDTIPOTXTPROPFAS = P_IDTIPOTXTPROPFAS
										 AND NRORG = P_NRORG)
	 WHERE NRPROPOSFAS = P_NRPROPOSFAS 
	   AND NRVERPROPFAS = P_NRVERPROPFAS    
	   AND CDTXTPROPFAS = P_CDTXTPROPFAS 	
	   AND IDTIPOTXTPROPFAS = P_IDTIPOTXTPROPFAS
	   AND NRORG = P_NRORG;

	UPDATE TXTPROPFAS 
		SET TXPROPFAS_CLOB = GET_TXPROPFAS_CLOB(TXPROPFAS_BLOB),
		    TXPROPFAS_BLOB = NULL
	 WHERE NRPROPOSFAS = P_NRPROPOSFAS 
	   AND NRVERPROPFAS = P_NRVERPROPFAS    
	   AND CDTXTPROPFAS = P_CDTXTPROPFAS 	
	   AND IDTIPOTXTPROPFAS = P_IDTIPOTXTPROPFAS
	   AND NRORG = P_NRORG;
	   
	COMMIT;

END;
/
