<?php
namespace Zeedhi\Queue;

use Stomp\SimpleStomp;
use Stomp\Transport\Frame;
use Stomp\Transport\Message;

class Connection {

    /** @var string */
    protected $host;
    /** @var string */
    protected $port;
    /** @var string */
    protected $login;
    /** @var string */
    protected $password;
    /** @var string */
    protected $vhost;
    /** @var SimpleStomp */
    protected $stomp = null;
    /** @var bool */
    protected $isSubscribed = false;

    public function __construct($host, $port, $login, $password, $vhost) {
        $this->host = $host;
        $this->port = $port;
        $this->login = $login;
        $this->password = $password;
        $this->vhost = $vhost;
    }

    protected function connect() {
        if ($this->stomp === null) {
            $client = new \Stomp\Client('tcp://' . $this->host . ':' . $this->port);
            $client->setLogin($this->login, $this->password);
            $client->setVhostname($this->vhost);
            $client->connect();
            $this->stomp = new SimpleStomp($client);
        }
    }

    /**
     * @param string $queue
     * @param string $message
     * @param array  $headers
     *
     * @return void
     */
    public function sendMessage($queue, $message, $headers = array()) {
        $this->connect();
        $this->stomp->send($queue, new Message($message, $headers));
    }

    /**
     * @param string $queue
     * @param string $subscriptionId
     * @param string $ackMode
     * @param array  $headers
     *
     * @return void
     */
    public function subscribe($queue, $subscriptionId = null, $ackMode = 'client', $headers = array()) {
        $this->connect();
        $this->isSubscribed = $this->stomp->subscribe($queue, $subscriptionId, $ackMode, null, $headers);
    }

    /**
     * @return false|\Stomp\Transport\Frame
     *
     * @throws \Exception if not subscribded
     */
    public function waitForFrame() {
        if (!$this->isSubscribed)
            throw new \Exception("Can't wait for a message without be subscribed to a queue.");

        return $this->stomp->read();
    }

    /**
     * @param Frame $frame
     *
     * @return void
     */
    public function ackFrame(Frame $frame) {
        $this->stomp->ack($frame);
    }

    /**
     * @param Frame $frame
     *
     * @return void
     */
    public function nackFrame(Frame $frame) {
        $this->stomp->nack($frame);
    }
}